/*
* Asterisk-GUI	- an Asterisk configuration interface
*
* Backup functions
*
* Copyright (C) 2006-2011, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
* Erin Spiceland <espiceland@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Added support for file backups
 * Added new layout to backup according to new requirement
 * Added size limitation on the upload backup file
 */
var $P = top.$;
var mWindow = top.window.frames["mainscreen"]; // this will be the main window 
var bkpPath = top.sessionData.directories.ConfigBkp;
var udisk_bkpPath = "";
var upload_Filename = ""; // will be updated by upload_form.html
var starteduploading = 0;
var upload_Path; // path for 'uploads' as defined in http.conf - this variable will be automatically updated from http.conf
var tables = [];
var schemas = {};
var cached_data = new Object();
var usb_path = "";
var fileArray = [];

window.onload = function()
{
    $P.lang(document, true);
    var mode = gup("mode");
    var item = gup("item");
    if (mode)
    {
        if (mode == "add")
        {
            prepareAddItemForm();
        }
        init_validator();
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG633"));
        $("title").attr("locale", "LANG633");
        localajaxinit_2();
        if (typeof tmp_check_perms_guibkps !== "undefined")
        {
            tmp_check_perms_guibkps();
        };
    }
}

function HideDownloadLink()
{
    $('#backup_download_Link').hide();
}
function DataBindListfile(val)
{
    var _bft = _$('bkpfilesTable');
    fileArray = [];
    if ($P.gs.isObject(val))
    {
        $.each(val["backup-files"], function (key, fileList)
        {
            var filename_arr = fileList["filelist"].split(".tar ");
            for(var i = 0; i < filename_arr.length; i++ )  //to eliminate empty string in the array
            {
                var namepath_pair = {
                    filename: "",
                    filepath: ""
                };
                if(filename_arr[i].length == 0 || /^[-a-zA-Z0-9_]+$/.test(filename_arr[i]) == false)
                {
                    continue;
                }
                else
                {
                    namepath_pair.filename = filename_arr[i];
                    namepath_pair.filepath = fileList["path"];
                    fileArray.push(namepath_pair);
                }    
                
            }
            
        });
        var sortedFileArray = sortBackupFile(fileArray);
        for (var i = 0; i < sortedFileArray.length; i++)
        {
            if (_bft) {
                var size = _bft.rows.length;
                if (sortedFileArray[i].full_name.length > 0)
                {
                    var filename = sortedFileArray[i].full_name + ".tar";
                    var filepath = sortedFileArray[i].path;
                    addrow_totable(sortedFileArray[i], size);
                }
            }
        }
        
    }
    top.dialog.clearDialog();
}
function localajaxinit_2()
{
    var bkpfiles = "";
    var udisk_path = "";
	var tmp_cmd = "action=fileOpMkdir&type=asterisk_dir&data=gui_backups";
	$.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: function(){}});

	var tmp_cmd = "action=fileOpDelete&type=backup&data=tmp";
	var cb = function()
    {
        ASTGUI.domActions.clear_table(_$('bkpfilesTable'));
        $P.gs.actionRequest(document, "getBackupFile", null, {
            "request": ["list-file"],
            "auto": false,
            "onInvalidResponse": onResponseError,
            "onResponseProcessed": DataBindListfile
        });
    }
	$.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: cb});



    $P.gs.actionRequest(document, "getBackupConfig", null, {
        "request": ["backup-conf"],
        "auto": false,
        "onInvalidResponse": onResponseError,
        "onResponseProcessed": DataBind2
    });
}

function restore_uploadedbkpfile()
{

}

function sortCompareTime(a, b)
{

    var cmpa = a.timestamp;
    var cmpb = b.timestamp;
    return cmpb - cmpa;
}

function sortBackupFile(file_arr)
{
    var sorted_filelist = [];
    for (var i = 0; i < file_arr.length; i++)
    {
        var backupfile = {
            path: file_arr[i].filepath,
            full_name: file_arr[i].filename + ".tar",
            display_name : file_arr[i].filename,
            timestamp : 0,
            year: "",
            month: "",
            day: "",
            hour: "",
            minute: "",
            second: ""
        };
        var splitter_index = file_arr[i].filename.lastIndexOf("__"); 
        //split backup file to two parts; first part is the name that user input; second is the date string like 2008Jan03_15_12_14. 
        if(splitter_index > -1) 
        {
            var date_str = "";
            var file_date;
            var second_part = "";
            second_part = file_arr[i].filename.substring(splitter_index+2);
            //filedate = tmp_a.rChop('.tar');
            if(second_part.length > 0)
            {
                filedate_arr = second_part.split("_");
                var day = filedate_arr[0].substr(7);
                var month = filedate_arr[0].substr(4, 3);
                var year = filedate_arr[0].substr(0, 4);
                if(day.length > 0 && month.length > 0 && year.length > 0)
                {
                    date_str = month + " " + day + ", " + year;
                    backupfile.display_name = file_arr[i].filename.substring(0, splitter_index);
                    backupfile.year = year;
                    backupfile.month = month;
                    backupfile.day = day;
                    if (filedate_arr.length >= 4)
                    {
                        backupfile.hour = filedate_arr[1];
                        backupfile.minute = filedate_arr[2];
                        backupfile.second = filedate_arr[3];
                        date_str = date_str + " " + backupfile.hour + ":" + backupfile.minute + ":" + backupfile.second;
                    }
                    filedate = new Date(date_str);
                    backupfile.timestamp = filedate.getTime();
                }
                
            }
            
        }
        sorted_filelist.push(backupfile);   
        
    }
    sorted_filelist.sort(sortCompareTime);
    return sorted_filelist;
}

function addrow_totable(file)
{
    var newRow = _$('bkpfilesTable').insertRow(-1);
    ASTGUI.domActions.tr_addCell(newRow, { html: _$('bkpfilesTable').rows.length, width: '5%'});
    ASTGUI.domActions.tr_addCell(newRow, { html: file.display_name, width: '37%'});
    if (file.year == "" || file.month == "" || file.day == "")
    {
        ASTGUI.domActions.tr_addCell(newRow, { html: "", width: '30%'});
    }
    else if (file.hour == "" || file.minute == "" || file.second == "")
    {
        ASTGUI.domActions.tr_addCell(newRow, { html: file.month.capitalizeFirstChar() + " " + file.day + ", " + file.year, width: '30%'});
    }
    else
    {
        ASTGUI.domActions.tr_addCell(newRow, { html: file.hour + ":" + file.minute + ":" + file.second + " " + file.month.capitalizeFirstChar() + " " + file.day + ", " + file.year, width: '30%'});
    }

    var tmp = "<button class='options download' onclick='dld_bkp(\"" + file.path + "\",\"" + file.full_name + "\")' localeTitle='LANG759' title='"+$P.lang('LANG759')+"'></button>&nbsp;" +
		      "<button class='options restore' onclick='restore_bkp(\"" + file.path + file.full_name + "\")' localeTitle='LANG760' title='"+$P.lang('LANG760')+"'></button>&nbsp;" +
		      "<button class='options del' onclick='delete_bkp(\"" + file.path + "\",\"" + file.full_name + "\")' localeTitle='LANG739' title='"+$P.lang('LANG739')+"'></button>";
    ASTGUI.domActions.tr_addCell(newRow, { html: tmp, align: 'center', width: '28%' });
    $("#bkpfilesTable tbody tr:even").addClass("even");
    $("#bkpfilesTable tbody tr:odd").addClass("odd");
}

function dld_bkp(path, filename)
{
    var tmp_cmd = "action=fileOpMkdir&type=guiinstall_dir&data=config/private";
	$.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: function(){}});
	tmp_cmd = "action=fileOpMkdir&type=guiinstall_dir&data=config/private/bkps";
	$.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: function(){}});	
	tmp_cmd = "action=systemcmdwithoutput&rm -rf " + top.sessionData.directories.ConfigBkp_dldPath + "*";
	var cb1 = function()
    {
        if(!filename.contains(' '))
		    var tmp_cmd = "action=systemcmdwithoutput&ln -s " + path + filename + " " + top.sessionData.directories.ConfigBkp_dldPath + filename;
		else
		    var tmp_cmd = "action=systemcmdwithoutput&ln -s " + path + "'" + filename + "' " + top.sessionData.directories.ConfigBkp_dldPath + "'" + filename + "'";
		var cb2 = function()
        {
            var location_dir = window.location.href;
            /*
            if(location_dir.endsWith('backup.html'))
                location_dir = location_dir.rChop('backup.html');
            else if(location_dir.endsWith('backup.html#'))
                location_dir = location_dir.rChop('backup.html#');
            */
            var px_idx = location_dir.indexOf('backup.html');
            if(px_idx > -1)
                location_dir = location_dir.slice(0,px_idx);

            var download_link = location_dir + "private/bkps/" + filename;
            var save_text = (jQuery.browser.msie) ? "LANG1413" : "LANG1414";
            _$('backup_download_Link_url').innerHTML = '<center><a href="' + download_link + '" onclick="HideDownloadLink();"><b locale="LANG224">'+$P.lang("LANG224")+'</b></a>'+"<br /><font locale='LANG1413 "+save_text+"'>" + $P.lang("LANG1412").format($P.lang(save_text)) + "<font></center>";
            $('#backup_download_Link').show();
        }
		$.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: cb2});
    }
	$.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: cb1});
}


function restore_bkp(filename)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG855"),
        buttons: {
            ok: function () { restore_bkp_step3(filename); }
        }
    });
}

function restore_bkp_step3(file_fullpath)
{
    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG1415") });
    $P.gs.actionRequest(document, "updateRestoreFile", null, {
        "request": ["file-restore=" + file_fullpath],
        "auto": false,
        "onInvalidResponse": onResponseErrorRestore,
        "onResponseProcessed": dataBindRestore
    });
}

function delete_bkp(dir, filename)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(filename),
        buttons: {
            ok: function () { delete_bkp2(dir + filename); }
        }
    });
}

function delete_bkp2(file_fullpath)
{
    if ( file_fullpath.substring(0, 6) == '/media' )
    {
        var device = file_fullpath.substring(7); 
        if ((device.substring(0, 2) == "sd" && 
           device.charAt(2) >= 'a' && device.charAt(2) <= 'z') ||
           (device.substring(0, 6) == "mmcblk" && device.charAt(6) >= '1' && 
           device.charAt(6) <= '2'))
        {
            var tmp_cmd = "action=fileOpDelete&type=media&data=" + top.UrlEscape(device);
            var cb = function()
		    {
            	top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG819"), callback: function () { window.location.reload(); } });
		    }
		    $.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: cb});
        }
	} else {
	    var tmp_str = 'gui_backups/';
	    var pos = file_fullpath.indexOf(tmp_str);
	    if ( pos > -1 ) {
		    var tmp_path = file_fullpath.substring(pos+tmp_str.length, file_fullpath.length);
		    var tmp_cmd = "action=fileOpDelete&type=backup&data="+top.UrlEscape(tmp_path);
		    var cb = function()
		    {
            	top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG819"), callback: function () { window.location.reload(); } });
		    }
		    $.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: cb});
	    } else {
	        /*  error  */
	        top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG820"), callback: function () { window.location.reload(); } });
	    }
	}
}


function prepareAddItemForm()
{
    var months = ["jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec"];
    var today = new Date()
    var year = today.getFullYear();
    var month = months[today.getMonth()];
    var day = today.getDate().addZero();
    var hour = today.getHours().addZero();
    var minute = today.getMinutes().addZero();
    var seconds = today.getSeconds().addZero();
    var bkpfile = "backup_" + year + month + day + "_" + hour + minute + seconds;
    ASTGUI.updateFieldToValue('newbkp_name', bkpfile);
    $(".AA50only").hide();
}

function take_bkp()
{
    top.dialog.dialogInnerhtml({
        dialogTitle: $P.lang("LANG227"),
        frameSrc: "backup_modal.html?mode=add"
    });
}

function check_filetype()
{
    var bkp_data = false;
    usb_path = "";
    var selected = 0;
    $.each(tables, function (key, val)
    {

        $("#" + val + " input").each(function ()
        {
            var id = $(this).attr("id");
            var id_arr = id.split("-");
            var rc_arr = id_arr[1].split("_");
            var pri_key = rc_arr[0];

            if ($(this).attr("checked") == "checked")
            {
                selected++;
            }

            if (pri_key != "1" && $(this).attr("checked") == "checked")
            {
                bkp_data = true;
                return false;
            }

        });

        if (bkp_data == true)
            return false;

    });

    if (bkp_data == true)
    {
        top.dialog.dialogConfirm({
            confirmStr: $P.lang("LANG985"),
            buttons: {
                ok: function ()
                {
                    parent.ASTGUI.listSystemFiles("/media/", function (bkpfiles)
                    {

                        if (bkpfiles.length > 0)
                        {
                            for (var i = 0; i < bkpfiles.length; i++)
                            {
                                if ((bkpfiles[i].substring(0, 2) == "sd" && 
                                     bkpfiles[i].charAt(2) >= 'a' && bkpfiles[i].charAt(2) <= 'z') ||
                                    (bkpfiles[i].substring(0, 6) == "mmcblk" && bkpfiles[i].charAt(6) >= '1' && 
                                    bkpfiles[i].charAt(6) <= '2'))
                                {
                                    usb_path = "/media/" + bkpfiles[i] + "/";
                                    take_bkp();
                                    break;
                                }

                            }

                            if (i >= bkpfiles.length)
                                top.dialog.dialogMessage({ type: 'warning', content: $P.lang("LANG851") });
                        }
                        else
                        {
                            top.dialog.dialogMessage({ type: 'warning', content: $P.lang("LANG851") });
                        }

                    });
                }
            }
        });
    }
    else
    {
        if (selected > 0)
            take_bkp();
        else
            top.dialog.dialogMessage({ type: 'warning', content: $P.lang("LANG852") });
    }

}

function confirm_reboot()
{
    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG832") });

    $.ajax({
        type: "GET",
        url: "../webcgi?action=reboot",
        success: function ()
        {
            setTimeout("top.window.location.reload();", 120000);
        },
        error: function (jqXHR, textStatus, errorThrown)
        {
            top.dialog.clearDialog();
            if (errorThrown != null && errorThrow.length > 0)
                top.dialog.dialogMessage({ type: 'error', content: errorThrown });
            else
                top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG853") });
        }
    })
}

function backup_file()
{

    var _nn = _$('newbkp_name');
    if (!ASTGUI.checkRequiredFields([_nn]) || !ASTGUI.validateFields([_nn]))
    {
        top.dialog.clearDialog();
        return;
    }
    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG854") });

    var months = ["jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec"];
    var today = new Date();
    var year = today.getFullYear();
    var month = months[today.getMonth()];
    var day = today.getDate().addZero();
    var hour = today.getHours().addZero();
    var minute = today.getMinutes().addZero();
    var seconds = today.getSeconds().addZero();
    var bkpfile = _nn.value + "__" + year + month + day + "_" + hour + "_" + minute + "_" + seconds + ".tar";
    var bkppath = "";
    if (mWindow.usb_path.length > 0)
        bkppath = mWindow.usb_path;
    else
        bkppath = "default";

    $P.gs.actionRequest(document, "updateBackupFile", null, {

        "request": ["file-backup=" + bkppath + "," + bkpfile],
        "auto": false,
        "onInvalidResponse": onResponseErrorBackup,
        "onResponseProcessed": dataBindBackup

    });
}

var init_validator = function () {
		if ($("#form").tooltip) {
			$("#form").tooltip();
		}
        $P("#form", document).validate({
            rules: {
                "newbkp_name": {
                    required: true,
                    letterDigitUndHyphen: true
                }
            }
        });
};

var save_entrance = function() {
    if ($P("#form", document).valid())
    {
        saveChanges();
    }
};

function saveChanges()
{
    var post_data = { "backup-conf": {} };
    $.each(mWindow.tables, function (key, val)
    {  //val is table name;
        mWindow.$("#" + val + " input").each(function ()
        {
            var id = $(this).attr("id");
            if ($(this).attr("checked") == "checked")
                cached_data[id] = 1;
            else
                cached_data[id] = 0;
        });
    });

    var tables_all = {};
    $.each(cached_data, function (key, val)
    {
        var id_arr = key.split("-");
        var table_name = id_arr[0];
        var r_c = id_arr[1];
        var rc_arr = id_arr[1].split("_");
        var row_id = rc_arr[0];
        var column_id = rc_arr[1];
        var field_name = mWindow.schemas[table_name][column_id];


        if (tables_all[table_name] != undefined && tables_all[table_name].ids != undefined)
        {
            var data_arr = post_data["backup-conf"][table_name].data;
            if (tables_all[table_name].ids[row_id] != undefined)
            {
                for (var i = 0; i < data_arr.length; i++)
                {
                    if (data_arr[i].id == row_id)
                    {
                        data_arr[i].fields = data_arr[i].fields + "|" + field_name + ": " + val + " ";
                        break;
                    }
                }
            }
            else
            {
                tables_all[table_name].ids[row_id] = 1;
                var data_object = {
                    id: row_id,
                    fields: ""
                };
                data_object.fields = field_name + ": " + val + " ";
                data_arr.push(data_object);
            }
        }
        else
        {
            tables_all[table_name] = {};
            tables_all[table_name].ids = {};
            tables_all[table_name].ids[row_id] = 1;
            var table_object = {
                table: table_name,
                data: []
            }
            var data_object = {
                id: row_id,
                fields: ""
            };
            data_object.fields = field_name + ": " + val + " ";
            table_object.data.push(data_object);
            post_data["backup-conf"][table_name] = table_object;

        }
    });

    var json = JSON.stringify(post_data["backup-conf"]);
    post_data["backup-conf"] = json;

    $P.gs.actionSubmit(document, "updateBackupConfig", null, {
        "dataType": "json",
        "checkReload": false,
        "postToBody": true,
        "data": post_data,
        "onResponseProcessed": backup_file,
        "feedback": false
    });
}

function onResponseError()
{
}

function onResponseErrorBackup(val)
{
    top.dialog.clearDialog();
    top.dialog.dialogConfirm({
        confirmStr: val.body,
        buttons: {
            ok: top.dialog.clearDialog()
        }
    });
}

function onResponseErrorRestore(val)
{
    top.dialog.clearDialog();
    top.dialog.dialogConfirm({
        confirmStr: val.body,
        buttons: {
            ok: top.dialog.clearDialog()
        }
    });
}

function dataBindBackup()
{
    top.dialog.clearDialog();
    mWindow.location.reload();
}

function dataBindRestore(msg)
{
    top.dialog.clearDialog();
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG833"),
        buttons: {
            ok: confirm_reboot
        }
    });
}


function DataBind2(val)
{
    if ($P.gs.isObject(val))
    {
        $.each(val, function (key, table)
        {
            tables.push(key);
            if (!$P.gs.isEmpty(table))
            {
                var container = $("div#outputBody");
                var table_container = $("<table id='" + key + "' align='center' class='tableWidget' cellpadding='0' cellspacing='0' border='0'/>");
                var thead = $("<thead>").appendTo(table_container).addClass("thead");
                var title_tr = $("<tr>");
                var schema_arr = table.schema.split("$");
                schemas[key] = schema_arr;
                var j;
                var col_type = -1;
                var col_selected = -1;
                var col_id = -1;
                var has_rowselector = false;
                for (j = 0; j < schema_arr.length; j++)
                {
                    if (schema_arr[j] == "id")
                        col_id = j;
                    else if (schema_arr[j] == "TYPE")
                        col_type = j;
                    else if (schema_arr[j] == "SELECTED")
                        col_selected = j;
                }
                title_tr.append("<th width='180px' locale='LANG573'>"+$P.lang("LANG573")+"</th>");
                title_tr.append("<th width='290px' locale='LANG1416'>"+$P.lang("LANG1416")+"</th>");
                thead.append(title_tr);

                var record_arr = table.result.split("|");
                for (j = 0; j < record_arr.length; j++)
                {
                    var tr_id = key + "-" + j;
                    var record_tr = $("<tr id='" + tr_id + "'/>");
                    var row_arr = record_arr[j].split("$");
                    var input_size = 0;
                    for (var k = 0; k < row_arr.length; k++)
                    {
                        if (k == col_type)
                        {
                            record_tr.append("<td>" + row_arr[k] + "</td>");
                        }
                        else if (k == col_selected)
                        {
                            id = key + "-" + row_arr[col_id] + "_" + k;
                            if (row_arr[k] == 1)
                                record_tr.append("<td><input type='checkbox' id='" + id + "' name='" + id + "' checked='checked'></td>");
                            else
                                record_tr.append("<td><input type='checkbox' id='" + id + "' name='" + id + "' onclick='confirm(this)'></td>");
                        }
                    }
                    table_container.append(record_tr);
                }
                container.append(table_container);
                $(".tableWidget tbody tr:even").addClass("even");
                $(".tableWidget tbody tr:odd").addClass("odd");
            }
        });
        top.Custom.init(document);
    }
}

function confirm (obj) {
    if(obj.id == "ast_modules-4_2" && !obj.checked) {
        top.dialog.dialogConfirm({
            confirmStr: $P.lang("LANG2642"),
            buttons: {
                cancel: function () { 
                    obj.checked = false;
                    top.Custom.init(document, obj); 
                }
            }
        });
    }
}
function uploadBackupFile()
{
    top.dialog.dialogInnerhtml({
        dialogTitle: $P.lang("LANG2256"),
        frameSrc: "backupfile_upload.html"
    });
}
 
 
