/* Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */
$P = top.$;
var blacklist_array = [];
var origin_blacklist_array = [];
var BLACKLIST = 'blacklist';

window.onload = function ()
{
    $P.lang(document, true);
    top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG667"));
    $("title").attr("locale", "LANG667");
    loadDOMelements();
    initForm();
    return false;
};


var loadDOMelements = function ()
{
    $select_blacklist = $('#select_blacklist');
};

var initForm = function ()
{
    ASTGUI.resetTheseFields(['blacklist_enable', 'select_blacklist']);
    init_blacklist_select();
    init_blacklist_options();

    bind_event();
    $select_blacklist.trigger('lengthchange');
    top.Custom.init(document);
    init_validator();
}

var bind_event = function ()
{
    $select_blacklist.on('lengthchange', function (ev) {
        $('#blacklist_total').html(blacklist_array.length);
    });

    $('#del_btn').click( function (ev) {
        $(':selected', $select_blacklist).each( function (idx, item) {
            $item = $(item);
            blacklist_array.splice(blacklist_array.indexOf($item.val()), 1);
            $item.remove();
        });
        $select_blacklist.trigger('lengthchange');
    });

};

var init_blacklist_options = function () {
    var exten_globals = context2json( {
        "filename": 'extensions.conf',
        "context": 'globals',
        "usf": 1
    });
    if (exten_globals)
    {
        if (exten_globals['BLACKLIST_CHECK'] && exten_globals['BLACKLIST_CHECK'] === '1')
        {
            $('#blacklist_enable').attr('checked', 'checked');
        }
    }
};

//fetch blacklist and append them to select
var init_blacklist_select = function () {
    blacklist_array = [];
    var blacklist_obj = ASTGUI.ASTDB.getAllKeys({ dbname: BLACKLIST});
    $.each(blacklist_obj, function (key, value) {
        blacklist_array.push(key);
    });

    blacklist_array.sortExtension();
    $.each(blacklist_array, function (index, value) {
        var $option = $('<option></option>');
        $option.val(value).text(value).attr('title', value);
        $select_blacklist.append($option);
    });

    origin_blacklist_array = blacklist_array.slice();
};

//check the new number is already exists in blacklist or not
var check_blacklist_existance = function (value, element) {
    if (!value)
    {
        return true;
    }

    if ( blacklist_array.contains(value) )
    {
        return false;

    }
    return true;
};

var init_validator = function ()
{
    $("#add_form").tooltip();
    $P("#add_form", document).validate({
        debug: true,
        rules: {
            "new_number": {
                digits: true,
                //callerid: true,
                customCallback: [$P.lang('LANG2285'), check_blacklist_existance]
            }
        },
        submitHandler: function (ev) {
            if (ev.preventDefault)
            {
                ev.preventDefault();
            }

            var $new_number = $('#new_number');
            var num = $new_number.val();

            if (num == null || num.trim() === "" )
            {
                $new_number.val('');
                return;
            }

            //append new number to select 
            var $option = $('<option></option>');
            $option
                .val(num)
                .attr('title', num)
                .text(num);
            $select_blacklist.append($option);
            blacklist_array.push(num);

            $select_blacklist.val([]);  //clear selection
            $select_blacklist.prop('selectedIndex', $select_blacklist.prop('length') - 1);

            $select_blacklist.trigger('lengthchange');

            //clear input and get focus
            $new_number.val('');
            $new_number.trigger('focus');
        }
    });
};


var save_blacklist = function () 
{
    var number_add = $(blacklist_array).not(origin_blacklist_array);
    var number_del = $(origin_blacklist_array).not(blacklist_array);

    $.each(number_del, function (idx, value) {
        ASTGUI.ASTDB.deleteKey( {
            "dbname": BLACKLIST,
            "key": value
        });
    });

    $.each(number_add, function (idx, value) {
        ASTGUI.ASTDB.updateKey( {
            "dbname": BLACKLIST,
            "key": value,
            "keyvalue": '1'
        });
    });
}

var save_changes = function ()
{
    top.dialog.dialogMessage({
        type: 'loading',
        content: $P.lang("LANG905")
    });

    save_blacklist();

    var save_after = function (success, errMsg) {
        if (success)
        {
            top.dialog.clearDialog();
            top.dialog.dialogMessage({
                "type": 'success',
                "content": $P.lang("LANG815"),
                "callback": function () {
                    top.dialog.clearDialog();
                }
            });
        }
        else
        {
            top.dialog.clearDialog();
            top.dialog.dialogMessage({
                    type: 'error',
                    content: $P.lang("LANG861"),
                    callback: function () {
                        top.dialog.clearDialog();
                    }
            });
        }
    };

    //save blacklist option
    var blacklist_enable = $('#blacklist_enable').attr('checked') ? '1' : '0';
    var action = new listOfActions('extensions.conf');
    action.new_action('autoupdate', 'globals', 'BLACKLIST_CHECK', blacklist_enable);
    action.callActions(save_after);
};

