/*
* Asterisk-GUI  - an Asterisk configuration interface
*
* Calling Rules functions
*
* Copyright (C) 2006-2010, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
* Ryan Brindley <rbrindley@digium.com>
* Erin Spiceland <espiceland@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Order to safari                                
 * add prepend to dial number in outbound route   
 * Added the trunk type for calling rules         
 */
$P = top.$;
var mWindow = window;
var EDIT_CR; //calling rule being edited
var EDIT_CR_RULE;
var isNew;
var OLD_RULES;
var OLD_PRI_TRUNK_IDX;  // for trunk ldap
var typeObj = {
    "voip": "LANG241",
    "analog": "LANG105"
};
window.onload = function ()
{
    $P.lang(document, true);

    //var url = top.$.url(window.location.href, true);
    var mode = gup("mode");
    var item = gup("item");
    load_DOMelements(mode ? true : false);
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        var tmp_someArray = parent.miscFunctions.getAllDestinations();
        tmp_someArray.push({ optionText: $P.lang('LANG231'), optionValue: 'CUSTOM' });
        ASTGUI.selectbox.populateArray('new_crl_localDest', tmp_someArray);
        ASTGUI.domActions.showHideClassByCheckBox('toLocalDest', 'STT_TR_OPTIONS', true);

        ASTGUI.events.add('restore_default_clrs_button', 'click', restore_default_callingRules);

        $('#new_crl_localDest').change(function ()
        {
            if (this.value == 'CUSTOM')
            {
                $('#new_crl_localDest_CUSTOM_container').show();
                $('#new_crl_localDest_CUSTOM').val('').focus();
            } else
            {
                $('#new_crl_localDest_CUSTOM_container').hide();
            }
        });

        if (mode == "add")
        {
            prepareAddItemForm();
        }
        else if (mode == "edit")
        {
            prepareEditItemForm(item);
        }

        OrderTable.table_id = 'table_failover';
        OrderTable.max_failover = (parent.sessionData.GUI_PREFERENCES['failover_trunk_max'] && Number(parent.sessionData.GUI_PREFERENCES['failover_trunk_max']));
        OrderTable.refresh_failover();
        OrderTable.bind_event();
        init_validator();
        isEnableWeakPw();
        top.Custom.init(document);
        var levelVal = $("#new_crl_pri_level").val();
        if (levelVal == "internal") {
            $(".levelTip").show();
        }
        $("#new_crl_pri_level").bind("change", function(ev) {
            levelVal = $("#new_crl_pri_level").val();
            if (levelVal == "internal") {
                $(".levelTip").show();
            }
            else {
                $(".levelTip").hide();
            }
            ev.stopPropagation();
        });
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG655"));
        $("title").attr("locale", "LANG655");
        update_CRLSTable();
        OrderTable.bind_event();
    }
};

var isEnableWeakPw = function () {
    var GUI_PREFERENCES = top.sessionData.GUI_PREFERENCES;
    if (!GUI_PREFERENCES.getProperty('weak_password')) {
        GUI_PREFERENCES  = context2json({ filename: 'guipreferences.conf', context: 'general', usf: 1 });
    }
    if (GUI_PREFERENCES.weak_password == "yes") {
        var new_crl_name = $P.lang("LANG1533");
        var password = $P.lang("LANG73");
        $P("#new_crl_pinset", document).rules("add", {
            minlength: 4,
            checkNumericPw: [ 
                showCheckPassword, {
                    type: "digital",
                    pwsId: "#new_crl_pinset"
                }
            ],
            identical: [new_crl_name, password, $("#new_crl_name")] 
        });
    }
}
var init_validator = function () {
    //init validator
    $P = top.$;
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "new_crl_name": {
                required: true,
                letterDigitUndHyphen: true,
                outgoingRuleNameExists: [OLD_RULES]
            },
            "new_crl_pattern": {
                required: true,
                /*
                dialpattern: ['dial pattern'],
                dialpattern_additional: ['dial pattern']
                */
                //customCallback: [$P.lang('LANG2139'), check_pattern_format]
                customCallback1: [$P.lang('LANG2139'), check_pattern_format],
                customCallback2: ["Invalid format! The format of CID Identification should be numbers.", check_pattern_with_cid]
            },
            "new_crl_pri_level": {
                required: true
            },
            "new_crl_pinset": {
                digits: true
            },
            "toLocalDest": {
                required: true
            },
            "new_crl_trunk": {
                required: true,
                customCallback: [$P.lang("LANG2140"), failoverTrunkConflictValidate]
            },
            "new_crl_tr_stripx": {
                digits: true,
                stripMax: ['pattern length', $('#new_crl_pattern')]
            },
            "new_crl_tr_prepend": {
                phoneNumberOrExtension: true
            },
            "new_crl_fotrunk": {
                //required: '#new_crl_foChkbx:checked',
                required: true,
                notEqualTo: ['Primary Trunk', $('#new_crl_trunk')],
                customCallback: [$P.lang("LANG2140"), failoverTrunkConflictValidate]
            },
            "new_crl_fotr_stripx": {
                digits: true,
                stripMax: ['pattern', $('#new_crl_pattern')]
            },
            "new_crl_fotr_prepend": {
                phoneNumberOrExtension: true
            }
        },
        submitHandler: function(ev) {
            var target = this.submitButton;
            if ($(target).attr('id') !== 'save')
            {
                return;
            }
            new_crl_save_go(ev);
        }
    });

};

var failoverTrunkConflictValidate = function (value, element) {
    if (!value)
    {
        return true;
    }

    var len = OrderTable.lists.length;
    var lists = OrderTable.lists;
    for (var i = 0; i < len; i++)
    {
        if (lists[i].trunk === value)
        {
            //except editing failover itself
            if ( OrderTable.operater_switch !== 'edit' || lists[i].trunk !== OrderTable.click_val)
            {
                return false;
            }
        }
    }
    return true;
};

/*  added by dcwei 2012-09-06   */
var prepareAddItemForm = function ()
{
    isNew = true;
    EDIT_CR = '';
    EDIT_CR_RULE = '';
    OLD_RULES = '';
    OLD_PRI_TRUNK_IDX = '';

    ASTGUI.resetTheseFields([DOM_new_crl_name, DOM_new_crl_pattern, DOM_new_crl_trunk, DOM_new_crl_pri_level, DOM_new_crl_pinset, DOM_new_crl_tr_stripx, /*DOM_new_crl_tr_prepend, DOM_new_crl_tr_filter,*/DOM_new_crl_foChkbx, DOM_new_crl_fotrunk, DOM_new_crl_fotr_stripx, /*DOM_new_crl_fotr_prepend , DOM_new_crl_fotr_filter,*/'toLocalDest', 'new_crl_localDest' /*'new_crl_caller_id'*/]);
	en_db_fofields();
    $("#editForm").show();
    _$('toLocalDest').updateStatus();
};

/*  added by dcwei 2012-09-06   */
var prepareEditItemForm = function (a)
{
    isNew = false;

    //	DOM_new_crl_tr_filter.value = '';
    //	DOM_new_crl_fotr_filter.value = '';

    var c = ASTGUI.cloneObject(parent.sessionData.pbxinfo.callingRules);
    var b = c[a];

    EDIT_CR = a;
    OLD_RULES = a;
    EDIT_CR_RULE = b;
    var tmp_cr = ASTGUI.parseContextLine.obCallingRuleCustom(b);
    //get name from trunk val
    var failover_list_len = tmp_cr.failover_list.length;
    for (var i = 0; i < failover_list_len; i++)
    {
        tmp_cr.failover_list[i].name = getTrunkName(tmp_cr.failover_list[i].trunk);
    }

    OrderTable.lists = tmp_cr.failover_list.slice();

    DOM_new_crl_trunk.selectedIndex = -1;
    DOM_new_crl_fotrunk.selectedIndex = -1;
    DOM_new_crl_pri_level.selectedIndex = -1;

    DOM_new_crl_name.value = EDIT_CR.withOut(ASTGUI.contexts.CallingRulePrefix);
//    DOM_new_crl_name.disabled = true;

    var patterns = [];
    var str_pattern = '';
    for ( var i = 0; i < b.length; i++ )
    {
        var ptn = fetch_pattern(b[i]);
        if ( !patterns.contains(ptn) )
        {
            /*	add '_' to pattern automatically	*/
            patterns.push(ptn);
            if (!(ptn.indexOf('_') == 0))
                ptn = '_' + ptn;
            str_pattern += ptn+'\n';
        }
    }
    DOM_new_crl_pattern.value = str_pattern;
    //	DOM_new_crl_caller_id.value = tmp_cr.callerID;
    ASTGUI.updateFieldToValue(DOM_new_crl_pri_level, tmp_cr.priviledgeLevel);
    DOM_new_crl_pinset.value = tmp_cr.pinset;
    _$('new_crl_localDest').selectedIndex = -1;
    if (tmp_cr.destination)
    {
        ASTGUI.selectbox.selectOption('new_crl_localDest', tmp_cr.destination);
        _$('toLocalDest').checked = true;
    } else
    {
        _$('toLocalDest').checked = false;
        /* if the trunk is not analog, we can use this value as is. If it is,
        we need to figure out if it has been converted to a group_X name */


        if (!parent.pbx.trunks.isAnalog(tmp_cr.firstTrunk))
        {
            ASTGUI.updateFieldToValue(DOM_new_crl_trunk, tmp_cr.firstTrunk);
        } else if (tmp_cr.firstTrunk.indexOf('group_') > -1)
        {
            ASTGUI.updateFieldToValue(DOM_new_crl_trunk, tmp_cr.firstTrunk);
        } else
        {
            //			var trunk1_name = parent.pbx.trunks.getName(tmp_cr.firstTrunk);
            //			var ded_group = parent.pbx.trunks.getDedicatedGroup(trunk1_name);
            // if ded_group is null, the trunk is misconfigured; this should never happen
            // 			ded_group = "group_" + (ded_group ? ded_group : "0");
            // 			ASTGUI.updateFieldToValue(DOM_new_crl_trunk, ded_group);
            ASTGUI.updateFieldToValue(DOM_new_crl_trunk, tmp_cr.firstTrunk);
        }

        OLD_PRI_TRUNK_IDX = DOM_new_crl_trunk.value;

        //	DOM_new_crl_tr_stripx.value = tmp_cr.stripdigits_firstTrunk  ;
        DOM_new_crl_tr_stripx.value = tmp_cr.firstStripx;
        //		DOM_new_crl_tr_prepend.value = tmp_cr.firstPrepend ;
        //		DOM_new_crl_tr_filter.value = tmp_cr.firstFilter;
        ASTGUI.updateFieldToValue('new_crl_tr_prepend', tmp_cr.prepend_firstTrunk);


        if (tmp_cr.secondTrunk)
        {
            DOM_new_crl_foChkbx.checked = true;
            if (!parent.pbx.trunks.isAnalog(tmp_cr.secondTrunk))
            {
                //ASTGUI.selectbox.selectOption(DOM_new_crl_fotrunk, tmp_cr.secondTrunk);
                ASTGUI.updateFieldToValue(DOM_new_crl_fotrunk, tmp_cr.secondTrunk);
            } else if (tmp_cr.secondTrunk.indexOf('group_') > -1)
            {
                ASTGUI.updateFieldToValue(DOM_new_crl_fotrunk, tmp_cr.secondTrunk);
            } else
            {
                //	var trunk2_name = parent.pbx.trunks.getName(tmp_cr.secondTrunk);
                //	var ded_group = parent.pbx.trunks.getDedicatedGroup(trunk2_name);
                // if ded_group is null, the trunk is misconfigured; this should never happen
                //	ded_group = "group_" + (ded_group ? ded_group : "0");
                //	ASTGUI.updateFieldToValue(DOM_new_crl_fotrunk, ded_group);
                ASTGUI.updateFieldToValue(DOM_new_crl_fotrunk, tmp_cr.secondTrunk);
            }
            DOM_new_crl_fotr_stripx.value = tmp_cr.secondStripx;
            ASTGUI.updateFieldToValue('new_crl_fotr_prepend', tmp_cr.prepend_secondTrunk);
            //			DOM_new_crl_fotr_prepend.value = tmp_cr.secondPrepend ;
            //			DOM_new_crl_fotr_filter.value = tmp_cr.secondFilter;
        } else
        {
            DOM_new_crl_foChkbx.checked = false;
        }

        en_db_fofields();
    }
    $("#editForm").show();

    _$('toLocalDest').updateStatus();
};

var en_db_fofields = function ()
{
    /* to remove the failover-enable check box
    var t = !DOM_new_crl_foChkbx.checked;
    DOM_new_crl_fotrunk.disabled = t;
    DOM_new_crl_fotr_stripx.disabled = t;
    DOM_new_crl_fotr_prepend.disabled = t;
    */
};

var load_DOMelements = function (editor)
{
    DOM_tbl_crls = _$('table_CRLS_list');
    // new calling rule dom elements
    DOM_new_cr_button = _$('new_cr_button');

    DOM_new_crl_name = _$('new_crl_name');
    DOM_new_crl_pattern = _$('new_crl_pattern');
    /*  modified by dcwei 2012-09-06    */
    //		DOM_new_crl_caller_id= _$('new_crl_caller_id');
    DOM_new_crl_trunk = _$('new_crl_trunk');
    DOM_new_crl_pri_level = _$('new_crl_pri_level'); /*	added by dcwei 2012-08-27	*/
    DOM_new_crl_pinset = _$('new_crl_pinset');
    DOM_new_crl_tr_stripx = _$('new_crl_tr_stripx');
    //			DOM_new_crl_tr_prepend = _$('new_crl_tr_prepend');
    //			DOM_new_crl_tr_filter = _$('new_crl_tr_filter');
    DOM_new_crl_foChkbx = _$('new_crl_foChkbx');
    DOM_new_crl_fotrunk = _$('new_crl_fotrunk');
    DOM_new_crl_fotr_stripx = _$('new_crl_fotr_stripx');
    //			DOM_new_crl_fotr_prepend = _$('new_crl_fotr_prepend');
    //			DOM_new_crl_fotr_filter = _$('new_crl_fotr_filter');
    DOM_new_crl_fotr_prepend = _$('new_crl_fotr_prepend');
    DOM_new_crl_tr_filter = _$('new_crl_tr_filter');

    // new calling rule dom elements
    if (editor)
    {
        en_db_fofields();
        ASTGUI.events.add(DOM_new_crl_foChkbx, 'change', en_db_fofields);
        //ASTGUI.events.add(DOM_new_cr_button, 'click', prepareAddItemForm);

        // list non-analog trunks by trunk.
        var t = parent.pbx.trunks.list({ iax: true, providers: true, sip: true, analog: true, bri: true, pri: true });
        var TMP_FORSORT = [];
        t.each(function (item)
        {
            TMP_FORSORT.push(parent.pbx.trunks.getName(item) + ':::::::' + item);
        });
        TMP_FORSORT.sort();

        TMP_FORSORT.each(function (this_str)
        {
            var a = this_str.split(':::::::');
			var type = parent.sessionData.pbxinfo.trunks_info[a[1]].trunkstyle;
            var name = "";
            if ( type == 'voip' )
            {
                var tmp_type = parent.sessionData.pbxinfo.trunks_info[a[1]].trunkflags;
                if ( /^sip1$/.test(tmp_type) )
                    name = 'SIP Peer ' + $P.lang('LANG83') + ' -- ' + a[0];
                else if ( /^sip2$/.test(tmp_type) )
                    name = 'SIP Register ' + $P.lang('LANG83') + ' -- ' + a[0];
                else if ( /^iax1$/.test(tmp_type) )
                    name = 'IAX Peer ' + $P.lang('LANG83') + ' -- ' + a[0];
                else if ( /^iax2$/.test(tmp_type) )
                    name = 'IAX Register ' + $P.lang('LANG83') + ' -- ' + a[0];
            } else {
			    name = $P.lang(typeObj[type]) + $P.lang('LANG83') + ' -- ' + a[0];
            }
            ASTGUI.selectbox.append(DOM_new_crl_trunk, name, a[1]);
            ASTGUI.selectbox.append(DOM_new_crl_fotrunk, name, a[1]);
        });

        /*  added by dcwei 2012-09-06   */
        // ybiaozhou move to static html page
        //ASTGUI.selectbox.append(DOM_new_crl_pri_level, "Local", "local");
        //ASTGUI.selectbox.append(DOM_new_crl_pri_level, "National", "national");
        //ASTGUI.selectbox.append(DOM_new_crl_pri_level, "International", "international");

        // list analog trunks by group.
        //		var g = parent.pbx.trunks.listAllGroups();
        //
        //		g.each( function(group){
        //			ASTGUI.selectbox.append( DOM_new_crl_trunk, parent.pbx.trunks.getGroupDescription(group), 'group_' + group);
        //			ASTGUI.selectbox.append( DOM_new_crl_fotrunk , parent.pbx.trunks.getGroupDescription(group), 'group_' + group);
        //		});
        //
        var modules_show = ASTGUI.cliCommand('module show');

        if (modules_show.contains('res_skypeforasterisk') && modules_show.contains('chan_skype'))
        {
            ASTGUI.selectbox.append(DOM_new_crl_trunk, $P.lang('LANG239'), 'Skype');
            ASTGUI.selectbox.append(DOM_new_crl_fotrunk, $P.lang('LANG239'), 'Skype');
        }
    }
};

moveCrUpDown = function (context, rule, updown)
{
    ASTGUI.miscFunctions.moveUpDown_In_context(context, rule, updown, function (newcontext)
    {
        parent.sessionData.pbxinfo.callingRules[context] = newcontext;
        top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG815"), callback: function () { mWindow.location.reload(); } });
    });
};

/*  added by dcwei 2012-09-06   */
var update_CRLSTable = function ()
{
    //DOM_tbl_crls
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function
    (function ()
    { // add first row
        if (DOM_tbl_crls) {
            var newRow = DOM_tbl_crls.insertRow(-1);
            newRow.className = "frow";
            addCell(newRow, { html: $P.lang('LANG240'), locale: 'LANG240', width: '100px' });
            addCell(newRow, { html: $P.lang('LANG656'), locale: 'LANG656' });
            addCell(newRow, { html: $P.lang('LANG246'), locale: 'LANG246' });
            addCell(newRow, { html: $P.lang('LANG1543'), locale: 'LANG1543' });   //
            addCell(newRow, { html: $P.lang('LANG74'), locale: 'LANG74' });
        }
    })();

    var PreviousTRColor = 'odd'; // 'odd' : 'even' ;

    (function ()
    {
        var c = ASTGUI.cloneObject(parent.sessionData.pbxinfo.callingRules);
        var outbound_lists = parent.sessionData.pbxinfo.outbound_allroutes;
        var priviledgeLangMap = {
            "INTERNAL"       : 'LANG1071',
            "LOCAL"          : 'LANG1072',
            "NATIONAL"       : 'LANG1073',
            "INTERNATIONAL"  : 'LANG1074' 
        };

        //for (var d in c)
        for ( var idx = 0; idx < outbound_lists.length; idx++ )
        {
            var d = outbound_lists[idx];
            d = d.substring(d.indexOf('=')+1);
            if (c.hasOwnProperty(d))
            {
                var crd = c[d]; // temporarily store all the details of this calling  rule
                var tmp = idx+1;
                PreviousTRColor = (PreviousTRColor == 'odd') ? 'even' : 'odd';

                var this_crl_set = ASTGUI.cloneObject(c[d]);
                var crl = d.withOut(ASTGUI.contexts.CallingRulePrefix);

                var tmp_cr = ASTGUI.parseContextLine.obCallingRuleCustom(this_crl_set);
                if (DOM_tbl_crls) {
                    var newRow = DOM_tbl_crls.insertRow(-1);
                    newRow.className = PreviousTRColor;
                    newRow.id = 'rule_' + crl;
                    //            addCell( newRow , { html: tmp_movepriorities } )
                    addCell(newRow, { html: ''+tmp });
                    addCell(newRow, { html: crl });

                    var patterns = [];
                    var str_patterns = '';
                    for ( var i = 0; i < this_crl_set.length; i++ )
                    {
                        var ptn = fetch_pattern(this_crl_set[i]);
                        if ( !patterns.contains(ptn) )
                        {
                            patterns.push(ptn);
                            str_patterns += '<div>' + ptn + '</div>';
                        }
                    }
                    addCell(newRow, { html: str_patterns});

                    if (tmp_cr.firstTrunk)
                    {
                    priviledgeLangMap 
                        var priviledgeLang = priviledgeLangMap[tmp_cr.priviledgeLevel.toUpperCase()];
                        //               addCell( newRow , { html: tmp_cr.callerID });
                        addCell(newRow, { html: $P.lang(priviledgeLang), locale: priviledgeLang});
                    } else
                    {
                        addCell(newRow, { html: '<i><font color=blue>' + $P.lang('LANG1554') +': ' + ASTGUI.parseContextLine.showAs(tmp_cr.destination) + '</font></i>', align: 'left', colspan: 2 });
                    }

                    tmp = "<button class='options edit' onclick=\"showEditForm('" + d + "')\" localeTitle='LANG738' title='" + $P.lang("LANG738") + "'></button>&nbsp;" +
                      "<button class='options del' onclick=\"delete_CR_confirm('" + d + "','')\" localeTitle='LANG739' title='" + $P.lang("LANG739") + "'></button>" +
                      "<button class='rule_clickTop options' localeTitle='LANG793' title='" + $P.lang('LANG793') + "' onClick='rule_top(" + "\"rule_" + crl + "\")'></button>" +
                      "<button class='rule_clickUp options' localeTitle='LANG794' title='" + $P.lang('LANG794') + "'onClick='rule_up(" + "\"rule_" + crl + "\")'></button>" +
                      "<button class='rule_clickDown options' localeTitle='LANG795' title='" + $P.lang('LANG795') +"' onClick='rule_down(" + "\"rule_" + crl + "\")'></button>" +
                      "<button class='rule_clickBottom options' localeTitle='LANG796' title='" + $P.lang('LANG796') +"' onClick='rule_bottom(" + "\"rule_" + crl + "\")'></button>";
                    addCell(newRow, { html: tmp });
                    $(newRow).attr('value', idx);
                }
            } 
        }
        if (DOM_tbl_crls) {
            if (DOM_tbl_crls.rows.length == 1)
            {
                ASTGUI.domActions.clear_table(DOM_tbl_crls);
                var newRow = DOM_tbl_crls.insertRow(-1);
                newRow.className = 'even';
                addCell(newRow, { html: $P.lang('LANG129').format($P.lang('LANG655')), locale: 'LANG129 LANG655' });
                return;
            }
        }
    })();
};

var trunkHtml = function (trunk)
{
    if (!trunk) { return '<i><font color=red>None Assigned</font></i>'; }
    var trunk_name = parent.pbx.trunks.getName(trunk);
    if (trunk_name) { return trunk_name; }
    if (trunk.indexOf('group_' > -1))
    { // it's a new-style analog trunk
        var group = trunk.replace('group_', "");
        var tr = parent.pbx.trunks.getTrunkNamesByGroup(group);
        var trstr = tr.join(", ");
        if (trstr.length > 30)
        {
            trstr = trstr.substr(30) + '...';
        }
        return "Group " + group + "(" + trstr + ")";
    }
    return '<i><font color=red>Invalid Trunk\"' + trunk + '\"</font></i>';
};




var restore_default_callingRules = function ()
{
    if (!confirm('This would restore the factory default Calling Rules.'
		+ '\n Any other custom calling rules will not be edited or deleted.'
		+ '\n Click OK to continue ')) return;


    var x = new listOfActions('extensions.conf');
    x.new_action('delcat', 'CallingRule_Longdistance', '', '');
    x.new_action('newcat', 'CallingRule_Longdistance', '', '');
    x.new_action('append', 'CallingRule_Longdistance', 'exten', '_91XXXXXXXXXX!,1,Macro(' + ASTGUI.contexts.dialtrunks + ',${}/${FILTER(0123456789,${EXTEN:1})}, , , )');

    x.new_action('delcat', 'CallingRule_IAXTEL', '', '');
    x.new_action('newcat', 'CallingRule_IAXTEL', '', '');
    x.new_action('append', 'CallingRule_IAXTEL', 'exten', '_91700XXXXXXX!,1,Macro(' + ASTGUI.contexts.dialtrunks + ',${}/${FILTER(0123456789,${EXTEN:1})}, , , )');

    x.new_action('delcat', 'CallingRule_Local_AreaCode', '', '');
    x.new_action('newcat', 'CallingRule_Local_AreaCode', '', '');
    x.new_action('append', 'CallingRule_Local_AreaCode', 'exten', '_9256XXXXXXX!,1,Macro(' + ASTGUI.contexts.dialtrunks + ',${}/${FILTER(0123456789,${EXTEN:4})}, , , )');

    x.new_action('delcat', 'CallingRule_International', '', '');
    x.new_action('newcat', 'CallingRule_International', '', '');
    x.new_action('append', 'CallingRule_International', 'exten', '_9011XXXXX.,1,Macro(' + ASTGUI.contexts.dialtrunks + ',${}/${FILTER(0123456789,${EXTEN:1})}, , , )');

    x.new_action('delcat', 'CallingRule_Local_7_digits', '', '');
    x.new_action('newcat', 'CallingRule_Local_7_digits', '', '');
    x.new_action('append', 'CallingRule_Local_7_digits', 'exten', '_9XXXXXXX!,1,Macro(' + ASTGUI.contexts.dialtrunks + ',${}/${FILTER(0123456789,${EXTEN:1})}, , , )');

    x.new_action('delcat', 'CallingRule_Emergency', '', '');
    x.new_action('newcat', 'CallingRule_Emergency', '', '');
    x.new_action('append', 'CallingRule_Emergency', 'exten', '_911!,1,Macro(' + ASTGUI.contexts.dialtrunks + ',${}/${FILTER(0123456789,${EXTEN:1})}, , , )');


    x.callActions(function ()
    {
        top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG942"), callback: function () { top.window.location.reload(); } });
    });
};

/*  added by dcwei 2012-09-06   */
var new_crl_save_go = function (ev)
{
	var levelVal = $("#new_crl_pri_level").val();
    if (levelVal == "internal") {
        top.dialog.dialogConfirm({
            type: "warning",
            confirmStr: $P.lang("LANG2534").format($P.lang("LANG1071"), $P.lang("LANG1071")),
            buttons: {
                ok:  function (){
                    doNew_crl_save_go();
                },
                cancel:  function (){
                    top.dialog.container.show();
                    top.dialog.shadeDiv.show();
                }
            }
        });        
    }
    else {
        doNew_crl_save_go();  
    }
}
var doNew_crl_save_go = function (ev) {
    var tmp_crl_name = DOM_new_crl_name.value;
    tmp_crl_name = 'outrt_' + tmp_crl_name;
    tmp_crl_name = tmp_crl_name.toLowerCase();
    //	var caller_id = ASTGUI.getFieldValue('new_crl_caller_id');
    //	if(/[^\w\d\s_\-"<>]/.test(caller_id)){
    //		ASTGUI.feedback( { msg:'Invalid Caller ID format!', showfor:2, color:'red' });
    //		return ;
    //	}
    //	if (caller_id && caller_id != 'undefined'){
    //		caller_id = ',' + caller_id;
    //	}else{
    //		caller_id = '';
    //	}

    if (_$('toLocalDest').checked)
    {
        var tmp_new_crl_localDest = ASTGUI.getFieldValue('new_crl_localDest');
        if (tmp_new_crl_localDest == 'CUSTOM')
        {
            /*	added '_' to pattern automatically when needed	*/
            if (!(DOM_new_crl_pattern.value.indexOf('_') == 0))
                DOM_new_crl_pattern.value = '_' + DOM_new_crl_pattern.value;
            var as = DOM_new_crl_pattern.value + ',1,' + ASTGUI.getFieldValue('new_crl_localDest_CUSTOM'); ////// <<<<<<<<<<<<<<<<<<<
        } else
        {
            /*			var as = DOM_new_crl_pattern.value + ',1,';
            if(caller_id){
            var args = ASTGUI.parseContextLine.getArgs(DOM_new_crl_pattern.value + ',1,'+ tmp_new_crl_localDest);
            as += "Macro(local-callingrule-cid-0.1," + args[0] + ',' + args[1] + ',' + args[2] + caller_id + ')';
            }else{
            as += tmp_new_crl_localDest;
            }
            */
        }
    } else
    {
        var t1 = ASTGUI.getFieldValue(DOM_new_crl_trunk);
        var t2 = (DOM_new_crl_foChkbx.checked) ? ASTGUI.getFieldValue(DOM_new_crl_fotrunk) : '';
        var pri_level = ASTGUI.getFieldValue(DOM_new_crl_pri_level);
        var pinset = DOM_new_crl_pinset.value.trim();

        /*
        var g1cid = '';
        var g2cid = '';
        if(t1.indexOf("group_") > -1){
        g1cid = parent.pbx.trunks.getTrunkIdByName(t1);
        }
        if(t2.indexOf("group_") > -1){
        g2cid = parent.pbx.trunks.getTrunkIdByName(t2);
        }
        */
        var trunk_index = (t1.split("_"))[1];
        var tmp_stripx = Number(DOM_new_crl_tr_stripx.value) || '0';
        var trunk_ldap_info = { 
                                "old_idx": '',
                                "idx": trunk_index,
                                "strip": tmp_stripx
        };

        var tmp_prepend = $('#new_crl_tr_prepend').val().trim() || '';
        var tmp_fotr_prepend = $('#new_crl_fotr_prepend').val().trim() || '';

        var tmp_fotr_stripx = Number(DOM_new_crl_fotr_stripx.value) || '0';
        var tmp_checkThis = (DOM_new_crl_pattern.value.beginsWith('_')) ? DOM_new_crl_pattern.value.length - 1 : DOM_new_crl_pattern.value.length;

        //		var t1_braces = (t1 == 'Skype') ? t1 : '${' + t1 + '}' ;
        var t1_braces = '';
        var Trunk_Build_str = tmp_prepend + '${EXTEN:' + tmp_stripx + '}';
        //		if(DOM_new_crl_tr_filter.value != ''){
        //			Trunk_Build_str = '${FILTER(' + DOM_new_crl_tr_filter.value +',' + Trunk_Build_str + ')}' ;
        //		}
        //		Trunk_Build_str = ',' + t1_braces + '/' + DOM_new_crl_tr_prepend.value + Trunk_Build_str;
        var foTrunk_Build_str = '';

        if (DOM_new_crl_foChkbx.checked)
        {
            //			var t2_braces = (t2 == 'Skype') ? t2 : '${' + t2 + '}' ;
            foTrunk_Build_str += tmp_fotr_prepend + '${EXTEN:' + tmp_fotr_stripx + '}';
            //			if(DOM_new_crl_fotr_filter.value != ''){
            //				foTrunk_Build_str = '${FILTER(' + DOM_new_crl_fotr_filter.value + ',' + foTrunk_Build_str + ')}' ;
            //			}
            //			foTrunk_Build_str = t2_braces + '/' + DOM_new_crl_fotr_prepend.value + foTrunk_Build_str;
        }
        //		foTrunk_Build_str = ',' + foTrunk_Build_str;

        //		var t1_cidarg = ( t1 == 'Skype') ? ',' : ',' + (g1cid ? g1cid : t1);
        //		var t2_cidarg = ( t2 == 'Skype') ? ',' : ',' + (g2cid ? g2cid : t2);
        //		var as = DOM_new_crl_pattern.value + ',1,Macro(' + ASTGUI.contexts.dialtrunks + Trunk_Build_str + foTrunk_Build_str + t1_cidarg + t2_cidarg + caller_id + ')' ;
        /*	added '_' to pattern automatically when needed	*/
        var patterns = DOM_new_crl_pattern.value.split('\n');
        var as_rules = [];
        for ( var ii = 0; ii < patterns.length; ii++ )
        {
            var pattern = patterns[ii];
            if ( pattern == '' )
                continue;
            if (pattern == '.' || pattern == '_.' ) {
                top.dialog.dialogMessage({
                    type: 'warning',
                    content: $P.lang("LANG941").format(pattern)
                });
                return ;
            }
            if (!(pattern.indexOf('_') == 0))
                pattern = '_' + pattern;
            var tmp_rule = pattern + ',1,Macro(user-callerid,SKIPTTL,)';
            as_rules.push(tmp_rule);
            tmp_rule = pattern + ',n,Set(_NODEST=)';
            as_rules.push(tmp_rule);
            tmp_rule = pattern + ',n,Macro(check-permission,${AAUSER},' + pri_level + ')';
            as_rules.push(tmp_rule);
            tmp_rule = pattern + ',n,Macro(dialout-trunk,' + trunk_index + ',' + Trunk_Build_str + ',' + pinset + ')';
            as_rules.push(tmp_rule);

            //generate failover trunk dial plan
            var list_len = OrderTable.lists.length;
            var as_fo = [];
            for (var i = 0; i < list_len; i++)
            {
                //var trunk_index_fo = (t2.split("_"))[1];
                //var as_fo = DOM_new_crl_pattern.value + ',n,Macro(dialout-trunk,' + trunk_index_fo + ',' + foTrunk_Build_str + ',)';
                //
                var item = OrderTable.lists[i];
                foTrunk_Build_str = item.prepend + '${EXTEN:' + item.strip + '}';
                var trunk_index_fo = item.trunk.split('_')[1];
                var dialstr = pattern + ',n,Macro(dialout-trunk,' + trunk_index_fo + ',' + foTrunk_Build_str + ',)';
                as_fo.push(dialstr);
                as_rules.push(dialstr);
            }

            tmp_rule = pattern + ',n,Macro(outisbusy,)';
            as_rules.push(tmp_rule);
        }
    }


    if (isNew)
    {
        var tmp_crl = 'include=' + tmp_crl_name;
        parent.sessionData.pbxinfo['outbound_allroutes'].push(tmp_crl);
        parent.pbx.calling_rules.add1(tmp_crl_name, as_rules);
        top.dialog.dialogMessage({
            type: 'success',
            content: $P.lang("LANG815"),
            callback: function ()
            {
                mWindow.location.reload();
            }
        });
    } 
    else
    {
        //top.dialog.dialogMessage({ type: 'loading', content: 'Updating ...' });
        var old_index = OLD_PRI_TRUNK_IDX.match('trunk_([0-9]+)')[1];
        trunk_ldap_info['old_idx'] = old_index;
        
        EDIT_CR_RULE = as_rules;
        if (parent.pbx.calling_rules.edit1(EDIT_CR, tmp_crl_name, EDIT_CR_RULE, trunk_ldap_info))
        {
            top.dialog.dialogMessage({
                type: 'success',
                content: $P.lang("LANG815"),
                callback: function ()
                {
                    mWindow.location.reload();
                }
            });
        }
        else
        {
            top.dialog.dialogMessage({
                type: 'error',
                content: $P.lang("LANG862")
            });
        }
    }
};

// TODO [AH] bad param name, what is a and b???
var delete_CR_confirm = function (a, b)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(a),
        buttons: {
            ok: function ()
            {
                /*  delete from sessionData  */
                var lists = parent.sessionData.pbxinfo['outbound_allroutes'];
                var rule = 'include='+a;
                for ( var i = 0; i < lists.length; i++ )
                {
                    if ( lists[i] == rule )
                    {
                        lists.splice(i,1);
                        break;
                    }
                }
                parent.pbx.calling_rules.remove1(a, b);
                top.dialog.dialogMessage({
                    type: 'success',
                    content: $P.lang("LANG816"),
                    callback: function ()
                    {
                        mWindow.location.reload();
                    }
                });
            }
        }
    });
};



var OrderTable = {
    "lists": [],
    "table_id": '',
    "operater_switch": true,
    "newCell": [],
    "click_val" : null,
    "bind_event": function () {
        var TBL = _$(this.table_id);

        $(TBL).on('click', '.table-add', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }

            if (OrderTable.max_failover > 0 && OrderTable.max_failover <= OrderTable.lists.length)
            {
                var target = $(this);
                // XXX it doesn't work until I set tooltip position twice.
                $(this).tooltip({ position: {
                         my: "right center", 
                         at: "left center" 
                     }
                });
                    
                var max_msg = $P.lang("LANG2141").format(OrderTable.max_failover);
                $(this).attr("titles", max_msg).addClass("ui-state-highlight").trigger("focusin");
                setTimeout( function () {
                   target.removeAttr("titles").removeClass("ui-state-highlight").trigger("focusout");

                }, 2000);
                return;
            }

            var clickRow = $(this).parent().parent();
            var newCell = document.createElement('tr');
            OrderTable.newCell = $(newCell);
            $(newCell).css('background', '#CAECFF');

            var cell = document.createElement('td');
            $(cell).attr('colspan', 4);
            newCell.appendChild(cell);
            //$(newCell).append($('<td /><td /><td /><td />'));


            //new add div
            var div = document.createElement('div');
            $(div).attr('class', 'testdiv');
            cell.appendChild(div);
            $('#table_edit_template').appendTo($(div));
            clickRow.after(newCell);
            $('#modal_btn').css('visibility', 'hidden');
            $('#table-add-btn').show();
            $('#table-edit-btn').hide();

            //default edit value
            OrderTable.set_edit_values(-1);

            if (clickRow.attr('data-item-val') === 'add_item')
            {
                clickRow.hide();
            }

            //$('#table-add-btn').show();
            OrderTable.click_val = $(this).parent().parent().attr('data-item-val');
            //top.Custom.init(document, $('#table_edit_template')[0]);
            //reset innerhtml height
            if (window.frameElement)
            {
                $(window.frameElement).css("height","0px");
            }
            top.dialog.repositionDialog();

            OrderTable.operater_switch = 'add';
            //TODO
            // disable operator -- add/edit/up/down...
        });


        $(TBL).on('click', '.table-del', function (ev) {

            if (OrderTable.operater_switch !== true)
            {
                return false;
            }

            var clickRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('trunk', clickRow.attr('data-item-val'));

            //remove select item from lists
            OrderTable.lists.splice(curIdx, 1);

            OrderTable.refresh_failover();


            //reset innerhtml height
            if (window.frameElement)
            {
                $(window.frameElement).css("height","0px");
            }
            top.dialog.repositionDialog("none");
        });


        $(TBL).on('click', '.table-edit', function (ev) {

            if (OrderTable.operater_switch !== true)
            {
                return false;
            }

            var clickRow = $(this).parent().parent();
            var newCell = document.createElement('tr');
            OrderTable.newCell = $(newCell);
            $(newCell).css('background', '#CAECFF');

            var cell = document.createElement('td');
            $(cell).attr('colspan', 4);
            newCell.appendChild(cell);
            //$(newCell).append($('<td /><td /><td /><td />'));

            var curIdx = OrderTable.getOptionOrder('trunk', clickRow.attr('data-item-val'));
            OrderTable.set_edit_values(curIdx);

            //new add div
            var div = document.createElement('div');
            $(div).attr('class', 'testdiv');
            cell.appendChild(div);
            $('#table_edit_template').appendTo($(div));

            clickRow.after(newCell);
            $('#table-add-btn').hide();
            $('#table-edit-btn').show();
            $('#modal_btn').css('visibility', 'hidden');
            clickRow.hide();

            //$('#table-add-btn').show();
            OrderTable.click_val = $(this).parent().parent().attr('data-item-val');

            //reset innerhtml height
            if (window.frameElement)
            {
                $(window.frameElement).css("height","0px");
            }
            top.dialog.repositionDialog("none");

            OrderTable.operater_switch = 'edit';
        });


        $(TBL).on('click', '.table-up', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }

            var currentRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('trunk', currentRow.attr('data-item-val'));
            if (curIdx <= 0)
            {
                return;
            }

            var prevRow = currentRow.prev();
            var prevIdx = OrderTable.getOptionOrder('trunk', prevRow.attr('data-item-val'));

            //row.before(prev);
            var lists = OrderTable.lists;
            var tmp = lists[curIdx];
            lists[curIdx] = lists[prevIdx];
            lists[prevIdx] = tmp;

            OrderTable.refresh_failover();
        });

        $(TBL).on('click', '.table-down', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }
            var currentRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('trunk', currentRow.attr('data-item-val'));
            var last = OrderTable.lists.length - 1;
            if (curIdx >= last)
            {
                return;
            }

            var nextRow = currentRow.next();
            var nextIdx = OrderTable.getOptionOrder('trunk', nextRow.attr('data-item-val'));

            //row.before(prev);
            var lists = OrderTable.lists;
            var tmp = lists[curIdx];
            lists[curIdx] = lists[nextIdx];
            lists[nextIdx] = tmp;

            OrderTable.refresh_failover();
        });

        $(TBL).on('click', '.table-top', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }
            var currentRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('trunk', currentRow.attr('data-item-val'));
            if (curIdx <= 0)
            {
                return;
            }

            var item = OrderTable.lists[curIdx];
            OrderTable.lists.splice(curIdx, 1);
            OrderTable.lists.splice(0, 0, item);

            OrderTable.refresh_failover();
        });


        $(TBL).on('click', '.table-bottom', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }
            var currentRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('trunk', currentRow.attr('data-item-val'));
            var lastIdx = OrderTable.lists.length - 1;
            if (curIdx >= lastIdx)
            {
                return;
            }

            var item = OrderTable.lists[curIdx];
            OrderTable.lists.splice(curIdx, 1);
            OrderTable.lists.splice(lastIdx, 0, item);

            OrderTable.refresh_failover();
        });

    },
    "getOptionOrder": function (key, val) {
        var len = this.lists.length;
        for ( var i = 0; i < len; i++)
        {
            if (this.lists[i][key] === val)
            {
                return i;
            }
        }

        return -1;
    },
    "tbl_btn_cancel": function (ev) {
        $('#table_edit_template').appendTo($('#table_template_placeholder'));
        $('#modal_btn').css('visibility', 'visible');
        $('[data-item-val="' + this.click_val + '"]').show();

        this.newCell.remove();
        this.newCell = [];

        //reset innerhtml height
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog("none");
        OrderTable.operater_switch = true;
        OrderTable.click_val = null;
    },
    "tbl_btn_add": function (ev) {

        if (!$P("#new_crl_fotrunk, #new_crl_fotr_stripx, #new_crl_fotr_prepend", document).valid())
        {
            return;
        }

        var trunk_val = $('#new_crl_fotrunk').val() || '';
        /*
        var pbxinfo = parent.sessionData.pbxinfo.trunks_info[trunk_val];
        var type = pbxinfo.trunkstyle || '';
        var name = pbxinfo.trunkname || '';
        */
        var trunkName = getTrunkName(trunk_val);
        var newTrunk = {
            "trunk": trunk_val,
            "name": trunkName || '',
            "strip": Number( $('#new_crl_fotr_stripx').val() ) || 0,
            "prepend": $('#new_crl_fotr_prepend').val().trim() || ''
        };
        $('#table_edit_template').appendTo($('#table_template_placeholder'));
        $('#modal_btn').css('visibility', 'visible');

        //this.newCell.remove();
        this.newCell = [];
       
        var order = this.getOptionOrder('trunk', this.click_val);
        this.lists.splice(order + 1, 0, newTrunk);
        this.refresh_failover();

        //reset innerhtml height
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog("none");
        OrderTable.operater_switch = true;
        OrderTable.click_val = null;
    },
    "tbl_btn_update": function (ev) {
        if (!$P("#new_crl_fotrunk, #new_crl_fotr_stripx, #new_crl_fotr_prepend", document).valid())
        {
            return;
        }

        var trunk_val = $('#new_crl_fotrunk').val() || '';
        var order = this.getOptionOrder('trunk', this.click_val);
        /*
        var pbxinfo = parent.sessionData.pbxinfo.trunks_info[trunk_val];
        var type = pbxinfo.trunkstyle || '';
        var name = pbxinfo.trunkname || '';
        */
        var trunkName = getTrunkName(trunk_val);

        this.lists[order] = {
            "trunk": trunk_val,
            "name": trunkName || '',
            "strip": Number( $('#new_crl_fotr_stripx').val() ) || 0,
            "prepend": $('#new_crl_fotr_prepend').val().trim() || ''
        };

        $('#table_edit_template').appendTo($('#table_template_placeholder'));
        $('#modal_btn').css('visibility', 'visible');

        //this.newCell.remove();
        this.newCell = [];

        this.refresh_failover();

        //reset innerhtml height
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog("none");
        OrderTable.operater_switch = true;
        OrderTable.click_val = null;
    },
    "refresh_failover": function () {
        var TBL = _$(this.table_id);
        $(TBL).children().remove();
        var addCell = ASTGUI.domActions.tr_addCell;



        (function ()
        { // add first row
            var newRow = TBL.insertRow(-1);
            newRow.className = "frow";
            addCell(newRow, { html: $P.lang('LANG83'), locale: 'LANG83' });
            addCell(newRow, { html: $P.lang('LANG1547'), locale: 'LANG1547' });
            addCell(newRow, { html: $P.lang('LANG1541'), locale: 'LANG1541' });
            addCell(newRow, { html: $P.lang('LANG74'), locale: 'LANG74' });
        })();

        var len = this.lists.length;
        if (len <= 0)     
        {
            var newRow = TBL.insertRow(-1);
            newRow.className = "odd";
            var td = $('<td colspan=4 style="text-align: center;"/>');
            $(newRow).append(td);
            $(newRow).attr('data-item-val', 'add_item');
            $(td).append('<span class="table-add">' + $P.lang('LANG1555') + '</span>');
            return;
        }

        //show item list
        for (var i = 0; i < len; i++)
        {
            var item = this.lists[i];
            var newRow = TBL.insertRow(-1);
            newRow.className = ((TBL.rows.length) % 2 == 1) ? 'odd' : 'even';
            addCell(newRow, { html: item.name || '' });
            addCell(newRow, { html: item.strip });
            addCell(newRow, { html: item.prepend || '' });
            var newCell = newRow.insertCell(newRow.cells.length);
            var $tmp;
            //add icon
            $tmp = $('<span />').addClass('table-add tableIcon table-add-icon').attr('localeTitle', 'LANG769').attr('title', $P.lang('LANG769')).appendTo(newCell);

            //top,up,down,bottom icons
            $tmp = $('<span />').addClass('table-top clickTop tableIcon').attr('localeTitle', 'LANG793').attr('title', $P.lang('LANG793')).appendTo(newCell);

            $tmp = $('<span />').addClass('table-up clickUp tableIcon').attr('localeTitle', 'LANG794').attr('title', $P.lang('LANG794')).appendTo(newCell);

            $tmp = $('<span />').addClass('table-down clickDown tableIcon').attr('localeTitle', 'LANG795').attr('title', $P.lang('LANG795')).appendTo(newCell);

            $tmp = $('<span />').addClass('table-bottom clickBottom tableIcon').attr('localeTitle', 'LANG796').attr('title', $P.lang('LANG796')).appendTo(newCell);

            //empty icon 
            //$tmp = $('<span />').addClass('tableIcon').appendTo(newCell);

            //edit, delete icons
            $tmp = $('<span />').addClass('table-edit tableIcon').attr('localeTitle', 'LANG738').attr('title', $P.lang('LANG738')).appendTo(newCell);

            $tmp = $('<span />').addClass('table-del tableIcon').attr('localeTitle', 'LANG739').attr('title', $P.lang('LANG739')).appendTo(newCell);

            $(newRow).attr('data-item-val', item.trunk);
        }


        $('.table-add').tooltip({ position: {
             my: "right center", 
             at: "left center" 
             }
        });

        //TODO add title to icons, title attr conflict with validate-tooltip
        //$('.table-del').attr('title', 'delete');
        //$('.table-del').tooltip({disabled: true});
        //$('.table-del').tooltip('disable');

    },
    "set_edit_values": function (idx) {
        var item = this.lists[idx] || {};
        if (item.trunk)
        {
            $('#new_crl_fotrunk').val(item.trunk || '');
        }
        else
        {
            DOM_new_crl_fotrunk.selectedIndex = -1;
        }


        $('#new_crl_fotr_stripx').val(item.strip || 0);
        $('#new_crl_fotr_prepend').val(item.prepend || '');

    }
};

var getTrunkName = function (trunk_val) {
    var pbxinfo = parent.sessionData.pbxinfo.trunks_info[trunk_val];
    if ( pbxinfo == undefined )
        return '';
    var type = pbxinfo.trunkstyle || '';
    var name = pbxinfo.trunkname || '';
    return $P.lang(typeObj[type]) + $P.lang('LANG83') + '  -- ' + name;
};


var reloadTable = function()
{
    $('#table_CRLS_list').children().remove();
    update_CRLSTable();
}

var rule_top = function(item)
{
    if ( item )
    {
        var idx = document.getElementById(item).value;

        if ( !/^(0|[1-9][0-9]*)$/.test(idx) )
            return false;
        idx = parseInt(idx);

        if ( idx == 0 )
        {
            return false;
        }
        var lists = parent.sessionData.pbxinfo['outbound_allroutes'];
        var it = lists[idx];
        lists.splice(idx, 1);
        lists.unshift(it);
        
        var actions = listOfSynActions('extensions.conf');
        actions.new_action('delcat', 'outbound-allroutes', '', '')
        actions.new_action('newcat', 'outbound-allroutes', '', '')

        for ( var i = 0; i < lists.length; i++ )
        {
            var iter = lists[i];
            iter = iter.substring(iter.indexOf('=')+1);
            actions.new_action('append', 'outbound-allroutes', 'include', iter);
        }
        var resp = actions.callActions();
        if ( !resp.contains('Response: Success') )
        {
            top.log.error('pbx.call_plans.add: Error update extensions.conf.');
            return false;
        }
        reloadTable();
        return true;
    }
    return false;
}
var rule_bottom = function(item)
{
    if ( item )
    {
        var idx = document.getElementById(item).value;

        if ( !/^(0|[1-9][0-9]*)$/.test(idx) )
            return false;
        idx = parseInt(idx);
        var lists = parent.sessionData.pbxinfo['outbound_allroutes'];
        if ( idx == lists.length-1 )
            return false;

        var it = lists[idx];
        lists.splice(idx, 1);
        lists.push(it);
        
        var actions = listOfSynActions('extensions.conf');
        actions.new_action('delcat', 'outbound-allroutes', '', '')
        actions.new_action('newcat', 'outbound-allroutes', '', '')

        for ( var i = 0; i < lists.length; i++ )
        {
            var iter = lists[i];
            iter = iter.substring(iter.indexOf('=')+1);
            actions.new_action('append', 'outbound-allroutes', 'include', iter);
        }
        var resp = actions.callActions();
        if ( !resp.contains('Response: Success') )
        {
            top.log.error('pbx.call_plans.add: Error update extensions.conf.');
            return false;
        }
        reloadTable();
        return true;
    }
    return false;
}
var rule_up = function(item)
{
    if ( item )
    {
        var idx = document.getElementById(item).value;

        if ( !/^(0|[1-9][0-9]*)$/.test(idx) )
            return false;
        idx = parseInt(idx);
        if ( idx == 0 )
            return false;

        var lists = parent.sessionData.pbxinfo['outbound_allroutes'];
        var it = lists[idx];
        var tmp = lists[idx-1];
        lists[idx-1] = it;
        lists[idx] = tmp;
        
        var actions = listOfSynActions('extensions.conf');
        actions.new_action('delcat', 'outbound-allroutes', '', '')
        actions.new_action('newcat', 'outbound-allroutes', '', '')

        for ( var i = 0; i < lists.length; i++ )
        {
            var iter = lists[i];
            iter = iter.substring(iter.indexOf('=')+1);
            actions.new_action('append', 'outbound-allroutes', 'include', iter);
        }
        var resp = actions.callActions();
        if ( !resp.contains('Response: Success') )
        {
            top.log.error('pbx.call_plans.add: Error update extensions.conf.');
            return false;
        }
        reloadTable();
        return true;
    }
    return false;
}
var rule_down = function(item)
{
    if ( item )
    {
        var idx = document.getElementById(item).value;

        if ( !/^(0|[1-9][0-9]*)$/.test(idx) )
            return false;
        idx = parseInt(idx);
        var lists = parent.sessionData.pbxinfo['outbound_allroutes'];
        if ( idx == lists.length-1 )
            return false;

        var it = lists[idx];
        var tmp = lists[idx+1];
        lists[idx+1] = it;
        lists[idx] = tmp;
        
        var actions = listOfSynActions('extensions.conf');
        actions.new_action('delcat', 'outbound-allroutes', '', '')
        actions.new_action('newcat', 'outbound-allroutes', '', '')

        for ( var i = 0; i < lists.length; i++ )
        {
            var iter = lists[i];
            iter = iter.substring(iter.indexOf('=')+1);
            actions.new_action('append', 'outbound-allroutes', 'include', iter);
        }
        var resp = actions.callActions();
        if ( !resp.contains('Response: Success') )
        {
            top.log.error('pbx.call_plans.add: Error update extensions.conf.');
            return false;
        }
        reloadTable();
        return true;
    }
    return false;
}

var check_pattern_format = function(value, element)
{
    var additional = function(value) 
    {
	    var res = false;
	    var off1 = value.indexOf('[');
	    var off2 = value.indexOf(']');
	    if ( off1 > -1 && off1 < off2 )
	    {
	    	res = true;
	    	var str = value;
	    	var pos = str.indexOf('-');
	    	while ( pos > -1 ) 
	    	{
	    		var v1 = str[pos-1];
	    		var v2 = str[pos+1];
	    		
	    		if ( pos < 2 )
	    		{
	    			res = false;
	    			break;
	    		}
	    		if ( /[^0-9a-zA-Z]/.test(v1) || /[^0-9a-zA-Z]/.test(v2) )
	    		{
	    			res = false;
	    			break;
	    		}
	    		if ( (/[0-9]/.test(v1) && /[0-9]/.test(v2)) && v1 > v2 )
	    		{
	    			res = false;
	    			break;
	    		}
	    		if ( (/[a-z]/.test(v1) && /[a-z]/.test(v2)) && v1 > v2 )
	    		{
	    			res = false;
	    			break;
	    		}
	    		if ( (/[A-Z]/.test(v1) && /[A-Z]/.test(v2)) && v1 > v2 )
	    		{
	    			res = false;
	    			break;
	    		}
	    		if ( (/[0-9]/.test(v1) && /[^0-9]/.test(v2)) ||
	    				(/[^0-9]/.test(v1) && /[0-9]/.test(v2)) )
	    		{
	    			res = false;
	    			break;
	    		}
	    		if ( (/[a-z]/.test(v1) && /[^a-z]/.test(v2)) ||
	    				(/[A-Z]/.test(v1) && /[^A-Z]/.test(v2)) )
	    		{
	    			res = false;
	    			break;
	    		}
	    		str = str.substr(pos+1);
	    		pos = str.indexOf('-');
	    	} 
	    } else if ( off1 == -1 && off2 == -1 )
	    {
	    	res = true;
	    }
	    return res;
    }
    var str = value.split('\n');
    var res = false;
    for ( var i = 0; i < str.length; i++ )
    {
        if (/^_$/.test(str[i]) || /(\[.*?(\.|\!|X|x|N|n|Z|z).*?\])|(\[\])/g.test(str[i])) {
            return false;
        }
        if ( /^_/.test(str[i]) )
            str[i] = str[i].substr(1);
        if ( /^_$/.test(str[i]) || str[i].length > 32 || /^[!\.]/.test(str[i]) )
            return false;
        if ( !/[^a-zA-Z0-9\#\*\.!\[\]\-\+\/_]/.test(str[i]) && additional(str[i]) )
        {
            res = true;
        } else
        {
            return false;
        }
    }
    return res;
}

var check_pattern_with_cid = function(value, element)
{
    var str = value.split('\n');
    var res = false;
    for ( var i = 0; i < str.length; i++ )
    {
        if ( !/^_/.test(str[i]) )
            str[i] = '_'+str[i];
        if ( /^_[0-9a-zA-Z!\[\]\-\.\?\+\*\#]+\/[0-9]+$/.test(str[i]) )
            res = true;
        else if ( !/\//.test(str[i]) )
            res = true;
        else
            return false;
    }

    return res;
}
    

var fetch_pattern = function(str)
{
    var tpl = 'exten=';
    if ( str.indexOf(tpl) != 0 )
    {
        return NULL;
    }
    var pos = str.indexOf(',');
    if ( pos == -1 )
    {
        return NULL;
    }

    return str.substring(tpl.length, pos);
}
