/*
* Asterisk-GUI - an Asterisk configuration interface
*
* cdr.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Modified the cdr function
 *
 */
$P = top.$;
var records_too_big = false;
var backend = "CDR-CSV";
var records = [];
var viewCount = 10;     /* number of CDR rows displayed at once */
var offset = 0;         /* index into CDR records for first row of current display */
//var intDest = [];
//var intExt_str = "('conf','',";
var btn_enabled = "btn btn-paging";
var btn_disabled = "btn btn-paging disabled";
var opt_play_enabled = "options play";
var opt_play_disabled = "options play disabled";
var opt_download_enabled = "options download";
var opt_download_disabled = "options download disabled";
var opt_del_enabled = "options del";
var opt_del_disabled = "options del disabled";
var total_count = "count(AcctId)";
var condition = "";
var confrooms = [];
var ivrs = [];
var vmgroups = [];
var ringgroups = [];
var pagegroups = [];
var queues = [];
var extens = [];
var voip_trunks = [];
var analog_trunks = [];
var all_trunks = [];
var feature_codes = [];
var all_exts = [];

//original field name "fname"; 
//display name "dname";
var short_fields = new Array();
short_fields["field_name"] = ["", "start", "clid", "dst", "duration", "billsec", /*"calltype",*/ "disposition", "recordfiles","option"];
short_fields["display_name"] = [
            "LANG177", "LANG169", "LANG581", "LANG582", "LANG2239", "LANG2238", /*"LANG2235",*/ "LANG186", "disNone","LANG74"
            ];

var long_fields = new Array();
long_fields["field_name"] = [
        "AcctId",
        "accountcode", "src", "dst", "dcontext",
        "clid", "channel", "dstchannel", "lastapp",
        "lastdata", "start", "answer", "end",
        "duration", "billsec", "disposition", "amaflags",
        "uniqueid", "userfield", "channel_ext", "dstchannel_ext", "service", "","recordfiles","option"/*, "calltype"*/
        
        ];
long_fields["display_name"] = [
        "LANG177","LANG178", "LANG581", "LANG582", "LANG179","LANG78", "LANG101", "LANG180", "LANG181","LANG182", "LANG169", "LANG183", "LANG184","LANG172", "LANG185", "LANG186", "LANG187","LANG187", "LANG189", "","","","","", "LANG74"/*, "LANG2235"*/
        ];
var fields; 
var svindex = [];       
var longversion = false; /* long version displays all CDR-CSV fields, short only those selected above */
var sortby = 0; /* Which column (CDR field) to sort the displayed CDR list by. */
/* negative means descending. Subtract 1 to get index into the records fields array */
var maxColumns = 0;     /* maximum number of columns found in CDR records */

var recordsSelected;
var showOutboundCalls = true;
var showInboundCalls = true;
var showInternalCalls = true;
var showExternalCalls = true;
//var searchStartfrom = false;
//var searchStartto = false;
var searchDurFrom = false;
var searchDurTo = false;
var startfrom = "";
var startto = "";
var durfrom = "";
var durto = "";
//var currSrc = "";
//var currDest = "";
var callerNum = "";
var callerName = "";
var nSelected = 0;
var sortfield = 9; //sort start time in order to get statistics.
var page_id = -1; //every time reload the records, the page_id will change. 
var currRecordList = [];  //record current <uniqueID, filename> pair.
var curr_date = new Date();
var curr_year = curr_date.getFullYear();
var curr_month = curr_date.getMonth();
var curr_day = curr_date.getDate();
var mWindow = top.window.frames["mainscreen"]; // this will be the main window
var acctId_idx = -1;
var billsec_idx = -1;
var duration_idx = -1;
var uniqueid_idx = -1; 
var clid_idx = -1;
var src_idx = -1;
var dst_idx = -1;
var calltype_idx = -1;
var ch_ext_idx = -1;
var dstch_ext_idx = -1;
var start_idx = -1;
var recordfiles_idx = -1;
var voiceRecords = {};

var orderbyStr = function()
{
    var DESC = "";
    var orderby_str = "";
    var order_index = 0;
    var field = "";
    if(sortby < 0)
    {
        DESC = " DESC ";
        order_index = Math.abs(sortby);
    }
    else
    {
        order_index = sortby;
    }
    field = long_fields["field_name"][order_index];
    if(order_index == 10 || order_index == 11 || order_index == 12)
        orderby_str = " order by " + field + DESC;
    else if(order_index == 13 || order_index == 14)
        orderby_str = " order by " + "CAST(" + field + " as integer)" + DESC;
    else
        orderby_str = " order by " + field + DESC;
    return orderby_str;
}

var limitStr = function()
{
    var limit_str = " limit " + viewCount;
    return limit_str;
}

var offsetStr = function()
{
    var offset_str = " offset " + offset;
    return offset_str;
}

function and(condition)
{
    var and_cond = (condition.length > 0)? " AND " : "";
    return and_cond;
}

function where(condition)
{
    var where_cond = (condition.length > 0)? " WHERE " : "";
    return where_cond;
}

function or(cond)
{
    var or_cond = (cond.length > 0)? " OR " : "";
    return or_cond;
}

var createSearchCondition = function()
{
    condition = "";
        
    //currSrc = _$("Source").value;
    //currDest = _$("Destination").value;
    callerNum = _$("CallerNumber").value;
    callerName = _$("CallerName").value;
    startfrom = _$("startfrom").value;
    startto = _$("startto").value;
    durfrom =  _$("drfrom").value;
    durto = _$("drto").value;
    //searchStartfrom = _$("sfselect").checked; 
    //searchStartto = _$("stselect").checked;
    searchDurFrom = _$("dfselect").checked;
    searchDurTo = _$("dtselect").checked;
    showOutboundCalls = _$("showOutbound").checked;
    showInboundCalls = _$("showInbound").checked; 
    showInternalCalls = _$("showInternal").checked;
    showExternalCalls = _$("showExternal").checked;
    var ob = (showOutboundCalls)? 1 : 0;
    var ib = (showInboundCalls)? 2 : 0;
    var it = (showInternalCalls) ? 4 : 0;
    var et = (showExternalCalls)? 8 : 0;
    var checksum = ob + ib + it + et; 
    var calltype_cond = "";

    var ib_cond = "(instr(channel_ext, 'trunk') AND not instr(dstchannel_ext, 'trunk'))";
    var ob_cond = "(not instr(channel_ext, 'trunk') AND instr(dstchannel_ext, 'trunk'))";
    var et_cond = "(instr(channel_ext, 'trunk') AND instr(dstchannel_ext, 'trunk'))";
    var it_cond = "(not instr(channel_ext, 'trunk') AND not instr(dstchannel_ext, 'trunk'))";

/*        
    if(currSrc.length > 0)
        condition += and(condition) + " src='" + currSrc + "'";
    if(currDest.length > 0)
        condition += and(condition) + " dst='" + currDest + "'";
*/
    if(callerNum.length > 0 && callerName.length > 0)
        condition += and(condition) + " (src='" + callerNum + "' or caller_name='" + callerName + "')";
    else if(callerNum.length > 0)
        condition += and(condition) + " src='" + callerNum + "'";
    else if(callerName.length > 0)
        condition += and(condition) + " caller_name='" + callerName + "'";
    
    if(startfrom.length > 0)
        condition += and(condition) + " start >= datetime('" + startfrom + "')";
    if(startto.length > 0)
        condition += and(condition) + " start <= datetime('" + startto + "')" ;
    
    if(searchDurFrom && durfrom.length > 0)
        condition += and(condition) + " CAST(duration as integer) >= " + durfrom;
    if(searchDurTo && durto.length > 0)
        condition += and(condition) + " CAST(duration as integer) <= " + durto;
    
    switch(checksum)
    {
    case 0:      //!showOutboundCalls && !showInboundCalls && !showInternalCalls && !showExternalCalls
        calltype_cond = "(channel_ext in ('') and channel_ext not in (''))";
        break;
    case 1:      //showOutboundCalls only
        calltype_cond = ob_cond;
        break;
    case 2:     //showInboundCalls only
        calltype_cond = ib_cond;
        break;
    case 3:     //showOutboundCalls & showInboundCalls only
        calltype_cond = "(" + ob_cond + " OR " + ib_cond + ")";
        break;
    case 4:     //showInternalCalls only
        calltype_cond = it_cond;
        break;
    case 5:     //showOutboundCalls & showInternalCalls only
        calltype_cond = "(not instr(channel_ext, 'trunk'))";
        break;
    case 6:     //showInboundCalls & showInternalCalls only
        calltype_cond = "(not instr(dstchannel_ext, 'trunk'))";
        break;
    case 7:     //showOutboundCalls & showInboundCalls & showInternalCalls only
        calltype_cond = " NOT " + et_cond;
        break;
    case 8:     //showExternalCalls only
        calltype_cond = et_cond;
        break;
    case 9:     //showOutboundCalls & showExternalCalls only
        calltype_cond = "(instr(dstchannel_ext, 'trunk'))";
        break; 
    case 10:    //showInboundCalls & showExternalCalls only
        calltype_cond = "(instr(channel_ext, 'trunk'))";
        break;
    case 11:    //showOutboundCalls & showInboundCalls & showExternalCalls only
        calltype_cond = " NOT " + it_cond;
        break;        
    case 12:    //showInternalCalls & showExternalCalls only
        calltype_cond = "(" + it_cond + " OR " + et_cond + ")";
        break;
    case 13:    //showOutboundCalls & showInternalCalls & showExternalCalls only
        calltype_cond = " NOT " + ib_cond;
        break;
    case 14:    //showInboundCalls & showInternalCalls & showExternalCalls only
        calltype_cond = " NOT " + ob_cond;
        break;
    case 15:    //all;
        calltype_cond = ""; 
        break;
    }
    
    if(calltype_cond.length > 0)
        condition += and(condition) + calltype_cond;
    condition = where(condition) + condition;
        
}

var popups = {
    del: {
        el_id: "delete_all",
        title: "Delete All",
        content: $P.lang("LANG840"),
        submit: "sendDeleteRequest();",
        cancel: "ASTGUI.hideDrag(event);",
        show: function ()
        {
            top.dialog.dialogConfirm({
                confirmStr: $P.lang("LANG840"),
                buttons: {
                    ok: sendDeleteRequest
                }
            });
        }
    },
    
    del_r: {
        el_id: "delete_record",
        title: "Delete File",
        content: $P.lang("LANG842"),
        submit: "sendDelRecordRequest",
        cancel: "ASTGUI.hideDrag(event);",
        show: function (id)
        {
            top.dialog.dialogConfirm({
                confirmStr: $P.lang("LANG841"),
                buttons: {
                    ok: function(){
                       sendDelRecordRequest(id); 
                    }
                }
            });
        }
    }
    
};

function hideDragAndRefresh()
{
    ASTGUI.hideDrag(event);
    window.location.reload();
}


function playEmbedSound(uniqueID, el_id)
{
    top.dialog.dialogInnerhtml({
        dialogTitle: $P.lang("LANG977"),
        frameSrc: "cdr_modal.html?mode=play&item={0}".format("auto-1370134867-1002-1003.wav")
    });
}

function displayExtCharts(ext)
{
    top.dialog.dialogInnerhtml({
        dialogTitle: $P.lang("LANG2306"),
        frameSrc: "cdr_modal.html?mode=chart&item={0}".format(ext)
    });
}

function displayCallDetail(AcctId)
{
    top.dialog.dialogInnerhtml({
        dialogTitle: $P.lang("LANG2307"),
        frameSrc: "cdr_modal.html?mode=calldetail&item={0}".format(AcctId)
    });
}

function genRecordList(type, uniqueID, filelist, acctid)
{
    var title = "";
    var frameSrc = "cdr_modal.html?mode=" + type + "&uid=" + uniqueID + "&item={0}".format(filelist);
    if(type == "play")
        title = $P.lang("LANG225");
    else if(type == "download")
        title = $P.lang("LANG224");
    else if(type == "delete") {
        title = $P.lang("LANG226");
        frameSrc += "&acctid=" + acctid
    }
    
    top.dialog.dialogInnerhtml({
            dialogTitle: title,
            frameSrc: frameSrc
        });
}


function convertToInt(array)
{
    for (var i = 0; i < array.length; i++)
    {
        array[i] = parseInt(array[i], 10);
    }
}


function setSortbyField(index)
{        /* comes here on click on one of the table headers */
    offset = 0;
    if (Math.abs(sortby) == index) sortby = -sortby; /* if sorting on same column, simply reverse order */
    else sortby = index;
    performSearch(false);
}

function firstPage()
{
    if(_$("firstPageBtn").className == btn_enabled)
    {
        offset = 0;
        performSearch(false);
    }
}

function lastPage()
{
    if(_$("lastPageBtn").className == btn_enabled)
    {
        var pages = Math.floor(nSelected/viewCount);
        offset = (nSelected % viewCount > 0) ? pages * viewCount : (pages-1) * viewCount;
        performSearch(false);
    }
}

function nextPage()
{
    if(_$("nextPageBtn").className == btn_enabled)
    {
        if (nSelected > offset + viewCount) 
            offset += viewCount;
        performSearch(false);
    }
        
}

function prevPage()
{
    if(_$("prevPageBtn").className == btn_enabled)
    {
        if (offset) offset -= viewCount;
        if (offset < 0) offset = 0;
        performSearch(false);
    }
}

function VisibleHeight()
{      /* need to test to make sure works on all browsers */
    var h = 0;
    if (typeof (window.parent.innerHeight) == 'number')
    {
        h = window.parent.innerHeight;          /* Most browsers except MS IE */
    }
    else if (window.parent.document.documentElement &&
                  window.parent.document.documentElement.offsetHeight)
    {
        h = window.parent.document.documentElement.offsetHeight; /* MS IE 7 */
    }
    else if (document.body && (document.body.clientHeight))
    {
        h = document.body.clientHeight;
    }
    return h;
}
function VisibleWidth()
{       /* need to test to make sure works on all browsers */
    var w = 0;
    if (typeof (window.parent.innerWidth) == 'number')
    {
        w = window.parent.innerWidth;           /* Most browsers except MS IE */
    }
    else if (window.parent.document.documentElement &&
                window.parent.document.documentElement.offsetWidth)
    {
        w = window.parent.document.documentElement.offsetWidth; /* MS IE 7 */
    }
    else if (document.body && (document.body.clientWidth))
    {
        w = document.body.clientWidth;
    }
    return w;
}

function isset(obj)
{
    if (typeof obj != "object")
        return (typeof obj != "undefined");
    for (var i in obj)
        return true;
    return false;
}

//function deleteItemsFromArray(src, dst) //delete any elements of src from dst array. 
function sendDelRecordRequest(uniqueID, files, acctid, fileList)
{  
    var path = "";
    var url = "";
    if(files[0].indexOf("auto-") > -1)
    {
        path = "monitor";
        url = "../delete.php?type=cdr&path=" + path + "&file=" + files.slice().toString();
    }    
    else if(files[0].indexOf("meetme-") > -1)
    {
        path = "meetme";
        url = "../delete.php?type=cdr&path=" + path + "&file=" + files.slice().toString();
    }
    else
        return;
    var request = $.ajax({
        url: url,
        type: "GET",
        async: false,
        dataType: "text",
        success: function (msg)
        {
            if (msg.indexOf("Success") != -1)
            {
                var newfiles = "";
                for (var i = 0; i < files.length; i++) {
                    newfiles += files[i] + "@"; 
                }
                delFile = fileList.replace(newfiles, ""); 
                var cmd = "update cdr set recordfiles='"+delFile+"' where acctid=" + acctid;
                $P.gs.actionRequest(document, "updateCDRDB", null, {
                    "request": ["cdr-clear=" + cmd],
                    "auto": false,      
                    "onInvalidResponse": onResponseErrorClear,
                    "onResponseProcessed": dataBindClear
                });
                var filename = files[0].split(".");
                var fileparts = filename[0].split("-");
                var ext1 = fileparts[2];
                var ext2 = fileparts[3];
                var key1 = ext1 + "." + ext2;
                var key2 = ext2 + "." + ext1;                
                if(voiceRecords[key1] != undefined && voiceRecords[key1].length > 0)
                { 
                    var items = jQuery.grep(voiceRecords[key1],function (item) {
                        return jQuery.inArray(item, files) < 0;
                    });                
                    voiceRecords[key1] = items;
                }
                if(voiceRecords[key2] != undefined && voiceRecords[key2].length > 0)
                {
                    var items = jQuery.grep(voiceRecords[key2],function (item) {
                        return jQuery.inArray(item, files) < 0;
                    });                  
                    voiceRecords[key2] = items;
                } 
            }
        },
        error: function (msg)
        {
            top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG913") });
        }
    });
}

function sendDldRecordRequest(file)
{
    document.location.href = "../udata/monitor/" + file;
}

function createInputElement(type, title, id, value, disabled, style, filelist)
{
    var ele = $("<"+type+">");
    ele.attr({
        localeTitle: title,
        title: $P.lang(title),
        id: id,
        disabled: disabled

    }).addClass(style);
    return ele[0];
}

function getDurInSec(timestamp)
{
    var seconds = 0;
    tm_arr = timestamp.split(":");
    if(tm_arr.length == 3)
        seconds = 3600 * parseInt(tm_arr[0], 10) + 60 * parseInt(tm_arr[1], 10) + parseInt(tm_arr[2], 10);
    else if(tm_arr.length == 2)
        seconds = 60 * parseInt(tm_arr[0], 10) + parseInt(tm_arr[1], 10);
    else if(tm_arr.length == 1)
        seconds = parseInt(tm_arr[0], 10);
    return seconds;
}

function sendListRequest()
{
    var request_str = "../list.php?type=cdr";
    var request = $.ajax({
            url: request_str,
            type: "GET",
            async: true,
            dataType: "text",
            success: function (msg)
            {
                if(msg.length > 0)
                {
                    var msg_arr = msg.split(",");
                    for(var i = 0; i < msg_arr.length; i++)
                    {
                        if( msg_arr[i].length > 0 )
                        {
                            var nameSuffix_arr = msg_arr[i].split(".wav");
                            var filename_arr = nameSuffix_arr[0].split("-");
                            if(filename_arr.length == 4 && filename_arr[0] == "auto")  //need to have 3 "-" in the file name (auto-xxxx-xxx-xxx.wav)
                            {
                                var ext1 = filename_arr[2];
                                var ext2 = filename_arr[3];
                                var key = ext1 + "." + ext2;
                                
                                if(voiceRecords[key] == undefined)
                                    voiceRecords[key] = [];
                                voiceRecords[key].push(msg_arr[i]);
                            } 
                            else if(filename_arr.length == 5 && filename_arr[0] == "meetme")
                            {
                                var ext_conf = filename_arr[3];
                                var key = "conf." + ext_conf;
                                if(voiceRecords[key] == undefined)
                                    voiceRecords[key] = [];
                                voiceRecords[key].push(msg_arr[i]);
                            }   
                        }
                    }
                    
                }
            },
            error: function (jqXHR, textStatus, msg)
            {
            }
    });
}

function getVoiceRecords(uniqueID, src, dst, duration)
{
    var recordList = [];
    var dursec = getDurInSec(duration);
    var key = src + "." + dst;
    if(voiceRecords[key] != undefined )
    {
        var files = voiceRecords[key];
        for(var i = 0; i < files.length; i++ )
        {
            var fileparts = files[i].split("-");
            var rid = Math.floor(uniqueID); 
            var vid = Math.floor(fileparts[1]);
            if(rid <= vid && rid + dursec >= vid)
            {
                recordList.push(files[i]);
            }
        }
    }
    
    key = dst + "." + src;
    if(voiceRecords[key] != undefined ) 
    {
        var files = voiceRecords[key];
        for(var i = 0; i < files.length; i++ )
        {
            var fileparts = files[i].split("-");
            var rid = Math.floor(uniqueID); 
            var vid = Math.floor(fileparts[1]);
            if(rid <= vid && rid + dursec >= vid)
            {
                recordList.push(files[i]);
            }
        }
    } 
    
    key = "conf." + src;
    if(voiceRecords[key] != undefined ) 
    {
        var files = voiceRecords[key];
        for(var i = 0; i < files.length; i++ )
        {
            var filename_arr = files[i].split(".wav");
            var fileparts = filename_arr[0].split("-");
            var rid = Math.floor(uniqueID); 
            var vid = Math.floor(fileparts[4]);
            //console.log("src " + src + " dst " + dst + " id " + uniqueID + " time " + fileparts[4]);
            if(rid <= vid && rid + dursec >= vid)
            {
                recordList.push(files[i]);
            }
        }
    } 
    
    key = "conf." + dst;
    //console.log(voiceRecords[key]);
    if(voiceRecords[key] != undefined ) 
    {
        var files = voiceRecords[key];
        for(var i = 0; i < files.length; i++ )
        {
            var filename_arr = files[i].split(".wav");                           
            var fileparts = filename_arr[0].split("-");  
            var rid = Math.floor(uniqueID); 
            var vid = Math.floor(fileparts[4]);
            //console.log("src " + src + " dst " + dst + " id " + uniqueID + " time " + fileparts[4] + " dur " + dursec + " rid " + rid + " vid " + vid);
            if(rid <= vid && rid + dursec >= vid)
            {
                recordList.push(files[i]);
            }
        }
    } 
   //console.log(recordList); 
    if(recordList.length > 0)
        recordList.sort();
    return recordList.toString();  
}

function sendListRequest_backup()
{
    var request_str = "";
    
    for(var i = 0; i < recordsSelected.length; i++ )
    {
        var uniqueID = recordsSelected[i][uniqueid_idx];
        if(uniqueID != undefined && uniqueID.length > 0)
        {
            request_str = request_str + uniqueID + "," + 
                          getDurInSec(recordsSelected[i][duration_idx]) + "," + 
                          recordsSelected[i][src_idx] + "," + 
                          recordsSelected[i][dst_idx] + "|";
        }
    } 
    //answer time, end time, source, dest
    if( request_str.length > 0)
    {
        request_str = "../list.php?id=" + page_id + "&type=cdr&list=" + request_str;
        var request = $.ajax({
            url: request_str,
            type: "GET",
            async: true,
            dataType: "text",
            success: function (msg)
            {
                /* return string should be like:
                * "page_id\n"
                * "uniqueID1,file_name1\n"
                * "uniqueID2,file_name2\n"
                * ......
                */
                if (msg.length > 0)    //since it's plain text, so no need to do msg.responseText
                {
                    var msg_arr = msg.split("\n");

                    if (msg_arr[0] == page_id)  //if the page hasn't changed.
                    {
                        for (var i = 1; i < msg_arr.length; i++)
                        {
                            var record_arr = msg_arr[i].split(",");
                            if (record_arr.length == 2)
                            {
                                var rid = record_arr[0];
                                var filename = record_arr[1];
                                document.getElementById("play_" + rid).disabled = false;
                                document.getElementById("play_" + rid).className = "options play";
                                document.getElementById("download_" + rid).disabled = false;
                                document.getElementById("download_" + rid).className = "options download";
                                document.getElementById("delete_" + rid).disabled = false;
                                document.getElementById("delete_" + rid).className = "options del";
                                currRecordList[rid] = filename;
                            }
                        }
                    }
                }

            },
            error: function (jqXHR, textStatus, msg)
            {
            }

        });
     }  
}

function loadRecords(buildselected)
{
    recordsSelected = records;
    page_id = Number(new Date());
    var nCdrs = records.length;
    var e = _$("cdr_content_container");
    if(e == null)
        return;
    e.innerHTML = "";

    var d = document.createElement("TABLE");
    d.id = "table_CDR_list";
    d.className = "table_CDR";
    d.cellSpacing = 0;

    var thead = document.createElement("thead");
    var tr = document.createElement("tr");
    var total_columns;
    if (longversion) 
    {
        tr.style.fontSize = "8pt";
        total_columns = maxColumns;
        fields = long_fields;
    }
    else 
    {
        tr.style.fontSize = "14px";
        total_columns = short_fields["field_name"].length;
        fields = short_fields;
    }
    
    for (var i = 0; i < total_columns; i++)
    {
        var th = document.createElement("th");
        var x = longversion ? i : svindex[i];
        if( i > 0 )
        {
            if ($(d).fixedHeader) {
                th.setAttribute("onclick", "javascript:setSortbyField(" + x + ")");   
            }
            else {
                th.onclick = Function('setSortbyField("' + x + '")');    /* needed for IE with no fixedheader */    
            }
        }
        var disnameSpan = document.createElement("span");
        disnameSpan.innerHTML = $P.lang(fields["display_name"][i]);
        disnameSpan.setAttribute("locale", fields["display_name"][i]);
        if (fields["display_name"][i] == "LANG74" || fields["display_name"][i] == "LANG2235") {
            th.style.width = '150px';
        } 
        th.appendChild(disnameSpan); 
        if (x == parseInt(sortby, 10)) {
            var downSpan = document.createElement("span");
            th.className = "header-background";
            downSpan.className = "header_sort_down";  
            th.appendChild(downSpan);
            th.childNodes[0].className = "sort-font-color";          
        } 
        else if (x == 0 - sortby) {
            var upSpan = document.createElement("span");
            th.className = "header-background";
            upSpan.className = "header_sort_up";
            th.appendChild(upSpan);
            th.childNodes[0].className = "sort-font-color";            
        }
        if (fields["display_name"][i] == "disNone") {
            $(th).hide();
        }
        tr.appendChild(th);
    }
    thead.appendChild(tr);
    d.appendChild(thead);
    var tbody = document.createElement("tbody");
    var c = viewCount;
    var n = 1; /* will use as row numbers (left most column) */
    
    for (var i = 0; c-- && isset(recordsSelected[i]); i++)
    {
        var r = recordsSelected[i];
        var tr = document.createElement("tr");
        tr.className = (i % 2) ? "odd" : "even";
        if (r.duplicate) {
            tr.style.color = "ff0000";
        }
        for (var j = 0; j < total_columns; j++)
        {
            var x = longversion ? j : svindex[j];
            var td = document.createElement("td");
            //td.style.textAlign="right";
            if (j == 0)
            {    
                /* row numbers (first column) */
                td.appendChild(document.createTextNode(offset + n));
                n++;
            }
            else if (!longversion && fields["field_name"][j] == "start" )
            {
                td.appendChild(document.createTextNode(r[x]));
                td.onclick = Function('displayCallDetail("' + r[0] + '")');
                td.className = "selectable";
            }
            else if (!longversion && fields["field_name"][j] == "clid")
            {
                
                if(r.calltype == "outgoing call" || r.calltype == "internal call" )
                {
                    td.appendChild(document.createTextNode(r[x]));
                    if(all_exts.indexOf(r[src_idx]) > -1)
                    {
                        td.onclick = Function('displayExtCharts("' + r[src_idx] + '")');
                        td.className = "selectable";
                    }
                }
                else if(r.calltype == "incoming call" || r.calltype == "external call")
                {
                    var trunk_str = "";
                    all_trunks.each( function(trunk_obj)
                    {
                        if(trunk_obj.number == r[ch_ext_idx])
                        {
                            trunk_str = " [Trunk: " + trunk_obj.name + "]";
                            return false;
                        }
                    });
                    
                    td.appendChild(document.createTextNode(r[x] + trunk_str));
                }
            }
            else if(!longversion && fields["field_name"][j] == "dst" )
            {
                
                if(r.calltype == "incoming call" || r.calltype == "internal call")
                {
                    td.appendChild(document.createTextNode(r[x]));
                    if(all_exts.indexOf(r[dst_idx]) > -1)
                    {
                        td.onclick = Function('displayExtCharts("' + r[dst_idx] + '")');
                        td.className = "selectable";
                    }
                }
                else if(r.calltype == "outgoing call" || r.calltype == "external call")
                {
                    var trunk_str = "";
                    
                    all_trunks.each( function(trunk_obj)
                    {
                        if(trunk_obj.number == r[dstch_ext_idx])
                        {
                            trunk_str = " [Trunk: " + trunk_obj.name + "]";
                            return false;
                        }
                    });                   
                    
                    td.appendChild(document.createTextNode(r[x] + trunk_str));
                }
            }
            else if (!longversion && fields["field_name"][j] == "calltype")
            {
                var ext1Div = document.createElement("div");
                var connDiv = document.createElement("div");
                var ext2Div = document.createElement("div");
                var hint_str = "";
                //ext1Div.localetitle = "LANG738";
                
                if(r.calltype == "incoming call")
                {
                    hint_str = "Inbound call from trunk " + r[ch_ext_idx];
                    ext1Div.className = "pstndiv";
                    connDiv.className = "incomingdiv";
                    ext2Div.className = "phonediv";
                                        
                }
                else if(r.calltype == "outgoing call")
                {
                    hint_str = "Outbound call through trunk " + r[dstch_ext_idx];
                    ext1Div.className = "phonediv";
                    connDiv.className = "outgoingdiv";
                    ext2Div.className = "pstndiv";
                    
                }
                else if(r.calltype == "internal call")
                {
                    hint_str = "Internal call";
                    ext1Div.className = "phonediv";
                    connDiv.className = "connectdiv";
                    ext2Div.className = "phonediv";
                    
                }
                else if(r.calltype == "external call")
                {
                    hint_str = "External call through trunk " + r[ch_ext_idx] + " and trunk " + r[dstch_ext_idx];
                    ext1Div.className = "pstndiv";
                    connDiv.className = "connectdiv";
                    ext2Div.className = "pstndiv";
                    
                }
                ext1Div.title = hint_str;
                connDiv.title = hint_str;
                ext2Div.title = hint_str;
                td.appendChild(ext1Div);
                td.appendChild(connDiv);
                td.appendChild(ext2Div);
                //td.title = "Edit";
                //td.localetitle = "LANG738";
                    
                //td.appendChild(document.createTextNode(r.calltype));
            }
            else if (!longversion && fields["field_name"][j] == "disposition")
            {
                var newDiv = document.createElement("div");
                var hint_str = "";
                if(r[x].indexOf("ANSWERED") > -1)
                {
                    newDiv.className = "answereddiv";
                    newDiv.title = "Answered";
                }
                else if(r[x].indexOf("NO ANSWER") > -1)
                {
                    newDiv.className = "noanswerdiv";
                    newDiv.title = "No Answer";
                }
                else if(r[x].indexOf("FAILED") > -1)
                {
                    newDiv.className = "faileddiv";
                    newDiv.title = "Failed";
                }
                else if(r[x].indexOf("BUSY") > -1)
                {
                    newDiv.className = "busydiv";
                    newDiv.title = "Busy";
                }
                td.appendChild(newDiv);                
            }
            else if (!longversion && fields["field_name"][j] == "recordfiles" )
            {
                td.appendChild(document.createTextNode(r[x]));
                $(td).hide();
            }
            else if (!longversion && fields["field_name"][j] == "option" )
            {
                var disabled = true;
                //var record_list = getVoiceRecords(r.uniqueID, r[src_idx], r[dst_idx], r[duration_idx]);
                var record_list = r.recordfiles.trim();
                th.style.width = "150px";
                if(record_list.length > 0){
                    disabled = false;
                }
                var ele = createInputElement("button", "LANG777", "play_" + r.uniqueID, "", 
                                             disabled, disabled? opt_play_disabled: opt_play_enabled, record_list);
                td.appendChild(ele);
                ele = createInputElement("button", "LANG759", "download_" + r.uniqueID, "", 
                                         disabled, disabled? opt_download_disabled: opt_download_enabled, record_list);
                td.appendChild(ele);
                ele = createInputElement("button", "LANG739", "delete_" + r.uniqueID, "", 
                                         disabled, disabled? opt_del_disabled: opt_del_enabled, record_list);
                $(ele).attr("acctid", r.acctid);
                td.appendChild(ele);
            }
            else
            {
                td.appendChild(document.createTextNode(r[x]));
            }
            tr.appendChild(td);
        }
        tbody.appendChild(tr);
    }
    d.appendChild(tbody);
    e.appendChild(d);
    /* table is now built */
    $(".options").bind("click", function (ev) {
        if (!this.disabled) //this is HTML object; $(this) is JQuery object;
        {
            var acctid = $(this).attr("acctid");
            var id_arr = this.id.split("_");
            var filelist = $(this).parent().prev().html();
            if (id_arr.length > 1)
            {
                var op_type = id_arr[0];   //"play"/"download"/"delete"
                var uniqueID = id_arr[1];  //uniqueID of a record;
                genRecordList(op_type, uniqueID, filelist, acctid);
                
            }
        }
        ev.stopPropagation();
        return false;       
    });
    top.dialog.clearDialog();
}

function sendDeleteRequest()
{
    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG877") });
    var cmd = "DELETE FROM CDR";
    $P.gs.actionRequest(document, "updateCDRDB", null, {
        "request": ["cdr-clear=" + cmd],
        "auto": false,
        "onInvalidResponse": onResponseErrorClear,
        "onResponseProcessed": dataBindClear
    });
    
    var request = $.ajax({
        url: "../delete.php?type=cdr",
        type: "GET",
        async: false,
        dataType: "text",
        success: function (msg)
        {
            //top.dialog.clearDialog();
            
        },
        error: function (msg)
        {
            //top.dialog.clearDialog();
            
        }
    }); 
}

function sendDownloadRequest()
{  
/* 
    var request = $.ajax({
        url: "../list.php?type=cdr&file=Master.csv",
        type: "GET",
        async: false,
        dataType: "text",
        success: function (msg)
        {
            if (msg.indexOf("Success") > -1)
            {
                window.open("/webcgi?action=fileOpDownload&type=CDR_recording&data=Master.csv&_=" + (new Date().getTime()));    
            }
            else
            {
                top.dialog.dialogMessage({ type: 'error', content: msg });
            }
        },
        error: function (msg)
        {
            top.dialog.dialogMessage({ type: 'error', content: msg });
        }
    });
*/
    var tmp_cmd = "action=systemcmdwithoutput&sh /app/asterisk/var/lib/asterisk/scripts/exportcsv.sh > /app/asterisk/var/log/asterisk/cdr-csv/Master.csv";
	var cb1 = function()
    {
        window.open("/webcgi?action=fileOpDownload&type=CDR_recording&data=Master.csv&_=" + (new Date().getTime()));     
    }
    $.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: cb1});
}

function resetPageButtons()
{
    if (offset == 0)
    { 
        _$("firstPageBtn").className = btn_disabled;
        _$("prevPageBtn").className = btn_disabled;
    }
    else
    { 
        _$("firstPageBtn").className = btn_enabled;
        _$("prevPageBtn").className = btn_enabled;
    }
       
    if (offset + viewCount >= nSelected) 
    {            
        _$("lastPageBtn").className = btn_disabled;
        _$("nextPageBtn").className = btn_disabled;
    }
    else 
    { 
        _$("lastPageBtn").className = btn_enabled;
        _$("nextPageBtn").className = btn_enabled;
    }
}

function processCDRResult(result)
{
    delete records;
    if(result.length == 0)
    {
        records = [];
        return;
    }
    records = result.split("|");
    var total_num = records.length;
    for(var i = 0; i < total_num; i++ )
    {
        records[i] = records[i].split("$");
        var nColumns = records[i].length;
        maxColumns = Math.max(maxColumns, nColumns);
        if (nColumns < 2)
        {
            // humm, line didn't contain enough number of commas 
            // could be bogus data or blank line. Either way it will 
            // cause problems later. We should delete the record 
            records.splice(i, 1);
            // now it is gone, but next record has moved up one index 
            // so we need to decrement i so that we don't skip over it. 
            i--;
            // Which will work unless the browser javascript for-loop 
            // optimizer is buggy (records.length must be calculated each loop) 
            continue;
        }
        var r = records[i];
        
        for ( var j = 0; j < nColumns; j++)
        {
            // Strips quotation marks from each record
            if ((j == duration_idx) || (j == billsec_idx))
            { 
                // duration or billable seconds 
                // converts seconds to HH:MM:SS. 
                var s = parseInt(r[j], 10);
                var h = Math.floor(s / 3600); s = s % 3600;
                var m = Math.floor(s / 60); s = s % 60;
                r[j] = h + ":" + (m < 10 ? ("0" + m) : m) + ":" + (s < 10 ? ("0" + s) : s);
            }
            else if (j == acctId_idx) {
                r.acctid = r[j];
            }
            else if (j == clid_idx)
            { 
                // callerID, may have double quotation marks 
                r[j] = r[j].toString().replace(/\"\"/g, '\"');
            }
            else if (j == uniqueid_idx)
            {
                r.uniqueID = r[j];
            }
            else if (j == recordfiles_idx)
            {
                r.recordfiles = r[j];
            }  
            
        }

        var src_sum = r[ch_ext_idx].indexOf("trunk") > -1 ? 1 : 0;
        var dst_sum = r[dstch_ext_idx].indexOf("trunk") > -1 ? 2 : 0;
        var sum = src_sum + dst_sum;
        
        if( sum == 0){
            r.calltype = "internal call";
        }else if(sum == 1){
            r.calltype = "incoming call";
        }else if(sum == 2){
            r.calltype = "outgoing call";
        }else if(sum == 3){
            r.calltype = "external call";
        }
        
//        r[nColumns] = ""; //add an extra field for play/download/delete record file.
        records[i] = r;
    }
    maxColumns = Math.min(maxColumns, long_fields["field_name"].length );
}

function onResponseErrorCDR(msg)
{
    
}

function dataBindCDR(val)
{
    if ($P.gs.isObject(val) && !$P.gs.isEmpty(val["cdr"]))
    {        
        var cdr_table = val["cdr"];
        if(cdr_table.schema != total_count)
        {
            processCDRResult(cdr_table.result);
            loadRecords(true);
            resetPageButtons();
        }
        else
        {
            nSelected = Math.floor(cdr_table.result);
            resetPageButtons();
        }
    }
}

function onResponseErrorClear(msg)
{
    top.dialog.clearDialog();
    top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG913") });
}

function dataBindClear(val)
{
    top.dialog.clearDialog();
    if ($P.gs.isObject(val) )
    {
        top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG961"), callback: function () { window.location.reload(); } });
        
    }
}

function performSearch(search_with_cond)
{
    var query_str = "";
    if(search_with_cond)
    {
        offset = 0;
        createSearchCondition();
        query_str = "select " + total_count + " from cdr" + condition;
	    $P.gs.actionRequest(document, "searchCDRDB", null, {
            "request": ["cdr-query=" + query_str],
            "auto": false,
            "onInvalidResponse": onResponseErrorCDR,
            "onResponseProcessed": dataBindCDR
        });
    }
    
    query_str = "SELECT * FROM cdr" + condition + orderbyStr() + limitStr() + offsetStr();
    //console.log(query_str);
    $P.gs.actionRequest(document, "searchCDRDB", null, {
        "request": ["cdr-query=" + query_str],
        "auto": false,
        "onInvalidResponse": onResponseErrorCDR,
        "onResponseProcessed": dataBindCDR
    });
}

window.onload = function ()
{
    if (_$('engine') != null)
    {
        _$('engine').innerHTML = backend;
    }
    _$("lastPageBtn").className = "btn btn-paging disabled";
    _$("nextPageBtn").className = "btn btn-paging disabled";
    _$("showOutbound").checked = true; showOutboundCalls = true;
    _$("showInbound").checked = true; showInboundCalls = true;
    _$("showInternal").checked = true; showInternalCalls = true;
    _$("showExternal").checked = true; showExternalCalls = true;
    //_$("longFields").onclick = function () { longversion = this.checked; };
    _$("showOutbound").onclick = function () { showOutboundCalls = this.checked; };
    _$("showInbound").onclick = function () { showInboundCalls = this.checked; };
    _$("showInternal").onclick = function () { showInternalCalls = this.checked; };
    _$("showExternal").onclick = function () { showExternalCalls = this.checked; };
    _$("selectViewCount").onchange = function () { offset = 0; viewCount = parseInt(this.value, 10); performSearch(false); };
    //_$("sfselect").onclick = function () { searchStartfrom = this.checked; }
    //_$("stselect").onclick = function () { searchStartto = this.checked; }
    _$("dfselect").onclick = function () { searchDurFrom = this.checked; }
    _$("dtselect").onclick = function () { searchDurTo = this.checked; }
    var jc = context2json({ filename: "cdr.conf", context: "general", usf: 1 });

    if (jc.hasOwnProperty('enable') && !jc['enable'].isAstTrue())
    {
        top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG990") });
    }

    var c = context2json({ filename: 'http.conf', context: 'general', usf: 1 });

    if (c.hasOwnProperty('enablestatic') && !c['enablestatic'].isAstTrue())
    {
        top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG991") });
    }
	var tmp_cmd = "action=systemcmdwithoutput&"+top.sessionData.directories.script_mastercsvexists;
	//sendListRequest();
	
	$.each(short_fields["field_name"], function(key,value)
	{
	    var tmp_index = long_fields["field_name"].indexOf(value);
	    svindex.push(tmp_index);
	});
    acctId_idx = long_fields["field_name"].indexOf("AcctId");
	billsec_idx = long_fields["field_name"].indexOf("billsec");
    duration_idx = long_fields["field_name"].indexOf("duration");
    uniqueid_idx = long_fields["field_name"].indexOf("uniqueid"); 
	clid_idx = long_fields["field_name"].indexOf("clid");
	src_idx = long_fields["field_name"].indexOf("src");
	dst_idx = long_fields["field_name"].indexOf("dst");
	calltype_idx = long_fields["field_name"].indexOf("calltype");
	ch_ext_idx = long_fields["field_name"].indexOf("channel_ext");
	dstch_ext_idx = long_fields["field_name"].indexOf("dstchannel_ext");
	start_idx = long_fields["field_name"].indexOf("start");
    recordfiles_idx = long_fields["field_name"].indexOf("recordfiles");
		
	sortby = -10;
	var exts = parent.pbx.users.list();
	exts.each( function(ext)
	{
	    var uinfo = parent.sessionData.pbxinfo.users[ext];
	    var username = uinfo.getProperty('fullname');
	    var user_obj = { number: ext, name: username};
	    extens.push(user_obj);
	    all_exts.push(ext);
	});
	//console.log(extens);
	
	var sessionroom = parent.sessionData.pbxinfo.conferences;

	for(var id in sessionroom) {
                    
        if(!sessionroom.hasOwnProperty(id))
            continue;
        confrooms.push(id);
        all_exts.push(id);
    }
    
    var c = parent.sessionData.pbxinfo['pagegroups'];
    //console.log(c);
    c.each( function(line)
    {
        var this_exten = ASTGUI.parseContextLine.getExten(line);
        pagegroups.push(this_exten);
        all_exts.push(this_exten);
    });
    
    var QUEUES_CONF = config2json({ filename: 'queues.conf', usf: 1 });
    var m = parent.sessionData.pbxinfo.queues.exten;
    for (var l in m)
    {
        if (m.hasOwnProperty(l))
        {
            if (!QUEUES_CONF.hasOwnProperty(l))
                continue;
            var queue_obj = { number: l, name: QUEUES_CONF[l].getProperty('fullname')};
            queues.push(queue_obj); 
            all_exts.push(l);        
        }
    }
    
    c = parent.sessionData.pbxinfo.ringgroups;
    for (var d in c)
    {
        if (c.hasOwnProperty(d))
        {
            var ringgroup_obj = { number: c[d]['extension'], name: c[d]['NAME']};
            ringgroups.push(ringgroup_obj);
            all_exts.push(c[d]['extension']);
        }
    }
   
    var vms = parent.sessionData.pbxinfo.vmgroups.getOwnProperties();
    
    vms.each( function(this_vmg_exten)
    {
        var vmgroup_obj = { number: this_vmg_exten, name: parent.sessionData.pbxinfo.vmgroups[this_vmg_exten].label };
        vmgroups.push(vmgroup_obj);   
        all_exts.push(this_vmg_exten);     
    });
    
    var lvms = parent.sessionData.pbxinfo.voicemenus.getOwnProperties();
    lvms.each(function (vm_name)
    {
        var THIS_VM = parent.sessionData.pbxinfo.voicemenus[vm_name];
        var lbl = THIS_VM.comment || vm_name;
        var ae_line = THIS_VM.getProperty('alias_exten');
        var ae = (ae_line) ? ASTGUI.parseContextLine.getExten(ae_line) : '';
        var ivr_obj = { number: ae, name: lbl};
        ivrs.push(ivr_obj);    
        all_exts.push(ae);   
               
    }); 
    
    var trunk_list = parent.pbx.trunks.list({ sip: true, iax: true });
    trunk_list.each(function (item)
    {
        var ttype = parent.pbx.trunks.getType(item);
        var provider_name = parent.sessionData.pbxinfo.trunks[ttype][item]['trunkname'];
        var trunk_obj = { number: item, name: provider_name };
        voip_trunks.push(trunk_obj);
    });
    
    trunk_list = parent.pbx.trunks.list({ analog: true });
    trunk_list.each(function (item)
    {
        var provider_name = parent.sessionData.pbxinfo['trunks']['analog'][item]['trunkname'];
        var trunk_obj = { number: item, name: provider_name };
        analog_trunks.push(trunk_obj);
    });
    
    if(voip_trunks.length > 0 && analog_trunks.length >0)
        all_trunks = voip_trunks.concat(analog_trunks);
    else if(voip_trunks.length > 0)
        all_trunks = voip_trunks;
    else if(analog_trunks.length > 0)
        all_trunks = analog_trunks;
        
    var contexts = ASTGUI.contexts;
    var featuresObj = new ASTGUI.customObject;
    featuresObj["appVmmain"] = { context: contexts.AppVmmain, label: 'LANG1253', contentType: 'Features' };
    featuresObj["appDialvm"] = { context: contexts.AppDialvm, label: 'LANG1232', contentType: 'Features' };
    featuresObj["appPickup"] = { context: contexts.AppPickup, label: 'LANG1218', contentType: 'Features' };
    featuresObj["appPaging"] = { context: contexts.AppPaging, label: 'LANG1248', contentType: 'Features' };
    featuresObj["appIntercom"] = { context: contexts.AppIntercom, label: 'LANG1246', contentType: 'Features' };
    featuresObj["appAgentPause"] = { context: contexts.AppAgentPause, label: 'LANG1204', contentType: 'Features' };
    featuresObj["appAgentUnpause"] = { context: contexts.AppAgentUnpause, label: 'LANG1206', contentType: 'Features' };
    featuresObj['fcode_cfb_on' ] = { context: ASTGUI.contexts.AppCallForwardBusyOn, label: 'LANG1220', contentType: 'Features'};
    featuresObj['fcode_cfb_off'] = { context:  ASTGUI.contexts.AppCallForwardBusyOff, label: 'LANG1222', contentType: 'Features'};
    featuresObj['fcode_cfn_on' ] = { context: ASTGUI.contexts.AppCallForwardNoAnswerOn, label: 'LANG1224', contentType: 'Features'};
    featuresObj['fcode_cfn_off'] = { context:  ASTGUI.contexts.AppCallForwardNoAnswerOff, label: 'LANG1226', contentType: 'Features'};
    featuresObj['fcode_cfu_on' ] = { context: ASTGUI.contexts.AppCallForwardUnconditionOn, label: 'LANG1228', contentType: 'Features'};
    featuresObj['fcode_cfu_off'] = { context:  ASTGUI.contexts.AppCallForwardUnconditionOff, label: 'LANG1230', contentType: 'Features'};
    featuresObj['fcode_dnd_on' ] = { context: ASTGUI.contexts.AppDoNotDisturbOn, label: 'LANG1238', contentType: 'Features'};
    featuresObj['fcode_dnd_off'] = { context:  ASTGUI.contexts.AppDoNotDisturbOff, label: 'LANG1240', contentType: 'Features'};
    featuresObj['fcode_blacklist_add'] = { context:  ASTGUI.contexts.AppBlacklistAddWrap, label: 'LANG2282', contentType: 'Features'};
    featuresObj['fcode_blacklist_remove'] = { context:  ASTGUI.contexts.AppBlacklistRemoveWrap, label: 'LANG2281', contentType: 'Features'};
    
    var featureList = featuresObj.getOwnProperties();
    var pbxinfo = parent.sessionData.pbxinfo;
    featureList.each( function(ext) {
        if(featuresObj[ext] && featuresObj[ext].contentType == "Features")
        {
             if (pbxinfo['localextensions'] != undefined) {
                 var extension;
                 if (pbxinfo['localextensions'].hasOwnProperty(featuresObj[ext].context)) 
                 {
                     extension = pbxinfo['localextensions'][featuresObj[ext].context];
                     feature_codes.push(extension);
                 }
             }
        }
    });
    //console.log(feature_codes);
    
    //console.log(all_trunks);
		
	var query_str = "select " + total_count + " from cdr";
	$P.gs.actionRequest(document, "searchCDRDB", null, {
        "request": ["cdr-query=" + query_str],
        "auto": false,
        "onInvalidResponse": onResponseErrorCDR,
        "onResponseProcessed": dataBindCDR
    });
    
    query_str = "SELECT * FROM cdr" + orderbyStr() + limitStr() + offsetStr();
    $P.gs.actionRequest(document, "searchCDRDB", null, {
        "request": ["cdr-query=" + query_str],
        "auto": false,
        "onInvalidResponse": onResponseErrorCDR,
        "onResponseProcessed": dataBindCDR
    });
    top.Custom.init(document);
}
