/*
CUSTOM FORM ELEMENTS

Created by Ryan Fait
www.ryanfait.com

The only things you may need to change in this file are the following
variables: checkboxHeight, radioHeight and selectWidth (lines 24, 25, 26)

The numbers you set for checkboxHeight and radioHeight should be one quarter
of the total height of the image want to use for checkboxes and radio
buttons. Both images should contain the four stages of both inputs stacked
on top of each other in this order: unchecked, unchecked-clicked, checked,
checked-clicked.

You may need to adjust your images a bit if there is a slight vertical
movement during the different stages of the button activation.

The value of selectWidth should be the width of your select list image.

Visit http://ryanfait.com/ for more information.
*/

/*
 * Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 */

var checkboxHeight = "25";
var radioHeight = "25";
var selectHeight = "22";
var fontSize = "12px";
var defaultWidth = "35px";
var fontWeight = "normal";
var iconPosition = "-139px";
var iconWidth = 18;
var modelSelMaxWidth = 130;
var mainSelMaxWidth = 300;
var mDocument;
/* No need to change anything after this */

var Custom = {
	init: function(document, elDom) {
		mDocument = document;

		var inputs = new Array(), 
			div = new Array();
		if (elDom && elDom.tagName == "INPUT") {
			inputs = new Array();
			inputs[0] = elDom;
		}
		else {
			inputs = document.getElementsByTagName("input");
		}
		for(a = 0; a < inputs.length; a++) {
			if((inputs[a].type == "checkbox" || inputs[a].type == "radio")) {
				inputs[a].parentNode.style.position = "relative";
				if (!inputs[a].className.contains("styled")) {
					$(inputs[a]).addClass("styled");
				}
				var prev = inputs[a].previousSibling;
				var clsName;
				if (prev) {
					clsName = prev.className
				}
				if (clsName && prev && clsName.contains(inputs[a].type.toLowerCase())) {
					div[a] = inputs[a].previousSibling;
				}
				else {
					div[a] = document.createElement("div");
					div[a].className = inputs[a].type;
				}

				if(inputs[a].checked) {
					if(inputs[a].type == "checkbox") {
						position = "0 -" + (checkboxHeight*2) + "px";
						div[a].style.backgroundPosition = position;
					} else {
						position = "0 -" + (radioHeight*2) + "px";
						div[a].style.backgroundPosition = position;
					}
				}
				else if (!inputs[a].checked && !inputs[a].disabled) {
					position = "0 0";
					div[a].style.backgroundPosition = position;
				}
				inputs[a].parentNode.insertBefore(div[a], inputs[a]);
				inputs[a].onchange = Custom.clear;
				if(!inputs[a].disabled) {
					div[a].onmouseover = Custom.pushed;
					div[a].onmouseout = Custom.unpushed;
					div[a].onmouseup = Custom.check;
				} 
				else {
					div[a].className = div[a].className += " disabled";
					if(inputs[a].checked == true) {
						if(inputs[a].type == "checkbox") {
							position = "0 -" + (checkboxHeight*4) + "px";
							div[a].style.backgroundPosition = position;
						} 
						else {
							position = "0 -" + (radioHeight*4) + "px";
							div[a].style.backgroundPosition = position;
						}
					}
					else {
						if(inputs[a].type == "checkbox") {
							position = "0 -" + (checkboxHeight*5) + "px";
							div[a].style.backgroundPosition = position;
						} 
						else {
							position = "0 -" + (radioHeight*5) + "px";
							div[a].style.backgroundPosition = position;
						}	
					}
					div[a].onmouseover = null;
					div[a].onmouseout = null;
					div[a].onmouseup = null;
				}
			}
		}
		if (elDom && elDom.tagName == "SELECT") {
			inputs = new Array();
			inputs[0] = elDom;
		}
		else {
			inputs = document.getElementsByTagName("select");
		}
		for(a = 0; a < inputs.length; a++) {
			var textnode = "";
			var option = inputs[a].getElementsByTagName("option");
			var widthArr = new Array();
			var clsName;
			$(inputs[a]).removeAttr("title");
			if (!mDocument.body.className.contains("modal")) {
				selMaxWidth = mainSelMaxWidth;
			}
			else {
				selMaxWidth = modelSelMaxWidth;
			}
			if (!inputs[a].size) {
				if ($(inputs[a]).attr("mSelect")) {
					selMaxWidth = mainSelMaxWidth;
				}
				if (!inputs[a].className.contains("styled")) {
					$(inputs[a]).addClass("styled");
				}
				var optionSelectedValue = $(inputs[a]).val();
				for(b = 0; b < option.length; b++) {
					$(option[b]).removeAttr("title");
					var optionValue = option[b].text;
					var result = this.textSize(option[b]);
					// if (!optionSelectedValue && optionSelectedValue != "") {
					// 	inputs[a].selectedIndex = -1;
					// }
					if (option[b].childNodes[0] && inputs[a].selectedIndex != -1) {
						option[b].childNodes[0].nodeValue = optionValue;
					}
					if (result.width > selMaxWidth) {
						var pos = 0;
						for(var i = parseInt(optionValue.length/2); i < optionValue.length; i++){
							option[b].text = optionValue.substring(0, i);
							var optResult = this.textSize(option[b]);
							if (optResult.width > selMaxWidth) {
								result.width = optResult.width;
								pos = i;
								i = optionValue.length;
							}
						}
						if (pos != 0) {
							option[b].text = optionValue.substring(0, pos) + "...";
							option[b].title = optionValue;
						}
					}
					widthArr.push(result.width);
					if(option[b].selected) {
						textnode = document.createTextNode(optionValue);
					}
				}
				if(!optionSelectedValue && optionSelectedValue != "") {
					textnode = document.createTextNode("");
				}
				var prev = inputs[a].previousSibling;
				if (prev) {
					clsName = prev.className
				}
				if (prev && prev.className && prev.className.contains("divSelect")) {
					$(prev).remove();
				}
				div[a] = document.createElement("div");
				div[a].className = "divSelect";
				spanDes = document.createElement("span");
				if (!mDocument.body.className.contains("modal")) {
					spanDes.className = "mainSpanDes";
				}
				else{
					spanDes.className = "modelSpanDes";
				}
				if ($(inputs[a]).attr("mSelect")) {
					spanDes.className = "mainSpanDes";
				};
				spanIcon = document.createElement("span");
				spanIcon.className = inputs[a].type;
				div[a].appendChild(spanDes);
				div[a].appendChild(spanIcon);
				spanIcon.className = "select";
				div[a].id = "select" + inputs[a].name;
				inputs[a].parentNode.insertBefore(div[a], inputs[a]);
				spanDes.innerText = "";
				spanDes.appendChild(textnode);
				if (widthArr.length == 0) {
					spanDes.style.width = defaultWidth; 
				}
				else{
					spanDes.style.width = Math.max.apply(Math, widthArr) + "px";
				}
				if ((spanDes.offsetWidth == 0 || spanIcon.offsetWidth == 0) && widthArr.length != 0) {
					inputs[a].style.width = (Math.max.apply(Math, widthArr) + 9 + iconWidth) + "px";
				}
				else{
					inputs[a].style.width = (spanDes.offsetWidth + spanIcon.offsetWidth + 2) + "px";
				}
				if ($(inputs[a]).css("display") == "none") {
					$(div[a]).css("display", "none");
				}
				$(inputs[a]).bind({
					focus: function(ev) {
						Custom.select_focus_in(this);
						ev.stopPropagation();
					},
					blur: function(ev) {
						Custom.select_focus_out(this);
						ev.stopPropagation();
					}
				});
				$(inputs[a]).bind("mouseover", function(ev) {
					var span = this.previousSibling.children.item(0);
					var result = Custom.textSize(span);
	                if (result.width > span.offsetWidth && result.width > selMaxWidth) {
	                	var text;
		                if (typeof span.textContent != "undefined")
				            text = span.textContent;
				        else 
				        	text = span.innerText;
		                $(this).attr("title", text);
	                }
	                else {
	                    $(span).removeAttr("title");
	                }
	                ev.stopPropagation();
				});
				if(!inputs[a].disabled) {
					$(inputs[a]).bind("change", function(ev) {
						Custom.choose(this);
						ev.stopPropagation();
					});
				} 
				else {
					inputs[a].previousSibling.children.item(0).className += " spanDesDisable";
					inputs[a].previousSibling.children.item(1).style.backgroundPosition = iconPosition + " -" + selectHeight*2 + "px";
				}
			}
		}
		document.onmouseup = Custom.clear;
	},
	pushed: function() {
		element = this.nextSibling;
		if(element.checked == true && element.type == "checkbox") {
			this.style.backgroundPosition = "0 -" + checkboxHeight*3 + "px";
		} else if(element.checked == true && element.type == "radio") {
			this.style.backgroundPosition = "0 -" + radioHeight*3 + "px";
		} else if(element.checked != true && element.type == "checkbox") {
			this.style.backgroundPosition = "0 -" + checkboxHeight + "px";
		} else {
			this.style.backgroundPosition = "0 -" + radioHeight + "px";
		}
	},
	unpushed: function() {
		element = this.nextSibling;
		if(element.checked == true && element.type == "checkbox") {
			this.style.backgroundPosition = "0 -" + checkboxHeight*2 + "px";
		} else if(element.checked == true && element.type == "radio") {
			this.style.backgroundPosition = "0 -" + radioHeight*2 + "px";
		} else if(element.checked != true && element.type == "checkbox") {
			this.style.backgroundPosition = "0 0";
		} else {
			this.style.backgroundPosition = "0 0";
		}
	},
	check: function() {
		element = this.nextSibling;
		if(element.checked == true && element.type == "checkbox") {
			this.style.backgroundPosition = "0 0";
		} 
		else {
			if(element.type == "checkbox") {
				this.style.backgroundPosition = "0 -" + checkboxHeight*2 + "px";
			} else {
				this.style.backgroundPosition = "0 -" + radioHeight*2 + "px";
				group = this.nextSibling.name;
				inputs = mDocument.getElementsByTagName("input");
				for(a = 0; a < inputs.length; a++) {
					if(inputs[a].name == group && inputs[a] != this.nextSibling) {
						inputs[a].previousSibling.style.backgroundPosition = "0 0";
					}
				}
			}
		}
		$(element).trigger("click");
	},
	clear: function() {
		inputs = document.getElementsByTagName("input");
		for(var b = 0; b < inputs.length; b++) {
			if(inputs[b].type == "checkbox" && inputs[b].checked == true && inputs[b].className.contains("styled")) {
				inputs[b].previousSibling.style.backgroundPosition = "0 -" + checkboxHeight*2 + "px";
			} else if(inputs[b].type == "checkbox" && inputs[b].className) {
				inputs[b].previousSibling.style.backgroundPosition = "0 0";
			} else if(inputs[b].type == "radio" && inputs[b].checked == true && inputs[b].className.contains("styled")) {
				inputs[b].previousSibling.style.backgroundPosition = "0 -" + radioHeight*2 + "px";
			} else if(inputs[b].type == "radio" && inputs[b].className.contains("styled")) {
				inputs[b].previousSibling.style.backgroundPosition = "0 0";
			}
		}
	},
	choose: function(obj) {
		option = obj.getElementsByTagName("option");
		$(obj).removeAttr("title");
		var span = obj.previousSibling.children.item(0);
		var icon = obj.previousSibling.children.item(1);
		for(d = 0; d < option.length; d++) {
			if(option[d].selected == true) {
				var optionValue = option[d].childNodes[0].nodeValue;
				var optionTitle = option[d].title;
				if (optionValue.contains("...") && !/^\.+$/.test(optionValue)) {
					this.text(span, optionTitle);
				}
				else {
					this.text(span, optionValue);
				}
			}
		}
		icon.className = "select";
		if ($.browser.msie || $.browser.mozilla) {
			icon.className += " selectIe";
		}
		obj.style.width = (span.offsetWidth + icon.offsetWidth) + "px";
	},
	select_focus_in: function (obj){
		$(obj.previousSibling).addClass("divSelectFocus");
		obj.previousSibling.children.item(1).style.backgroundPosition = iconPosition + " -" + selectHeight + "px";
	},
    select_focus_out: function (obj) {
    	$(obj.previousSibling).removeClass("divSelectFocus");
		obj.previousSibling.children.item(1).style.backgroundPosition = iconPosition + " 0px";
	},
	select_disable: function (obj) {
    	$(obj.previousSibling.children.item(0)).addClass("spanDesDisable");
		obj.previousSibling.children.item(1).style.backgroundPosition = iconPosition + " -" + selectHeight*2 + "px";
	},
    textSize: function (obj) {
        var text = obj.text || obj.innerText;
        var span = document.createElement("span");
		if (!mDocument.body.className.contains("modal")) {
			span.className = "mainSpanDes";
		}
		else{
			span.className = "modelSpanDes";
		}
        var result = {};
        result.width = span.offsetWidth;
        result.height = span.offsetWidth;
        span.style.fontSize = fontSize;
        span.style.fontWeight = fontWeight;
        span.style.visibility = "hidden";
        document.body.appendChild(span);
        this.text(span, text);
        result.width = span.offsetWidth + 9 - result.width;
        result.height = span.offsetHeight - result.height;
        span.parentNode.removeChild(span);
        return result;
    },
    text: function(el, text) {
        if (typeof el.textContent != "undefined")
            el.textContent = text;
        else 
        	el.innerText = text;
    }
}