/*
 * Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
$P = top.$;

var datetime_names = [ "P30", "P144", "P64", "P143", "P246"];

function save_changes()
{
    if (!$P("#form", document).valid())
    {
        return;
    }

    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG905") });
    var data = { action : "setparam" };
    
    datetime_names.each(function (name) {
        var elements = $(":input[name=" + name + "]");
        if (elements)
        {
            var type = $(elements).attr('type');
            if (type == 'checkbox')
            {
                var value=0;
                if ( elements.attr('checked') == "checked" )
                {
                    value=1;
                }
            }
            else if (type == 'select')
            {
                var index=$(elements)[0].selectedIndex;
                var value=$(elements)[0].options[index].value;
            }
            else
            {
                var value = elements.val();
            }
            data[name] = value || '';
        }
    });
    
    function apply_changes()
    {
        top.dialog.clearDialog();
        $.ajax({
            type : 'GET',
            url : '../webcgi?action=reloadTimezone&timezoneupdate=',
            success : function(response) {
                // reset systime timer
                top.sessionData.systime.stoptimer = true;

                if (response.response.trim() == 'success') 
                {
                    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG945") });
                }
                else
                {
                    top.dialog.dialogMessage({ type: 'error', content: response.body });
                }
            }
        });
    }

    $.ajax({url: ASTGUI.paths.rawman, data: data , async : true, timeout : 3000,
             success: function(msg)
             {
                top.dialog.clearDialog();
                top.dialog.dialogConfirm({ confirmStr: $P.lang("LANG928"), 
                    buttons:{ ok: function () { apply_changes(); }, 
                    cancel: function () { window.location.reload(); } } });
             },
             error: function(msg)
             {
                top.dialog.clearDialog();
                setTimeout(function()
                {
                    top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG861")});
                }, 10);
             }
    });

};
var addRule = function () {
    if ( $("[name=P64]").get(0).value === "customize" )
    {
        $("[name=P246]").removeAttr("disabled").removeClass("disabled");
        $P("[name=P246]", document).rules("add", {
            required: true
        });
    }
    else
    {
        $("[name=P246]").attr("disabled", "disabled").addClass("disabled");
        $P("[name=P246]", document).rules("remove", "required");
    }
}
window.onload = function ()
{
    $P.lang(document, true);
    top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG718"));
    $("title").attr("locale", "LANG718");
    $("[name=P64]").change( function () {
        addRule();
    });
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "P30": {
                maxlength: 64,
                host: 'host'
            },
            "P246": {
                maxlength: 64
            }
        },
        submitHandler: function() {
            save_changes();
        }
    });

    var getparam_success = function(msg)
    {
        msg = unescape(msg);
        var verdata = msg.split("\r\n");
        var i, j;
        for (i = 0; i < verdata.length; ++i)
        {
            var data  = verdata[i].split("=");
            var elements = $(":input[name=" + data[0] + "]");
            if (elements)
            {
                    var type = $(elements).attr('type');
                    switch (type)
                    {
                        case 'select':
                            $(elements).find("[value='"+data[1]+"']").attr("selected", "true");
                            if ( data[1] === 'customize' )
                            {
                                $('[name=P246]').removeAttr('disabled').removeClass("disabled");
                            }
                            else
                            {
                                $('[name=P246]').attr('disabled', 'disabled').addClass("disabled");
                            }
                            break;
                        case 'checkbox':
                            if ( data[1] == "1" )
                            {
                                $(elements).prop('checked', 'checked');
                            }
                            break;
                        case 'text':
                            $(elements).val(data[1]);
                            break;
                        default:
                            break;
                    }
            }
        }
        addRule();
        top.Custom.init(document);
    };

    var verparam = datetime_names.join("&");
    $.ajax({url: ASTGUI.paths.rawman + '?action=getparam&' + verparam, success : getparam_success });

};
