/*
 * Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
$P = top.$;
var mWindow = window;
var DISA_NUMBER = 0;
/* the number of disa display on on page*/
var PAGE_NUMBER = 20;
var USERSLIST;
var NAMELIST;
var EDITUSERID;
var PAGE = 1;
var NPAGE = 1;
var NUMBER = -1;

function load_disa_number()
{
    var buf = "action=disagetall&keyword=displayname";
    $.ajax({
        type: "post",
        url: "/webcgi",
        data: buf,
		error: function (jqXHR, textStatus, errorThrown) { top.dialog.dialogMessage({ type: 'error', content: errorThrown }); },
        success: function (data)
        {
            NAMELIST = data.body ? data.body : [];
            DISA_NUMBER= NAMELIST.length;
            load_users_table();
        }
    });
}
        
window.onload = function ()
{
    $P.lang(document, true);
    var mode = gup("mode");
    var item = gup("item");
    var jump = gup("page_jump");

    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        if (mode === "edit")
        {
            EDITUSERID = mWindow.USERSLIST[item];
            prepareEditItemForm(item);
        }
        else if (mode === "add")
        {
            prepareAddItemForm();
        }
        init_validate();
        isEnableWeakPw();
        $("#edit_disa_div").show();
        return false; // bypass reload dialog
    }
    else if ( jump == undefined)
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG2353"));
        $("title").attr("locale", "LANG2353");
        load_disa_number();
        jump_event();
    }
};
var isEnableWeakPw = function () {
    var GUI_PREFERENCES = top.sessionData.GUI_PREFERENCES;
    if (!GUI_PREFERENCES.getProperty('weak_password')) {
        GUI_PREFERENCES  = context2json({ filename: 'guipreferences.conf', context: 'general', usf: 1 });
    }
    if (GUI_PREFERENCES.weak_password == "yes") {
        $P("#passwd", document).rules("add", { 
            checkNumericPw: [ 
                showCheckPassword, {
                    type: "digital",
                    pwsId: "#passwd"
                }
            ] 
        });
    }
}
var init_validate = function () {
    if ($("#form").tooltip) {
        $("#form").tooltip();
    }
    $P("#form", document).validate({
        rules:{
            "displayname": {
                required: true, 
                letterDigitUndHyphen: true,
                customCallback: [$P.lang("LANG2137"), function(){
                    var flag = true;
                    mWindow.NAMELIST.each(function(val)
                    {
                        if (_$("displayname").value == val.displayname && _$("displayname").value != EDITUSERID.displayname)
                        {
                            flag = false;
                            return flag;
                        }
                    });
                    
                    return flag;
                }]
            },
            "passwd": {
                required: true,
                digits: true,
                minlength: 4
            },
            "resptimeout": {
                required: true,
                digits: true,
                range: [3,180]
            },
            "digittimeout": {
                required: true,
                digits: true,
                range: [1,10]
            },
            "callerid": {
                digits: true,
                minlength: 2
            }
        },   
        submitHandler: function() {
            saveDisa();
        }
    });
}
var show_users_list = function (n)
{
    var TBL = _$('table_userslist');
    var addCell = ASTGUI.domActions.tr_addCell;
    ASTGUI.domActions.clear_table(TBL);
    (function ()
    { // add first row
        var newRow = TBL.insertRow(-1);
        newRow.className = "frow";
        addCell(newRow, { html: $P.lang("LANG135"), locale: "LANG135"});
        addCell(newRow, { html: $P.lang("LANG1069"), locale: "LANG1069"});
        addCell(newRow, { html: $P.lang("LANG2358"), locale: "LANG2358"});
        addCell(newRow, { html: $P.lang("LANG2360"), locale: "LANG2360" });
        //addCell(newRow, { html: $P.lang("LANG1359"), locale: "LANG1359" });
        addCell(newRow, { html: $P.lang("LANG2363"), locale: "LANG2363" });
        addCell(newRow, { html: $P.lang("LANG2299"), locale: "LANG2299"});
    })();

    var buf = "action=disagetall&pagesize=" + PAGE_NUMBER + "&page=" + n;
    $.ajax({
        type: "post",
        url: "/webcgi",
        data: buf,
		error: function (jqXHR, textStatus, errorThrown) { top.dialog.dialogMessage({ type: 'error', content: errorThrown }); },
        success: function (data)
        {
			USERSLIST = data.body ? data.body : [];
            set_table_option();
        }
    });
    
    var set_table_option = function()
    {
        for (var i = 0; i < 20 && i < USERSLIST.length; i++)
        {
            var user = USERSLIST[i];
            var newRow = TBL.insertRow(-1);
            var tmp = "<button type='button' class='options edit' onclick=showEditForm(" + i +") localetitle=\"LANG738\" title=" + $P.lang("LANG738") + "></button>" +
                     "<button type='button' class='options del' onclick=deleteDisa(" + i + ") localetitle=\"LANG739\" title=" + $P.lang("LANG739") + "></button>";
            
            newRow.className = ((TBL.rows.length) % 2 == 1) ? 'odd' : 'even';
            addCell(newRow, { html: user.displayname || ""});
            var permission = user.privilege || "internal";
            var str;
            if (permission.indexOf("international") != -1)
            {
                str = "LANG1074";
            }
            else if (permission.indexOf("national") != -1)
            {
                str = "LANG1073";
            }
            else if (permission.indexOf("local") != -1)
            {
                str = "LANG1072";
            }
            else
            {
                str = "LANG1071";
            }
                
            addCell(newRow, { html: $P.lang(str), locale: str});
            addCell(newRow, { html: user.resptimeout });
            addCell(newRow, { html: user.digittimeout });
            //addCell(newRow, { html: user.callerid });
            str = user.hangup == "yes" ? "LANG136" : "LANG137";
            addCell(newRow, { html: $P.lang(str), locale: str });
            addCell(newRow, { html: tmp });
        }
    
        _$('totle').innerHTML = DISA_NUMBER;
        _$('show').innerHTML = PAGE + '/' + NPAGE;
        _$("page_jump").value = "";
        btn_disable();
    };
}

var load_users_table = function ()
{
    var TBL = _$('table_userslist');
    var addCell = ASTGUI.domActions.tr_addCell;
    var i = DISA_NUMBER % PAGE_NUMBER;
    NPAGE = (DISA_NUMBER - i + (i ? PAGE_NUMBER : 0)) / PAGE_NUMBER;

    if (!DISA_NUMBER)
    {
        ASTGUI.domActions.clear_table(TBL);
        var newRow = TBL.insertRow(-1);
        newRow.className = 'even';
        addCell(newRow, { html: $P.lang("LANG129").format($P.lang("LANG2353")), locale: "LANG129 LANG2353"});
        _$('totle').innerHTML = '0';
        _$('show').innerHTML = '1/1';
        document.getElementById('pre_button').style.display = 'none';
        //document.getElementById('next_button').style.display = 'none';
        //document.getElementById('jump_button').style.display = 'none';
        return;
    }
    show_users_list(1);
};

var prepareAddItemForm = function ()
{
    EDITUSERID = "";
    ASTGUI.resetTheseFields(['displayname', 'passwd', 'privilege', 'resptimeout', 'digittimeout', 'callerid', 'hangup']);
    _$("resptimeout").value = "10";
    _$("digittimeout").value = "5";
    _$("privilege").value = "internal";
    top.Custom.init(document);
};

var prepareEditItemForm = function (i)
{
    var user = mWindow.USERSLIST[i];
    
    _$("displayname").value = user.displayname;
    _$("passwd").value = user.passwd;
    _$("privilege").value = user.privilege;
    _$("resptimeout").value = user.resptimeout;
    _$("digittimeout").value = user.digittimeout;
    _$("callerid").value = user.callerid;
    //_$("hangup").value = user.hangup;
    if (user.hangup && user.hangup == "yes")
    {
        _$("hangup").checked = true
    }
    top.Custom.init(document);
};

var after = function ()
{
    $.ajax({
        type: "post",
        url: "/webcgi",
        data: "action=deployDisa&disa-apply=",
        error: function (jqXHR, textStatus, errorThrown) { top.dialog.dialogMessage({ type: 'error', content: errorThrown }); },
        success: saveSuccess
    });  
};
    
var saveSuccess = function ()
{
    top.dialog.dialogMessage({
        type: 'success',
        content: $P.lang("LANG815"),
        callback: function ()
        {
            top.$.cookie('configFilesChanged', 'yes');
            top.$('#applyChanges_Button').css("visibility", "visible");
            top.$('#line_Button').css("visibility", "visible");
            top.$('#applyChanges_Button').effect("shake", {
                direction: "up", distance: 2, times:10000 
            }, 400);
            mWindow.location.reload();
        }
    });
};
    
var saveDisa = function ()
{
    var levelVal = $("#privilege").val();
    if (levelVal != "internal") {
        top.dialog.dialogConfirm({
            type: "warning",
            confirmStr: $P.lang("LANG2537").format($P.lang("LANG1071"), $P.lang("LANG2353")),
            buttons: {
                ok:  function (){
                    doSaveDisa();
                },
                cancel:  function (){
                    top.dialog.container.show();
                    top.dialog.shadeDiv.show();
                }
            }
        }); 
    }
    else {
        doSaveDisa();
    }
}
var doSaveDisa = function ()
{
    if ( EDITUSERID )
    {
        var buf = "action=disamode&disaid=" + EDITUSERID.disaid; 
    }
    else
    {
        var buf = "action=disaadd";
    }
    
    buf += "&displayname=" + _$("displayname").value + "&passwd=" + _$("passwd").value 
        + "&privilege=" + _$("privilege").value + "&resptimeout=" + _$("resptimeout").value
        + "&digittimeout=" + _$("digittimeout").value + "&callerid=" + _$("callerid").value
        + "&hangup=" +  ASTGUI.getFieldValue("hangup");

    $.ajax({
        type: "post",
        url: "/webcgi",
        data: buf,
        error: function (jqXHR, textStatus, errorThrown) { top.dialog.dialogMessage({ type: 'error', content: errorThrown }); },
        success: after
    });

};

var deleteDisa = function (i)
{   
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(USERSLIST[i].displayname),
        buttons: {
            ok: function ()
            {
                var buf = "action=disadel&disaidlist=" + USERSLIST[i].disaid;
                
                $.ajax({
                    type: "post",
                    url: "/webcgi",
                    data: buf,
                    error: function (jqXHR, textStatus, errorThrown) { top.dialog.dialogMessage({ type: 'error', content: errorThrown }); },
                    success: after
                });  
            }
        }
    });   
};

var select_page = function (a)
{
    if ($("#" + a).attr("disabled") == "disabled")
    {
        return;
    }
    
    if ( a == "first")
    {
        PAGE = 1;
    }
    else if (a == "next")
    {
        if (PAGE >= NPAGE)
        {
            return;
        }

        PAGE++;
    }
    else if (a == "pre")
    {
        if (PAGE <= 1)
        {
            return;    
        }
        PAGE--;
    }
    else if (a == "last")
    {
        PAGE = NPAGE;
    }
    else if (a == "jump")
    {
        var page = ASTGUI.getFieldValue('page_jump');
        if (page < 1 || page > NPAGE || isNaN(page)) {
            $(".jump").blur();
            top.dialog.dialogMessage({
                type: 'warning',
                content: $P.lang("LANG929")
            })
            _$("page_jump").value = "";
            return;
        }
        
        PAGE = Number(page);
    }

    show_users_list(PAGE, NUMBER);
};

function btn_disable()
{
    $("#first").removeClass("disabled").removeAttr("disabled");
    $("#pre").removeClass("disabled").removeAttr("disabled");
    $("#next").removeClass("disabled").removeAttr("disabled");
    $("#last").removeClass("disabled").removeAttr("disabled");
    
    if (PAGE == 1) {
        $("#first").addClass("disabled").attr("disabled", "disabled");
        $("#pre").addClass("disabled").attr("disabled", "disabled");
    }

    if (PAGE == NPAGE)
    {
        $("#next").addClass("disabled").attr("disabled", "disabled");
        $("#last").addClass("disabled").attr("disabled", "disabled");
    }
};

var jump_event = function(){

    $(".jump").bind({
        mousedown: function(ev){
            if(ev.button==2){
                $(this).val($(this).val().replace(/\D|^0/g,''));
                ev.stopPropagation();
                return false;
            }
        },
        keydown: function(ev){
            if (ev.keyCode == 13)
            {
                var page = parseInt($(".jump").val());
                if (page < 1 || page > NPAGE || isNaN(page)) {
                    $(".jump").blur();
                    top.dialog.dialogMessage({
                        type: 'warning',
                        content: $P.lang("LANG929")
                    })
                    _$("page_jump").value = "";
                    return;
                }

                PAGE = page;
                show_users_list(PAGE, NUMBER);  
                ev.stopPropagation();
                return false;
            }
        },
        keyup: function(ev){      //keyup event
            $(this).val($(this).val().replace(/\D|^0/g,''));
            ev.stopPropagation();
            return false;
        },
        paste: function(ev){
            $(this).val($(this).val().replace(/\D|^0/g,''));
            ev.stopPropagation();
            return false;
        }   
    });
};
