/*
 * Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
$P = top.$;
var mWindow = window;

window.onload = function()
{
    $P.lang(document, true);    
    load_DOMelements();
    update_dynamic_defense();
    init_validator();
    $('#dd_detect_enable').trigger("change");
    top.Custom.init(document);
};

var load_DOMelements = function ()
{
    var enable = $('#dd_detect_enable');
    enable.change( function()
    {
        var el = _$('dd_detect_enable');
        if ( el.checked ) {
            _$('dd_detect_timeout').disabled = false;
            _$('dd_detect_block_timeout').disabled = false;
            _$('dd_detect_threshold').disabled = false;
            _$('dd_detect_whitelist').disabled = false;
            _$('dd_dynamic_blacklist').disabled = false;
        } else {
            _$('dd_detect_timeout').disabled = true;
            _$('dd_detect_block_timeout').disabled = true;
            _$('dd_detect_threshold').disabled = true;
            _$('dd_detect_whitelist').disabled = true;
            _$('dd_dynamic_blacklist').disabled = true;
        }
    } );

    DOM_dd_dynamic_blacklist = _$('dd_dynamic_blacklist');
    DOM_dd_detect_enable = _$('dd_detect_enable');
    DOM_dd_detect_timeout = _$('dd_detect_timeout');
    DOM_dd_detect_block_timeout = _$('dd_detect_block_timeout');
    DOM_dd_detect_threshold = _$('dd_detect_threshold');
    DOM_dd_detect_whitelist = _$('dd_detect_whitelist');
};

var init_validator = function()
{
    //$P = top.$;
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "dd_detect_timeout": {
                digits: true,
                range: [1, 59],
                customCallback: ["This field is required, when enable=yes.", check_detect_timeout]
            },
            "dd_detect_block_timeout": {
                digits: true,
                range: [1, 86400],
                customCallback: ["This field is required, when enable=yes.", check_block_timeout]
            },
            "dd_detect_threshold": {
                digits: true,
                range: [1, 1000],
                customCallback: ["This field is required, when enable=yes.", check_detect_threshold]
            },
            "dd_detect_whitelist": {
                customCallback: ["Invalid format.", check_whitelist]
            }
        },
        submitHandler: function(ev) {
            var target = this.submitButton;
            if ($(target).attr('id') !== 'save')
            {
                return;
            }
            saveChanges();
        }
    });
};

var array2object = function( list )
{
    var obj = {};
    for ( var idx = 0; list && idx < list.length; idx++ )
    {
        var item = list[idx];
        var key = item.split('=')[0];
        var value = item.split('=')[1];
        key = key.replace(/^\s+|\s+$/g, '');
        value = value.replace(/^\s+|\s+$/g, '');
        obj[key] = value;
    }

    return obj;
}

var object2array = function( obj )
{
    var list = [];
    for ( var k in obj ) 
    {
        if ( typeof(k) == "string" )
        {
            var str = k + ' = ' + obj[k];
            list.push(str);
        }
    }

    return list;
}

var update_dynamic_defense = function()
{
    var general_settings = parent.sessionData.pbxinfo['dynamic_defense'];
    var general = general_settings['general'];
    general = array2object( general );

    var enable = general['enable'] || "";
    if ( enable == "yes" )
    {
        DOM_dd_detect_enable.checked = true;
        $('#dd_detect_enable').trigger('change');
    } else {
        DOM_dd_detect_enable.checked = false;
        $('#dd_detect_enable').trigger('change');
    }

    var timeout = general['timeout'] || "";
    ASTGUI.updateFieldToValue(DOM_dd_detect_timeout, timeout);

    var block_timeout = general['block_timeout'] || "";
    ASTGUI.updateFieldToValue(DOM_dd_detect_block_timeout, block_timeout);

    var threshold = general['threshold'] || "";
    ASTGUI.updateFieldToValue(DOM_dd_detect_threshold, threshold);

    var whitelist = general_settings['whitelist'];
    var tmp_value = "";
    for ( var idx = 0; whitelist && idx < whitelist.length; idx++ )
    {
        if ( whitelist[idx] && whitelist[idx] != "" ) {
            var item = whitelist[idx].split('=')[1].trim();
            tmp_value += item;
            tmp_value += "\n";
        }
    }
    ASTGUI.updateFieldToValue(DOM_dd_detect_whitelist, tmp_value);

    var addCell = ASTGUI.domActions.tr_addCell;
    var newRow = DOM_dd_dynamic_blacklist.insertRow(-1);
    newRow.className = "frow";
    addCell(newRow, { html: "" });
    addCell(newRow, { html: $P.lang("LANG2293"), locale: "LANG2293" });
    addCell(newRow, { html: $P.lang("LANG2299"), locale: "LANG2299" });
    var PreviousTRColor = 'odd'; // 'odd' : 'even' ;

    var blacklist = parent.sessionData.pbxinfo['dynamic_blacklist'];
    var list = blacklist['blacklist'] || [];
    for ( var idx = 0; list && idx < list.length; idx++ )
    {
        var item = list[idx];
        var value = item.split('=')[1];

        newRow = DOM_dd_dynamic_blacklist.insertRow(-1);
        newRow.className = PreviousTRColor;
        addCell(newRow, { html: ""});
        addCell(newRow, { html: value});
        var btn = "<button type='button' class='options del' onclick=\"removeItem('" + value + "')\" title=" + $P.lang("LANG739") + "></button>";
        addCell(newRow, { html: btn });
    }
};

var removeItem = function(item)
{
    var table = $('#dd_dynamic_blacklist');
    var ch = table.children();
    if ( ch )
        ch.remove();

    var addCell = ASTGUI.domActions.tr_addCell;
    var newRow = DOM_dd_dynamic_blacklist.insertRow(-1);
    newRow.className = "frow";
    addCell(newRow, { html: "" });
    addCell(newRow, { html: $P.lang("LANG2293"), locale: "LANG2293" });
    addCell(newRow, { html: $P.lang("LANG2299"), locale: "LANG2299" });
    var PreviousTRColor = 'odd'; // 'odd' : 'even' ;

    var blacklist = parent.sessionData.pbxinfo['dynamic_blacklist'];
    var list = blacklist['blacklist'] || [];
    for ( var idx = 0; list && list[idx]; )
    {
        var c = list[idx];
        var value = c.split('=')[1];
        if ( value != item )
        {
            newRow = DOM_dd_dynamic_blacklist.insertRow(-1);
            newRow.className = PreviousTRColor;
            addCell(newRow, { html: ""});
            addCell(newRow, { html: value});
            var btn = "<button type='button' class='options del' onclick=\"removeItem('" + value + "')\" title=" + $P.lang("LANG739") + "></button>";
            addCell(newRow, { html: btn });
            idx++;
        } else {
            // remove it from blacklist 
            list.splice(idx, 1);
        }
    }

    
    var removedItem = parent.sessionData.pbxinfo['dynamic_removed_list'];
    removedItem.push(item);
}


var saveChanges = function()
{
    var general_settings = parent.sessionData.pbxinfo['dynamic_defense'];
    var conf = listOfSynActions("dynamic_defense.conf");

    var enable = "no";
    if ( DOM_dd_detect_enable.checked )
    {
        enable = "yes";
    } 

    conf.new_action("update", "general", "enable", enable);

    var timeout = DOM_dd_detect_timeout.value
    conf.new_action("update", "general", "timeout", timeout);

    var block_timeout = DOM_dd_detect_block_timeout.value
    conf.new_action("update", "general", "block_timeout", block_timeout);

    var threshold = DOM_dd_detect_threshold.value;
    conf.new_action("update", "general", "threshold", threshold);

    var whitelist = DOM_dd_detect_whitelist.value;
    var list = whitelist.split('\n');
    var wlist = [];
    conf.new_action('delcat', 'whitelist', '', '');
    conf.new_action('newcat', 'whitelist', '', '');
    for ( var idx = 0; idx < list.length; idx++ )
    {
        if ( list[idx] != "" )
        {
            conf.new_action('append', 'whitelist', 'include', list[idx]);
            list[idx] = "include = " + list[idx];
            wlist.push(list[idx]);
        }
    }
    $P.cookie("disNoneApply", true);
    var resp = conf.callActions();
    if ( !resp.contains('Response: Success') )
    {
        top.log.error('pbx.call_plans.add: Error update extensions.conf.');
        return false;
    }
    conf.clearActions();

    var tmp_settings = array2object( general_settings['general'] );
    tmp_settings['enable'] = enable;
    tmp_settings['timeout'] = timeout; 
    tmp_settings['block_timeout'] = block_timeout;
    tmp_settings['threshold'] = threshold; 
    general_settings['whitelist'] = wlist;

    general_settings['general'] = object2array( tmp_settings );

    var cconf = listOfSynActions("blacklist");
    var blacklist = parent.sessionData.pbxinfo['dynamic_blacklist'];
    var blist = blacklist['blacklist'];
    cconf.new_action('delcat', 'blacklist', '', '');
    cconf.new_action('newcat', 'blacklist', '', '');
    for ( var idx = 0; blist && idx < blist.length; idx++ )
    {
        if ( blist[idx] && blist[idx] != "" )
        {
            var value = blist[idx].split('=')[1];
            cconf.new_action('append', 'blacklist', 'include', value);
        }
    }

    var removedList = parent.sessionData.pbxinfo['dynamic_removed_list'];
    cconf.new_action('delcat', 'removedlist', '', '');
    cconf.new_action('newcat', 'removedlist', '', '');
    for ( var idx = 0; removedList && idx < removedList.length; idx++ )
    {
        if ( removedList[idx] && removedList[idx] != "" )
            cconf.new_action('append', 'removedlist', 'exten', removedList[idx]);
    }
    $P.cookie("disNoneApply", true);
    var resp = cconf.callActions();
    if ( !resp.contains('Response: Success') )
    {
        top.log.error('pbx.call_plans.add: Error update extensions.conf.');
        return false;
    }

    parent.sessionData.pbxinfo['dynamic_removed_list'] = [];
    applyChanges();
    return true;
}

 var applyChanges = function ()
{
    $.ajax({
        type: 'GET',
        url: '../webcgi?action=reloadDynamicDefense&dynamic_defense=',
        success: function( response )
        {
            if (response.response.trim() == 'success') 
            {
                top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG945") });
            }
            else
            {
                top.dialog.dialogMessage({ type: 'error', content: response.body });
            }
        }
    });
}

var check_detect_timeout = function()
{
    var enable = _$('dd_detect_enable');
    var timeout = _$('dd_detect_timeout');
    if ( enable.checked )
    {
        if ( timeout.value == "" )
            return false;
    }

    return true;
}

var check_block_timeout = function()
{
    var enable = _$('dd_detect_enable');
    var timeout = _$('dd_detect_block_timeout');
    if ( enable.checked )
    {
        if ( timeout.value == "" )
            return false;
    }

    return true;
}

var check_detect_threshold = function()
{
    var enable = _$('dd_detect_enable');
    var threshold = _$('dd_detect_threshold');
    if ( enable.checked )
    {
        if ( threshold.value == "" )
            return false;
    }

    return true;
}

var check_whitelist = function()
{
    var enable = _$('dd_detect_enable');
    var whitelist = _$('dd_detect_whitelist');
    if ( enable.checked )
    {
        var values = whitelist.value;
        var list = values.split('\n');
        for ( var i = 0; i < list.length; i++ )
        {
            var listVal = list[i];
            if (listVal.indexOf(" ") < 0) {
                listVal = listVal.trim();
            }
            if ( listVal != "" && !/^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$/.test(listVal) )
                return false;
        }
    }
    return true;
}
