/*
 * Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
$P = top.$;
window.event_list;
window.event_user_list = {};
var mWindow = window;
var isNewBridge;
var EDIT_BRIDGE;

window.onload = function ()
{
    $P.lang(document, true);

    var mode = gup("mode");
    var item = gup("item");
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        init_event_validate();
        
        if (mode == "add")
        {
            prepareAddItemForm();
            
            $("#editForm").show();
            $("#edit_uri").focus();
        }
        else if (mode == "edit")
        {
            prepareEditItemForm(item);
            $("#editForm").show();
        }

        top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG2474"));
        $("title").attr("locale", "LANG595");
        load_event_list();
    }
};

var init_event_validate = function()
{
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "edit_uri": {
                required: true,
                letterDigitUndHyphen: true,
                customCallback: [$P.lang("LANG2527"), function(){
                    var res = true;
                    mWindow.window.parent.pbx.users.list().each(function(val){
                        if ( _$('edit_uri').value == val )
                        {
                            res = false;
                            return res;
                        }
                    });
                    return res;
                }],
                customCallback1: [$P.lang("LANG2137"), function(){
                    var res = true;
                    mWindow.window.event_list.each(function(val){
                        if ( _$('edit_uri').value == val.URI )
                        {
                            res = false;
                            return res;
                        }
                    });
                    return res;
                }]
            },
            "select_allow_local": {
                customCallback: [$P.lang("LANG2168").format("1"), function(){
                    var total = ASTGUI.selectbox.readAllValues('select_allow_local').length
                        + ASTGUI.selectbox.readAllValues('select_allow_remote').length;
                    if ( _$('edit_ext').value )
                    {
                        total += _$('edit_ext').value.split(',').length;
                    }

                    return  total > 0;
                }],
                customCallback1: [$P.lang("LANG2199"), function(){
                    return ASTGUI.selectbox.readAllValues('select_allow_local').join(',').length < 900;
                }]
            },
            "select_allow_remote": {
                customCallback: [$P.lang("LANG2199"), function(){
                    var total = ASTGUI.selectbox.readAllValues('select_allow_local').join(',').length
                        + ASTGUI.selectbox.readAllValues('select_allow_remote').join(',').length;
                    return total < 900;
                }]
            },
            "edit_ext": {
                customCallback: [$P.lang("LANG2167").format("'5000,5001,96000'"), function(){
                    var res = true;
                    var value = _$('edit_ext').value;
                    if (value)
                    {
                        value.split(',').each(function(ext){
                            if (!ext || /[^\d]/.test(ext) || ext.length > 64 )
                            {
                                res = false;
                                return res;
                            }
                        });
                        
                    }
                    return res;
                }],
                customCallback1: [$P.lang("LANG2199"), function(){
                    var total = ASTGUI.selectbox.readAllValues('select_allow_local').join(',').length
                        + ASTGUI.selectbox.readAllValues('select_allow_remote').join(',').length
                        +  _$('edit_ext').value.length;
                    return total < 900;
                }]
            }
        },
        submitHandler: function() {
            save_eventlist();
        }
    });
};

var loadEvent = function (a)
{
    var buf = "action=getEventlist&eventlist-info=" + a.URI;
    var res = $.ajax({type: "post",url: "/webcgi",data: buf,async:false});
    var data = eval("("+res.responseText+")").body["eventlist-info"];
    var event = {};
    var sub_num;
    
    event.locallist = [];
    event.remotelist = [];
    event.remote_num = 0;
    event.local_num = 0;
    event.uri = a.URI;
    
    if (data.contains("Eventlist "+ a.URI +" not found"))
    {
        event.subscriber = "";
        if (a.local_extens != "" )
        {
            event.local_num = a.local_extens.split(',').length;
        }

        if (a.remote_extens != "" )
        {
            event.remote_num = a.remote_extens.split(',').length;
        }
    }
    else
    {
        data = data.split('\n');
        
        var buf = data[0].split('!');
        var total_ext = buf[5];
    
        for(var i = 1; i <= total_ext; i++)
        {
            var tmp = data[i].split('!');
            if (tmp[0])
            {
                var user = {};
                user.id = tmp[0];
                user.ext = tmp[1];
                user.state = tmp[2];
                user.type = tmp[3];
                user.peer = tmp[4];
                if (user.type == "local")
                {
                    ( parent.pbx.users.list().indexOf(user) != -1 )
                    {
                        event.locallist.push(user);
                        user.peer = "--";
                        event.local_num += 1;
                    }
                }
                else
                {
                    event.remotelist.push(user); 
                    event.remote_num += 1;
                }
            }
        }
        
        sub_num = buf[3];

        event.subscriber = buf[4];
    }

    event.sub_num = sub_num ? Number(sub_num) : 0;
    window.event_user_list[a.ID] = event;
}

var load_event_list = function()
{
    $.ajax({
        type:"post",
        url:"/webcgi",
        data:"action=getEventlist&eventlist-uri=",
        success:function (data, textStatus, jqXHR) {
                
            var event = data.body.eventlist ? data.body.eventlist : [];
            window.event_list = event;           
            window.event_list.each(function(a){
                window.event_user_list[a.ID] = {};
                loadEvent(a);
            });
            eacheventtd(window.event_user_list);
            load_sub_URI();
        }
    });
}

function eacheventtd(data) {
    for (var x in data) {
        var id = x;
        if (id)
        {
            if ( !document.getElementById("event"+id) ) 
            {
                var eventhtml = "<tr id=event" + id + "  class='event'>"
                        + "<td ><span id=event" + id + "_select name = 'selecttag' class='options room_noselect'></span></td>"
                        + "<td id=event" + id + "_uri>" + data[x].uri + "</td>"
                        + "<td id=event" + id + "_local_extens>" + data[x].local_num + "</td>"
                        //+ "<td ></td>"
                        + "<td id=event" + id + "_remote_extens>" + data[x].remote_num + "</td>"
                        + "<td id=event" + id + "_sub_num>" + data[x].sub_num + "</td>"
                        + "<td class='option' align='left' > ";
                       /* + "<button  class='btn btn-update create_user' name=" + id + ">Invite</button>" ;*/
    
                eventhtml += "<button localeTitle='LANG738' class='options medit' id='edit_" + id + "' ></button>";
                eventhtml += "<button localeTitle='LANG739' class='options mdel' id='del_" + id + "' ></button>";
                eventhtml += "</td></tr>";
                $("#eventcontain").append(eventhtml);
             
                userhtml = "<tr class='tr_table_user' id='User"+id+"' >"
                 + " <td colspan='8' width='100%'>"
                 + "<table id='table_userlist' style='width: 100%' cellpadding='0' cellspacing='0' border='0' align='center'> "
                 + "<tbody id = 'usercontain"+id+"'>"
                 + " <tr class='frow threaduser' >"
                 + "<td ></td> "
                 + "<td width='15%' locale='LANG85'  ></td>"
                 + "<td width='20%' locale='LANG0'   ></td> "
                 + "<td width='20%' locale='LANG84'  ></td>"
                 + "<td width='20%' locale='LANG1351'></td>"
                 + "<td width='20%'></td></tr> "
                 + "</tbody> "
                 + "<tr id=userend"+id+" style='display: none'></tr> "
                 + " </table> "
                 + "</td>"
                 + "</tr>";
                
                $("#event" + id).after(userhtml);
                $P.lang(document, true);
                $("#User"+id).hide();
            }
            else
            {
                $("#event"+id+"_local_extens").text(data[x].local_num);
                $("#event"+id+"_remote_extens").text(data[x].remote_num);
                $("#event"+id+"_sub_num").text(data[x].sub_num);
            }
            
            clean_user(id);
            load_user_list(id, data[x].locallist);
            load_user_list(id, data[x].remotelist);
        }
    }
}

var clean_user = function(id)
{
        
    var list = $("#User" + id + " tbody").eq(1).children();
    for(var c = 0; c < list.length -1;c ++ )
    {
        $("#" + list[c].id).remove();
    }
    
}

function load_user_list(id, list)
{
    for (var x = 0; x < list.length; x++ )
    {
        var userhtml;
        var user = list[x];
        if (user.ext)
        {
            if ( _$(id+"_user_"+ user.ext) == null )
            {
                userhtml = "<tr id="+id+"_user_"+ user.ext + " class='user'>";
                userhtml += "<td></td><td id=" + id + "_ext_" + user.ext + ">" + user.ext + "</td>" 
                    + "<td id=" + id + "_state_" + user.state + ">" + user.state + "</td>"
                    + "<td id=" + id + "_type_" + user.type + ">" + user.type + "</td>"
                    + "<td id=" + id + "_peer_" + user.peer + ">" + user.peer + "</td>"
                    + "<td></td></tr>";

                $("#userend"+id).before(userhtml);
            }/*
            else
            {
                $("#"+id+"_ext_"+user.ext).text(user.ext);
                $("#"+id+"_state_"+user.state).text(user.state);
                $("#"+id+"_state_"+user.type).text(user.type);
                $("#"+id+"_peer_"+user.peer).text(user.peer);
            }*/
        }
    }
}
 
function reflesh_status() {
    if ( $("#en_auto_reflesh").attr("checked") == "checked" ) {
       load_event_list();
    }
    setTimeout(reflesh_status, 3000);
}
        
var do_action = function(buf, lang)
{
    var deplay = function()
    {
        $.ajax({
            type: "post",
            url: "/webcgi",
            data: "action=deployEventList&eventlist=",
            error: function (jqXHR, textStatus, errorThrown) { top.dialog.dialogMessage({ type: 'error', content: errorThrown }); },
            success: function (data)
            {
                top.dialog.dialogMessage({
                    type: 'success',
                    content: $P.lang(lang),
                    callback: function ()
                    {
                        top.$.cookie('configFilesChanged', 'yes');
                        top.$('#applyChanges_Button').css("visibility", "visible");
                        top.$('#line_Button').css("visibility", "visible");
                        top.$('#applyChanges_Button').effect("shake", {
                            direction: "up", distance: 2, times:10000 
                        }, 400);
                        mWindow.location.reload();
                    }
                });
            }
        });
    };
    
    $.ajax({
        type: "post",
        url: "/webcgi",
        data: buf,
        error: function (jqXHR, textStatus, errorThrown) { top.dialog.dialogMessage({ type: 'error', content: errorThrown }); },
        success: function (data)
        {
            deplay();
        }
    });
}

var delete_eventlist_confirm = function (id)
{
    var event = window.event_user_list[id];
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(event.uri),
        buttons: {
            ok: function ()
            {
                var buf = "action=deleteEventlist&eventlist-uri={\"PRIMARY\": [{\"key\":\"uri\", \"value\":\"" + event.uri + "\"}]}";
                var lang = "LANG816";
                do_action(buf, lang);
            }
        }
    });
};

var prepareAddItemForm = function ()
{
    isNewBridge = true;
    EDIT_BRIDGE = '';
    reset_Eventlist_Form();
};

var prepareEditItemForm = function (list)
{
    isNewBridge = false;
    EDIT_BRIDGE = list;
    reset_Eventlist_Form();
};

var get_ext_list = function ()
{
    var ext_list = {};
    var list = [];
    var tmp_cnf = [];
    var tmp;
    var pre = "";
    var trunk = "";
    var conf = top.sessionData.FileCache["phonebookdn.conf"];

    conf ? conf.modified = true : "";
    var cnf = config2json({ filename: 'phonebookdn.conf', usf: 0 });
    list = cnf.getOwnProperties();
    list.each(function(n){
        if (n != "general")
        {
            trunk = cnf[n][0];
            trunk = trunk.split('=')[2];
            pre = (n == "1" ? "" : (trunk + " -- "));
            trunk = n + "_" + trunk;
            ext_list[trunk] =[];
            name = "phonebook" + n + ".conf";
            conf = top.sessionData.FileCache[name];
            conf ? conf.modified = true : "";
            tmp_cnf = config2json({ filename: name, usf: 0 });
            tmp_cnf.getOwnProperties().each(function(ext){
                var tmp_ext = {};
                tmp_ext.id = ext;
                tmp = tmp_cnf[ext][1];
                if (tmp && tmp.split('=')[1])
                {
                    tmp = " \""+tmp.split('=')[1] + "\"";
                }
                else
                {
                    tmp = "";
                }

                tmp_ext.name = pre + tmp_ext.id + tmp;
                ext_list[trunk].push(tmp_ext);
            });
        }
    });
    
    return ext_list;
};

var reset_Eventlist_Form = function ()
{
    var locallist = [];
    var remotelist = [];
    var localallow = [];
    var remoteallow = [];
    var extobj = {};
    
    extobj = get_ext_list();
    for( var name in extobj)
    {
        if (name == "1_pbx")
        {
            locallist = extobj[name];
        }
        else
        {
            remotelist = remotelist.concat(extobj[name]);
        }
    }
            
    if (isNewBridge == false)
    {
        _$('edit_uri').value = mWindow.window.event_user_list[EDIT_BRIDGE].uri;
        _$('edit_uri').disabled = true;
        
        mWindow.window.event_list.each(function(event){
            if( event.ID == EDIT_BRIDGE )
            {
                if (event.remote_extens)
                {
                    remoteallow = event.remote_extens.split(",");
                }
                
                if (event.local_extens)
                {
                    localallow = event.local_extens.split(",");
                }
            }
        });
    }

    bind_select_action('local', locallist);
    bind_select_action('remote', remotelist);
    reset_form_ext('local', localallow, locallist);
    reset_form_ext('remote', remoteallow, remotelist);
};

var save_eventlist = function ()
{
    var uri = _$('edit_uri').value;
    var local_ext = ASTGUI.selectbox.readAllValues('select_allow_local').join(',') || '';
    var remote_ext = ASTGUI.selectbox.readAllValues('select_allow_remote').join(',') || '';
    if (_$('edit_ext').value)
    {
        if (remote_ext != "")
        {
            remote_ext += ",";
        }
        remote_ext += _$('edit_ext').value;
    }
    var buf = "action=updateEventlist&eventlist-uri={\"PRIMARY\": [{\"key\":\"uri\", \"value\":\"" 
        + uri + "\"}],\"CONTENT\": {\"URI\":\"" + uri + "\",\"event\": \"dialog\",\"local_extens\": \""
        + local_ext + "\",\"remote_extens\":\"" + remote_ext + "\"}}";
    var lang = "LANG815";
    do_action(buf, lang);   

};

var bind_select_action = function (id, listOfext)
{
    var DOM_allow = _$('select_allow_' + id);
    var allow = 'select_allow_' + id;
    var DOM_available = _$('select_available_' + id);
    var add = 'button_add_' + id;
    var rm = 'button_remove_' + id;
    var rmall = 'button_removeall_' + id;
    var addall = 'button_addall_' + id;
    
    ASTGUI.events.add(add, 'click', function ()
    {
        if (!DOM_available.value)
        {
            return;
        }

        $(':selected', DOM_available).each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_allow, text, val);
        });
        $P("#" + allow, document).valid();
    });

    ASTGUI.events.add(rm, 'click', function ()
    {
        if (!DOM_allow.value)
        {
            return;
        }

        $(':selected', DOM_allow).each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_available, text, val);
        });
        $P("#" + allow, document).valid();
    });

    ASTGUI.events.add(rmall, 'click', function ()
    {
        ASTGUI.selectbox.clear(DOM_allow);
        ASTGUI.selectbox.clear(DOM_available);

        for (var item in listOfext)
        {
            if (!listOfext.hasOwnProperty(item))
            {
                continue;
            }
            ASTGUI.selectbox.append(DOM_available, listOfext[item].name, listOfext[item].id);
        }
        $P("#" + allow, document).valid();
    });

    ASTGUI.events.add(addall, 'click', function ()
    {
        // modified to do not override former order in selected box, just append it
        $(DOM_available).children().each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_allow, text, val);
        });
        $P("#" + allow, document).valid();
    });
};

var reset_form_ext = function (id, allow_ext, listOfext) {

    var allow_id = 'select_allow_' + id;
    var avai_id = 'select_available_' + id;
    allow_ext = allow_ext ? allow_ext : [];
    var buf = "";

    listOfext.each(function (item)
    {
         if (allow_ext.contains(item.id))
         {
             if (!ASTGUI.selectbox.readAllValues(allow_id).contains(item.id))
             {
                 ASTGUI.selectbox.append(allow_id, item.name, item.id);
             }
         }
         else
         {
             ASTGUI.selectbox.append(avai_id, item.name, item.id);
         }
    });
    
    allow_ext.each(function (item)
    {
        if (!ASTGUI.selectbox.readAllValues(allow_id).contains(item))
        {
            if (!buf)
            {
                buf = item;
            }
            else
            {
                buf = buf + "," + item;
            }
        }
    });
    
    _$("edit_ext").value = buf;
};

var load_sub_URI = function()
{
    if (_$('edit_sub')) {
        _$('edit_sub').options.length = 0;
        window.event_list.each(function(event){
            if (window.event_user_list[event.ID].sub_num)
            {
                ASTGUI.selectbox.append('edit_sub', event.URI, event.ID);
            }
        });
        
        if (_$('edit_sub').value)
        {
            $('#sub_div').show();
            check_subscribers();
        }
        else
        {
            $('#sub_div').hide();
        }
        top.Custom.init(document);
    }
};

var check_subscribers = function()
{
    var TBL = _$('table_userslist');
    var addCell = ASTGUI.domActions.tr_addCell;
    ASTGUI.domActions.clear_table(TBL);
    id = _$('edit_sub').value;
            
    if( id )
    {
        var sub = window.event_user_list[id].subscriber;
        if (sub)
        {
            sub = sub.split(',');

            // add first row
            var newRow = TBL.insertRow(-1);
            newRow.className = "frow";
            addCell(newRow, { html: $P.lang("LANG2477"), width:1000,locale: "LANG2477"});
            sub.each(function(user){
                if (user)
                {
                    var newRow = TBL.insertRow(-1);
                    newRow.className = ((TBL.rows.length) % 2 == 1) ? 'odd' : 'even';
                    addCell(newRow, { html: user || ""});
                }
            });
        }
    }
};

