/*
 * Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
$P = top.$;
var mWindow = window;

ASTGUI.contexts.FaxPrefix = 'ext-fax-';
ASTGUI.contexts.ExtFax = 'ext-fax';

var edit_exten; // page group being edited
var listOfDialDevices = [];

window.onload = function ()
{
    $P.lang(document, true);
    var mode = gup("mode");
    var item = gup("item");
    loadDOMelements();
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 

        if (mode === "add")
        {
            prepareAddItemForm();
        }
        else if (mode === "edit")
        {
            prepareEditItemForm(item);
        }
        init_validator();
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG614"));
        $("title").attr("locale", "LANG614");
        update_table_fax_group();
    }
};

var init_validator = function ()
{
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "text_fax_exten": {
                required: true,
                digits: true,
                extensionExists: [edit_exten]
            },
            "text_fax_name": {
                required: true,
                letterDigitUndHyphen: true
            },
            "text_fax_email": {
                required: true,
                serveremail: true
            }
        },
        submitHandler: function() {
            save_fax_exten_work();
        }
    });
};

var loadDOMelements = function ()
{
    DOM_fax_exten_editdiv = _$('fax_exten_editdiv');
    DOM_table_fax_list = _$('table_fax_list');
    //fax extension
    DOM_text_fax_exten = _$('text_fax_exten');
    DOM_text_fax_name = _$('text_fax_name');
    DOM_text_fax_email = _$('text_fax_email');
};
/* 
[ext-fax-8001]
exten => 8001,1,Noop(---custom public receive fax----)
exten => 8001,n,set(FAXMAIL=ybiaozhou@grandstream.com)
exten => 8001,n,Goto(my-fax,s,1)

*/
var pbx_new_fax_exten = function (exten, name, email, cb)
{
    var context = ASTGUI.contexts.FaxPrefix + exten;

    var action = new listOfActions('extensions.conf');
    action.new_action('append', ASTGUI.contexts.ExtFax, 'include', context);
    action.new_action('newcat', context, '', '');
    //action.new_action('append', context, 'exten', exten + ',1,Log(NOTICE, "fax extension ' + exten + ' - ' + name + '")');
    action.new_action('append', context, 'exten', exten + ',1,set(FAXNAME=' + name + ')');
    action.new_action('append', context, 'exten', exten + ',n,set(FAXMAIL=' + email + ')');
    action.new_action('append', context, 'exten', exten + ',n,set(RECEIVEEXTEN=${EXTEN})');
    action.new_action('append', context, 'exten', exten + ',n,Goto(' + ASTGUI.contexts.Fax2Email + ',s,1)');
    var update_pbxinfo = function ()
    {
        parent.sessionData.pbxinfo.fax.exten[exten] = new ASTGUI.customObject;
        parent.sessionData.pbxinfo.fax.exten[exten]['name'] = name;
        parent.sessionData.pbxinfo.fax.exten[exten]['email'] = email;
        cb();
    };
    action.callActions(update_pbxinfo);
    return;
}

var pbx_del_fax_exten = function (exten, cb)
{
    var context = ASTGUI.contexts.FaxPrefix + exten;
    var sync_action = new listOfSynActions('extensions.conf');

    sync_action.new_action('delcat', context, '', '');
    sync_action.new_action('delete', ASTGUI.contexts.ExtFax, 'include', '', context);
    var resp = sync_action.callActions();

    if (!resp.contains('Response: Success'))
    {
        top.log.error('pbx_del_fax_exten: error modifying extensions.conf');
        return false;
    }
    delete parent.sessionData.pbxinfo.fax.exten[exten];
    if (cb && typeof cb == 'function')
    {
        cb();
    }
    return true;
}

var save_fax_exten_work = function ()
{
    var fax_exten = ASTGUI.getFieldValue(DOM_text_fax_exten);
    var name = ASTGUI.getFieldValue(DOM_text_fax_name);
    var email = ASTGUI.getFieldValue(DOM_text_fax_email);

    var tmp_after = function ()
    {
        var later = function ()
        {
            top.dialog.clearDialog();
            top.dialog.dialogMessage({
                type: 'success',
                content: $P.lang("LANG815"),
                callback: function ()
                {
                    mWindow.location.reload();
                }
            });
        };
        //parent.pbx.paging.add( tmp_new_line, later );
        pbx_new_fax_exten(fax_exten, name, email, later);
    };

    if (edit_exten)
    { // if editing existing page group
        top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826') });
        pbx_del_fax_exten(edit_exten, tmp_after);
    }
    else
    {
        top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826') });
        tmp_after();
    }
};

var resetFields = function ()
{
    if (!edit_exten) //new extension
    {
        ASTGUI.resetTheseFields([DOM_text_fax_exten, DOM_text_fax_name, DOM_text_fax_email]);
        return;
    }
    else
    {
        var c = parent.sessionData.pbxinfo.fax.exten[edit_exten];
        ASTGUI.updateFieldToValue(DOM_text_fax_exten, edit_exten);
        DOM_text_fax_exten.disabled = true;
        ASTGUI.updateFieldToValue(DOM_text_fax_name, c['name']);
        ASTGUI.updateFieldToValue(DOM_text_fax_email, c['email']);
    }

};
var delete_fax_exten_confirm = function (item)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(item),
        buttons: {
            ok: function ()
            {
                top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826') });

                pbx_del_fax_exten(item, function ()
                {
                    top.dialog.clearDialog();
                    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG816"), callback: function () { mWindow.location.reload(); } });
                });
            }
        }
    });
};
var prepareAddItemForm = function ()
{
    edit_exten = '';
    resetFields();
    $("#editForm").show();
};
var prepareEditItemForm = function (item)
{
    edit_exten = item;
    resetFields();
    $("#editForm").show();
};
var update_table_fax_group = function ()
{
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function
    (function (){ // add first row
        if (DOM_table_fax_list) {
            var newRow = DOM_table_fax_list.insertRow(-1);
            newRow.className = "frow";
            addCell(newRow, { html: '', width: '10px' });
            addCell(newRow, { html: $P.lang('LANG85'), locale: 'LANG85' });
            addCell(newRow, { html: $P.lang('LANG135'), locale: 'LANG135' });
            addCell(newRow, { html: $P.lang('LANG126'), locale: 'LANG126' });
            addCell(newRow, { html: $P.lang('LANG74'), locale: 'LANG74' });
        }
    })();
    var c = parent.sessionData.pbxinfo.fax.exten;
    for (var fax_exten in c)
    {
        if (!c.hasOwnProperty(fax_exten))
        {
            continue;
        }

        var item = c[fax_exten];
        if (DOM_table_fax_list) {
            var newRow = DOM_table_fax_list.insertRow(-1);
            var rn = DOM_table_fax_list.rows.length;
            newRow.className = ((rn) % 2 == 1) ? 'odd' : 'even';
            addCell(newRow, { html: '' });
            addCell(newRow, { html: fax_exten });
            addCell(newRow, { html: item.name || '' });
            addCell(newRow, { html: item.email || '' });
            var tmp = "<button class='options edit' onclick=\"showEditForm('" + fax_exten + "')\"  localeTitle='LANG738' title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                      "<button class='options del' onclick=\"delete_fax_exten_confirm('" + fax_exten + "')\" localeTitle='LANG739' title=" + $P.lang("LANG739") + "></button>";
            addCell(newRow, { html: tmp });
        }
    }
    if (DOM_table_fax_list) {
        if (DOM_table_fax_list.rows.length == 1)
        {
            ASTGUI.domActions.clear_table(DOM_table_fax_list);
            var newRow = DOM_table_fax_list.insertRow(-1);
            newRow.className = 'even';
            addCell(newRow, { html: $P.lang('LANG129').format($P.lang('LANG1268')), locale: 'LANG129 LANG1268'});
            return;
        }
    }
};
