/*
 * Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
$P = top.$;

window.onload = function ()
{
    $P.lang(document, true);
    //top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG753"));
    loadDOMelements();
    initForm();
    top.Custom.init(document);
    return false;
};


var loadDOMelements = function ()
{
    //fax setting
    DOM_edit_ecm = _$('edit_ecm_mode');
    DOM_edit_maxrate = _$('edit_maxrate');
    DOM_edit_minrate = _$('edit_minrate');
    DOM_text_default_email = _$('text_default_email');
};

var initForm = function ()
{
    ASTGUI.resetTheseFields([DOM_edit_ecm, DOM_edit_maxrate, DOM_edit_minrate, DOM_text_default_email]);
    var context = context2json({ filename: 'res_fax.conf', context: 'general', usf: 1 });

    if (context) 
    {
        DOM_edit_ecm.checked = (context['ecm'] && context['ecm'] == 'yes') ? true : false;
        context['maxrate'] ? ASTGUI.updateFieldToValue(DOM_edit_maxrate, context['maxrate']) : null;
        context['minrate'] ? ASTGUI.updateFieldToValue(DOM_edit_minrate, context['minrate']) : null;
        context['emailsubject'] ? ASTGUI.updateFieldToValue('emailsubject', context['emailsubject']) : null;
        context['emailbody'] ? ASTGUI.updateFieldToValue('emailbody', context['emailbody'].replace('\\n', '\n')) : null;
    }

    var default_email = ASTGUI.ASTDB.getKey({ dbname: 'FAX/DEFAULT', key: 'EMAIL' });
    ASTGUI.updateFieldToValue(DOM_text_default_email, default_email);

    parent.sessionData.pbxinfo.fax.setting['ecm'] = ASTGUI.getFieldValue(DOM_edit_ecm);
    parent.sessionData.pbxinfo.fax.setting['maxrate'] = ASTGUI.getFieldValue(DOM_edit_maxrate);
    parent.sessionData.pbxinfo.fax.setting['minrate'] = ASTGUI.getFieldValue(DOM_edit_minrate);
    parent.sessionData.pbxinfo.fax.setting['email'] = default_email;

    init_validator();
}

var init_validator = function ()
{
    var minRateLabel = $P.lang("LANG1266");
    var maxRateLabel = $P.lang("LANG1264");
    var bigArr = [maxRateLabel, minRateLabel, $("#edit_minrate"), {equal: true}];
    var smallArr = [minRateLabel, maxRateLabel, $("#edit_maxrate")];

	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "text_default_email": {
                serveremail: true
            },
            "edit_minrate": {
                smaller: smallArr
            },
            "edit_maxrate": {
                bigger: bigArr
            },
            "serveremail": {
                required: true
            },
            "serverbody": {
                required: true
            }
        },
        submitHandler: function() {
            saveForm_work();
        }
    });
};

var saveForm_work = function ()
{

    var maxrate = ASTGUI.getFieldValue(DOM_edit_maxrate);
    var minrate = ASTGUI.getFieldValue(DOM_edit_minrate);
    var email = ASTGUI.getFieldValue(DOM_text_default_email);
    var emailbody = $('#emailbody').val() || '';
    emailbody = emailbody.replace('\n', '\\n');

    var action = new listOfSynActions('res_fax.conf');
    action.new_action('autoupdate', 'general', 'ecm', DOM_edit_ecm.checked ? 'yes' : 'no');
    action.new_action('autoupdate', 'general', 'maxrate', maxrate);
    action.new_action('autoupdate', 'general', 'minrate', minrate);
    action.new_action('autoupdate', 'general', 'emailsubject', $('#emailsubject').val() || '');
    action.new_action('autoupdate', 'general', 'emailbody', emailbody);
    action.callActions();

    ASTGUI.ASTDB.updateKey({ dbname: 'FAX/DEFAULT', key: 'EMAIL', keyvalue: ASTGUI.getFieldValue(DOM_text_default_email) });


    parent.sessionData.pbxinfo.fax.setting['ecm'] = ASTGUI.getFieldValue(DOM_edit_ecm);
    parent.sessionData.pbxinfo.fax.setting['maxrate'] = maxrate;
    parent.sessionData.pbxinfo.fax.setting['minrate'] = minrate;
    parent.sessionData.pbxinfo.fax.setting['email'] = email;

    top.dialog.clearDialog();
    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG815") });

};

var load_defaults = function ()
{
    ASTGUI.updateFieldToValue('emailbody', 'Hello ${RECEIVEEXTEN}, you have received a new fax ${FILENAME} (${FAXPAGES} pages) at ${FAXDATE} from ${CALLERIDNUM} ${CALLERIDNAME}.');
    ASTGUI.updateFieldToValue('emailsubject', 'New Fax from ${CALLERIDNUM} ${CALLERIDNAME}');
};

