/* Asterisk-GUI	- an Asterisk configuration interface
 *
 * Call Features javascript
 *
 * Copyright (C) 2007-2008, Digium, Inc.
 *
 * Ryan Brindley <rbrindley@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * add CDR userfield to specific call type                              
 * modify select,checkbox and radiobox elements style parkext_timeout   
 * add blacklist function
 * 
 */
$P = top.$;
var exten_conf;
var feat_conf;
var gui_conf;
var vals = {};
var feature_code_list = {
        'fcode_pickup' : ASTGUI.contexts.AppPickup,
        'fcode_dialvm' : ASTGUI.contexts.AppDialvm,
        'fcode_vmmain' : ASTGUI.contexts.AppVmmain,
        'fcode_agentpause'   : ASTGUI.contexts.AppAgentPause,
        'fcode_agentunpause' : ASTGUI.contexts.AppAgentUnpause,
        'fcode_paging_prefix' : ASTGUI.contexts.AppPaging,
        'fcode_intercom_prefix' : ASTGUI.contexts.AppIntercom,
        'fcode_cfb_on' : ASTGUI.contexts.AppCallForwardBusyOn,
        'fcode_cfb_off' : ASTGUI.contexts.AppCallForwardBusyOff,
        'fcode_cfn_on' : ASTGUI.contexts.AppCallForwardNoAnswerOn,
        'fcode_cfn_off' : ASTGUI.contexts.AppCallForwardNoAnswerOff,
        'fcode_cfu_on' : ASTGUI.contexts.AppCallForwardUnconditionOn,
        'fcode_cfu_off' : ASTGUI.contexts.AppCallForwardUnconditionOff,
        'fcode_dnd_on' : ASTGUI.contexts.AppDoNotDisturbOn,
        'fcode_dnd_off' : ASTGUI.contexts.AppDoNotDisturbOff,
        'fcode_blacklist_add': ASTGUI.contexts.AppBlacklistAddWrap,
        'fcode_blacklist_remove': ASTGUI.contexts.AppBlacklistRemoveWrap,
        'fcode_direct_vm': "VM_PRE"
};
var otherExtensions;
var edit_funcs = {};
var remove_funcs = {};
var new_exten = {};

edit_funcs['fgeneral_featuredigittimeout'] = function(val, actions) {
sendUpdate(actions, {cxt: 'general', name: 'featuredigittimeout', val: val});
};

edit_funcs['fgeneral_parkext'] = function(val, actions) {

if (val)
{
    parent.sessionData.pbxinfo['localextensions']['parkext'] = val;
}
else
{
    delete parent.sessionData.pbxinfo['localextensions']['parkext'];
}

sendUpdate(actions, {cxt: 'general', name: 'parkext', val: val});
};

edit_funcs['fgeneral_parkpos'] = function(val, actions) {

sendUpdate(actions, {cxt: 'general', name: 'parkpos', val: val});
}

edit_funcs['fgeneral_parkingtime'] = function(val, actions) {

sendUpdate(actions, {cxt: 'general', name: 'parkingtime', val: val});
};

edit_funcs['fmap_blindxfer'] = function(val, actions) {

sendUpdate(actions, {cxt: 'featuremap', name: 'blindxfer', val: val});
};

edit_funcs['fmap_disconnect'] = function(val, actions) {

sendUpdate(actions, {cxt: 'featuremap', name: 'disconnect', val: val});
};

edit_funcs['fmap_atxfer'] = function(val, actions) {

sendUpdate(actions, {cxt: 'featuremap', name: 'atxfer', val: val});
};

edit_funcs['fmap_parkcall'] = function(val, actions) {

sendUpdate(actions, {cxt: 'featuremap', name: 'parkcall', val: val});
};

edit_funcs['fmap_automon'] = function(val, actions) {

sendUpdate(actions, {cxt: 'featuremap', name: 'automon', val: val});
};

edit_funcs['fmap_automixmon'] = function(val, actions) {

sendUpdate(actions, {cxt: 'featuremap', name: 'automixmon', val: val});
};

edit_funcs['fgeneral_pickupexten'] = function(val, actions) {

sendUpdate(actions, {cxt: 'general', name: 'pickupexten', val: val});
};

new_exten['fcode_pickup'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppPickup;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', '_' + val + '.,1,Log(NOTICE, "Attempt to Pickup ${EXTEN:2} by ${CALLERID(num)}")');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(${EXTEN:' + len + '})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(${EXTEN:' + len + '}@ext-local)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(${EXTEN:' + len + '}@from-internal)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(${EXTEN:' + len + '}@from-internal-xfer)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(${EXTEN:' + len + '}@from-did-direct)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(FMPR-${EXTEN:' + len + '})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(LC-${EXTEN:' + len + '}@from-internal)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(LC-${EXTEN:' + len + '}@from-internal-xfer)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(FMPR-${EXTEN:' + len + '}@from-internal)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(FMPR-${EXTEN:' + len + '}@from-internal-xfer)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Pickup(FMPR-${EXTEN:' + len + '}@from-did-direct)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Hangup()');


return;
};

edit_funcs['fcode_pickup'] = function(val, actions) {
var context = ASTGUI.contexts.AppPickup;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
    return true;
}

//update
new_exten['fcode_pickup'](val, actions);
parent.sessionData.pbxinfo['localextensions'][context] = val;

return true;
};


new_exten['fcode_dialvm'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppDialvm;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n(start),Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Log(NOTICE, "app-dialvm: Asking for mailbox")');
actions.new_action('append', context, 'exten', val + ',n,set(i=0)');
actions.new_action('append', context, 'exten', val + ',n,Read(MAILBOX,vm-login,,,,5)');
actions.new_action('append', context, 'exten', val + ',n(check),Log(NOTICE, "app-dialvm: Got Mailbox ${MAILBOX}")');
actions.new_action('append', context, 'exten', val + ',n,Set(i=$[${i} + 1])');
actions.new_action('append', context, 'exten', val + ',n,Macro(get-vmcontext,${MAILBOX})');
actions.new_action('append', context, 'exten', val + ',n,MailBoxExists(${MAILBOX}@${VMCONTEXT})');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($["${VMBOXEXISTSSTATUS}" = "SUCCESS" & "${DB(GROUP/${MAILBOX}/HASVM)}" != "no"]?good:bad)');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');
actions.new_action('append', context, 'exten', val + ',n(good),Log(NOTICE, "app-dialvm: Good mailbox ${MAILBOX}@${VMCONTEXT}")');
actions.new_action('append', context, 'exten', val + ',n,VoiceMailMain(${MAILBOX}@${VMCONTEXT})');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');
actions.new_action('append', context, 'exten', val + ',n(bad),Log(WARNING, "app-dialvm: BAD mailbox ${MAILBOX}@${VMCONTEXT}")');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($[${i} >= 3]?thankyou)');
actions.new_action('append', context, 'exten', val + ',n,Log(NOTICE, "app-dialvm: Asking for mailbox again")');
actions.new_action('append', context, 'exten', val + ',n,Read(MAILBOX,vm-incorrect-mailbox,,,,5)');
actions.new_action('append', context, 'exten', val + ',n,Goto(check)');
actions.new_action('append', context, 'exten', val + ',n(thankyou),playback(vm-incorrect&auth-thankyou)');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');
actions.new_action('append', context, 'exten', '_' + val + '.,1,Answer');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Wait(1)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(MAILBOX=${EXTEN:'+ len +'})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Log(NOTICE, "app-dialvm: Asking for mailbox")');
actions.new_action('append', context, 'exten', '_' + val + '.,n,set(i=0)');
actions.new_action('append', context, 'exten', '_' + val + '.,n(check),Log(NOTICE, "app-dialvm: Got Mailbox ${MAILBOX}")');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(i=$[${i} + 1])');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(get-vmcontext,${MAILBOX})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,MailBoxExists(${MAILBOX}@${VMCONTEXT})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,GotoIf($["${VMBOXEXISTSSTATUS}" = "SUCCESS" & "${DB(GROUP/${MAILBOX}/HASVM)}" != "no"]?good:bad)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(hangupcall,)');
actions.new_action('append', context, 'exten', '_' + val + '.,n(good),Log(NOTICE, "app-dialvm: Good mailbox ${MAILBOX}@${VMCONTEXT}")');
actions.new_action('append', context, 'exten', '_' + val + '.,n,VoiceMailMain(${MAILBOX}@${VMCONTEXT})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(hangupcall,)');
actions.new_action('append', context, 'exten', '_' + val + '.,n(bad),Log(WARNING, "app-dialvm: BAD mailbox ${MAILBOX}@${VMCONTEXT}")');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Wait(1)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,GotoIf($[${i} >= 3]?thankyou)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Log(NOTICE, "app-dialvm: Asking for mailbox again")');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Read(MAILBOX,vm-incorrect-mailbox,,,,5)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Goto(check)');
actions.new_action('append', context, 'exten', '_' + val + '.,n(thankyou),playback(vm-incorrect&auth-thankyou)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(hangupcall,)');
return;
};

edit_funcs['fcode_dialvm'] = function(val, actions) {
var context = ASTGUI.contexts.AppDialvm;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
    return true;
}

//update
var len = val.length;
new_exten['fcode_dialvm'](val, actions);

parent.sessionData.pbxinfo['localextensions'][context] = val;
return true;
};


new_exten['fcode_vmmain'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppVmmain;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,Macro(get-vmcontext,${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n(check),MailBoxExists(${AAUSER}@${VMCONTEXT})');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($["${VMBOXEXISTSSTATUS}" = "SUCCESS"]?mbexist)');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');
actions.new_action('append', context, 'exten', val + ',check+101(mbexist),Set(SKIP_PW_CHECK=${DB(AAUSER/${AAUSER}/SKIPVMAUTH)})');
actions.new_action('append', context, 'exten', val + ',n,ExecIf($["${SKIP_PW_CHECK}" = "YES"]?VoiceMailMain(${AAUSER}@${VMCONTEXT},s):VoiceMailMain(${AAUSER}@${VMCONTEXT}))');
//actions.new_action('append', context, 'exten', val + ',check+101(mbexist),VoiceMailMain(${AAUSER}@${VMCONTEXT})');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');

return;
};

edit_funcs['fcode_vmmain'] = function(val, actions) {

var context = ASTGUI.contexts.AppVmmain;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
    return true;
}

//update
var len = val.length;
new_exten['fcode_vmmain'](val, actions);
parent.sessionData.pbxinfo['localextensions'][context] = val;
return true;
};


new_exten['fcode_blacklist_add'] = function(val, actions) {
    val = val || '';
    var len = val.length;
    var context = ASTGUI.contexts.AppBlacklistAddWrap;

    actions.new_action('delcat', context, '', '');
    actions.new_action('newcat', context, '', '');
    actions.new_action('append', context, 'exten', val + ',1,Macro(app-blacklist-add,${EXTEN},1)');

    return;
};

edit_funcs['fcode_blacklist_add'] = function(val, actions) {

    var context = ASTGUI.contexts.AppBlacklistAddWrap;
    if (val === '') //remove
    {
        actions.new_action('delcat', context, '', '');
        delete parent.sessionData.pbxinfo['localextensions'][context];
    }
    else    //update
    {
        new_exten['fcode_blacklist_add'](val, actions);
        parent.sessionData.pbxinfo['localextensions'][context] = val;
    }

    return true;
};

new_exten['fcode_blacklist_remove'] = function(val, actions) {
    val = val || '';
    var len = val.length;
    var context = ASTGUI.contexts.AppBlacklistRemoveWrap;

    actions.new_action('delcat', context, '', '');
    actions.new_action('newcat', context, '', '');
    actions.new_action('append', context, 'exten', val + ',1,Macro(app-blacklist-remove,${EXTEN},1)');

    return;
};

edit_funcs['fcode_blacklist_remove'] = function(val, actions) {

    var context = ASTGUI.contexts.AppBlacklistRemoveWrap;
    if (val === '') //remove
    {
        actions.new_action('delcat', context, '', '');
        delete parent.sessionData.pbxinfo['localextensions'][context];
    }
    else    //update
    {
        new_exten['fcode_blacklist_remove'](val, actions);
        parent.sessionData.pbxinfo['localextensions'][context] = val;
    }

    return true;
};





new_exten['fcode_agentpause'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppAgentPause;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Macro(agent-pause)');

return;
};
edit_funcs['fcode_agentpause'] = function(val, actions) {

if (val === '') //remove
{
    actions.new_action('delcat', ASTGUI.contexts.AppAgentPause, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][ASTGUI.contexts.AppAgentPause];
}
else    //update
{
    new_exten['fcode_agentpause'](val, actions);

    parent.sessionData.pbxinfo['localextensions'][ASTGUI.contexts.AppAgentPause] = val;
}

return true;

};

new_exten['fcode_agentunpause'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppAgentUnpause;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Macro(agent-unpause)');

return;
};

edit_funcs['fcode_agentunpause'] = function(val, actions) {

if (val === '') //remove
{
    actions.new_action('delcat', ASTGUI.contexts.AppAgentUnpause, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][ASTGUI.contexts.AppAgentUnpause];
}
else    //update
{
    new_exten['fcode_agentunpause'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][ASTGUI.contexts.AppAgentUnpause] = val;
}

return true;

};

new_exten['fcode_paging_prefix'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppPaging;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
var pg_exten = '_' + val + '.' + ',1,Macro(pagingintercom,Local/${EXTEN:' + len + '}@from-internal,q)';
actions.new_action('append', context, 'exten', pg_exten);

return;
};

edit_funcs['fcode_paging_prefix'] = function(val, actions) {

if (val === '') //remove
{
    actions.new_action('delcat', ASTGUI.contexts.AppPaging, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][ASTGUI.contexts.AppPaging];
}
else    //update
{
    new_exten['fcode_paging_prefix'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][ASTGUI.contexts.AppPaging] = val;
}

return true;

};

new_exten['fcode_intercom_prefix'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppIntercom;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
var intercom_exten = '_' + val + '.' + ',1,Macro(pagingintercom,Local/${EXTEN:' + len + '}@from-internal,qd)';
actions.new_action('append', context, 'exten', intercom_exten);

return;
};

edit_funcs['fcode_intercom_prefix'] = function(val, actions) {

if (val === '') //remove
{
    actions.new_action('delcat', ASTGUI.contexts.AppIntercom, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][ASTGUI.contexts.AppIntercom];
}
else    //update
{
    new_exten['fcode_intercom_prefix'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][ASTGUI.contexts.AppIntercom] = val;
}

return true;

};

new_exten['fcode_cfb_on'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppCallForwardBusyOn;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,Set(fromext=${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Set(hastry=0)');
actions.new_action('append', context, 'exten', val + ',n(startread),Set(hastry=$[${hastry} + 1])');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($[${hastry} > 3]?end)');
actions.new_action('append', context, 'exten', val + ',n,Read(toext,call-fwd-on-busy&please-enter-your&target-attendant&then-press-pound,,,,)');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($["foo${toext}"="foo"]?startread)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Set(DB(AAUSER/${fromext}/CFB)=${toext})');
actions.new_action('append', context, 'exten', val + ',n,Playback(call-fwd-on-busy&for&extension)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${fromext})');
actions.new_action('append', context, 'exten', val + ',n,Playback(is-set-to)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${toext})');
actions.new_action('append', context, 'exten', val + ',n(end),Macro(hangupcall,)');

actions.new_action('append', context, 'exten', '_' + val + '.,1,Answer');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Wait(1)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(DB(AAUSER/${AAUSER}/CFB)=${EXTEN:' + len + '})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Playback(call-fwd-on-busy&for&extension)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,SayDigits(${AAUSER})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Playback(is-set-to)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,SayDigits(${EXTEN:' + len + '})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(hangupcall,)'); 

return;
};


edit_funcs['fcode_cfb_on'] = function(val, actions) {

var context = ASTGUI.contexts.AppCallForwardBusyOn;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
}
else    //update
{
    new_exten['fcode_cfb_on'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][context] = val;
}

return true;
};


new_exten['fcode_cfb_off'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppCallForwardBusyOff;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,dbDel(AAUSER/${AAUSER}/CFB)');
actions.new_action('append', context, 'exten', val + ',n,Playback(call-fwd-on-busy&for&extension)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n,Playback(cancelled)');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');

return;
};

edit_funcs['fcode_cfb_off'] = function(val, actions) {

var context = ASTGUI.contexts.AppCallForwardBusyOff;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
}
else    //update
{
    new_exten['fcode_cfb_off'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][context] = val;
}

return true;
};

new_exten['fcode_cfn_on'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppCallForwardNoAnswerOn;
actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,Set(fromext=${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Set(hastry=0)');
actions.new_action('append', context, 'exten', val + ',n(startread),Set(hastry=$[${hastry} + 1])');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($[${hastry} > 3]?end)');
actions.new_action('append', context, 'exten', val + ',n,Read(toext,call-fwd-no-ans&please-enter-your&target-attendant&then-press-pound,,,,)');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($["foo${toext}"="foo"]?startread)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Set(DB(AAUSER/${fromext}/CFN)=${toext})');
actions.new_action('append', context, 'exten', val + ',n,Playback(call-fwd-no-ans&for&extension)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${fromext})');
actions.new_action('append', context, 'exten', val + ',n,Playback(is-set-to)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${toext})');
actions.new_action('append', context, 'exten', val + ',n(end),Macro(hangupcall,)');

actions.new_action('append', context, 'exten', '_' + val + '.,1,Answer');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Wait(1)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(DB(AAUSER/${AAUSER}/CFN)=${EXTEN:' + len + '})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Playback(call-fwd-no-ans&for&extension)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,SayDigits(${AAUSER})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Playback(is-set-to)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,SayDigits(${EXTEN:' + len + '})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(hangupcall,)');

return;
};

edit_funcs['fcode_cfn_on'] = function(val, actions) {

var context = ASTGUI.contexts.AppCallForwardNoAnswerOn;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
}
else    //update
{
    new_exten['fcode_cfn_on'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][context] = val;
}

return true;
};


new_exten['fcode_cfn_off'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppCallForwardNoAnswerOff;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,dbDel(AAUSER/${AAUSER}/CFN)');
actions.new_action('append', context, 'exten', val + ',n,Playback(call-fwd-no-ans&for&extension)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n,Playback(cancelled)');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');

return;
};

edit_funcs['fcode_cfn_off'] = function(val, actions) {

var context = ASTGUI.contexts.AppCallForwardNoAnswerOff;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
}
else    //update
{
    new_exten['fcode_cfn_off'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][context] = val;
}

return true;
};


new_exten['fcode_cfu_on'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppCallForwardUnconditionOn;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,Set(fromext=${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Set(hastry=0)');
actions.new_action('append', context, 'exten', val + ',n(startread),Set(hastry=$[${hastry} + 1])');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($[${hastry} > 3]?end)');
actions.new_action('append', context, 'exten', val + ',n,Read(toext,call-fwd-unconditional&please-enter-your&target-attendant&then-press-pound,,,,)');
actions.new_action('append', context, 'exten', val + ',n,GotoIf($["foo${toext}"="foo"]?startread)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Set(DB(AAUSER/${fromext}/CFU)=${toext})');
actions.new_action('append', context, 'exten', val + ',n,Playback(call-fwd-unconditional&for&extension)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${fromext})');
actions.new_action('append', context, 'exten', val + ',n,Playback(is-set-to)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${toext})');
actions.new_action('append', context, 'exten', val + ',n(end),Macro(hangupcall,)');


actions.new_action('append', context, 'exten', '_' + val + '.,1,Answer');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Wait(1)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Set(DB(AAUSER/${AAUSER}/CFU)=${EXTEN:' + len + '})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Playback(call-fwd-unconditional&for&extension)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,SayDigits(${AAUSER})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Playback(is-set-to)');
actions.new_action('append', context, 'exten', '_' + val + '.,n,SayDigits(${EXTEN:' + len + '})');
actions.new_action('append', context, 'exten', '_' + val + '.,n,Macro(hangupcall,)');

return ;
}
edit_funcs['fcode_cfu_on'] = function(val, actions) {

var context = ASTGUI.contexts.AppCallForwardUnconditionOn;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
}
else    //update
{
    new_exten['fcode_cfu_on'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][context] = val;
}

return true;
};


new_exten['fcode_cfu_off'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppCallForwardUnconditionOff;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,dbDel(AAUSER/${AAUSER}/CFU)');
actions.new_action('append', context, 'exten', val + ',n,Playback(call-fwd-unconditional&for&extension)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n,Playback(cancelled)');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');

return ;
}
edit_funcs['fcode_cfu_off'] = function(val, actions) {

var context = ASTGUI.contexts.AppCallForwardUnconditionOff;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
}
else    //update
{
    new_exten['fcode_cfu_off'](val, actions)
    parent.sessionData.pbxinfo['localextensions'][context] = val;
}

return true;
};


new_exten['fcode_dnd_on'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppDoNotDisturbOn;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,Set(DB(AAUSER/${AAUSER}/DND)=YES)');
actions.new_action('append', context, 'exten', val + ',n,Playback(do-not-disturb&activated&for&extension)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');

return ;
}
edit_funcs['fcode_dnd_on'] = function(val, actions) {

var context = ASTGUI.contexts.AppDoNotDisturbOn;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
}
else    //update
{
    new_exten['fcode_dnd_on'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][context] = val;
}

return true;
};


new_exten['fcode_dnd_off'] = function(val, actions) {
val = val || '';
var len = val.length;
var context = ASTGUI.contexts.AppDoNotDisturbOff;

actions.new_action('delcat', context, '', '');
actions.new_action('newcat', context, '', '');
actions.new_action('append', context, 'exten', val + ',1,Answer');
actions.new_action('append', context, 'exten', val + ',n,Set(CDR(userfield)=FC)');
actions.new_action('append', context, 'exten', val + ',n,Wait(1)');
actions.new_action('append', context, 'exten', val + ',n,Macro(user-callerid,)');
actions.new_action('append', context, 'exten', val + ',n,dbDel(AAUSER/${AAUSER}/DND)');
actions.new_action('append', context, 'exten', val + ',n,Playback(do-not-disturb&de-activated&for&extension)');
actions.new_action('append', context, 'exten', val + ',n,SayDigits(${AAUSER})');
actions.new_action('append', context, 'exten', val + ',n,Macro(hangupcall,)');
return ;
}

edit_funcs['fcode_dnd_off'] = function(val, actions) {

var context = ASTGUI.contexts.AppDoNotDisturbOff;

if (val === '') //remove
{
    actions.new_action('delcat', context, '', '');
    delete parent.sessionData.pbxinfo['localextensions'][context];
}
else    //update
{
    new_exten['fcode_dnd_off'](val, actions);
    parent.sessionData.pbxinfo['localextensions'][context] = val;
}

return true;
};

new_exten['fcode_direct_vm'] = function(val, actions) {
    val = val || '*';
    actions.new_action('update', 'globals', 'VM_PRE', val);
    return ;
}

edit_funcs['fcode_direct_vm'] = function(val, actions) {

    new_exten['fcode_direct_vm'](val, actions);
    parent.sessionData.pbxinfo.GLOBALS["VM_PRE"] = val;

    return true;
};


/**
* function to load options
*/
var load = function() {
exten_conf = context2json({
    filename: 'extensions.conf',
    context: 'globals',
    usf: 1
});


feat_conf = config2json({
    filename: 'features.conf',
    usf: 1
});

if (!feat_conf['general'] || !feat_conf['featuremap'] || !feat_conf['applicationmap']) {
    var u = new listOfSynActions('features.conf');
    if( !feat_conf.hasOwnProperty('general') ) {
        u.new_action('newcat', 'general', '', '');
    }
    if( !feat_conf.hasOwnProperty('featuremap') ) {
        u.new_action('newcat', 'feature_map', '', '');
    }
    u.callActions();
    //window.location.reload();
		return;
	}

    var all_exten = parent.miscFunctions.getAllExtensions();
    //remove localextensions and parkext, parkpos
    var local_exten = featureMisc.getLocalExtensions();
    otherExtensions = $(all_exten).not(local_exten).get();

    //load parkpos range
    if (feat_conf.general.parkpos)
    {
        var parkpos = feat_conf.general.parkpos.split('-');
        featureMisc.parkpos_start = parkpos[0];
        featureMisc.parkpos_end = parkpos[1];
    }


};


/**
 * sends updates to Asterisk.
 * @actions actions.
 * @param cxt variable's context.
 * @param name variable name.
 * @param val variable value.
 * modified the function to reuse edit_funcs
 */
var sendUpdate = function(actions, params) {
	actions.new_action('autoupdate', params.cxt, params.name, params.val);
    //feat_conf[params.cxt][params.name] = params.val;
}

var featureMisc = {
    parkpos_start: null,
    parkpos_end : null,
    makeResetBtn : function (id) {
        var reset_btn = document.createElement('button');
        reset_btn = $(reset_btn);
        reset_btn.attr('type', 'button');
        reset_btn.addClass('btn');
        reset_btn.html('Reset');
        //var id = $(":input[type='text']", content).attr('id');
        reset_btn.attr('onclick', 'featureMisc.resetSingle(\'' + id + '\')');

        return reset_btn;
    },
    makeDefaultBtn : function (id) {
        var default_btn = document.createElement('button');
        default_btn = $(default_btn);
        default_btn.attr('type', 'button');
        default_btn.addClass('btn');
        default_btn.html('Default');
        //var id = $(":input[type='text']", content).attr('id');
        default_btn.attr('onclick', 'featureMisc.defaultSingle(\'' + id + '\')');

        return default_btn;
    },
    resetClass : function (className) {
        $('.' + className + ' > input[type="text"]').each( function (idx, item) {
            featureMisc.resetSingle($(item).attr('id')); 
        });

        //if feature map, reset the caller/callee permission
        if (className === 'feature_map')
        {
            this.loadDialOption();
        }
    },
    defaultClass : function (className) {
        $('.' + className + ' > input[type="text"]').each( function (idx, item) {
            featureMisc.defaultSingle($(item).attr('id')); 
        });

        if (className === 'feature_map')
        {
            //pass the default DIAL_OPTION = ''
            this.loadDialOption('');
        }
    },
    resetSingle : function (id) {
        var oldval = this.getSessionValueByID(id) || '';
        oldval = (oldval ? oldval : $('#' + id).attr('dfalt'));
        $('#' + id).val(oldval);
    },
    defaultSingle : function (id) {
        var target = $('#' + id);
        var dfval = target.attr('dfalt') || '';
        target.val(dfval);
    },
    getSessionValueByID : function (id) {
        var oldval;
        if (id.beginsWith('fcode_'))
        {
            var fcode_name = feature_code_list[id];
            
            if (id == "fcode_direct_vm")
            {
                oldval = parent.sessionData.pbxinfo.GLOBALS[fcode_name];
            }
            else
            {
                oldval = parent.sessionData.pbxinfo.localextensions[fcode_name];
            }
        }
        else if (id.beginsWith('fmap_'))
        { 
		    var vari = id.split('_')[1];
			oldval = feat_conf.featuremap[vari];
        }
        else if (id.beginsWith('fgeneral_'))
        {
		    var vari = id.split('_')[1];
			oldval = feat_conf.general[vari];
        }
        return oldval;
    },
    checkFeatureCode: function (value, except_id) {
        //var texts = $('input', '.feature_code, .feature_dnd').remove('#' + except_id).each( function (idx, content) {
        var texts = $('input', '.feature_code, .feature_dnd', '[id!=#' + except_id + ']');
        texts = texts.not($('#' + except_id));
        var len  = texts.length;
        for ( var i = 0; i < len; i++)
        {
            if (texts[i].value === value)
            {
                return true;
            }
        }

        return false;
    },
    getLocalExtensions: function () {
        var local_extens = [];
        //get feature code /dnd extensions
        var code_extens = parent.sessionData.pbxinfo.localextensions;
        for (var key in code_extens)
        {
            if( code_extens.hasOwnProperty(key) )
            {
                local_extens.push(code_extens[key]);
            }
        }

        if (feat_conf.general.parkext)
        {
            local_extens.push(feat_conf.general.parkext);
        }

        //parkpos range
        if (feat_conf.general.parkpos)
        {
            var parkpos = feat_conf.general.parkpos.split('-');
            var parkpos_end = Number(parkpos[1]);
            for ( var i = Number(parkpos[0]); i <= parkpos[1]; i++)
            {
                local_extens.push(i.toString());
            }
        }

        return local_extens;
    },
    saveChanges: function () {
        var exten_actions = new listOfActions('extensions.conf');
        var feature_actions = new listOfActions('features.conf');

        //extensions.conf
        this.saveDialOpt(exten_actions);
        this.saveFeatureCode(exten_actions);
        //features.conf
        this.saveFeatureConf(feature_actions);

        var successHandler = function () {
            top.dialog.dialogMessage({ 
                type: 'success', 
                content: $P.lang("LANG844")
            });
        };

        exten_actions.batchActions({
            "feedback": true ,
            "checkReload": true,
            "onResponseProcessed": successHandler
        });

        feature_actions.batchActions({
            "feedback": false,
            "checkReload": true,
            "onResponseProcessed": successHandler
        });
        return;

    },
    saveDialOpt: function (actions) {
        var default_dial_opt = '';      // former 'r'
        var to_trunk_opt_arr = [];      //uppercase
        var from_trunk_opt_arr = [];    //lowercase

        //pairs= [ {"Neither": 0x00}, {"Caller Enable": 0x01}, {"Callee Enable": 0x02} ,{"Both": 0x03}];

        $('.feature_map > [id^=dial_]').each( function(idx, content) {
            content = $(content);
            var opts = content.attr('id').split('_');
            if (opts.length != 2)
            {
                //dial_t_fake case
                return;
            }
            var opt = opts[1];
            var val = content.val();

            if ( val & 0x01 )
            {
                to_trunk_opt_arr.push(opt.toUpperCase() );
            }

            if ( val & 0x02 )
            {
                from_trunk_opt_arr.push(opt);
            }

        });

        var to_trunk_opt = to_trunk_opt_arr.join('') + default_dial_opt;
        var from_trunk_opt = from_trunk_opt_arr.join('') + default_dial_opt;
        var dial_opt = to_trunk_opt_arr.join('') + from_trunk_opt_arr.join('') + default_dial_opt;

        actions.new_action('autoupdate', 'globals', 'DIAL_OPTIONS', dial_opt);
        actions.new_action('autoupdate', 'globals', 'FROM_TRUNK_OPTIONS', from_trunk_opt);
        actions.new_action('autoupdate', 'globals', 'TRUNK_OPTIONS', to_trunk_opt);
    },
    saveFeatureConf: function (actions) {
        $('input[type="text"]', '.feature_map, .feature_general').each( function (idx, item) {
            var id = $(item).attr('id');
            if (edit_funcs[id])
            {
                edit_funcs[id]($(item).val(), actions);
            }
        });
        
        id = "fgeneral_pickupexten";
        if (edit_funcs[id])
        {
            edit_funcs[id](_$("fgeneral_pickupexten").value, actions);
        }
    },
    saveFeatureCode: function (actions) {
        $('input', '.feature_code, .feature_dnd').each( function (idx, item) {

            var id = $(item).attr('id');
            var val = $(item).val();
            // if (val == featureMisc.getSessionValueByID(id) || id == "fgeneral_pickupexten" )
            // {
            //     //not modified the value
            //     return;
            // }
            if (edit_funcs[id])
            {
                edit_funcs[id](val, actions);
            }
        });
    },
    //@param dial_options   specific dial options (optional)
    loadDialOption : function (dial_options) {
        // set all the enabled dial options 
        
        // clear the dial options
        var opts = $('[id^=dial_]');
        opts.val(0);

        if (typeof dial_options != 'string')
        {
            //if no dial option specified, to get from extension.conf 
            dial_options = exten_conf['DIAL_OPTIONS'] || '';
        }
        dial_opts = dial_options.split('');
        for (var i=0; i<dial_opts.length; i++) {
            //$('#dial_'+dial_opts[i]).attr('checked', 'checked');
            //pairs= [ {"Neither": 0x00}, {"Caller Enable": 0x01}, {"Callee Enable": 0x02} ,{"Both": 0x03}];
            var lowercase = dial_opts[i].toLowerCase();
            var opt = $('#dial_' + lowercase);
            var val = opt.val();
            if (dial_opts[i] === lowercase)
            {
                //callee LowerCase 
                val |= 0x02;
            }
            else
            {
                //caller UpperCase
                val |= 0x01;
            }
            opt.val(val);
        }

        //attend transfer same to blind transfer (share the value)
        $('#dial_t_fake').val( $('#dial_t').val() );

    }
};

window.onload = function () {
    $P.lang(document, true);
    top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG610"));
    load();
    $("title").attr("locale", "LANG610");
    $('#fgeneral_parkpos').focusout( function () {
        if ( $P('#fgeneral_parkpos', document).valid() )
        {
            var parkpos = $('#fgeneral_parkpos').val().split('-');
            featureMisc.parkpos_start = parkpos[0];
            featureMisc.parkpos_end = parkpos[1];
        }
    });


    //display input text value 
    $('.feature_code, .feature_dnd, .feature_map, .feature_general').each( function(idx, content) {
        //display value
        var id = $(":input[type='text']", content).attr('id');
        featureMisc.resetSingle(id);

    });

    //display select value , caller/callee enable
    var pairs= [
        {"LANG1254": 0x00},
        {"LANG1255": 0x01},
        {"LANG1256": 0x02},
        {"LANG1257": 0x03}
    ];
    $('.feature_map > [id^=dial_]').each( function(idx, content) {
        for ( var i=0; i < pairs.length; i++)
        {
            var option = document.createElement('option');
            var opts = $(option);
            for (var key in pairs[i])
            {
                if (pairs[i].hasOwnProperty(key))
                {
                    opts.html($P.lang(key));
                    opts.val(pairs[i][key]);
                    opts.attr('locale', key);
                    break;
                }

            }
            //option.appendChild(selectone);
            content.appendChild(option);
        }
        //$(selectone).appendTo(content);
    });

    featureMisc.loadDialOption();

    $('.refresh_icon').click(function () {
        window.location.reload();
    });

    //bind blind/attend transfer checkbox
    $('.share_dial').bind('change', function () {
        var val = $(this).val();
        $('.share_dial').val(val);
        $('.share_dial').prev().text($(this).find("option[value="+val+"]").text());
        top.Custom.init(document);
    });
    init_validator();
    top.Custom.init(document);
};

/*
 * check extension (other module) existance
 */
var check_extension_exists = function (value, element) {
    if (!value)
    {
        return true;
    }

    //otherExtensions are strings, to make sure we can detect exstince ,add value.toString();
    return !otherExtensions.contains(value.toString());
};

/* check local extensions
 */
var check_edit_value_existance = function (value, element) {
    if (!value)
    {
        return true;
    }

    var id = $(element).attr('id') || '';
    if (featureMisc.checkFeatureCode(value, id))
    {
        return false;
    }

    if ( id !== 'fgeneral_parkext' 
            && $('#fgeneral_parkext').val() === value)
    {
        return false;
    }

    if (featureMisc.parkpos_start)
    {
        if (Number(value) >= featureMisc.parkpos_start 
                && Number(value) <= featureMisc.parkpos_end)
        {
            return false;
        }
    }

    return true;
};
var validate_parkpos_formate = function (value) {
    if(/^[1-9]\d*\-[1-9]\d*$/.test(value)) {
        return true;
    }
    return false;
}
var validate_parkpos= function (value, element) {
    if (!value)
    {
        return true;
    }

	if (!value.match(/^[0-9]+\-[0-9]+$/)) {
        //pattern will take care it
        return true;
	}

	/* lets split */
	var splits = value.split('-');
	var pos_start = parseInt(splits[0], 10);
	var pos_end = parseInt(splits[1], 10);

    if (pos_start > pos_end)
    {
        return false;
    }
    
	/* we need to do a special validate for extens */
	gui_conf = context2json({
		filename: 'guipreferences.conf',
		context: 'general',
		usf: 1
	});

    if (gui_conf.disable_extension_ranges && gui_conf.disable_extension_ranges != 'yes')
    {
        var prefs = {ue: 'User Extensions', mm: 'Conference Extensions', qe: 'Queue Extensions', vme: 'Voicemail Extensions', rge: 'Ring Group Extensions', vmg: 'Voicemail Group Extensions'};
        for (pref in prefs) {
            if (!prefs.hasOwnProperty(pref)) {
                continue;
            }

            var start = gui_conf[pref + '_start'];
            var end = gui_conf[pref + '_end'];

            if ((pos_start >= start && pos_start <= end) || (pos_end >= start && pos_end <= end) || (pos_start < start && pos_end >= start)) {
                return false;
            }
        }
    }


    var ret;
    for (i = pos_start; i <= pos_end; i++)
    {
        ret = check_extension_exists(i, element);
        if (!ret)
        {
            return false;
        }
    }

    if (ret)  
    {
        for (i = pos_start; i <= pos_end; i++)
        {
            //check other editing extensions ( feature_code and parkext)
            var val = i.toString();
            ret = !featureMisc.checkFeatureCode(val, 'fgeneral_parkpos');      //the id param does not work actually
            if (ret)
            {
                ret = $('#fgeneral_parkext').val() !== val;
            }

            if (!ret)
            {
                return false;
            }
        }
    }

    return true;
};

var validate_feature_map= function (value, element) {
    if (!value)
    {
        return true;
    }

    var texts = $('.feature_map > input[type="text"]').not($(element));
    var len = texts.length;
    for (var i = 0; i < len; i++)
    {
        if (texts[i].value === value)
        {
            return false;
        }
    }
    return true;
};

var init_validator = function () {
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {

            "fmap_blindxfer": {
                numeric_pound_star: true,
                customCallback: [$P.lang("LANG2208"), validate_feature_map]
            },
            "fmap_atxfer": {
                numeric_pound_star: true,
                customCallback: [$P.lang("LANG2208"), validate_feature_map]
            },
            "fmap_disconnect": {
                numeric_pound_star: true,
                customCallback: [$P.lang("LANG2208"), validate_feature_map]
            },
            "fmap_parkcall": {
                numeric_pound_star: true,
                customCallback: [$P.lang("LANG2208"), validate_feature_map]
            },
            "fmap_automon": {
                numeric_pound_star: true,
                customCallback: [$P.lang("LANG2208"), validate_feature_map]
            },
            "fmap_automixmon": {
                numeric_pound_star: true,
                customCallback: [$P.lang("LANG2208"), validate_feature_map]
            },
            "fcode_dnd_on": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_dnd_off": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_cfb_on": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_cfb_off": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_cfn_on": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_cfn_off": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_cfu_on": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_cfu_off": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fgeneral_featuredigittimeout": {
                digits: true
            },
            "fgeneral_parkext": {
                required: true, 
                digits: true,
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fgeneral_parkpos": {
                required: true,
                pattern: /[0-9]+\-[0-9]+/,
                customCallback: ['Range invalid or conflict with other extensions', validate_parkpos],
                customCallback1: [$P.lang("LANG2644"), validate_parkpos_formate]
            },
            "fgeneral_parkingtime": {
                required: true,
                digits: true,
                min: 1
            },
            "fcode_pickup": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_dialvm": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_vmmain": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_agentpause": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_agentunpause": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_paging_prefix": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_intercom_prefix": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_blacklist_add": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_blacklist_remove": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fgeneral_pickupexten": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            },
            "fcode_direct_vm": {
                numeric_pound_star: true,
                customCallback2: [$P.lang("LANG2209") , check_edit_value_existance],
                customCallback: [$P.lang("LANG2126"), check_extension_exists]
            }
        }
    });
};


var save_entrance = function() {
    if ($P("#form", document).valid())
    {
        featureMisc.saveChanges();
    }
};




