/*
* Asterisk-GUI	- an Asterisk configuration interface
*
* follow.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * modify the followme function and dialplan                     
 *
 */
$P = top.$;
var mWindow = window;


var EDIT_USER;
var CURRENT_DESTINATIONS = [];
var FOLLOWME_OPTIONS = '';
var RINGTIME;
var USERSLIST = [];
var PAGE = 1;
var NPAGE = 1;
var NUMBER = -1;
var type = "save";

var is_same_user = function()
{
    if (_$('newFM_Number_radio_local').checked)
    {
        var tmp_number = ASTGUI.getFieldValue('FMU_newNumber_local');
    }

    if (_$('newFM_Number_radio_Externals').checked)
    {
        var tmp_number = ASTGUI.getFieldValue('FMU_newNumber_External');
    }

    var tmp_seconds = ASTGUI.getFieldValue('FMU_newNumber_seconds');

    if (tmp_number.contains('-')) tmp_number = tmp_number.withOut('-');
    var tmp_dest = tmp_number + ',' + tmp_seconds;

    var tmp_last = CURRENT_DESTINATIONS.lastValue();

    if (_$('newFM_Order_radio_alongWith').checked && tmp_last)
    {
        var a = tmp_last.split(',')[0];
        if (a)
        {
            var b = a.split('&');
            if (b)
            {
                for (var t = 0; t < b.length; t++)
                {
                    if (b[t] == tmp_dest.split(',')[0])
                    {
                        return false;
                    }
                }
            }
        }
    }
    return true;
}

window.onload = function ()
{
    $P.lang(document, true);
    var mode = gup("mode");
    var item = gup("item");
    var param = gup("param");
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        initForm();
        if (mode === "edit")
        {
        
            if ($("#form").tooltip) {
				$("#form").tooltip();
			}
            $P("#form", document).validate({
                rules:{
                    "time": {
                        required: true,
                        digits: true,
                        range: [10, 60],
                        cmpareRingtime: true
                    },
                    "FMU_newNumber_External": {
                        required: true,
                        numeric_pound_star: true
                    },
                    "FMU_newNumber_local": {
                        customCallback: [ $P.lang("LANG2207"), is_same_user]
                    }
                },
                submitHandler: function() {
                    if (type == "add") {
                        followMe_MiscFunctions.push_newdest();
                        type = "save";
                    }
                    else if (type == "save") {
                        saveFollowMeUser();
                    }
                }
            });
            prepareEditItemForm(item, param);
            $("#editForm").show();
        }
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG568"));
        $("title").attr("locale", "LANG568");
        load_users_table();
    }
};

var initForm = function ()
{

    ASTGUI.events.add('newFM_Number_radio_local', 'click', function ()
    {
        $('#FMU_newNumber_local').hide();
        $('#FMU_newNumber_External').hide();

        if (_$('newFM_Number_radio_local').checked)
        {
            $('#FMU_newNumber_local').show();
        }
    });

    ASTGUI.events.add('newFM_Number_radio_Externals', 'click', function ()
    {
        $('#FMU_newNumber_local').hide();
        $('#FMU_newNumber_External').hide();

        if (_$('newFM_Number_radio_Externals').checked)
        {
            $('#FMU_newNumber_External').show();
        }
    });
    (function ()
    {
        var mcls = config2json({ filename: 'musiconhold.conf', usf: 1 });
        for (var this_class in mcls)
        {
            if (mcls.hasOwnProperty(this_class))
            {
                ASTGUI.selectbox.append('FMU_moh', this_class, this_class);
            }
        }
        _$('FMU_moh').selectedIndex = -1;
    })();

    $('#sqDestinations').click(function (event)
    {
        followMe_MiscFunctions.save_allDestinations();

        var s = ASTGUI.events.getTarget(event);
        var cl = $(s).attr("class");
        if (!cl || !cl.beginsWith('step_')) { return; }
        var stepNo = Number(s.parentNode.STEPNO);
        switch (cl)
        {
            case 'step_delete':
                CURRENT_DESTINATIONS.splice(stepNo, 1);
                if(NUMBER == stepNo)
                {
                    followMe_MiscFunctions.hide_FORM_newFM_Number();
                }
                if (NUMBER != -1 && NUMBER > stepNo )
                {
                    NUMBER --;
                }

                break;
            case 'step_up':
                if (stepNo == 1) return;
                var tmp = CURRENT_DESTINATIONS[stepNo];
                CURRENT_DESTINATIONS.splice(stepNo, 1);
                CURRENT_DESTINATIONS.splice(stepNo - 1, 0, tmp);
                break;
            case 'step_down':
                if (stepNo == (CURRENT_DESTINATIONS.length - 1)) return;
                var tmp = CURRENT_DESTINATIONS[stepNo];
                CURRENT_DESTINATIONS.splice(stepNo + 2, 0, tmp);
                CURRENT_DESTINATIONS.splice(stepNo, 1);
                break;
            case 'step_add':
                NUMBER = stepNo;
                followMe_MiscFunctions.show_FORM_newFM_Number();
                break;
            default:
                break;
        }
        followMe_MiscFunctions.refresh_allDestinations();
    });
}
var show_users_list = function (n)
{
    var TBL = _$('table_userslist');
    var addCell = ASTGUI.domActions.tr_addCell;
    ASTGUI.domActions.clear_table(TBL);
    (function ()
    { // add first row
        var newRow = TBL.insertRow(-1);
        newRow.className = "frow";
        addCell(newRow, { html: $P.lang("LANG85"), locale: "LANG85"});
        addCell(newRow, { html: $P.lang("LANG568"), locale: "LANG568"});
        addCell(newRow, { html: $P.lang("LANG1974"), locale: "LANG1974" });
        addCell(newRow, { html: $P.lang("LANG74"), locale: "LANG74"});
    })();
    var followme_cnf = config2json({ filename: 'followme.conf', usf: 0 });
    for (var i = (n - 1) * 20; i < n * 20 && i < USERSLIST.length; i++)
    {
        var user = USERSLIST[i];
        /* remove by yjwu */
        var newRow = TBL.insertRow(-1);
        newRow.className = ((TBL.rows.length) % 2 == 1) ? 'odd' : 'even';
        addCell(newRow, { html: user });
        /* modify by yjwu */
        var value = ASTGUI.ASTDB.getKey({ dbname: 'AAUSER/' + user, key: 'FOLLOWME' });
        if (value == 'ENABLE')
        {
            /* modify end*/
            addCell(newRow, { html: '<font color=green locale="LANG274">' + $P.lang("LANG274") + '</font>'});
            var tmp_a = "<button class='options edit' onclick=\"showEditForm('" + user + "', 1)\" localetitle=\"LANG738\" title=" + $P.lang("LANG738") + "></button>"
        } else
        {
            addCell(newRow, { html: '<font locale="LANG273">'+ $P.lang("LANG273") + '</font>'});
            var tmp_a = "<button class='options edit' onclick=\"showEditForm('" + user + "', 0)\" localetitle=\"LANG738\" title=" + $P.lang("LANG738") + "></button>"
        }
        if (followme_cnf.hasOwnProperty(user) && followme_cnf[user].length > 3)
        {
            var tmp_followorder = [];
            followme_cnf[user].each(function (line)
            {
                if (line.beginsWith('number='))
                {

                    line = line.lChop('number=');
                    var fl_number = line.split(',')[0];
                    if (fl_number != user)
                    {
                        tmp_followorder.push(fl_number.split('&').join(' & '));
                    }
                }
            });
            tmp_followorder = tmp_followorder.join(', ');
            addCell(newRow, { html: (tmp_followorder.length > 50) ? tmp_followorder.substr(0, 50) + '....' : tmp_followorder, align: 'center' });
        } else
        {
            addCell(newRow, { html: $P.lang("LANG1988"), locale: "LANG1988"});
        }
        addCell(newRow, { html: tmp_a, align: 'center' });
    }

    _$('totle').innerHTML = USERSLIST.length;
    _$('show').innerHTML = PAGE + '/' + NPAGE;
}

var load_users_table = function ()
{
    var TBL = _$('table_userslist');
    var addCell = ASTGUI.domActions.tr_addCell;
    USERSLIST = parent.pbx.users.list();
    USERSLIST = USERSLIST.sortExtension();
    var i = USERSLIST.length % 20;
    NPAGE = (USERSLIST.length - i + (i ? 20 : 0)) / 20;

    if (!USERSLIST.length)
    {
        ASTGUI.domActions.clear_table(TBL);
        var newRow = TBL.insertRow(-1);
        newRow.className = 'even';
        addCell(newRow, { html: $P.lang("LANG129"), locale: "LANG129"});
        _$('totle').innerHTML = '0';
        _$('show').innerHTML = '1/1';
        document.getElementById('pre_button').style.display = 'none';
        document.getElementById('next_button').style.display = 'none';
        document.getElementById('jump_button').style.display = 'none';
        return;
    }
    show_users_list(1);
};

var prepareEditItemForm = function (u, fm_ed)
{
    EDIT_USER = u;
    followMe_MiscFunctions.reset_Fields();
    //reset moh to default 'default'
    ASTGUI.updateFieldToValue('FMU_moh', 'default');
    followMe_MiscFunctions.load_LocalExtensionsList(u);
    RINGTIME = null;

    // load fields
    if (!fm_ed)
    {
        fm_ed = 0;
    } else
    {
        fm_ed = Number(fm_ed);
    }

    if (fm_ed)
    {
        _$('FMU_Enable').checked = true;
        _$('FMU_Disable').checked = false;
    } else
    {
        _$('FMU_Enable').checked = false;
        _$('FMU_Disable').checked = true;
    }

    CURRENT_DESTINATIONS = [];
    var cxt = context2json({ filename: 'followme.conf', context: EDIT_USER, usf: 0 });
    var time;
    if (!cxt)
    {
        cxt = [];
        time = parent.sessionData.pbxinfo.GLOBALS['RINGTIME'];
        time =  Number(time) < 30 ? time : "30";
        CURRENT_DESTINATIONS.push(EDIT_USER + "," + time );
    }
    var flag = 0;
    for (var t = 0; t < cxt.length; t++)
    {
        if (cxt[t].beginsWith('musicclass='))
        {
            ASTGUI.updateFieldToValue('FMU_moh', cxt[t].afterChar('='));
            if (_$('FMU_moh').selectedIndex == -1)      //former musicclass has been deleted 
            {
                ASTGUI.updateFieldToValue('FMU_moh', 'default');
            }
        }
        if (cxt[t].beginsWith('number='))
        {
            CURRENT_DESTINATIONS.push(cxt[t].afterChar('='));
        }
    }

    followMe_MiscFunctions.hide_FORM_newFM_Number();
    followMe_MiscFunctions.refresh_allDestinations();   
    return;
};

var save_FollowMeUser = function ()
{
    // update follow me status of the user in extensions.conf
    // update followme.conf with all other chosen preferences
    // reload the page
    for(var i = 0; i < CURRENT_DESTINATIONS.length; i++)
    {
        var value = _$("step_" + i).value;
        AskRingtime(value);
    }
};
var saveFollowMeUser = function () {
    followMe_MiscFunctions.save_allDestinations();
    /* modify by yjwu */
    var value = (_$('FMU_Enable').checked) ? 'ENABLE' : 'DISABLE';
    ASTGUI.ASTDB.updateKey({ dbname: 'AAUSER/' + EDIT_USER, key: 'FOLLOWME', keyvalue: value });
    var FOLLOWME_INCLUDE = config2json({ filename: 'extensions.conf', usf: 0 });
    var e = new listOfSynActions('extensions.conf');
    var FOLLOWME_CONTEXT = ASTGUI.contexts.FollowmePrefix + EDIT_USER;
    if (FOLLOWME_INCLUDE.hasOwnProperty(FOLLOWME_CONTEXT) == false)
    {
        e.new_action('newcat', FOLLOWME_CONTEXT, '', '');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',1,GotoIf($["${IN_FOLLOWME}" = "YES" | "${RECORD_VMENU}"="ON" | "${CONF_ROOM}" != ""]?disable)');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',n,Set(FM_ENABLE=${DB(AAUSER/${EXTEN}/FOLLOWME)})');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',n,GotoIf($[${FM_ENABLE} = ENABLE]?enable:disable)');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',n(enable),Set(__IN_FOLLOWME=YES)');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',n, Answer()');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',n,Macro(findme-follow,${EXTEN})');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',n,Macro(vm,${EXTEN})');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',n,Macro(hangupcall)');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', EDIT_USER + ',n(disable),Goto(ext-local,${EXTEN},1)');
        e.new_action('append', FOLLOWME_CONTEXT, 'exten', 'h,1,Macro(hangupcall)');
        e.new_action('append', ASTGUI.contexts.ExtFindmefollow, 'include', FOLLOWME_CONTEXT);
        e.callActions();
    }
    var FOLLOWME_CONF = config2json({ filename: 'followme.conf', usf: 0 });
    var u = new listOfActions('followme.conf');
    if (FOLLOWME_CONF.hasOwnProperty(EDIT_USER)) { u.new_action('delcat', EDIT_USER, '', ''); }
    u.new_action('newcat', EDIT_USER, '', '');
    u.new_action('append', EDIT_USER, 'musicclass', ASTGUI.getFieldValue('FMU_moh'));
    u.new_action('append', EDIT_USER, 'context', parent.sessionData.pbxinfo.users[EDIT_USER].getProperty('context'));
    for (var t = 0; t < CURRENT_DESTINATIONS.length; t++)
    {
        u.new_action('append', EDIT_USER, 'number', CURRENT_DESTINATIONS[t]);
    }
    u.callActions(function ()
    {
        top.dialog.dialogMessage({
            type: 'success',
            content: $P.lang("LANG815"),
            callback: function ()
            {
                mWindow.location.reload();
            }
        });
    });
}

var followMe_MiscFunctions = {
    load_LocalExtensionsList: function (u)
    { //  followMe_MiscFunctions.load_LocalExtensionsList()
        ASTGUI.selectbox.clear('FMU_newNumber_local');
        var ul = parent.pbx.users.list(); ul = ul.sortExtension();
        ul.each(function (user)
        {
            if (user != u)
            {
                var fullname = parent.sessionData.pbxinfo.users[user].getProperty('fullname');
                fullname = fullname ? ' \"' + fullname + '\"' : '';
                ASTGUI.selectbox.append('FMU_newNumber_local', user + fullname, user);
            }
        });
    },

    show_FORM_newFM_Number: function ()
    { // followMe_MiscFunctions.show_FORM_newFM_Number()

        var show = true;
        type = "add";
        $('#newFM_Number_radio_local').show();
        $('#newFM_Number_radio_local_label').show();
        $('#FMU_newNumber_local').show();
        $('#FMU_newNumber_External').hide();

        if ($('#FMU_newNumber_local > option').length < 1 )
        {
            show = false;
            $('#newFM_Number_radio_local').hide();
            $('#newFM_Number_radio_local_label').hide();
            $('#FMU_newNumber_local').hide();
            $('#FMU_newNumber_External').show();
        }

        var ringtime = parent.sessionData.pbxinfo.GLOBALS['RINGTIME'];

        ASTGUI.updateFieldToValue('FMU_newNumber_seconds', ringtime < 30 ? ringtime : '30');
        ASTGUI.updateFieldToValue('FMU_newNumber_External', '');
        _$('FMU_newNumber_local').selectedIndex = 0;
        _$('newFM_Number_radio_local').checked = show;
        _$('newFM_Number_radio_Externals').checked = !show;
        _$('newFM_Order_radio_after').checked = true;
        _$('newFM_Order_radio_alongWith').checked = false;

        $('.FORM_newFM_Number').show();
        //$($('.FORM_newFM_Number')[0]).hide();
        $(".new-fm-cmds").hide();
        $('#lastRow_Edit').hide();

        if(NUMBER == 0)
        {
             $('#newFM_Order_radio_alongWith').hide();
             $('#newFM_Order_radio_alongWith_label').hide();
        }
        else
        {
             $('#newFM_Order_radio_alongWith').show();
             $('#newFM_Order_radio_alongWith_label').show();
        }

        if (window.frameElement)
            $(window.frameElement).css("height", "0px");
        top.dialog.repositionDialog("none");
    },

    hide_FORM_newFM_Number: function ()
    { // followMe_MiscFunctions.hide_FORM_newFM_Number()
        if ( $("#step_" + NUMBER).parent().parent()[0] != undefined)
        {
            $("#step_" + NUMBER).parent().parent()[0].style.background = "";
        }
        NUMBER = -1;

        $('.FORM_newFM_Number').hide();
        //$($('.FORM_newFM_Number')[0]).show();
        $(".new-fm-cmds").show();
        $('#lastRow_Edit').show();

        if (window.frameElement)
            $(window.frameElement).css("height", "0px");
        top.dialog.repositionDialog("none");
    },

    reset_Fields: function ()
    { // followMe_MiscFunctions.reset_Fields()

        /* modify by yjwu */
        //ASTGUI.resetTheseFields ([ 'FMU_Enable', 'FMU_Disable', 'FMU_moh', 'FMU_context','FMU_newNumber_local','FMU_newNumber_seconds' ]);
        ASTGUI.resetTheseFields(['FMU_Enable', 'FMU_Disable', 'FMU_moh', 'FMU_newNumber_local', 'FMU_newNumber_seconds']);

        ASTGUI.domActions.removeAllChilds('sqDestinations'); CURRENT_DESTINATIONS = [];

    },

    refresh_allDestinations: function ()
    { // followMe_MiscFunctions.refresh_allDestinations()
        ASTGUI.domActions.removeAllChilds('sqDestinations');
        var add_sqStep = function (a)
        {
            var txt = CURRENT_DESTINATIONS[a];
            var tmp = document.createElement('div');
            tmp.STEPNO = a;
            var sp_desc = document.createElement('span');
            sp_desc.className = 'step_desc';
            sp_desc.innerHTML = txt.split(',')[0].split('&').join(' <B>&</B> ') + $P.lang("LANG569") + "<input id='step_" + t + "' name=step_'" + t +"' class='inputWidth20'> " + $P.lang("LANG570");
            //_$("step_" + t).value = txt.split(',')[1];
            tmp.appendChild(sp_desc);
            var sp_add = document.createElement('span');
            sp_add.className = 'step_add';
            sp_add.innerHTML = '&nbsp;';

            tmp.appendChild(sp_add);
            if ( a > 0 ) {
                var sp_up = document.createElement('span');
                sp_up.className = 'step_up';
                sp_up.innerHTML = '&nbsp;';
                var sp_down = document.createElement('span');
                sp_down.className = 'step_down';
                sp_down.innerHTML = '&nbsp;';
                var sp_delete = document.createElement('span');
                sp_delete.className = 'step_delete';
                sp_delete.innerHTML = '&nbsp;';

                tmp.appendChild(sp_delete);
                tmp.appendChild(sp_up);
                tmp.appendChild(sp_down);
            }
            _$('sqDestinations').appendChild(tmp);
            _$("step_" + t).value = txt.split(',')[1];
            $P('#step_' + t, document).rules('add', {
                        required: true,
                        digits: true,
                        range: [10, 60],
                        cmpareRingtime: true
                });
        };
        for (var t = 0; t < CURRENT_DESTINATIONS.length; t++)
        {
            add_sqStep(t);
        }

        if (NUMBER != -1)
        {
            $("#step_" + NUMBER).parent().parent()[0].style.background = "#ACACAD";
        }
    },

    save_allDestinations: function ()
    {
        for (var t = 0; t < CURRENT_DESTINATIONS.length; t++)
        {
            CURRENT_DESTINATIONS[t] = CURRENT_DESTINATIONS[t].split(",")[0] + "," + _$("step_" + t).value
        }
    },

    push_newdest: function ()
    { // followMe_MiscFunctions.push_newdest() ;
        //AskRingtime(_$('self_user_time').value);
        //AskRingtime(_$('FMU_newNumber_seconds').value);
        this.save_allDestinations();
        if (_$('newFM_Number_radio_local').checked)
        {
            var tmp_number = ASTGUI.getFieldValue('FMU_newNumber_local');
        }

        if (_$('newFM_Number_radio_Externals').checked)
        {
            var tmp_number = ASTGUI.getFieldValue('FMU_newNumber_External');
        }

        var tmp_seconds = ASTGUI.getFieldValue('FMU_newNumber_seconds');

        if (tmp_number.contains('-')) tmp_number = tmp_number.withOut('-');
        var tmp_dest = tmp_number + ',' + tmp_seconds;
        var tmp = CURRENT_DESTINATIONS[NUMBER];

        if (_$('newFM_Order_radio_after').checked || !tmp)
        {
            CURRENT_DESTINATIONS.splice(NUMBER + 1, 0, tmp_dest);
        }

        if (_$('newFM_Order_radio_alongWith').checked && tmp)
        {
            var a = tmp.split(',')[0];
            CURRENT_DESTINATIONS.splice(NUMBER + 1, 0, a + '&' + tmp_dest);
            CURRENT_DESTINATIONS.splice(NUMBER, 1);
        }

        this.refresh_allDestinations();
        this.hide_FORM_newFM_Number();
        //ASTGUI.resetTheseFields (['FMU_newNumber','FMU_newNumber_seconds' ]);
    }
};

var update_FollowMe_Options = function ()
{
    FOLLOWME_OPTIONS = '';

    if (_$('chk_fmoptions_n').checked) FOLLOWME_OPTIONS = FOLLOWME_OPTIONS + 'n';
    if (_$('chk_fmoptions_a').checked) FOLLOWME_OPTIONS = FOLLOWME_OPTIONS + 'a';
    if (_$('chk_fmoptions_s').checked) FOLLOWME_OPTIONS = FOLLOWME_OPTIONS + 's';

    ASTGUI.updateaValue({ file: 'extensions.conf', context: 'globals', variable: 'FOLLOWMEOPTIONS', value: FOLLOWME_OPTIONS });
    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG844"), callback: function () { mWindow.location.reload();} });
};

var cancel_FollowMe_Options = function ()
{

    _$('chk_fmoptions_s').checked = (FOLLOWME_OPTIONS.contains('s')) ? true : false;
    _$('chk_fmoptions_a').checked = (FOLLOWME_OPTIONS.contains('a')) ? true : false;
    _$('chk_fmoptions_n').checked = (FOLLOWME_OPTIONS.contains('n')) ? true : false;
};

var select_page = function (a)
{
    if (a == "next")
    {
        if (PAGE < NPAGE)
        {
            PAGE++;
        }
        else
        {
           // ASTGUI.feedback({ msg: 'This page is the last, no next page!', showfor: 2, color: 'green', bgcolor: '#FFFFFF' });
            return;
        }
    }
    else if (a == "pre")
    {
        if (PAGE > 1)
        {
            PAGE--;
        }
        else
        {
            //ASTGUI.feedback({ msg: 'This page is the first, no previous page!', showfor: 2, color: 'green', bgcolor: '#FFFFFF' });
            return;
        }
    }
    else if (a == "jump")
    {
        $("#mform").tooltip();
        $P("#mform", document).validate({
            rules:{
                "page_jump": {
                    required: true,
                    digits: true,
                    range: [1, NPAGE]
                }
            }
        });
    }

    if (a == "jump")
    {
        //$P('#page_jump', document).rules('add')
        if ($P("#mform", document).valid())
        {
            var page_jump = ASTGUI.getFieldValue('page_jump');

            if (Number(page_jump) == PAGE)
            {
                return;
            }
            else
            {
                PAGE = Number(page_jump);
                show_users_list(PAGE);
            }
        }
    }
    else
    {
        show_users_list(PAGE);
    }
};

function AskRingtime(time){

    var res = 0;

    if (!time || !/^([0-9]\d+)$/.test(time)) {
        return;
    }

    time = Number(time);
    if (time < 0 || time > 60)
    { 
        return;
    }

    var ringtime = parent.sessionData.pbxinfo.GLOBALS['RINGTIME'];
    if (time > ringtime) {
        top.dialog.dialogConfirm({
            confirmStr: $P.lang("LANG930").format(ringtime),
            buttons: {
                ok: function ()
                {
                    top.module.jumpMenu('preferences.html');
                },
                cancel: function ()
                {
                    top.dialog.container.show();
                    top.dialog.shadeDiv.show();
                }
            }
        });
    }
};

var add_number = function()
{
    NUMBER =  CURRENT_DESTINATIONS.length - 1;
    $("#step_" + NUMBER).parent().parent()[0].style.background = "#ACACAD";
    followMe_MiscFunctions.show_FORM_newFM_Number();
}
