/*
* Description: UCM6100 WebGUI 
*
* Copyright (C) 2013 Grandstream Networks, Inc.
*
*/
(function ($, document, undefined)
{
    var self = this;

    var inited = false;
    var locales = new Object();
    var default_locale = top.ddlData[0].value;
    var current_locale = default_locale;

    var _init = function ()
    {
        if (inited)
            return;
        var langArr = new Array;
        for (var i = 0; i < top.ddlData.length; i++) {
            langArr[i] = top.ddlData[i].value;
        }
        if ($.inArray($.cookie('locale'), langArr) < 0) {
            $.cookie('locale', default_locale);
        }
        loadLocale(default_locale); // load default locale first. NOTE: if this fail, then locale won't work
        var _selected = $.cookie('locale');
        if (_selected != undefined && _selected != null)
        {
            if (loadLocale(_selected))
                current_locale = _selected;
        }
        else
            setCurrentLocale(default_locale);
        inited = true;
    };

    var loadLocale = function (code)
    {
        if (code.length == 0)
            return;

        if (locales[code] == undefined || locales[code] == null)
        {
            var request = $.ajax({
                url: "../config/locale/locale." + code + ".json",
                type: "GET",
                async: false,
                dataType: "json"
            });

            request.done(function (json)
            {
                locales[code] = json;
                return true;
            });

            request.fail(function (jqXHR, textStatus)
            {
                alert("Request failed: " + textStatus);
                return false;
            });
        };

        return true; // locale loaded
    };
    var setCurrentLocale = function (code)
    {
        if (code == undefined || code == null || code.length == 0)
            return;

        current_locale = code;
        $.cookie('locale', code, { expires: 365 });
    };

    var _switch = $.langSwitch = function (code, document)
    {
        if (!inited)
            _init();

        if (loadLocale(code))
        {
            setCurrentLocale(code);
            _translate(document, true);
        }
    };

    var _translate = $.lang = function (req, isDoc)
    {
        if (!inited)
            _init();

        if (!isDoc)
        {
            var cur = locales[current_locale];

            var result = cur.translate[req];
            if (result == undefined && current_locale != default_locale)
                result = locales[default_locale].translate[req];

            return result != undefined ? result : "XXXXX";
        }
        else
        {
            if (req == null || req == undefined)
                req = $(document);

            // TODO [AH] this should be moved to different script file
            $("[gLabel]", $(req).contents().find("body")).each(function (i)
            {
                $(this).html(""); // clean inner contents first

                // checking to see if contains tooltip
                var label = $(this).attr("gLabel");
                var tooltip = $(this).attr("tooltip");

                if (label.charAt(0) == '@'){
                    label = label.substr(1);
                    var labelArr = label.split(" ");
                    var labelArrStr = _translate(labelArr[0]);
                    if (label.indexOf(" ") > 0) {
                        for (var i = 1; i < labelArr.length; i++) {
                            if (labelArrStr.indexOf("{") < 0) {
                                labelArrStr += "{" + i +"}";
                            }
                            labelArrStr = labelArrStr.replace("{"+i+"}", "{0}");
                            if (/[\']/.test(labelArr[i])) {
                                labelArrStr = labelArrStr.format(labelArr[i].replace(/(\')/g, ""));
                            }
                            else {
                                labelArrStr = labelArrStr.format(_translate(labelArr[i]));
                            }
                        }
                        label = labelArrStr;
                    }
                    else {
                        label = _translate(label);
                    }
                }
                var labelContainer = $("<span class=\"label\"></span>");
                var tipContainer = $("<span class=\"tooltip\"></span>");

                // append ':' for display 
                if (label.slice(-1) != ':')
                {
                    label = label + ':';
                }
                labelContainer.html(label);

                if (tooltip && tooltip.length > 0)
                {
                    $(tipContainer).addClass("on");
                    $(tipContainer).attr("tip", tooltip);
                }

                var container = $("<div class=\"glab\" />");

                container.append(tipContainer).append(labelContainer);

                $(this).append(container[0].outerHTML);

            });

            $("[locale]", $(req).contents().find("body")).each(function (i)
            {
                // only process when given translate keyword...
                var locale = $(this).attr("locale");
                var localeArr = locale.split(" ");
                var localeArrStr = _translate(localeArr[0]);
                var transLocale = _translate(locale);
                var nameLen = transLocale.length;
                $(this).removeAttr("title");
                if (locale.contains(" ")) {
                    for (var i = 1; i < localeArr.length; i++) {
                        if (/[\']/.test(localeArr[i])) {
                            if(localeArrStr.contains("{0}")){
                                localeArrStr = localeArrStr.format(localeArr[i].replace(/(\')/g, ""));
                            }
                            else{
                                localeArrStr += " " + localeArr[i].replace(/(\')/g, "");
                            }
                        }
                        else if(localeArrStr.contains("{0}")){
                            localeArrStr = localeArrStr.format(_translate(localeArr[i]));
                        }
                        else {
                            localeArrStr += " " + localeArr[i];
                        }
                        localeArrStr = localeArrStr.replace("{"+i+"}", "{0}");
                    }
                    $(this).html(localeArrStr);
                }
                else {
                    if (transLocale.contains("{0}")) {
                        transLocale = transLocale.replace("{0}", "").trim();
                    }
                    $(this).html(transLocale);
                }
            });
            $("[localeTitle]", $(req).contents().find("body")).each(function (i)
            {
                // only process when given translate keyword...
                $(this).attr("title", _translate($(this).attr("localeTitle")));
            });

            // this will handle the tooltip locale prcoess
            if ($.fn.qtip)
            {
                $("[tip]", $(req).contents().find("body")).each(function (i)
                {
                    var value = $(this).attr('tip');
                    if (value.charAt(0) == '@')
                        value = _translate(value.substr(1));

                    // add label at beginning of tooltip
                    var header = $(this).next().html();
                    if (value.slice(0, 3) !== '<B>')
                    {
                        value = '<B>' + header  + '</B> ' + value;
                    }

                    $(this).qtip({
                        content: { text: value },
                        style: {
                            classes: 'ui-tooltip-plain ui-tooltip-shadow ui-tooltip-rounded'
                        },
                        position: {
                            my: "bottom center", // Use the corner...
                            at: "top center", // ...and opposite corner
                            target: "mouse"
//                            adjust: {
//                                x: 216, // [AH] hardcoded for current style, should adjust in different skin layout
//                                y: 125   // [AH] hardcoded for current style, should adjust in different skin layout
//                            }
                        }
                    });
                });
            }
        }
    };

})(jQuery, document);
