/*
* Asterisk-GUI	- an Asterisk configuration interface
*
* iax.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Support multiple languages of IAX global settings                 
 * modified widget for codec settings                                
 * Modified for change the externip to externaddr for new sip.conf   
 *
 */
$P = top.$;

var fieldnames = ['adsi', 'authdebug', 'autokill', 'bandwidth', 'bindaddr', 'bindport', 'codecpriority', 'delayreject', 'dropcount', 'forcejitterbuffer', 'iaxcompat', 'iaxmaxthreadcount', 'iaxthreadcount', 'jitterbuffer', 'jittershrinkrate', /* 'language', */ 'maxexcessbuffer', 'maxjitterbuffer', 'maxjitterinterps', 'maxregexpire', 'minexcessbuffer', 'minregexpire', 'mohinterpret', 'mohsuggest', 'nochecksums', 'resyncthreshold', 'tos', 'trunkfreq', 'trunktimestamps', 'calltokenoptional', 'maxcallnumbers', 'maxcallnumbers_nonvalidated'];
var callNumberLimitsList = [];

window.onload = function ()
{
    $P.lang(document, true);
    top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG677"));
    $("title").attr("locale", "LANG677");
    var c = context2json({ filename: 'iax.conf', context: 'general', usf: 1 });
    var AU = ASTGUI.updateFieldToValue; // temporarily cache function
    fieldnames.each(function (fld)
    {
        var val = (c[fld]) ? c[fld] : '';
        AU(fld, val);
        if (_$(fld)) {
            if ( (fld === 'mohinterpret' || fld === 'mohsuggest') && _$(fld).selectedIndex === -1 )
            {
                ASTGUI.updateFieldToValue(fld, 'default');
            }
        }
    });

    var d = context2json({ filename: 'iax.conf', context: 'callnumberlimits', usf: 0 });
    if ($("#callNumberLimits")[0])
    {
        if (d)
        {
            d.each(function (fld)
            {
                pushNewCallNumberLimit(fld);
            });
        }
    }


    var disallowed = false;
    var real_codecs;
    ASTGUI.CODECSLIST.populateCodecsList(_$('allow'));
    if (c.hasOwnProperty('allow')) { real_codecs = c['array']; }
    if (c.hasOwnProperty('disallow')) { disallowed = c['disallow'].split(','); }
    var default_selected = ['ulaw', 'alaw', 'gsm'];
    default_selected.each(function (val)
    {
        if (!disallowed.contains(val))
        {
            real_codecs = real_codecs + "," + val;
        }
    });
    ASTGUI.CODECSLIST.selectCodecs(_$('allow'), real_codecs);
    top.Custom.init(document);
}
var isExist = function() {
    var limits = $("#callNumberLimits").children();
    var limitsLen = limits.length
    var limitsArr = new Array;
    var limitsIp = $("#form_newCallNumberLimit_ip").val();
    var maxcallnumbers = $("#form_newCallNumberLimit_maxcallnumbers").val();
    var limit = limitsIp + "=" + maxcallnumbers;
    for(var i = 0; i < limitsLen; i++)
    {
        limitsArr[i] = limits.eq(i).attr("id");
    }
    if ($.inArray(limit, limitsArr) < 0) {
        return true
    }   
    else{
        return false;
    }
}
var saveChanges = function ()
{
    var cat = 'general';
    var skip_ifempty = ['register', 'localnet', 'externhost', 'externaddr'];
    var x = new listOfActions('iax.conf');
    /* Can't use fieldnames.each here because the return statement
    would return from the anonymous function passed to the iterator, 
    not saveChanges(). */
    for (var i = 0; i < fieldnames.length; i++)
    {
        var fld = fieldnames[i];
        var value = ASTGUI.getFieldValue(fld);
        if (value !== null)
        {
            value = value.trim();
            if (value == "")
            {
                if (skip_ifempty.contains(fld))
                {
                    return;
                }
                x.new_action('delete', cat, fld, '');
            } else
            {
                if (!ASTGUI.validateFields([fld])) { return; }
                x.new_action('update', cat, fld, value);
            }
        }
    }

    x.new_action('delcat', 'callnumberlimits', '', '');
    if (callNumberLimitsList.length >= 0)
    {
        x.new_action('newcat', 'callnumberlimits', '', '')
    }

    callNumberLimitsList.each(function (eachOne)
    {
        var pieces = eachOne.split('=');
        x.new_action('append', 'callnumberlimits', pieces[0], pieces[1]);
    });

    /* move codec to single file 
    x.new_action('delete', cat, 'disallow', '');
    x.new_action('delete', cat, 'allow', '');
    x.new_action('append', cat, 'disallow', 'all');
    //x.new_action('append', cat, 'allow', ASTGUI.CODECSLIST.getSelectedCodecs(_$('allowCodecs')));
    var codeList = ASTGUI.CODECSLIST.getSelectedCodecs(_$('allow'));
    if (codeList !== null)
        x.new_action('append', cat, 'allow', codeList);
    */


    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG826") });

    var after = function ()
    {
        top.dialog.clearDialog();
        top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG844") });
    };
    setTimeout(function () { x.callActions(after); }, 300);
    hideCallNumberLimitsForm();
}

var showCallNumberLimitsForm = function ()
{
    ASTGUI.updateFieldToValue('form_newCallNumberLimit_ip', '');
    ASTGUI.updateFieldToValue('form_newCallNumberLimit_maxcallnumbers', '');
    $('.form_newCallNumberLimit').show();
    $("#dAddCallNumberLimit").hide();
}

var hideCallNumberLimitsForm = function ()
{
    $('.form_newCallNumberLimit').hide();
    $("#dAddCallNumberLimit").show();
    //$($('.form_newCallNumberLimit')[0]).show();
}

var pushNewCallNumberLimit = function (limit_str)
{
    if (limit_str)
    {
        var p = limit_str.split("=");
        var new_ip = p[0];
        var new_maxcallnumbers = p[1];
    } else
    {
        var new_ip = ASTGUI.getFieldValue('form_newCallNumberLimit_ip');
        var new_maxcallnumbers = ASTGUI.getFieldValue('form_newCallNumberLimit_maxcallnumbers');
    }
    var tmp_pieces = new_ip.split("/");
    if (!limit_str && !ASTGUI.validateFields(['form_newCallNumberLimit_ip']))
    {
        return;
    } 
    else if (!callNumberLimitsList.contains(new_ip + "=" + new_maxcallnumbers))
    {
        callNumberLimitsList.push(new_ip + "=" + new_maxcallnumbers);
        clearCallNumberLimitsForm();
        refreshCallNumberLimitsList();
    }
}
var AddNewCallNumberLimit = function () {
    $P("#form_newCallNumberLimit_ip", document).rules("add", {
        required: true,
        iporrange: true,
        isLimitsExist: [$P.lang("LANG2389"), isExist]
    });
    $P("#form_newCallNumberLimit_maxcallnumbers", document).rules("add", {
        required: true,
        digits: true
    });
    if ($P("#form", document).validate().element($("#form_newCallNumberLimit_ip")) && $P("#form", document).validate().element($("#form_newCallNumberLimit_maxcallnumbers"))) {
        pushNewCallNumberLimit();
        $P("#form_newCallNumberLimit_ip", document).rules("remove", "required iporrange isLimitsExist");
        $P("#form_newCallNumberLimit_maxcallnumbers", document).rules("remove", "required digits");
    }   
}
var deleteCallNumber = function (indexno)
{
    callNumberLimitsList.splice(indexno, 1);
    refreshCallNumberLimitsList();
}

var clearCallNumberLimitsForm = function ()
{
    ASTGUI.updateFieldToValue('form_newCallNumberLimit_ip', '');
    ASTGUI.updateFieldToValue('form_newCallNumberLimit_maxcallnumbers', '');
}

var refreshCallNumberLimitsList = function ()
{
    ASTGUI.domActions.removeAllChilds('callNumberLimits');

    for (var i = 0; i < callNumberLimitsList.length; i++)
    {
        var pieces = callNumberLimitsList[i].split("=");

        var row_div = document.createElement('div');
        row_div.id = callNumberLimitsList[i];
        row_div.innerHTML = pieces[0] + " = " + pieces[1];

        var sp_delete = document.createElement('span');
        sp_delete.className = 'callnumber_delete';
        sp_delete.innerHTML = '&nbsp;';
        sp_delete.id = "callNumberLimit" + i;
        row_div.appendChild(sp_delete);

        _$('callNumberLimits').appendChild(row_div);
        $('#callNumberLimit' + i).click(function (e)
        {
            deleteCallNumber($(this).attr('id').substr(15));
        });
    }

}

