/*
* Asterisk-GUI	- an Asterisk configuration interface
*
* incoming.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * reset Strip number when destination is 'ByDID'                               
 * added variable OLDPATTERN                                                    
 * added a row to ignore extension "NoOP" when show all incoming rules          
 * 
 */
$P = top.$;
var mWindow = window;
var DESTINATION_LIST = ['Extension', 'Voicemail', 'Conference Room', 'Queue', 'Ringgroup', 'Page', 'Vmgroup', 'Fax', 'DISA', 'IVR'];
var dest_dialplan_list = [
    'Goto(ext-local,${args},1)',
    'Voicemail(${args},u)',
    'Goto(ext-conference,${args},1)',
    'Goto(ext-queues,${args},1)',
    'Goto(ext-group,${args},1)',
    'Goto(pagegroups,${args},1)',
    'Goto(voicemailgroups,${args},1)',
    'Goto(ext-fax,${args},1)',
    'Goto(disa,${args},1)',
    'Goto(${args},s,1)'
];
var dest_sData_list = ['users', 'users', 'conferences', 'queues', 'voicemenus', 'ringgroups', 'vmgroups', 'fax'];
var STRING_SEPERATOR = '#######'; // will be used to store 'actual context line + contextname' as a DOM attribute, will not be written to any file
var isNewIR = false;
var EDIT_CONTEXT_IR = '';
var EDIT_CONTEXT_IR_LINE = '';
var EX_CF;
var OLDPATTERN;
var selectedRecord;
var selectedContext;    //additional DID settings context
var datactol = {
    "clicked": false,
    "timeout": null
};
var typeObj = {
    "voip": "LANG241",
    "analog": "LANG105"
};
window.onload = function ()
{

    $P.lang(document, true);
    var mode = gup("mode");
    var item = gup("item");
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        initForm(mode);
        append_time_value(mode);
        if (mode == "add")
        {
            incomingRules_MiscFunctions.new_IR_form();
            $("#editForm").show();
        }
        else if (mode == "edit")
        {
            incomingRules_MiscFunctions.edit_IR(mWindow.selectedRecord);
            $("#editForm").show();
        }

        OrderTable.table_id = 'table_timecondition';
        OrderTable.refresh_timecondition();
        OrderTable.bind_event();
        init_validator();
        top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG658"));
        $("title").attr("locale", "LANG658");
        incomingRules_MiscFunctions.listAllRulesInTable();
    }
    $P.lang(document, true);
};

var tm_condition_change_init = function ()
{
    var week_all = $('#new_itrl_week_all');
    week_all.bind("click", function ()
    {
        var week_chkbox_all = function(value)
        {
            var children = $('#new_itrl_week_container').children().find("input");
            for ( var i = 0; i < children.length; i++ )
            {
                if ( children[i].type == 'checkbox' )
                    children[i].checked = value;
                    if (value) {
                        $(children[i]).prev().css("backgroundPosition", "0px -50px");
                    }
                    else {
                        $(children[i]).prev().css("backgroundPosition", "0px 0px");
                    }
            }
        }
        var all = _$('new_itrl_week_all');
        if ( all.checked )
        {
            week_chkbox_all(true);
        } else
        {
            week_chkbox_all(false);
        }
    });
    var month_all = $('#new_itrl_month_all');
    month_all.bind("click", function ()
    {
        var month_chkbox_all = function(value)
        {
            var children = $('#new_itrl_month_container').children().find("input");
            for ( var i = 0; i < children.length; i++ )
            {
                if ( children[i].type == 'checkbox' )
                    children[i].checked = value;
                                    if (value) {
                        $(children[i]).prev().css("backgroundPosition", "0px -50px");
                    }
                    else {
                        $(children[i]).prev().css("backgroundPosition", "0px 0px");
                    }
            }
        }
        var all = _$('new_itrl_month_all');
        if ( all.checked )
        {
            month_chkbox_all(true);
        } else
        {
            month_chkbox_all(false);
        }
    });
    var chk_week = $(".chk_week");
    chk_week.bind("click", function (ev) {
        if (chk_week.filter(":checked").length != chk_week.length) {
            week_all.removeAttr("checked");
            week_all.prev().css("backgroundPosition", "0px 0px");
        }
        else {
            week_all.attr("checked", "checked");
            week_all.prev().css("backgroundPosition", "0px -50px");
        }
        ev.stopPropagation();        
    });
    var chk_month = $(".chk_month");
    chk_month.bind("click", function (ev) {
        if (chk_month.filter(":checked").length != chk_month.length) {
            month_all.removeAttr("checked");
            month_all.prev().css("backgroundPosition", "0px 0px");
        }
        else {
            month_all.attr("checked", "checked");
            month_all.prev().css("backgroundPosition", "0px -50px");
        }
        ev.stopPropagation();        
    });
}

var dayMouseEnter = function (ev) {
    if (datactol.clicked === false)
    {
        return;
    }
    var clickedElement = $(datactol.clicked);
    var clickedDay = parseInt(clickedElement.attr('id').afterChar('_'), 10);
    var mouseDay = parseInt(this.id.afterChar('_'), 10);
    var newClass = clickedElement.hasClass('day_selected') ? 'day_selected_pending' : 'day_unselected_pending';

    var start = (clickedDay < mouseDay) ? clickedDay : mouseDay;
    var end = (clickedDay > mouseDay) ? clickedDay : mouseDay;

    $('.day_selected_pending').removeClass('day_selected_pending');
    $('.day_unselected_pending').removeClass('day_unselected_pending');

    for (var i = start; i <= end; i++)
    {
        $('#day_' + i).addClass(newClass);
    }

};
var initForm = function (type)
{
    /* Populate Destinations <select> */
    var trunkstyle = '';
    var k = mWindow.selectedRecord;
    if ( k ) {
        k = k.parentNode;
        while (k.tagName != 'TR') { k = k.parentNode; }
        var ALC = $(k).attr('ACTUAL_LINE__CONTEXT') || '';
        if (!ALC || !ALC.contains(STRING_SEPERATOR)) return;

        var context_ir = ALC.split(STRING_SEPERATOR)[1];
        var idx = context_ir.match(/^ext-did-([0-9][0-9]*)$/)[1];
        var trunk = parent.sessionData.pbxinfo.trunks_info['trunk_'+idx];
        if ( trunk )
            trunkstyle = trunk.trunkstyle;
    }
    var dest = $('#edit_itrl_dest');
    var tc_dest = $('#new_itrl_tc_dest');
    /*
    var list = parent.miscFunctions.getAllDestinations_4incomings();
    for (var d = 0; d < list.length; d++)
    {
        dest.append($('<option>').html(list[d].optionText).val(list[d].optionValue));
        tc_dest.append($('<option>').html(list[d].optionText).val(list[d].optionValue));
    }
    */
    dest.append($('<option>').html($P.lang('LANG1563')).val('ByDID'));
    tc_dest.append($('<option>').html($P.lang('LANG1563')).val('ByDID'));
    for ( var idx = 0; idx < DESTINATION_LIST.length; idx++ )
    {
        dest.append($('<option>').html(DESTINATION_LIST[idx]).val(idx));
        tc_dest.append($('<option>').html(DESTINATION_LIST[idx]).val(idx));
    }
    if (type == "add") {
        dest[0].selectedIndex = -1;
        tc_dest[0].selectedIndex = -1;
    }
    

    /* now lets add some events!! */
    dest.change(function ()
    {
        if ($(this).val() != 'ByDID')
        {
            $('.localext_byDid').css('display', 'none');
            $('#edit_itrl_dest_val').show();
            $('#edit_itrl_dest_val').attr('value', '');
            var idx = _$('edit_itrl_dest').value;
            _$('edit_itrl_dest_val').length = 0;
            _$('edit_itrl_dest_val').textContent = '';
            top.Custom.init(document, $('#edit_itrl_dest_val')[0]);
            append_destination_by_type('edit_itrl_dest_val', DESTINATION_LIST[idx]);
        } 
        else
        {
            $('.localext_byDid').css('display', 'block');
            $('#edit_itrl_dest_val').hide();
            $('#selectedit_itrl_dest_val').css("display", "none");
        }
        //reset innerhtml height
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog("none");
    });
    tc_dest.change(function ()
    {
        if ($(this).val() != 'ByDID')
        {
            $('.tc_byDid').css('display', 'none');
            $('#new_itrl_tc_dest_val').show();
            $('#new_itrl_tc_dest_val').attr('value', '');
            var idx = _$('new_itrl_tc_dest').value;
            _$('new_itrl_tc_dest_val').length = 0;
            _$('new_itrl_tc_dest_val').textContent = '';
            top.Custom.init(document, $('#new_itrl_tc_dest_val')[0]);
            append_destination_by_type('new_itrl_tc_dest_val', DESTINATION_LIST[idx]);
        } else
        {
            $('.tc_byDid').css('display', 'block');
            $('#new_itrl_tc_dest_val').hide();
            $('#selectnew_itrl_tc_dest_val').css("display", "none");
        }
    });

    var byweek = $('#new_itrl_date_week');
    var bymonth = $('#new_itrl_date_month');
    byweek.bind("click", function(ev)
    {
        var radio = document.getElementById('new_itrl_date_week');
        var tmp_radio = document.getElementById('new_itrl_date_month');
        $('.local_week').show();
        $('.local_month').hide();
        $('.local_day').hide();
        $('#new_itrl_day_container').hide();
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog();
    });
    bymonth.bind("click", function()
    {
        var radio = document.getElementById('new_itrl_date_month');
        var tmp_radio = document.getElementById('new_itrl_date_week');
        $('.local_week').hide();
        $('.local_month').show();
        $('.local_day').show();
        $('#new_itrl_day_container').show();
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog();
    });

    $('#edit_itrl_trunk').change(function ()
    {
        incomingRules_MiscFunctions.enableDisablePattern();
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog("none");
    });


    //init days
    var day_container = document.getElementById('new_itrl_day_container');
    var tbl = document.createElement('table');
    var addCell = ASTGUI.domActions.tr_addCell;
    var days_add = function(table, length)
    {
        var addCell = ASTGUI.domActions.tr_addCell;
        for ( var i = 0; i < length; i++ )
        {
            if ( (i%7) == 0 )
            {
                var newRow = table.insertRow(-1);
                $(newRow).css('background', '#CAECFF');
            }
            //var cell = "<input type='checkbox' id='new_itrl_day_" + i + "' value='" + i + "'>" + i;
            var cell = i + 1;
            addCell(newRow, {html: cell, className: 'day_unselected', title: $P.lang('LANG563') , localeTitle: 'LANG563', id: 'day_' + (i + 1)});
        }
    }
    days_add(tbl, 31);
    day_container.appendChild(tbl);

    $('#new_itrl_day_container').delegate('td', 'mousedown', function (ev) {
        var $this = $(this);
        if ( datactol.clicked === false)
        {
            datactol.clicked = $this;
        }
        //if (datactol.clicked == true)

        if ($(this).hasClass('day_selected'))
        {
            $this.removeClass('day_selected');
            $this.addClass('day_unselected');
        }
        else
        {
            $this.removeClass('day_unselected');
            $this.addClass('day_selected');
        }
        //this.className = 'day_selected';
        //
        //$('#new_itrl_day_container').delegate('td', 'mouseenter', dayMouseEnter);
        $('#new_itrl_day_container').delegate('td', 'mouseover', dayMouseEnter);
        $('#new_itrl_day_container').css('cursor', 'col-resize');

        if (ev.preventDefault)
        {
            ev.preventDefault();
        }
    });


    $('#new_itrl_day_container').delegate('td', 'mouseup', function (ev) {
        datactol.clicked = false;
        $('.day_selected_pending').attr('class', 'day_selected');
        $('.day_unselected_pending').attr('class', 'day_unselected');
        $('#new_itrl_day_container').undelegate('td', 'mouseover', dayMouseEnter);
        $('#new_itrl_day_container').css('cursor', 'auto');

        if (ev.preventDefault)
        {
            ev.preventDefault();
        }
    });



    EX_CF = config2json({ filename: 'extensions.conf', usf: 0 });
    var t = parent.pbx.trunks.list() || [];

    var TMP_FORSORT = [];
    t.each(function (item)
    {
        TMP_FORSORT.push(parent.pbx.trunks.getName(item) + STRING_SEPERATOR + item);
    });

    TMP_FORSORT.sort();
    TMP_FORSORT.each(function (this_str)
    {
        var a = this_str.split(STRING_SEPERATOR); // a[0] is trunkname , a[1] is trunk 
        var this_trunk = a[1];
        var this_trunk_label = a[0];
        var ttype = parent.pbx.trunks.getType(this_trunk);
        var defaultContext_ContextName = ASTGUI.contexts.TrunkDIDPrefix + (this_trunk.split('_'))[1];
    	var type = parent.sessionData.pbxinfo.trunks_info[this_trunk].trunkstyle;
        var name = "";
        if ( type == 'voip' )
        {
            var tmp_type = parent.sessionData.pbxinfo.trunks_info[a[1]].trunkflags;
            if ( /^sip1$/.test(tmp_type) )
                name = 'SIP Peer ' + $P.lang('LANG83') + ' -- ' + a[0];
            else if ( /^sip2$/.test(tmp_type) )
                name = 'SIP Register ' + $P.lang('LANG83') + ' -- ' + a[0];
            else if ( /^iax1$/.test(tmp_type) )
                name = 'IAX Peer ' + $P.lang('LANG83') + ' -- ' + a[0];
            else if ( /^iax2$/.test(tmp_type) )
                name = 'IAX Register ' + $P.lang('LANG83') + ' -- ' + a[0];
        } else {
            name = $P.lang(typeObj[type]) + $P.lang('LANG83') + ' -- ' + a[0];
        }
    	ASTGUI.selectbox.append('edit_itrl_trunk', name, this_trunk);
    });

    tm_condition_change_init()
}
var init_validator = function () {
    $("#edit_itrl_pattern").bind("blur", function () {
        $P("#edit_did_pattern", document).valid(); 
    });
    $("#edit_did_pattern").bind("blur", function () {
        $P("#edit_itrl_pattern", document).valid();
    });
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "edit_itrl_trunk": {
                required: true 
            },
            "edit_itrl_pattern": {
                required: true,
                customCallback3: [$P.lang('LANG2139'), check_pattern_format],
                dialpattern: ['dial pattern'],
				dialpattern_additional: ['dial pattern'],
                customCallback1: ["An incoming rule already exists for this pattern in the selected Trunk", check_pattern],
                customCallback2: ["Invalid format! The format of CID Identification should be numbers.", check_pattern_with_cid]
            },
            "edit_did_pattern": {
                customCallback3: [$P.lang('LANG2139'), check_pattern_format],
                dialpattern: ['dial pattern'],
                dialpattern_additional: ['dial pattern'],
                customCallback1: ["An incoming rule already exists for this pattern in the selected Trunk", check_pattern],
                customCallback2: ["Invalid format! The format of CID Identification should be numbers.", check_pattern_with_cid]
            },
            "edit_itrl_dest": {
                required: true
            },
            "edit_itrl_dest_val": {
                required: true,
                customCallback: ["The Number doesn't exist, please check it", check_dest_val]
            },
            "new_itrl_tc_dest": {
                required: true
            },
            "new_itrl_tc_dest_val": {
                required: true,
                customCallback: ["The Number doesn't exist, please check it", check_tc_dest_val] 
            },
            "edit_itrl_LocalDest_Pattern": {
                digits: true,
                stripMax: ['pattern length', $('#edit_itrl_pattern')]
            },
            "new_itrl_tc_digits": {
                digits: true,
                stripMax: ['pattern length', $('#edit_itrl_pattern')]
            },
            "new_itrl_stime_hour": {
            //    required: true,
                digits: true,
                range: [0,23],
                // LANG184  LANG169
                customCallback: [$P.lang('LANG2142').format($P.lang('LANG169'), $P.lang('LANG184')), check_time]     //end time's value cannot small than start time's value.
            },
            "new_itrl_ftime_hour": {
             //   required: true,
                digits: true,
                range: [0,23],
                customCallback: [$P.lang('LANG2142').format($P.lang('LANG169'), $P.lang('LANG184')), check_time]
            },
            "new_itrl_stime_minute": {
            //    required: true,
                digits: true,
                range: [0,59],
                customCallback: [$P.lang('LANG2142').format($P.lang('LANG169'), $P.lang('LANG184')), check_time]
            },
            "new_itrl_ftime_minute": {
            //    required: true,
                digits: true,
                range: [0,59],
                customCallback: [$P.lang('LANG2142').format($P.lang('LANG169'), $P.lang('LANG184')), check_time]
            },
            "new_itrl_date_week": {
                required: true
                //customCallback: ['Choose one detailed date', check_date]
            },
            "new_itrl_month_jan": {
                customCallback: [$P.lang("LANG2135").format('1', $P.lang('LANG244').toLowerCase()), check_month_required]
            },
            "new_itrl_week_sun": {
                customCallback: [$P.lang("LANG2135").format('1', $P.lang('LANG243').toLowerCase()), check_week_required]
            },
            "edit_itrl_level": {
                required: true
            }
        },
        submitHandler: function() {
            var target = this.submitButton;

            if ($(target).attr('id') !== 'save')
            {
                //non-input-addtional-validation
                if ($('#new_itrl_date_month:checked').length)
                {
                    if (!check_days_required() )
                    {
                        //check failed
                        return;
                    }
                }

                if ($(target).attr('id') == 'table-add-btn')
                {
                    OrderTable.tbl_btn_add();
                    return;
                }
                else if ($(target).attr('id') == 'table-edit-btn')
                {
                    OrderTable.tbl_btn_update();
                    return;
                }
                else
                {
                    return;
                }

            }
            incomingRules_MiscFunctions.update_IR();
        }
    });

}
var check_month_required = function (element, value) {
    if ( $('.chk_month:checked').length )
    {
        return true;
    }
    return false;

};

var check_week_required = function (element, value) {
    if ( $('.chk_week:checked').length )
    {
        return true;
    }
    return false;

};

var check_days_required = function () {
    if ( !$('.day_selected').length )
    {
        var target = $('#day_1');
        
        if (datactol.timeout)
        {
            clearTimeout(datactol.timeout);
        }
        target.attr("titles", $P.lang("LANG2135").format('1', $P.lang('LANG242').toLowerCase())).addClass("ui-state-highlight").trigger("focusin");

        datactol.timeout = setTimeout( function () {
           target.removeAttr("titles").removeClass("ui-state-highlight").trigger("focusout");

        }, 2000);
        return false;
    }
    return true;
};


var incomingRules_MiscFunctions = {
    moveUpDown_In_context: function (context, rule, updown)
    { 	
        top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826') });
        ASTGUI.miscFunctions.moveUpDown_In_context(context, rule, updown, function (newcontext)
        {
            top.dialog.clearDialog();
            top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG815"), callback: function () { mWindow.location.reload(); } });
        });
    },

    edit_IR: function (s)
    {
        isNewIR = false;
        var k = s.parentNode;
        while (k.tagName != 'TR') { k = k.parentNode; }

        var ALC = $(k).attr('ACTUAL_LINE__CONTEXT') || '';

        if (!ALC || !ALC.contains(STRING_SEPERATOR)) return;

        EDIT_CONTEXT_IR = ALC.split(STRING_SEPERATOR)[1];
        EDIT_CONTEXT_IR_LINE = ALC.split(STRING_SEPERATOR)[0];

        if (!EDIT_CONTEXT_IR.contains(ASTGUI.contexts.TimeIntervalPrefix) && EDIT_CONTEXT_IR.endsWith(ASTGUI.contexts.TrunkDefaultSuffix))
        {
            var tmp_TRUNK = EDIT_CONTEXT_IR.lChop(ASTGUI.contexts.TrunkDIDPrefix).beforeStr(ASTGUI.contexts.TrunkDefaultSuffix);
        } else
        {
            var tmp_TRUNK = EDIT_CONTEXT_IR.lChop(ASTGUI.contexts.TrunkDIDPrefix).beforeStr('_' + ASTGUI.contexts.TimeIntervalPrefix);
            tmp_TRUNK = 'trunk_' + tmp_TRUNK;
        }

        // modified by dcwei 2012-09-07
        //		var tmp_edit_itrl_tf = EDIT_CONTEXT_IR.afterStr( ASTGUI.contexts.TimeIntervalPrefix ) ;

        $('.hideOnEdit').hide();
        // modified by dcwei 2012-09-07
        //		_$('div_ir_edit_title').innerHTML = 'Edit Incoming Calling Rule, Trunk: ' + tmp_TRUNK + ' , Time Interval: ' + ( tmp_edit_itrl_tf || 'none' ) ;

        ASTGUI.updateFieldToValue('edit_itrl_trunk', tmp_TRUNK);
        //		ASTGUI.updateFieldToValue( 'edit_itrl_tf', tmp_edit_itrl_tf ) ;		//	modified by dcwei 2012-09-07
        var context_ir_line_val = ASTGUI.parseContextLine.getExten(EDIT_CONTEXT_IR_LINE).split("/");
        
        ASTGUI.updateFieldToValue('edit_itrl_pattern', context_ir_line_val[0]);
        if (context_ir_line_val[1]) {
            ASTGUI.updateFieldToValue('edit_did_pattern', context_ir_line_val[1]);
        }
        OLDPATTERN = ASTGUI.parseContextLine.getExten(EDIT_CONTEXT_IR_LINE);

        //$('.localext_byDid').hide();
       $('.localext_byDid').css('display', 'none');
        /*	modified by dcwei 2012-09-27	changed from "default" to "from-internal"	*/

        var tmp_dt_str = 'Set(__DISA_PERMISSION=';
        //ASTGUI.updateFieldToValue('edit_itrl_level', '');

        var tmp_line = EDIT_CONTEXT_IR_LINE;
        var content = config2json({filename: 'extensions.conf', usf: 0});
        if ( content.hasOwnProperty(EDIT_CONTEXT_IR) )
        {
            var lists = content[EDIT_CONTEXT_IR];

            var exten = ASTGUI.parseContextLine.getExten(tmp_line);
            OrderTable.lists = [];
            lists.each( function( item ) 
            {
                /*  dial trunk supported  */
                var tmp_exten = ASTGUI.parseContextLine.getExten(item);
                if ( exten == tmp_exten && item.indexOf(tmp_dt_str) != -1 )
                {
                    var a = item.indexOf(tmp_dt_str);
                    var b = item.lastIndexOf(')'); 
                    var level = item.substring(a+tmp_dt_str.length, b);
                    ASTGUI.updateFieldToValue('edit_itrl_level', level);
                }

                if ( tmp_exten == exten && item.contains('ExecIfTime(') )
                {
                    var it = item.substring(item.indexOf('ExecIfTime('));
                    var pos1 = it.indexOf('(');
                    var pos2 = it.indexOf(',');
                    var idx = OrderTable.lists.length;
                    var el = {};
                    el['tc_index'] = '' + idx;
                    /*  time  */
                    el['tc_time'] = it.substring(pos1+1, pos2);
                    /*  week  */
                    it = it.substring(pos2+1);
                    pos1 = it.indexOf(',');
                    el['tc_week'] = it.substring(0, pos1);

                    /*  day  */
                    it = it.substring(pos1+1);
                    pos1 = it.indexOf(',');
                    el['tc_day'] = it.substring(0, pos1);
                    /*  month  */
                    it = it.substring(pos1+1);
                    pos1 = it.indexOf('?');
                    el['tc_month'] = it.substring(0, pos1);

                    /*  destination  */
                    it = it.substring(pos1+1);
                    pos1 = it.lastIndexOf(')');
                    el['tc_dest'] = it.substring(0, pos1);

                    pos1 = el['tc_dest'].indexOf('${EXTEN:');
                    /*  digits  */
                    if ( pos1 > -1 )
                    {
                        it = it.substring(pos1);
                        pos1 = it.indexOf(':');
                        el['tc_digits'] = it.substring(pos1+1, it.indexOf('}'));
                    }

                    OrderTable.lists.push(el);
                }
            } );
        }
        if (tmp_line.contains( ASTGUI.globals.didAdditionalPostfix + ',${EXTEN'))
        {
            //$('.localext_byDid').show();
            $('.localext_byDid').css('display', 'block');
            ASTGUI.updateFieldToValue('edit_itrl_dest', 'ByDID');
            var tmp_ldp = tmp_line.betweenXY('{', '}').lChop('EXTEN').lChop(':');
            if (!tmp_ldp.length) { tmp_ldp = '0'; }
            ASTGUI.updateFieldToValue('edit_itrl_LocalDest_Pattern', tmp_ldp);
        } else
        {
            //ASTGUI.updateFieldToValue('edit_itrl_dest', ASTGUI.parseContextLine.getAppWithArgs(tmp_line));
            trans_dest('edit_itrl_dest', tmp_line);
        }

        incomingRules_MiscFunctions.enableDisablePattern();
    },

    delete_IR: function (s)
    { // incomingRules_MiscFunctions.delete_IR()
        //var s = ASTGUI.events.getTarget(event);
        var ALC;
        var k = s.parentNode;
        while (k.tagName != 'TR') { k = k.parentNode; }

        ALC = $(k).attr('ACTUAL_LINE__CONTEXT') || '';

        if (!ALC || !ALC.contains(STRING_SEPERATOR)) return;

        var TMP_CONTEXT = ALC.split(STRING_SEPERATOR)[1];
		var t = 'trunk_' + TMP_CONTEXT.split('-')[2];
        var tmp_msg = 'The incoming route for trunk - ' + top.sessionData.pbxinfo.trunks_info[t].trunkname;
		var TMP_LINE = ALC.split(STRING_SEPERATOR)[0];

        top.dialog.dialogConfirm({
            confirmStr: $P.lang("LANG818").format(tmp_msg),
            buttons:
            {
                ok: function ()
                {
                    var resp = top.pbx.trunks.rules.remove({ cxt: TMP_CONTEXT, line: TMP_LINE });
                    if (!resp)
                    {
                        top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG820") });
                    }
                    else
                    {
                        top.dialog.dialogMessage({
                            type: 'success',
                            content: $P.lang("LANG816"),
                            callback: function ()
                            {
                                mWindow.location.reload();
                            }
                        });
                    }
                }
            }
        });
    },

    listAllRulesInTable: function ()
    { // incomingRules_MiscFunctions.listAllRulesInTable();
        EX_CF = config2json({ filename: 'extensions.conf', usf: 0 });
        var t = parent.pbx.trunks.list() || [];

        if (!t.length)
        {

            var TBL = document.createElement('TABLE');
            TBL.cellPadding = 0;
            TBL.cellSpacing = 0;
            TBL.border = 0;
            TBL.className = 'table_incomingRulesList tableWidget';

            var newRow = TBL.insertRow(-1);
            newRow.className = 'even';
            ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang('LANG1561'), width: '95%' });

            var tbl_heading = document.createElement('div');
            tbl_heading.style.marginTop = '35px';
            if (tbl_heading) {
                tbl_heading.appendChild(TBL);
                _$('IR_CONTAINER').appendChild(tbl_heading);
            }
            $('.top_buttons').hide();
            return;
        }

        var TMP_FORSORT = [];
        t.each(function (item)
        {
            TMP_FORSORT.push(parent.pbx.trunks.getName(item) + STRING_SEPERATOR + item);
        });

        TMP_FORSORT.sortExtension();
        TMP_FORSORT.each(function (this_str)
        {
            var a = this_str.split(STRING_SEPERATOR); // a[0] is trunkname , a[1] is trunk 
            var this_trunk = a[1];
            var this_trunk_label = a[0];
			var type = parent.sessionData.pbxinfo.trunks_info[this_trunk].trunkstyle;
            var name = $P.lang(typeObj[type]) + $P.lang('LANG83') + ' -- ' + this_trunk_label;
            var ttype = parent.pbx.trunks.getType(this_trunk);
            var defaultContext_ContextName = ASTGUI.contexts.TrunkDIDPrefix + (this_trunk.split('_'))[1];
            var cxt_name = ASTGUI.contexts.TrunkDIDPrefix + (this_trunk.split('_'))[1];

            ASTGUI.selectbox.append('edit_itrl_trunk', name, this_trunk);

            var TBL = document.createElement('TABLE');
            TBL.cellPadding = 0;
            TBL.cellSpacing = 0;
            TBL.border = 0;
            TBL.className = 'table_incomingRulesList tableWidget';

            var newRow = TBL.insertRow(-1);
            newRow.className = 'frow';
            ASTGUI.domActions.tr_addCell(newRow, { html: '', width: '25px' });
            ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang('LANG246'), locale: 'LANG246' });
            ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang('LANG247'), locale: 'LANG247' });
            ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang('LANG84'), locale: 'LANG84' });
            ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang('LANG168'), locale: 'LANG168' });

            //ASTGUI.domActions.tr_addCell(newRow, { html: 'Destination' });
            ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang('LANG74'), locale: 'LANG74' });

            var PREVIOUS_SET_BGClass = 'odd'; // odd or even
            /*	if( ! EX_CF.hasOwnProperty( ASTGUI.contexts.TrunkDIDPrefix + this_trunk ) ){ return ; }	*/
            if (!EX_CF.hasOwnProperty(cxt_name)) { return; }


            var trunk_idx = a[1];
            var rule_lists = [];
            EX_CF[cxt_name].each(function (did_this_rule, main_did_index)
            {
                if (did_this_rule == 'include=' + defaultContext_ContextName)
                {

                    if (!EX_CF.hasOwnProperty(defaultContext_ContextName)) { return; }

                    var defaultContext = EX_CF[defaultContext_ContextName];
                    var tmp_defaultContext_length = EX_CF[defaultContext_ContextName].length;

                    defaultContext.each(function (this_defaultRule, this_defaultRule_Index)
                    {
                        if (!this_defaultRule.beginsWith('exten=')) { return; } // RULES in the main DID_trunk_x
                        var tmp_exten = ASTGUI.parseContextLine.getExten(this_defaultRule);
                        if (tmp_exten == 's' && this_defaultRule.contains('ExecIf')) { return; }
                        var newRow = TBL.insertRow(-1);
                        newRow.className = (PREVIOUS_SET_BGClass == 'odd') ? 'even' : 'odd';
                        $(newRow).attr('ACTUAL_LINE__CONTEXT', this_defaultRule + STRING_SEPERATOR + defaultContext_ContextName);
                        ASTGUI.domActions.tr_addCell(newRow, { html: '' });
                        //I18N has not been used.
                        ASTGUI.domActions.tr_addCell(newRow, { html: '<font color=red>none (no TimeIntervals matched)</font>' });   
                        if (ttype != 'analog')
                        {
                            if (tmp_exten == 's')
                            {
                                ASTGUI.domActions.tr_addCell(newRow, { html: "'s'" });
                            } else if (tmp_exten == '_X.' || tmp_exten == '_x.')
                            {
                                ASTGUI.domActions.tr_addCell(newRow, { html: "'_X.' (CatchAll)" });
                            } else
                            {
                                ASTGUI.domActions.tr_addCell(newRow, { html: tmp_exten });
                            }
                        }

                        ASTGUI.domActions.tr_addCell(newRow, { html: ASTGUI.parseContextLine.showAs(this_defaultRule) });
                        if (ttype != 'analog')
                        {
                            var tmp_movepriorities = '';

                            if (this_defaultRule_Index != 0)
                            {
                                tmp_movepriorities += "<A href=#  title='LANG794'><img src=images/arrow_up.png border=0 onclick=\"incomingRules_MiscFunctions.moveUpDown_In_context('" + defaultContext_ContextName + "','" + this_defaultRule + "', 1)\"></A>&nbsp; ";
                            } else
                            {
                                tmp_movepriorities += '<img src=images/arrow_blank.png border=0>&nbsp;';
                            }

                            if (this_defaultRule_Index != (tmp_defaultContext_length - 1))
                            {
                                tmp_movepriorities += "<A href=# title='LANG795'><img src=images/arrow_down.png border=0 onclick=\"incomingRules_MiscFunctions.moveUpDown_In_context('" + defaultContext_ContextName + "','" + this_defaultRule + "', 0)\"></A>";
                            } else
                            {
                                tmp_movepriorities += '<img src=images/arrow_blank.png border=0>&nbsp;';
                            }

                            ASTGUI.domActions.tr_addCell(newRow, { html: tmp_movepriorities });
                        }

                        var tmp_editstr = [];
                        tmp_editstr[0] = "<button class='options edit' onclick=\"showEditForm(this)\" title=" + $P.lang("LANG738") + "></button>&nbsp;";
                        tmp_editstr[1] = "<button class='options del' onclick=\"incomingRules_MiscFunctions.delete_IR(this)\" title=" + $P.lang("LANG739") + "></button>";
                        ASTGUI.domActions.tr_addCell(newRow, { html: tmp_editstr.join(' ') });
                    });
                }
                else if (did_this_rule == 'exten=fax,1,Goto(' + ASTGUI.contexts.Fax2Email + ',s,1)')
                {
                    //do not display fax extension
                    return;
                }
                else
                {
                    var this_defaultRule = did_this_rule;
                    if (!this_defaultRule.beginsWith('exten=')) { return; } // RULES in the main DID_trunk_x
                    var tmp_exten = ASTGUI.parseContextLine.getExten(this_defaultRule);
                    if (tmp_exten == 's' && this_defaultRule.contains('ExecIf') && !this_defaultRule.contains('ExecIfTime') ) { return; }
                    /*	added by dcwei 2012-09-10	*/
                    if (this_defaultRule.contains('NoOp')) { return; }
                    if (this_defaultRule.contains('Set')) { return; }

                    // ignore blacklist: exten = _XXX,n,GosubIf($["${BLACKLIST_CHECK}" = "1"]?app-blacklist-check,s,1)
                    if (this_defaultRule.contains('GosubIf($["${BLACKLIST_CHECK}'))
                    {
                        return;
                    }

                    if ( this_defaultRule.contains('ExecIfTime') )
                    {
                        var date_buf = {};
                        rule_fetch_date(this_defaultRule, date_buf);

                        var tmp_type = (date_buf.week == '*') ? 'LANG200': 'LANG199';
                        var tmp_time = "'"+date_buf.time+"'";
                        var tmp_dest = dest_translate(date_buf.dest);
                        var new_rule = {
                            'type': tmp_type,
                            'time': tmp_time,
                            'dest': tmp_dest
                        };
                        rule_lists.push(new_rule);
                        
                        return;
                    }


                    
                    var tmp_str = 'exten=' + tmp_exten +',n,';
                    var tmp_dest = dest_translate(this_defaultRule.substring(tmp_str.length));
                    var new_rule = {
                        'type': "'--'",
                        'time': 'LANG257',
                        'dest': tmp_dest
                    };
                    rule_lists.push(new_rule);

                    var newRow = TBL.insertRow(-1);
                    $(newRow).attr('ACTUAL_LINE__CONTEXT', this_defaultRule + STRING_SEPERATOR + defaultContext_ContextName);
                    newRow.className = (PREVIOUS_SET_BGClass == 'odd') ? 'even' : 'odd';
                    ASTGUI.domActions.tr_addCell(newRow, { html: '' });
                    if (ttype != 'analog')
                    {

                        if (tmp_exten == 's')
                        {
                            ASTGUI.domActions.tr_addCell(newRow, { html: "'s'" });
                        } else if (tmp_exten == '_X.' || tmp_exten == '_x.')
                        {
                            ASTGUI.domActions.tr_addCell(newRow, { html: "'_X.' (CatchAll)" });
                        } else
                        {
                            ASTGUI.domActions.tr_addCell(newRow, { html: tmp_exten });
                        }

                    }
                    else
                    {
                        ASTGUI.domActions.tr_addCell(newRow, { html: 'ByDID' });
                    }
                    var tmp_time = '';
                    var tmp_type = '';
                    var tmp_dest = '';
                    for ( var i = 0; i < rule_lists.length; i++ )
                    {
                        var it = rule_lists[i];
                        var dest = it.dest;
                        if ( /\${EXTEN:[0-9]+}/.test(it.dest) ) {
                            var digit = it.dest.match(/\${EXTEN:([0-9]+)}/)[1];
                            dest = it.dest.replace(/\${EXTEN:[0-9]+}/, 'Strip'+digit);
                        }
                        tmp_time += "<div class='odd' locale=\"LANG565 "+it.time+"\"></div>";
                        tmp_type += "<div class='odd' locale=\"LANG565 "+it.type+"\"></div>";
                        tmp_dest += "<div class='odd' locale=\"LANG566 "+dest+"\"></div>";
                    }
                    rule_lists = [];
                    ASTGUI.domActions.tr_addCell(newRow, { html: tmp_time});
                    ASTGUI.domActions.tr_addCell(newRow, { html: tmp_type});
                    ASTGUI.domActions.tr_addCell(newRow, { html: tmp_dest});

                    var tmp_editstr = [];
                    tmp_editstr[0] = "<button class='options edit' onclick=\"showEditForm(this)\" title=" + $P.lang("LANG738") + "></button>&nbsp;";
                    tmp_editstr[1] = "<button class='options del' onclick=\"incomingRules_MiscFunctions.delete_IR(this)\" title=" + $P.lang("LANG739") + "></button>";

                    ASTGUI.domActions.tr_addCell(newRow, { html: tmp_editstr.join(' ') });
                }
            });

            var tbl_heading = document.createElement('div');
            tbl_heading.style.marginTop = '15px';
            tbl_heading.innerHTML = '<div class="trunksName">' + $P.lang('LANG83') + ' - ' + this_trunk_label + '</div>';
            var $innerhtml = $('<div class="trunksName"></div>');
            $innerhtml.attr('locale', "{0} {1} '-{2}'".format('LANG566', 'LANG83', this_trunk_label));
            $(tbl_heading).html($innerhtml);



            if (type === 'voip')
            {
                // add DID features button to voip trunk
                var $additionalBtn = $('<button></button>');
                $additionalBtn.attr('data-context', cxt_name + ASTGUI.globals.didAdditionalPostfix);
                $additionalBtn.attr('onclick', 'showDIDSettingForm(this)');
                $additionalBtn.attr('class', 'btn btn-update didAddtional');
                $additionalBtn.attr('locale', 'LANG802');
                $additionalBtn.html($P.lang('LANG802'));
                
                $(tbl_heading).append($additionalBtn);
            }


            if (_$('IR_CONTAINER')) {
                _$('IR_CONTAINER').appendChild(tbl_heading);
                _$('IR_CONTAINER').appendChild(TBL);
            }
        });
    },

    new_IR_form: function ()
    {
        isNewIR = true;
        OLDPATTERN = "";
        ASTGUI.resetTheseFields(['edit_itrl_trunk', /*	modified by dcwei 'edit_itrl_tf', */'edit_itrl_pattern', 'edit_did_pattern', 'edit_itrl_dest', 'edit_itrl_LocalDest_Pattern', 'edit_itrl_level']);
        incomingRules_MiscFunctions.enableDisablePattern();
        $('.localext_byDid').css('display', 'none');
        $('.hideOnEdit').show();
    },

    enableDisablePattern: function ()
    { // incomingRules_MiscFunctions.enableDisablePattern() ;
        var tn = _$('edit_itrl_trunk').value;
        if (!tn)
        {
            _$('edit_itrl_pattern').disabled = false;
            return;
        }
        var ttype = parent.pbx.trunks.getType(tn);
        _$('edit_itrl_pattern').disabled = (ttype == 'analog') ? true : false;
        if (ttype === 'analog')
        {
            _$('edit_itrl_pattern').value = 's';

            var tmp_value = $('#edit_itrl_dest').val();
            $('#edit_itrl_dest > [value="ByDID"]').remove();
            if ( tmp_value && tmp_value != "ByDID" )
                $('#edit_itrl_dest').attr('value', tmp_value);
            else
                $('#edit_itrl_dest').attr('value', -1);
                
            tmp_value = $('#new_itrl_tc_dest').val();
            $('#new_itrl_tc_dest > [value="ByDID"]').remove();
            if ( tmp_value && tmp_value != "ByDID" )
                $('#new_itrl_tc_dest').attr('value', tmp_value);
            else
                $('#new_itrl_tc_dest').attr('value', -1);
            // modified for bug 15808 by dcwei
            // $('#edit_itrl_dest > [value="Hangup"]').remove();
            // $('#new_itrl_tc_dest > [value="Hangup"]').remove();
            // $('#edit_itrl_dest > [value="Congestion"]').remove();
            // $('#new_itrl_tc_dest > [value="Congestion"]').remove();
            // modified end bug 15808
            $('#edit_itrl_dt').hide();
            $('#edit_itrl_prilevel').hide();
        }
        else
        {
            if ( isNewIR )
                _$('edit_itrl_pattern').value = '';
            if (!$('#edit_itrl_dest > [value="ByDID"]').length)
            {
                var tmp_value = $('#edit_itrl_dest').val();
                $('#edit_itrl_dest').append($('<option>').html($P.lang('LANG1563')).val('ByDID').attr('locale', 'LANG1563'));
                if ( tmp_value )
                    $('#edit_itrl_dest').attr('value', tmp_value);
                else
                    $('#edit_itrl_dest').attr('value', -1);
            }

            if (!$('#new_itrl_tc_dest > [value="ByDID"]').length)
            {
                var tmp_value = $('#new_itrl_tc_dest').val();
                $('#new_itrl_tc_dest').append($('<option>').html($P.lang('LANG1563')).val('ByDID').attr('locale', 'LANG1563'));
                if ( tmp_value )
                    $('#new_itrl_tc_dest').attr('value', tmp_value);
                else
                    $('#new_itrl_tc_dest').attr('value', -1);
            }

            // modified for bug 15808 by dcwei
            // if (!$('#edit_itrl_dest > [value="Hangup"]').length)
            // {
            //     $('#edit_itrl_dest').append($('<option>').html($P.lang('LANG97')).val('Hangup').attr('locale', 'LANG97'));
            // }

            // if (!$('#new_itrl_tc_dest > [value="Hangup"]').length)
            // {
            //     $('#new_itrl_tc_dest').append($('<option>').html($P.lang('LANG97')).val('Hangup').attr('locale', 'LANG97'));
            // }
            // if (!$('#edit_itrl_dest > [value="Congestion"]').length)
            // {
            //     $('#edit_itrl_dest').append($('<option>').html($P.lang('LANG578')).val('Congestion').attr('locale', 'LANG578'));
            // }

            // if (!$('#new_itrl_tc_dest > [value="Congestion"]').length)
            // {
            //     $('#new_itrl_tc_dest').append($('<option>').html($P.lang('LANG578')).val('Congestion').attr('locale', 'LANG578'));
            // }
            // modified end bug 15808
            
            $('#edit_itrl_dt').show();
            $('#edit_itrl_prilevel').show();
        }

        if ( isNewIR )
        {
            $('#new_itrl_tc_dest')[0].selectedIndex = -1;
            $('#edit_itrl_dest')[0].selectedIndex = -1;
        }
        top.Custom.init(document, $('#edit_itrl_trunk')[0]);
        top.Custom.init(document, $('#edit_itrl_dest')[0]);
        top.Custom.init(document, $('#new_itrl_tc_dest')[0]);
    },

    update_IR: function ()
    { // incomingRules_MiscFunctions.update_IR() ;
        var this_trunk = ASTGUI.getFieldValue('edit_itrl_trunk');
        //		var this_tiName = ASTGUI.getFieldValue('edit_itrl_tf') ;
        var edit_itrl_pattern_val = ASTGUI.getFieldValue('edit_itrl_pattern');
        var edit_did_pattern_val = ASTGUI.getFieldValue('edit_did_pattern');
        var TMP_NEW_PATTERN = edit_itrl_pattern_val;
        if (edit_did_pattern_val) {
            TMP_NEW_PATTERN += "/" + edit_did_pattern_val;
        }
        var trunks_info = parent.sessionData.pbxinfo.trunks_info[this_trunk];
        if ((trunks_info && trunks_info.trunkstyle == "voip") || (edit_itrl_pattern_val !== 's' && !(TMP_NEW_PATTERN.indexOf('_') == 0)))
        {
            /*	added '_' to pattern automatically when needed	*/
            if (!/^_/.test(TMP_NEW_PATTERN)) {
                TMP_NEW_PATTERN = '_' + TMP_NEW_PATTERN;
            }
        }

//        var dest = $('#edit_itrl_dest').val();
        var dest = '';
        var dest_idx = $('#edit_itrl_dest').val();
        if ( /^[0-9][1-9]*$/.test(dest_idx) ) 
        {
            var dest_val = $('#edit_itrl_dest_val').val();
            dest = dest_dialplan_list[dest_idx].replace(/\${args}/, dest_val);
        } else {
            dest = dest_idx;
        }
        
        var local_dest = ASTGUI.getFieldValue('edit_itrl_LocalDest_Pattern');
        //		var time_int_name = $('#edit_itrl_tf option:selected').text();
        var time_int_name = ''; 	
        var trunk_level = _$('edit_itrl_level').value;

        if (isNewIR == true)
        { // create new Incoming Rule
            var levelVal = $("#edit_itrl_level").val();
            if (levelVal && levelVal != "internal") {
                top.dialog.dialogConfirm({
                    type: "warning",
                    confirmStr: $P.lang("LANG2537").format($P.lang("LANG1071"), $P.lang("LANG658")),
                    buttons: {
                        ok:  function (){
                            top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826')});

                            if (parent.pbx.trunks.rules.add({ trunk: this_trunk, name: time_int_name, time_interval: time_int_name, dest: dest, pattern: TMP_NEW_PATTERN, digits: local_dest, time_condition: OrderTable, level: trunk_level }))
                            {
                                top.dialog.clearDialog();
                                top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG815"), callback: function () { mWindow.location.reload(); } });
                            }
                            else
                            {
                                top.dialog.clearDialog();
                                top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG862") });
                            }
                        },
                        cancel:  function (){
                            top.dialog.container.show();
                            top.dialog.shadeDiv.show();
                        }
                    }
                });
            }
            else {
                top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826')});

                if (parent.pbx.trunks.rules.add({ trunk: this_trunk, name: time_int_name, time_interval: time_int_name, dest: dest, pattern: TMP_NEW_PATTERN, digits: local_dest, time_condition: OrderTable, level: trunk_level }))
                {
                    top.dialog.clearDialog();
                    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG815"), callback: function () { mWindow.location.reload(); } });
                }
                else
                {
                    top.dialog.clearDialog();
                    top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG862") });
                }
            }
        }
        else
        {
            var levelVal = $("#edit_itrl_level").val();
            if (levelVal && levelVal != "internal") {
                top.dialog.dialogConfirm({
                    type: "warning",
                    confirmStr: $P.lang("LANG2537").format($P.lang("LANG1071"), $P.lang("LANG658")),
                    buttons: {
                        ok:  function (){
                            // edit/update existing incoming rule
                            var resp = parent.pbx.trunks.rules.edit({
                                line: EDIT_CONTEXT_IR_LINE,
                                dest: dest,
                                digits: local_dest,
                                cxt: EDIT_CONTEXT_IR,
                                pattern: TMP_NEW_PATTERN,
                                oldpattern: OLDPATTERN,
                                time_condition: OrderTable,
                                level: trunk_level
                            });
                            if (resp)
                            {
                                top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG815"), callback: function () { mWindow.location.reload(); } });
                            }
                        },
                        cancel:  function (){
                            top.dialog.container.show();
                            top.dialog.shadeDiv.show();
                        }
                    }
                }); 
            }
            else {
                // edit/update existing incoming rule
                var resp = parent.pbx.trunks.rules.edit({
                    line: EDIT_CONTEXT_IR_LINE,
                    dest: dest,
                    digits: local_dest,
                    cxt: EDIT_CONTEXT_IR,
                    pattern: TMP_NEW_PATTERN,
                    oldpattern: OLDPATTERN,
                    time_condition: OrderTable,
                    level: trunk_level
                });
                if (resp)
                {
                    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG815"), callback: function () { mWindow.location.reload(); } });
                }
            }
        }
    }
};

var OrderTable = {
    "lists": [],
    "table_id": '',
    "operater_switch": true,
    "newCell": [],
    "click_val" : null,
    "bind_event": function () {
        var TBL = _$(this.table_id);

        $(TBL).on('click', '.table-add', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }

            if (OrderTable.max_failover > 0 && OrderTable.max_failover <= OrderTable.lists.length)
            {
                var target = $(this);
                // XXX it doesn't work until I set tooltip position twice.
                $(this).tooltip({ position: {
                         my: "right center", 
                         at: "left center" 
                     }
                });
                    
                var max_msg = $P.lang("LANG2141").format(OrderTable.max_failover);
                $(this).attr("titles", max_msg).addClass("ui-state-highlight").trigger("focusin");
                setTimeout( function () {
                   target.removeAttr("titles").removeClass("ui-state-highlight").trigger("focusout");

                }, 2000);
                return;
            }

            append_time_value();

            var clickRow = $(this).parent().parent();
            var newCell = document.createElement('tr');
            OrderTable.newCell = $(newCell);
            $(newCell).css('background', '#CAECFF');
            newCell.id = 'new_time_condition';

            var cell = document.createElement('td');
            $(cell).attr('colspan', 4);
            newCell.appendChild(cell);


            //new add div
            var div = document.createElement('div');
            $(div).attr('class', 'testdiv');
            cell.appendChild(div);
            $('#table_edit_template').appendTo($(div));


            clickRow.after(newCell);
            $('#modal_btn').css('display', 'none');
            $('#table-add-btn').show();
            $('#table-edit-btn').hide();

            //default edit value
            OrderTable.set_edit_values(-1);

            if (clickRow.attr('data-item-val') === 'add_item')
            {
                clickRow.hide();
            }

            //$('#table-add-btn').show();
            OrderTable.click_val = $(this).parent().parent().attr('data-item-val');
            $("#table_edit_template input[type=checkbox]:checked").each(function(index) {
                this.checked = false;
                this.previousSibling.style.backgroundPosition = "0 0";
            });
            $("#table_edit_template input[type=radio]").each(function(index) {
                if (!this.checked) {
                    this.previousSibling.style.backgroundPosition = "0 0";
                }
            });
            top.Custom.init(document, $('#table_edit_template')[0]);
            $("#new_itrl_stime_hour").change(function () {
                if (!$("#new_itrl_stime_minute").val()) {
                    $("#new_itrl_stime_minute").val("00");
                    top.Custom.init(document, $('#new_itrl_stime_minute')[0]);
                }
            });
            $("#new_itrl_ftime_hour").change(function () {
                if (!$("#new_itrl_ftime_minute").val()) {
                    $("#new_itrl_ftime_minute").val("00");
                    top.Custom.init(document, $('#new_itrl_ftime_minute')[0]);
                }
            });
            //reset innerhtml height
            if (window.frameElement)
            {
                $(window.frameElement).css("height","0px");
            }
            //top.dialog.repositionDialog("none");
            top.dialog.repositionDialog();

            OrderTable.operater_switch = 'add';
            //TODO
            // disable operator -- add/edit/up/down...
        });


        $(TBL).on('click', '.table-del', function (ev) {

            if (OrderTable.operater_switch !== true)
            {
                return false;
            }

            var clickRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('tc_index', clickRow.attr('data-item-val'));

            //remove select item from lists
            OrderTable.lists.splice(curIdx, 1);

            OrderTable.refresh_timecondition();


            //reset innerhtml height
            if (window.frameElement)
            {
                $(window.frameElement).css("height","0px");
            }
            top.dialog.repositionDialog("none");
        });


        $(TBL).on('click', '.table-edit', function (ev) {

            if (OrderTable.operater_switch !== true)
            {
                return false;
            }
            var clickRow = $(this).parent().parent();
            var newCell = document.createElement('tr');
            OrderTable.newCell = $(newCell);
            $(newCell).css('background', '#CAECFF');

            var cell = document.createElement('td');
            $(cell).attr('colspan', 4);
            newCell.appendChild(cell);
            var curIdx = OrderTable.getOptionOrder('tc_index', clickRow.attr('data-item-val'));
            OrderTable.set_edit_values(curIdx);

            //new add div
            var div = document.createElement('div');
            $(div).attr('class', 'testdiv');
            cell.appendChild(div);
            $('#table_edit_template').appendTo($(div));

            clickRow.after(newCell);
            $('#table-add-btn').hide();
            $('#table-edit-btn').show();
            $('#modal_btn').css('display', 'none');
            clickRow.hide();

            OrderTable.click_val = $(this).parent().parent().attr('data-item-val');

            //reset innerhtml height
            if (window.frameElement)
            {
                $(window.frameElement).css("height","0px");
            }
            top.dialog.repositionDialog();
            OrderTable.operater_switch = 'edit';
            top.Custom.init(document);
        });


        $(TBL).on('click', '.table-up', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }

            var currentRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('tc_index', currentRow.attr('data-item-val'));
            if (curIdx <= 0)
            {
                return;
            }

            var prevRow = currentRow.prev();
            var prevIdx = OrderTable.getOptionOrder('tc_index', prevRow.attr('data-item-val'));
            var lists = OrderTable.lists;
            var tmp = lists[curIdx];
            lists[curIdx] = lists[prevIdx];
            lists[prevIdx] = tmp;

            OrderTable.refresh_timecondition();
        });

        $(TBL).on('click', '.table-down', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }
            var currentRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('tc_index', currentRow.attr('data-item-val'));
            var last = OrderTable.lists.length - 1;
            if (curIdx >= last)
            {
                return;
            }

            var nextRow = currentRow.next();
            var nextIdx = OrderTable.getOptionOrder('tc_index', nextRow.attr('data-item-val'));
            var lists = OrderTable.lists;
            var tmp = lists[curIdx];
            lists[curIdx] = lists[nextIdx];
            lists[nextIdx] = tmp;

            OrderTable.refresh_timecondition();
        });

        $(TBL).on('click', '.table-top', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }
            var currentRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('tc_index', currentRow.attr('data-item-val'));
            if (curIdx <= 0)
            {
                return;
            }

            var item = OrderTable.lists[curIdx];
            OrderTable.lists.splice(curIdx, 1);
            OrderTable.lists.splice(0, 0, item);

            OrderTable.refresh_timecondition();
        });


        $(TBL).on('click', '.table-bottom', function (ev) {
            if (OrderTable.operater_switch !== true)
            {
                return false;
            }
            var currentRow = $(this).parent().parent();
            var curIdx = OrderTable.getOptionOrder('tc_index', currentRow.attr('data-item-val'));
            var lastIdx = OrderTable.lists.length - 1;
            if (curIdx >= lastIdx)
            {
                return;
            }

            var item = OrderTable.lists[curIdx];
            OrderTable.lists.splice(curIdx, 1);
            OrderTable.lists.splice(lastIdx, 0, item);

            OrderTable.refresh_timecondition();
        });

    },
    "getOptionOrder": function (key, val) {
        var len = this.lists.length;
        for ( var i = 0; i < len; i++)
        {
            if (this.lists[i][key] === val)
            {
                return i;
            }
        }

        return -1;
    },
    "tbl_btn_cancel": function (ev) {
        $('#table_edit_template').appendTo($('#table_template_placeholder'));
        $('#modal_btn').css('display', 'block');
        $('[data-item-val="' + this.click_val + '"]').show();

        this.newCell.remove();
        this.newCell = [];

        //reset innerhtml height
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog();
        OrderTable.operater_switch = true;
        OrderTable.click_val = null;
    },
    "tbl_btn_add": function (ev) {
        var stime_hour = _$('new_itrl_stime_hour').value;
        var stime_minute = _$('new_itrl_stime_minute').value;
        var ftime_hour = _$('new_itrl_ftime_hour').value;
        var ftime_minute = _$('new_itrl_ftime_minute').value;

        if ( stime_hour == "" && stime_minute == ""
            && ftime_hour == "" && ftime_minute == "" )
            var time = "00:00-23:59";
        else
            var time = stime_hour + ':' + stime_minute + '-' + ftime_hour + ':' + ftime_minute;
        
        var date_add = function (container, key, el_id)
        {
            el = document.getElementById(el_id);
            if ( el.checked == false )
                return;

            var offset = el_id.indexOf(key) + key.length + 1;
            var value = el_id.substring(offset);
            if ( container['value'] == '' )
            {
                container['value'] = value;
            } else
            {
                container['value'] = container['value'] + '&' + value;
            }
        }
        // week
        var week = {value: ''};
        if ( _$('new_itrl_date_week').checked == false )
        {
            week.value = '*';
        } else
        {
            date_add(week, 'week', 'new_itrl_week_sun');
            date_add(week, 'week', 'new_itrl_week_mon');
            date_add(week, 'week', 'new_itrl_week_tue');
            date_add(week, 'week', 'new_itrl_week_wed');
            date_add(week, 'week', 'new_itrl_week_thu');
            date_add(week, 'week', 'new_itrl_week_fri');
            date_add(week, 'week', 'new_itrl_week_sat');
        }

        /*  month  */ 
        var month = {value: ''};
        var day = {value: ''};
        if ( _$('new_itrl_date_month').checked == false )
        {
            month.value = '*';
            day.value = '*';
        } else
        {
            date_add(month, 'month', 'new_itrl_month_jan');
            date_add(month, 'month', 'new_itrl_month_feb');
            date_add(month, 'month', 'new_itrl_month_mar');
            date_add(month, 'month', 'new_itrl_month_apr');
            date_add(month, 'month', 'new_itrl_month_may');
            date_add(month, 'month', 'new_itrl_month_jun');
            date_add(month, 'month', 'new_itrl_month_jul');
            date_add(month, 'month', 'new_itrl_month_aug');
            date_add(month, 'month', 'new_itrl_month_sep');
            date_add(month, 'month', 'new_itrl_month_oct');
            date_add(month, 'month', 'new_itrl_month_nov');
            date_add(month, 'month', 'new_itrl_month_dec');

            var tmp_days = [];
            $('.day_selected').each( function (item) {
                var day_number = $(this).attr('id').afterChar('_');
                tmp_days.push(day_number);
            });
            day.value = tmp_days.join('&');

        }



        $('#table_edit_template').appendTo($('#table_template_placeholder'));
        $('#modal_btn').css('display', 'block');

        /* time condition destination */
//        var tc_dest = _$('new_itrl_tc_dest').value;
//        var tc_digits = _$('new_itrl_tc_digits').value;
        var tc_dest = '';
        var tc_dest_idx = $('#new_itrl_tc_dest').val();
        if ( /^[0-9][1-9]*$/.test(tc_dest_idx) ) 
        {
            var tc_dest_val = $('#new_itrl_tc_dest_val').val();
            tc_dest = dest_dialplan_list[parseInt(tc_dest_idx)].replace(/\${args}/, tc_dest_val);
        } else {
            tc_dest = tc_dest_idx;
        }

        var tc_digits = _$('new_itrl_tc_digits').value || '';
        if ( tc_digits == '' )
        {
            tc_digits = '0';
        }


        /*  display time condition  */
        this.newCell = [];
        var order = this.getOptionOrder('tc_index', this.click_val);
        order = parseInt(order, 10);
        /*  time condition object  */
        var newTimeCondition = {
            'tc_index': (++order).toString(),
            'tc_time': time,
            'tc_week': week['value'],
            'tc_month': month['value'],
            'tc_day': day['value'],
            'tc_dest': tc_dest,
            'tc_digits': tc_digits
        };
        this.lists.splice(order, 0, newTimeCondition);
        this.refresh_timecondition();

        // reset innerhtml height
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog("none");
        OrderTable.operater_switch = true;
        OrderTable.click_val = null;
    },
    "tbl_btn_update": function (ev) {      
        var order = this.getOptionOrder('tc_index', this.click_val);
        var stime_hour = _$('new_itrl_stime_hour').value;
        var stime_minute = _$('new_itrl_stime_minute').value;
        var ftime_hour = _$('new_itrl_ftime_hour').value;
        var ftime_minute = _$('new_itrl_ftime_minute').value;

        var time = stime_hour + ':' + stime_minute + '-' + ftime_hour + ':' + ftime_minute;
        this.lists[order]['tc_time'] = time;
        
        var date_add = function (container, key, el_id)
        {
            el = document.getElementById(el_id);
            if ( el.checked == false )
                return;

            var offset = el_id.indexOf(key) + key.length + 1;
            var value = el_id.substring(offset);
            if ( container['value'] == '' )
            {
                container['value'] = value;
            } else
            {
                container['value'] = container['value'] + '&' + value;
            }
        } 
        // week
        var week = {value: ''};
        if ( _$('new_itrl_date_week').checked == false )
        {
            week.value = '*';
        } else
        {
            date_add(week, 'week', 'new_itrl_week_sun');
            date_add(week, 'week', 'new_itrl_week_mon');
            date_add(week, 'week', 'new_itrl_week_tue');
            date_add(week, 'week', 'new_itrl_week_wed');
            date_add(week, 'week', 'new_itrl_week_thu');
            date_add(week, 'week', 'new_itrl_week_fri');
            date_add(week, 'week', 'new_itrl_week_sat');
        }
        this.lists[order]['tc_week'] = week.value;

        /*  month  */ 
        var month = {value: ''};
        var day = {value: ''};
        if ( _$('new_itrl_date_month').checked == false )
        {
            month.value = '*';
            day.value = '*';
        } else
        {
            date_add(month, 'month', 'new_itrl_month_jan');
            date_add(month, 'month', 'new_itrl_month_feb');
            date_add(month, 'month', 'new_itrl_month_mar');
            date_add(month, 'month', 'new_itrl_month_apr');
            date_add(month, 'month', 'new_itrl_month_may');
            date_add(month, 'month', 'new_itrl_month_jun');
            date_add(month, 'month', 'new_itrl_month_jul');
            date_add(month, 'month', 'new_itrl_month_aug');
            date_add(month, 'month', 'new_itrl_month_sep');
            date_add(month, 'month', 'new_itrl_month_oct');
            date_add(month, 'month', 'new_itrl_month_nov');
            date_add(month, 'month', 'new_itrl_month_dec');
            var tmp_days = [];
            $('.day_selected').each( function (item) {
                var day_number = $(this).attr('id').afterChar('_');
                tmp_days.push(day_number);
            });
            day.value = tmp_days.join('&');
        }
        this.lists[order]['tc_month'] = month.value;
        this.lists[order]['tc_day'] = day.value;


        /* time condition destination */
        var tc_dest = '';
        var tc_dest_idx = $('#new_itrl_tc_dest').val();
        if ( /^[0-9][1-9]*$/.test(tc_dest_idx) ) 
        {
            var tc_dest_val = $('#new_itrl_tc_dest_val').val();
            tc_dest = dest_dialplan_list[parseInt(tc_dest_idx)].replace(/\${args}/, tc_dest_val);
        } else {
            tc_dest = tc_dest_idx;
        }

        var tc_digits = _$('new_itrl_tc_digits').value;
        if ( tc_digits == '' )
        {
            tc_digits = '0';
        }
        this.lists[order]['tc_dest'] = tc_dest;
        this.lists[order]['tc_digits'] = tc_digits;
        $('#table_edit_template').appendTo($('#table_template_placeholder'));
        $('#modal_btn').css('display', 'block');

        //this.newCell.remove();
        this.newCell = [];

        this.refresh_timecondition();
        
        //reset innerhtml height
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog("none");
        OrderTable.operater_switch = true;
        OrderTable.click_val = null;
    },
    "refresh_timecondition": function () {
        var TBL = _$(this.table_id);
        $(TBL).children().remove();
        var addCell = ASTGUI.domActions.tr_addCell;
        (function ()
        { // add first row
            var newRow = TBL.insertRow(-1);
            newRow.className = "frow";
            addCell(newRow, { html: $P.lang('LANG247'), locale: 'LANG247' });
            addCell(newRow, { html: $P.lang('LANG168'), locale: 'LANG168' });
            addCell(newRow, { html: $P.lang('LANG74'), locale: 'LANG74' });
        })();

        var len = this.lists.length;
        if (len <= 0)     
        {
            var newRow = TBL.insertRow(-1);
            newRow.className = "odd";
            var td = $('<td colspan=3 />');
            $(newRow).append(td);
            $(newRow).attr('data-item-val', 'add_item');
            $(td).append('<span class="table-add">' + $P.lang('LANG1562') +'</span>');
            return;
        }

        //show item list
        for (var i = 0; i < len; i++)
        {
            var item = this.lists[i];
            var newRow = TBL.insertRow(-1);
            newRow.className = ((TBL.rows.length) % 2 == 1) ? 'odd' : 'even';
            addCell(newRow, { html: item.tc_time || '' });
            var tmp_dest = dest_translate(item.tc_dest);
//            var dest_val = item.tc_dest_val;
            if ( /^LANG[0-9]+/.test(tmp_dest) )
            {
                var lang = tmp_dest.match(/(LANG[1-9][0-9]*)/)[1];
                if ( /\${EXTEN:[0-9]+}/.test(tmp_dest) ) {
                    var tmp_str = 'Strip ' + tmp_dest.match(/\${EXTEN:([0-9]+)}/)[1];
                }
                else {                                                               
                    if(tmp_dest.match(/(LANG[1-9][0-9]*) '-- ([\w\*0-9]+)'/)) {                
                        tmp_str = tmp_dest.match(/(LANG[1-9][0-9]*) '-- ([\w\*0-9]+)'/)[2];    
                    }                                                                          
                    else {                                                                     
                        tmp_str = tmp_dest.split("'")[1];                                      
                    }                                                                          
                } 
                var innerHTML = $P.lang(lang) + '--' + tmp_str;
                addCell(newRow, { html: "<div locale=\"LANG566 "+tmp_dest+"\">"+innerHTML+"</div>"});
            } else {
                addCell(newRow, { html: tmp_dest});
            }
            //addCell(newRow, { html: "<div locale=\"LANG566 "+tmp_dest+"\">"+"$P.lang(\""+lang+"\") -- "+tmp_str+"</div>"});
            var newCell = newRow.insertCell(newRow.cells.length);
            var $tmp;
            //add icon
            $tmp = $('<span />').addClass('table-add tableIcon table-add-icon').attr('localeTitle', 'LANG769').attr('title', $P.lang('LANG769')).appendTo(newCell);
            //top,up,down,bottom icons
            $tmp = $('<span />').addClass('table-top clickTop tableIcon').attr('localeTitle', 'LANG793').attr('title', $P.lang('LANG793')).appendTo(newCell);
            $tmp = $('<span />').addClass('table-up clickUp tableIcon').attr('localeTitle', 'LANG794').attr('title', $P.lang('LANG794')).appendTo(newCell);
            $tmp = $('<span />').addClass('table-down clickDown tableIcon').attr('localeTitle', 'LANG795').attr('title', $P.lang('LANG795')).appendTo(newCell);
            $tmp = $('<span />').addClass('table-bottom clickBottom tableIcon').attr('localeTitle', 'LANG796').attr('title', $P.lang('LANG796')).appendTo(newCell);
            //empty icon 
            //$tmp = $('<span />').addClass('tableIcon').appendTo(newCell);
            //edit, delete icons
            $tmp = $('<span />').attr('title', $P.lang('LANG738')).attr('localeTitle', 'LANG738').addClass('table-edit tableIcon').appendTo(newCell);
            $tmp = $('<span />').attr('title', $P.lang('LANG739')).attr('localeTitle', 'LANG739').addClass('table-del tableIcon').appendTo(newCell);
            $(newRow).attr('data-item-val', item.tc_index);
        }
        $('.table-add').tooltip({ position: {
             my: "right center", 
             at: "left center" 
             }
        });
    },
    "set_edit_values": function (idx) {
        $(".chk_week").attr("checked", false);
        $("#new_itrl_week_all").attr("checked", false);
        $(".chk_month").attr("checked", false);
        $("#new_itrl_month_all").attr("checked", false);
        var set_chkbox_all = function(type, value)
        {
            var str = '#new_itrl_'+type+'_container';
            var container = $(str).children();
            for ( var i = 0; i < container.length; i++ )
            {
                var it = container[i];
                if ( it.type == 'checkbox' )
                {
                    it.checked = value;
                }
            }
        }

        var set_selected_chkbox = function(str, type, value)
        {
            var template = 'new_itrl_' + type + '_';
            if(str.split("&").length == 7) {
                $("#new_itrl_week_all").attr("checked", true); 
            }
            if (str.split("&").length == 12) {
                $("#new_itrl_month_all").attr("checked", true);
            }
            while ( str.indexOf('&') != -1 )
            {
                var offset = str.indexOf('&');
                var sstr = str.substring(0, offset);
                var chkbox = document.getElementById(template+sstr);
                if ( chkbox )
                {
                    chkbox.checked = value;
                }
                $('day_' + sstr).attr('class', 'day_selected');
                str = str.substring(offset+1);
            }
            if ( str != '' )
            {
                var chkbox = document.getElementById(template+str);
                if ( chkbox )
                {
                    chkbox.checked = value;
                }
            }
        }

        if ( idx == -1 )
        {
            ASTGUI.updateFieldToValue('new_itrl_tc_dest', '');
            ASTGUI.updateFieldToValue('new_itrl_tc_digits', '');
            ASTGUI.updateFieldToValue('new_itrl_tc_dest_val', '');
            ASTGUI.updateFieldToValue('new_itrl_stime_hour', '');
            ASTGUI.updateFieldToValue('new_itrl_stime_minute', '');
            ASTGUI.updateFieldToValue('new_itrl_ftime_hour', '');
            ASTGUI.updateFieldToValue('new_itrl_ftime_minute', '');

            _$('new_itrl_date_week').checked = true;
            $('#new_itrl_date_week').trigger('click');
            _$('new_itrl_date_month').checked = false;
            set_chkbox_all('week', false);
            set_chkbox_all('month', false);
            
            //var day = $('#new_itrl_day_container');
            //$(day).children().remove();
            $('#new_itrl_day_container td').removeClass('day_selected').addClass('day_unselected');
            return;
        }

        var item = this.lists[idx] || {};
        var tmp_str = item.tc_time;

        /*  time    */
        var offset1 = tmp_str.indexOf(':');
        ASTGUI.updateFieldToValue('new_itrl_stime_hour', tmp_str.substring(0, offset1));
        var offset2 = tmp_str.indexOf('-');
        ASTGUI.updateFieldToValue('new_itrl_stime_minute', tmp_str.substring(offset1+1, offset2));
        tmp_str = tmp_str.substring(offset2+1);
        offset1 = tmp_str.indexOf(':');
        ASTGUI.updateFieldToValue('new_itrl_ftime_hour', tmp_str.substring(0, offset1));
        ASTGUI.updateFieldToValue('new_itrl_ftime_minute', tmp_str.substring(offset1+1));

        /*  week    */
        tmp_str = item.tc_week;
        set_chkbox_all('week', false);
        if ( tmp_str == '*' ) 
        {
            _$('new_itrl_date_week').checked = false;
            $('.local_week').hide();
        } else 
        {
            _$('new_itrl_date_week').checked = true;
            $('.local_week').show();
            set_selected_chkbox(tmp_str, 'week', true);
        }
        /*  month   */
        tmp_str = item.tc_month;
        set_chkbox_all('month', false);
        if ( tmp_str == '*' ) 
        {
            _$('new_itrl_date_month').checked = false;
            $('.local_month').hide();
        } else 
        {
            _$('new_itrl_date_month').checked = true;
            $('.local_month').show();
            set_selected_chkbox(tmp_str, 'month', true);
        }
        $('#new_itrl_month_container').trigger('click');
        /*  day */
        
        var chkbox_all = function(value)
        {
            var i = 1;
            var t = document.getElementById('new_itrl_day_'+i);
            while ( t ) 
            {
                t.checked = value;
                i++;
                t = document.getElementById('new_itrl_day_'+i);
            }
        }
        var chkbox_selected = function(str, value)
        {
            $(".day_selected").removeClass("day_selected");
            str.split('&').each( function (item) {
                $('#day_' + item).attr('class', 'day_selected');
            });
        }
        tmp_str = item.tc_day;
        chkbox_all(false);
        if ( tmp_str == '*' ) 
        {
            $('.local_day').hide();
            $('#new_itrl_day_container').hide();
        } else 
        {
            $('.local_day').show();
            $('#new_itrl_day_container').show();
            chkbox_selected(tmp_str, true);
        }

        /*  time condition destination  */
        if ( /^Goto\(ext-did-/.test(item.tc_dest) )
        {
            ASTGUI.updateFieldToValue('new_itrl_tc_dest', 'ByDID');
            var digits = item.tc_dest.match(/^Goto\(ext-did-[0-9]+-did-additional,\${EXTEN:([0-9]+)},1\)$/)[1];
            ASTGUI.updateFieldToValue('new_itrl_tc_digits', digits);
            $('#new_itrl_tc_dest').trigger('change');
        }
        else
        {
        //    ASTGUI.updateFieldToValue('new_itrl_tc_dest', item.tc_dest || '');
        //    $('#new_itrl_tc_dest').trigger('change');
            trans_dest('new_itrl_tc_dest', item.tc_dest);
        }
    }
};

var append_time_value = function(type)
{
    var shour = $('#new_itrl_stime_hour');
    var fhour = $('#new_itrl_ftime_hour');
    shour.children().remove();
    fhour.children().remove();
    for ( var i = 0; i < 24; i++ )
    {
        var value = '';
        if ( i < 10 )
            value = '0' + i;
        else
            value = '' + i;
        shour.append($('<option>').html(value).val(value));
        fhour.append($('<option>').html(value).val(value));
    }

    var sminute = $('#new_itrl_stime_minute');
    var fminute = $('#new_itrl_ftime_minute');
    sminute.children().remove();
    fminute.children().remove();
    for ( var i = 0; i < 60; i++ )
    {
        var value = '';
        if ( i < 10 )
            value = '0' + i;
        else
            value = '' + i;
        sminute.append($('<option>').html(value).val(value));
        fminute.append($('<option>').html(value).val(value));
    }
    if (type == "add") {
        shour[0].selectedIndex = -1;
        fhour[0].selectedIndex = -1;
        sminute[0].selectedIndex = -1;
        fminute[0].selectedIndex = -1;
    }

}

var is_container_empty = function(type)
{
    var res = true;
    var container = '#new_itrl_' + type + '_container';
    var el = $(container).children();
    for ( var i = 0; el && i < el.length-1; i++ )
    {
        if ( el[i].checked )
        {
            res = false;
            break;
        }
    }
    return res;
}

var check_date_week = function(element) {
    if ( is_container_empty('week') )
        return false;
    return true;
}

var check_date_month = function(element)
{
    if ( is_container_empty('month') )
        return false;
    
    var res = false;
    var template = 'new_itrl_day_';
    for ( var i = 1; i <= 31; i++ )
    {
        var str = template + i;
        var el = document.getElementById(str);
        if ( el && el.checked )
        {
            res = true;
            break;
        }
    }
    return res;
}

var check_date = function(element)
{
    var week = _$('new_itrl_date_week');
    if ( week && week.checked )
    {
        return check_date_week(week);
    }

    var month = _$('new_itrl_date_month');
    if ( month && month.checked )
    {
        return check_date_month(month);
    }

    return false;
}

var check_time = function(value, element) 
{
    
    var stime_hour = parseInt( $('#new_itrl_stime_hour').val() || -1 , 10);
;
    var stime_min = parseInt( $('#new_itrl_stime_minute').val() || -1 , 10);
    var ftime_hour = parseInt( $('#new_itrl_ftime_hour').val() || -1 , 10);
    var ftime_min = parseInt( $('#new_itrl_ftime_minute').val() || -1 , 10 );


    if ( stime_hour == -1 && stime_min == -1 
        && ftime_hour == -1 && ftime_min == -1)
    {
        return true;
    }

    if ( stime_hour > -1 && stime_min > -1 
        && ftime_hour > -1 && ftime_min > -1
        && stime_hour * 60 + stime_min <= ftime_hour * 60 + ftime_min) 
    {
        return true;
    }

    return false;
}

var rule_fetch_date = function( str, buffer )
{
    var sstr = str.substring(str.indexOf('ExecIfTime'));
    var offset1 = sstr.indexOf('(');
    var offset2 = sstr.indexOf(',');
    buffer['time'] = sstr.substring(offset1+1, offset2);

    sstr = sstr.substring(offset2+1);
    offset1 = sstr.indexOf(',');
    buffer['week'] = sstr.substring(0, offset1);

    offset2 = sstr.lastIndexOf(')');
    offset1 = sstr.lastIndexOf('?')+1;
    buffer['dest'] = sstr.substring(offset1, offset2);
}

var dest_translate = function(str)
{
    var resp = '';
    var offset = '';
    var offset1 = '';
    
    if ( str == "ByDID" )
    {
        var digits = $('#new_itrl_tc_digits').val();
        if ( digits == "" )
            digits = '0';
        str = "Goto(ext-did-0-did-additional,${EXTEN:" + digits + "},1)";
    }

    if ( /^Goto\(/.test(str) )
    {
        var pattern;
        var exten = "";
        if ( /^Goto\(ext-local,/.test(str) )
        {
            pattern = RegExp('Goto\\\(ext-local,([0-9]+),1\\\)');
            var matchs = str.match(pattern);
            if (matchs) {
                exten = matchs[1];
            }
            if ( parent.sessionData.pbxinfo.users.getOwnProperties().contains(exten) )
                resp = "LANG248 '-- "+exten+"'";
            else
                resp = "LANG248 '-- "+exten+" has been removed'";
        } else if ( /^Goto\(ext-conference,/.test(str) )
        {
            pattern = RegExp('Goto\\\(ext-conference,([0-9]+),1\\\)');
            //exten = str.match(pattern)[1];
            var matchs = str.match(pattern);
            if (matchs) {
                exten = matchs[1];
            }
            if ( parent.sessionData.pbxinfo.conferences.getOwnProperties().contains(exten) )
                resp = "LANG98 '-- "+exten+"'";
            else
                resp = "LANG98 '-- "+exten+" has been removed'";
        } else if ( /^Goto\(ext-queues,/.test(str) )
        {
            pattern = RegExp('Goto\\\(ext-queues,([0-9]+),1\\\)');
            //exten = str.match(pattern)[1];
            var matchs = str.match(pattern);
            if (matchs) {
                exten = matchs[1];
            }
            if ( parent.sessionData.pbxinfo.queues.exten.getOwnProperties().contains(exten) )
                resp = "LANG91 '-- "+exten+"'";
            else
                resp = "LANG91 '-- "+exten+" has been removed'";
        } else if ( /^Goto\(ivr-/.test(str) )
        {
            pattern = RegExp('Goto\\\((ivr-[0-9]+),s,1\\\)');
            var matchs = str.match(pattern);
            var ivr_idx = "";
            if (matchs) {
                ivr_idx = matchs[1];
            }
            if ( parent.sessionData.pbxinfo.voicemenus.getOwnProperties().contains(ivr_idx) )
            {
                exten = parent.sessionData.pbxinfo.voicemenus[ivr_idx]['alias_exten'];
                resp = "LANG134 '-- "+exten+"'";
            } else {
                resp = "LANG134 '-- " + ivr_idx + " has been removed'";
            }
        } else if ( /^Goto\(ext-group,/.test(str) )
        {
            pattern = RegExp('Goto\\\(ext-group,([0-9]+),1\\\)');
            //var exten = str.match(pattern)[1];
            var matchs = str.match(pattern);
            if (matchs) {
                exten = matchs[1];
            }
            if ( parent.sessionData.pbxinfo.ringgroups.getOwnProperties().contains(exten) ) {
                var name = parent.sessionData.pbxinfo.ringgroups[exten]['NAME']
                resp = "LANG600 '-- "+name+"'";
            } else
                resp = "LANG600 '-- "+exten+" has been removed'";
        } else if ( /^Goto\(voicemailgroups,/.test(str) )
        {
            /*
            var exten = tmp_str.match(/voicemailgroups,([\d]+),1/)[1] || "";
            var label = parent.sessionData.pbxinfo.vmgroups[exten]['label'] || "";
            resp = "LANG89 '--"+label+"'";
        } else if ( tmp_str.indexOf('app-dialvm') > -1 )
        {
            var exten = tmp_str.match(/app-dialvm,([\*\d]+),1/)[1] || "";
            resp = "LANG17 '--"+exten+"'";
            */
            pattern = RegExp('Goto\\\(voicemailgroups,([0-9]+),1\\\)');
            //exten = str.match(pattern)[1];
            var matchs = str.match(pattern);
            if (matchs) {
                exten = matchs[1];
            }
            if ( parent.sessionData.pbxinfo.vmgroups.getOwnProperties().contains(exten) )
                resp = "LANG89 '-- "+exten+"'";
            else
                resp = "LANG89 '-- "+exten+" has been removed'";
        } else if ( /^Goto\(ext-fax,/.test(str) )
        {
            pattern = RegExp('Goto\\\(ext-fax,([0-9]+),1\\\)');
            //var fax_idx = str.match(pattern)[1];
            //exten = fax_idx
            var matchs = str.match(pattern);
            if (matchs) {
                exten = matchs[1];
            }
            if ( parent.sessionData.pbxinfo.fax.exten.getOwnProperties().contains(exten) )
                resp = "LANG95 '-- "+exten+"'";
            else
                resp = "LANG95 '-- "+exten+" has been removed'";
        } else if ( /^Goto\(pagegroups,/.test(str) )
        {
            pattern = RegExp('Goto\\\(pagegroups,([0-9]+),1\\\)');
            //exten = str.match(pattern)[1];
            var matchs = str.match(pattern);
            if (matchs) {
                exten = matchs[1];
            }
            var rule = "exten=" + exten + ",1,Macro(pagingintercom"
            var pgroups = parent.sessionData.pbxinfo.pagegroups;
            if ( pgroups.length != 0 )
            {
                for ( var i = 0; i < pgroups.length; i++ )
                    if ( pgroups[i].contains(rule) )
                        resp = "LANG94 '-- "+exten+"'";
            }
            if ( resp == "" )
                resp = "LANG94 '-- "+exten+" has been removed'";

        }  else if ( /^Goto\(disa,/.test(str) )
        {
            //var exten = str.match(/disa,([\d]+),1/)[1] || "";
            var matchs = str.match(/disa,([\d]+),1/);
            if (matchs) {
                exten = matchs[1];
            }
            var res = parent.getDisaMember();
            res.each(function (p)
            {
                if (p.disaid == exten)
                {
                    resp = "LANG2353 '-- " + p.displayname + "'";
                }
            });
            if ( resp == "" )
               resp = "LANG2353 '-- disa-" + exten + " has been removed'";
                
        } else if ( str.contains(ASTGUI.globals.didAdditionalPostfix))
        {
            /*
            var exten = tmp_str.match(/\${EXTEN:([0-9])+}/)[1];
            resp = "LANG1563 '--Strip" + exten +"'";
            */
            offset = str.indexOf(',');
            offset1 = str.lastIndexOf(',');
            var exten = str.substring(offset+1, offset1);
            resp = "LANG1563 '-- " + exten +"'";
        }
        else
        {
            /*
            var exten = tmp_str.match(/^\(default,([a-zA-Z0-9]+),1\)$/)[1];
            if ( exten == 'o' )
                exten = "Operator";
            resp = "LANG257 '--" + exten +"'";
            */
            resp += "'" + str + "' ''";
        }
    } else if ( str.indexOf('Voicemail') == 0 )
    {
        offset = str.indexOf('(');
        offset1 = str.indexOf(',');
        var exten = str.substring(offset+1, offset1);
        if ( parent.sessionData.pbxinfo.users.getOwnProperties().contains(exten) &&
                parent.sessionData.pbxinfo.users[exten]['hasvoicemail'] == "yes" )
            resp = "LANG249 '--"+exten+"'";
        else
            resp = "LANG249 '--"+exten+" has been removed'";
    } else
    {
        //resp += "LANG257 '--" + str + "'";
        resp += "'" + str + "' ''";
    }

    return resp;
}
var check_pattern_format = function(value, element)
{
    if (/^_$/.test(value) || /(\[.*?(\.|\!).*?\])|(\[\])/g.test(value)) {
        return false;
    }
    return true;
}
var check_pattern = function(value, element)
{
    var bool = true;
    var edit_itrl_pattern_val = $("#edit_itrl_pattern").val();
    var edit_did_pattern_val = $("#edit_did_pattern").val();
    if ( !/^_/.test(value) )
        value = '_'+value;
        
    var trunk = _$('edit_itrl_trunk').value;
    if ( !trunk )
        return true;
    var idx = trunk.match(/^trunk_([1-9][0-9]*)/)[1];
    var rules = parent.sessionData.pbxinfo.incomingrules['ext-did-'+idx];
    if (OLDPATTERN) {
        var oldpattern_itrl = OLDPATTERN.split("/")[0];
        var oldpattern_did = OLDPATTERN.split("/")[1];
        if ( !/^_/.test(oldpattern_did) )
            oldpattern_did = '_' + oldpattern_did;
        if (element.id == "edit_did_pattern") {
            bool = (value != oldpattern_did);
        }
        if (element.id == "edit_itrl_pattern") {
            bool = (value != oldpattern_itrl);
        };
    }
    if ( rules && bool)
    {
        for ( var k = 0; k < rules.length; ++k )
        {
            // added '/' in the pattern for incoming CallerID Identification
            var pattern = rules[k].match(/^exten=([_0-9a-zA-Z!\[\]\-\.\?\+\*\#\/_]+),.*/)[1];
            //var pattern = rules[k].match(/^exten=(_[0-9a-zA-Z!\[\]\-\.\?\+\*\#]+),.*/)[1];
            var pattern_itrl = pattern.split("/")[0];
            var pattern_did = pattern.split("/")[1];
            if ( !/^_/.test(edit_itrl_pattern_val) )
                edit_itrl_pattern_val = '_' + edit_itrl_pattern_val;
            if (pattern_itrl == edit_itrl_pattern_val && edit_did_pattern_val && pattern_did == edit_did_pattern_val)
                return false;
        }
    }
    return true;
}
var check_pattern_with_cid = function(value, element)
{
    if ( !/^_/.test(value) )
        value = '_'+value;
    if ( /^_[0-9a-zA-Z!\[\]\-\.\?\+\*\#]+\/[0-9]+$/.test(value) )
        return true;
    else if ( !/\//.test(value) )
        return true;
    else
        return false;
}


var check_dest_val = function(value, element)
{
    var idx = $('#edit_itrl_dest').val();
    var name = DESTINATION_LIST[parseInt(idx)];

    if ( value == name+' number' )
        return false;
    
    return true;
}

var check_tc_dest_val = function(value, element)
{
    var idx = $('#new_itrl_tc_dest').val();
    var name = DESTINATION_LIST[parseInt(idx)];

    if ( value == name+' number' )
        return false;
    
    return true;
}

var trans_dest = function ( DOM_id, str )
{
    var regstr = "";
    for ( var idx = 0; idx < dest_dialplan_list.length; idx++ )
    {
        regstr = dest_dialplan_list[idx];
        regstr = regstr.match(/^(.+)\${args}.+$/)[1];
        if ( !regstr )
            continue;
        regstr = regstr.replace(/\(/, "\\\(");
        reg = RegExp(regstr);
        if ( reg.test(str) )
        {
            ASTGUI.updateFieldToValue(DOM_id, idx);
            $('#'+DOM_id).trigger('change');
            var regnum = RegExp(regstr+"([\\\d\\\w-]+)");
            var num = "";
            var matchs = str.match(regnum);
            if (matchs) {
                num = matchs[1];
            }
            ASTGUI.updateFieldToValue(DOM_id+'_val', num);
            break;
        }
    }

}

var append_destination_by_type = function ( DOM_id, type ) 
{       
    switch ( type ) {
        case 'Extension':
            var y = parent.pbx.users.list();
            y.each(function (user)
            {
                $('#'+DOM_id).append($('<option>').html(user).val(user));
            });
            top.Custom.init(document);
            break;
        case 'Voicemail':
            var y = parent.pbx.users.list();
            var dom = $('#'+DOM_id);
            y.each(function (user)
            {
                var users = parent.sessionData.pbxinfo.users;
                if (users && users.getOwnProperties().contains(user) && users[user]['hasvoicemail'] == "yes") {
                    dom.append($('<option>').html(user).val(user));
                }
            });
            top.Custom.init(document);
            break;
        case 'Conference Room':
            var y = parent.sessionData.pbxinfo.conferences.getOwnProperties();
            y.each(function (meetme)
            {
                $('#'+DOM_id).append($('<option>').html(meetme).val(meetme));
            });
            top.Custom.init(document);
            break;
        case 'Queue':
            
            var y = parent.sessionData.pbxinfo.queues.exten.getOwnProperties();
            y.each(function (q)
            {
                $('#'+DOM_id).append($('<option>').html(q).val(q));
            });
            top.Custom.init(document);
            break;
        case 'Ringgroup':
            var y = parent.sessionData.pbxinfo.ringgroups.getOwnProperties();
            y.each(function (rg)
            {
                var name = parent.parent.sessionData.pbxinfo.ringgroups[rg]['NAME'];
                $('#'+DOM_id).append($('<option>').html(name).val(rg));
            });
            top.Custom.init(document);
            break;
        case 'Page':
            var y = parent.pbx.paging.list();
            y.each(function (pge)
            {
                $('#'+DOM_id).append($('<option>').html(pge).val(pge));
            });
            top.Custom.init(document);
            break;
        case 'Vmgroup':
            var y = parent.sessionData.pbxinfo.vmgroups.getOwnProperties();
            y.each(function (this_vmg_exten)
            {
                $('#'+DOM_id).append($('<option>').html(this_vmg_exten).val(this_vmg_exten));
            });
            top.Custom.init(document);
            break;
        case 'Fax':
            var y = parent.sessionData.pbxinfo.fax.exten.getOwnProperties();
            y.each(function (q)
            {
                $('#'+DOM_id).append($('<option>').html(q).val(q));
            });
            top.Custom.init(document);
            break;
        case 'DISA':
            var y = parent.getDisaMember();
            y.each(function (q)
            {
                $('#'+DOM_id).append($('<option>').html(q.displayname).val(q.disaid));
            });
            top.Custom.init(document);
            break;
        case 'IVR':
            var y = parent.sessionData.pbxinfo.voicemenus.getOwnProperties();
            y.each(function (vmenu)
            {
                var name = parent.sessionData.pbxinfo.voicemenus[vmenu]['alias_exten'];
                $('#'+DOM_id).append($('<option>').html(name).val(vmenu));
            });
            top.Custom.init(document);
            break;
        default:
            break;
    }
}
