/*
 * Description: UCM6100 WebGUI
 *
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
$P = top.$;
var AST_BASE = "/app/asterisk";

window.onload = function ()
{
    $P.lang(document, true);

    top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG712"));
    $("title").attr("locale", "LANG712");
    init_validate();

    var action = "action=systemcmdwithoutput";
    var cmd = "sh " + AST_BASE+"/bin/get_ldap_configs.sh";

    var cb = function(output){
        var configs = eval('('+output+')').body;
	configs = eval('('+configs+')');
        _$('basedn').value = configs['basedn'];
        _$('rootdn').value = configs['rootdn'];
        _$('rootpw').value = configs['rootpw'];
        _$('rootpwCfm').value = configs['rootpw'];

        var conf = config2json({ filename: 'phonebookdn.conf', usf: 1 });
        if (conf['1']) {
            var pbxdn = conf['1']['prefix']+','+configs['basedn'];
            _$('pbxdn').value = pbxdn;
        }
    }

    $.ajax({type:"GET", url:"/webcgi", data: action+'&'+cmd, success: cb});
}

function confUpdate()
{
    if($P("#form", document).valid())
    {
        var basedn = _$('basedn').value;
        var rootdn = _$('rootdn').value;
        var rootpw = _$('rootpw').value;
        var pbxdn  = _$('pbxdn').value;

        var comma_pos = pbxdn.indexOf(',');
        var prefix = pbxdn.substring(0, comma_pos);
    
        var action = "action=systemcmdwithoutput";
        var cmd = 'sh ' + AST_BASE + '/bin/update_slapd_conf.sh ' + basedn 
            + ' ' + rootdn + ' ' + rootpw + ' anonymous' + ' ' + prefix;

        var cb = function(op){
            var output = eval("("+ op +")").body;
            if (output.match(/LDAP_UPDATE_SUCCESS$/))
            {
                var after = function(){
                    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG940") });
                    window.location.reload();
                };
                var c = listOfActions("phonebookdn.conf");
                c.new_action('autoupdate', 'general', 'basedn', basedn);
                c.new_action('autoupdate', '1', 'prefix', prefix);
                c.callActions(after);
            }
            else if (output.match(/LDAP_UPDATE_ERROR$/))
            {
                top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG2010") });
            }
            else if (output.match(/LDAP_SERVER_ERROR$/))
            {
                /* If this happen, this is a internal server error of OpenLDAP */
                top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG2012") });
            }
            else
            {
                top.dialog.dialogMessage({type: 'error', content: $P.lang("LANG2013") });
            }
        }

        $.ajax({type:"GET", url:"/webcgi", data: action+'&'+cmd, success: cb});
    }
}

/*
 * LDAP tooltips:
 *
"LDAP_INVALID_CHARS":"{0} can only contain such characters in square brackets:[=,a-zA-Z0-9] .",
"LDAP_BASEDN":"Base dn",
"LDAP_ROOTDN":"Root dn",
"LDAP_ROOTPW":"Root password",
"LDAP_PASSWD_CONFIRM_FAILED":"Twice passwords input are not the same !",

"LDAP_PBXDN":"Pbx dn",
"LDAP_INVALID_DN":"Invilid DN syntax !",
"LDAP_INVALID_ATTR":"Unknown attribute",
"LDAP_PBXDN_UNDER_BASEDN":"Pbx dn should end with base dn and be a immediate child of base dn !",
"LDAP_PBXDN_PREFIX_CONFLICT":"Prefix of pbx dn conflicts with another existing phonebook !",
"LDAP_ROOTDN_ENDWITH_BASEDN":"Root dn should end with base dn !",
*/

function init_validate(){
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
     $P("#form", document).validate({
         rules:{
                    "basedn": {
                        required : true,
                        customCallback: [$P.lang("LANG2014").format($P.lang('LANG1999')), valid_ldap_chars],
                        customCallback1: [$P.lang("LANG2017"), valid_dn],
                        customCallback2: ["", valid_attr]
                    },

                    "pbxdn": {
                        required : true,
                        customCallback: [$P.lang("LANG2014").format($P.lang('LANG2016')), valid_ldap_chars],
                        customCallback1: [$P.lang("LANG2017"), valid_dn],
                        customCallback2: ["", valid_attr],
                        customCallback3: [$P.lang("LANG2019"), is_under_basedn],
                        customCallback4: [$P.lang("LANG2020"), is_prefix_exist]
                    },
                
                    "rootdn": {
                        required : true,
                        customCallback: [$P.lang("LANG2014").format($P.lang('LANG2001')), valid_ldap_chars],
                        customCallback1: [$P.lang("LANG2017"), valid_dn],
                        customCallback2: ["", valid_attr],
                        customCallback3: [$P.lang("LANG2021"), is_endWith_basedn]
                    },

                    "rootpw": {
                        required : true,
                        keyboradNoSpacesemicolon1: true
                        //customCallback: [$P.lang("LANG2014").format($P.lang('LANG2002')), valid_ldap_chars]
                    },

                    "rootpwCfm": {
                        required : true,
                        customCallback: [$P.lang("LANG2015"), function(val, ele){
                                if(val != _$('rootpw').value)                            
                                    return false;
                                return true;
                            }
                        ]
                    }
               }
     });
}

function is_prefix_exist(val, ele){
    var dn_list = config2json({ filename: 'phonebookdn.conf', usf: 1 });
    var bool = true;
    var prefix = val.rChop(','+_$("basedn").value);
    $.each(dn_list.getOwnProperties(), function(index, ele){
        if(ele != "1" && dn_list[ele]['prefix'] === prefix){
            bool = false;
            return false;
        }
    });

    return bool;
}

function valid_ldap_chars(val, ele){
    if (!val.match(/^[a-zA-Z0-9=,_]+$/))
        return false;
    return true;
}

function valid_dn(val, ele){
    /*
       * eg:
       *    dc=_ab
       *    cn=_0ab,dc=com   
       *    ou=ab,dc=com,dc=cc
     
       * attr:   [a-z]+
       * val:    [a-z0-9_]+
       * Reg expression match DN: (attr=val,)*(attr=val)
       * use "i" to ignore case sensitive
     */

    if(!val.match(/^([a-z]+=[a-z0-9_]+,)*([a-z]+=[a-z0-9_]+)$/i)){
        return false;
    }
    return true;
}

/*
# =====attribute types=====
st|stateorprovincename|street|streetaddress|l|localityname|c|countryname|o|organizationname|ou|organizationalunitname|cn|commonname|dc|domaincomponent|uid|userid
# =========================
 */
function valid_attr(val, ele){
    var pos_eq = 0;
    var pos_com = 0;
    var str = val;
    while((pos_eq = str.indexOf("=")) >= 0){
        $P(ele).removeData('msg-customcallback2');
        if(!str.match(/^(st|stateorprovincename|street|streetaddress|l|localityname|c|countryname|o|organizationname|ou|organizationalunitname|cn|commonname|dc|domaincomponent|uid|userid)=/i)){
            var attrname = str.substring(0, pos_eq);
            $P(ele).data('msg-customcallback2', $P.lang("LANG2018")+' : '+attrname);
            return false;
        }
        pos_com = str.indexOf(",");
        if(pos_com < 0) 
            break;
        str = str.substring(pos_com+1);
    }

    return true;
}

function is_endWith_basedn(val, ele){
    if(!val.endsWith(_$("basedn").value)){
        return false;
    }

    return true;
}

function is_under_basedn(val, ele){
    if(!val.endsWith(_$("basedn").value)){
        return false;
    }

    var prefix = val.rChop(_$("basedn").value);    
    if(!prefix.match(/^[a-z]+=[a-z0-9_]+,$/i)){
        return false;
    }

    return true;
}

