/*
 * Asterisk-GUI - an Asterisk configuration interface
 *
 * meetroomuser.html functions
 *
 * Copyright (C) 2006-2008, Digium, Inc.
 *
 * Pari Nannapaneni <pari@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Added language settings for conference function and other functions               
 * Modified for voice prompts language couldn't take efect on conference issue       
 * 
 */
$P = top.$;
var mWindow = window;

var isNewBridge;
var EDIT_BRIDGE;

window.onload = function ()
{
    $P.lang(document, true);
    loadDOMElements();
    mcls = config2json({ filename: 'musiconhold.conf', catlist: 'yes' });
    mcls.each(function (this_class)
    {
        if (this_class != "rbt")
        {
            ASTGUI.selectbox.append('edit_moh_class', this_class, this_class);
        }
    });
    var mode = gup("mode");
    var item = gup("item");
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        initForm();
        if (mode == "add")
        {
            prepareAddItemForm();
            init_room_validate();
            $("#editForm").show();
            $("#edit_Ext").focus();
        }
        else if (mode == "edit")
        {
            prepareEditItemForm(item);
            init_room_validate();
            $("#editForm").show();
        }
        else if (mode == "addUser")
        {
            prepareAddUserItemForm(item);
            init_add_user_validate();
            $('#addUserForm').show();
            $("#create_user_id").focus();
        }
		else if (mode == "addMcb")
        {
            prepareAddMcbItemForm(item);
            init_add_mcb_validate();
            $('#addMcbForm').show();
            $("#create_remote_room").focus();
        }
        isEnableWeakPw();
        top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG595"));
        $("title").attr("locale", "LANG595");
        if (!ASTGUI.miscFunctions.alertIfRangeisNotdefined('mm_start', 'mm_end', 'Conferences'))
        {
            $('.top_buttons').hide();
            return;
        }
        updateMeetmesTable();
    }
};
$(function () {
    $("#edit_public").bind("click", function (ev) {
        change_ps('p');
        ev.stopPropagation();
    });
    $("#edit_userInvite").bind("click", function (ev) {
        change_ps('i');
        ev.stopPropagation();        
    })
});
var isEnableWeakPw = function () {
    var GUI_PREFERENCES = top.sessionData.GUI_PREFERENCES;
    if (!GUI_PREFERENCES.getProperty('weak_password')) {
        GUI_PREFERENCES  = context2json({ filename: 'guipreferences.conf', context: 'general', usf: 1 });
    }
    if (GUI_PREFERENCES.weak_password == "yes") {
        var edit_Ext = $P.lang("LANG1029");
        var edit_AdminPinCode = $P.lang("LANG1020");
        var edit_PinCode = $P.lang("LANG1032");
        $P("#edit_PinCode", document).rules("add", { 
            identical: [edit_Ext, edit_PinCode, $("#edit_Ext")],
            checkNumericPw: [ 
                showCheckPassword, {
                    type: "digital",
                    pwsId: "#edit_PinCode"
                }
            ]
        });
        $P("#edit_AdminPinCode", document).rules("add", { 
            identical: [edit_Ext, edit_AdminPinCode, $("#edit_Ext")],
            checkNumericPw: [ 
                showCheckPassword, {
                    type: "digital",
                    pwsId: "#edit_AdminPinCode"
                }
            ] 
        });
    }
}
var init_room_validate = function()
{
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "edit_Ext": {
                required: true,
                digits: true,
                extensionRange: ['mm_start', 'mm_end'],
                extensionExists: true
            },
            "edit_PinCode": {
                digits: true,
                minlength: 4,
                notEqualTo: [ $P.lang('LANG1020'), $('#edit_AdminPinCode')]
            },
            "edit_AdminPinCode": {
                digits: true,
                minlength: 4,
                notEqualTo: [ $P.lang('LANG1032'), $('#edit_PinCode')],
                required: true
            }
        },
        submitHandler: function() {
            edit_meetMe_apply_work();
        }
    });
};

var init_add_user_validate = function()
{
    $("#formAddUser").tooltip();
    $P("#formAddUser", document).validate({
        rules: {
            "create_user_id": {
                required: true,
                //phoneNumberOrExtension: true
                digits: true
            }
        },
        submitHandler: function() {
            create_add_user_work();
        }
    });
};

var init_add_mcb_validate = function()
{
	$("#formAddMcb").tooltip();
    $P("#formAddMcb", document).validate({
        rules: {
            "create_remote_room": {
                required: true,
                digits: true
            },
			"create_remote_pass": {
				digits: true
			}
        },
        submitHandler: function() {
            create_add_mcb_work();
        }
    });
};


var initForm = function ()
{
    $('#edit_moh_firstcaller').click(function ()
    {
        this.checked ? $('#tr_moh_class').css('visibility', 'visible') : $('#tr_moh_class').css('visibility', 'hidden');
        top.Custom.init(document);
    });

    $('#edit_quietMode').change(function () {
        this.checked? $('#edit_announceCallers').attr('disabled', 'disabled') : $('#edit_announceCallers').removeAttr('disabled');
        top.Custom.init(document);
    });

    $('#edit_announceCallers').change(function () {
        this.checked? $('#edit_quietMode').attr('disabled', 'disabled') : $('#edit_quietMode').removeAttr('disabled');
        top.Custom.init(document);
    });
    
    //ASTGUI.domActions.enableDisableByCheckBox('edit_public', ['edit_PinCode', 'edit_AdminPinCode']);
    $('#edit_public').change(function () {
        this.checked? $('#edit_waitAdmin').attr('disabled', 'disabled') : $('#edit_waitAdmin').removeAttr('disabled');
        top.Custom.init(document);
    });

    $('#edit_waitAdmin').change(function () {
        this.checked? $('#edit_public').attr('disabled', 'disabled') : $('#edit_public').removeAttr('disabled');
        top.Custom.init(document);
    });
}

var loadDOMElements = function ()
{
    DOM_table_mml = _$('table_meetmelist');
    //DOM_edit_MeetMeDiv = _$('edit_MeetMeDiv');
    DOM_edit_MeetMe_title = _$('edit_MeetMe_title');
    DOM_edit_Ext = _$('edit_Ext');
    DOM_edit_PinCode = _$('edit_PinCode');
    DOM_edit_AdminPinCode = _$('edit_AdminPinCode');
    DOM_edit_moh_firstcaller = _$('edit_moh_firstcaller');
    DOM_edit_callerMenu = _$('edit_callerMenu');
    DOM_edit_announceCallers = _$('edit_announceCallers');
    DOM_edit_quietMode = _$('edit_quietMode');
    DOM_edit_waitAdmin = _$('edit_waitAdmin');
    //DOM_edit_closeLastAdmin = _$('edit_closeLastAdmin');
    DOM_edit_recording = _$('edit_recording');
    DOM_edit_moh_class = _$('edit_moh_class');
    DOM_edit_userInvite = _$('edit_userInvite');
    DOM_edit_skipAuth = _$('edit_skipauth');
    DOM_edit_public = _$('edit_public');
};


var updateMeetmesTable = function ()
{
    (function ()
    {
        var meetme_rooms = [];
        var m = parent.sessionData.pbxinfo.conferences;
        for (l in m)
        {
            if (m.hasOwnProperty(l))
            {
                meetme_rooms.push(l);
            }
        }

    });
    top.Custom.init(document);
};


var delete_meetMe_confirm = function (room)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(room),
        buttons: {
            ok: function ()
            {
                var u = new listOfSynActions('extensions.conf');
                if (parent.sessionData.pbxinfo.conferences[room]['extenLangStr'])
                {
                    u.new_action('delete', ASTGUI.contexts.CONFERENCES, 'exten', '', parent.sessionData.pbxinfo.conferences[room]['extenLangStr']);
                }
                
                if (parent.sessionData.pbxinfo.conferences[room]['extenStr'])
                {
                    u.new_action('delete', ASTGUI.contexts.CONFERENCES, 'exten', '', parent.sessionData.pbxinfo.conferences[room]['extenStr']);
                }
                
                if (parent.sessionData.pbxinfo.conferences[room]['dialLangStr'])
                {
                    u.new_action('delete', ASTGUI.contexts.CONFERENCES, 'exten', '', parent.sessionData.pbxinfo.conferences[room]['dialLangStr']);
                }

                if (parent.sessionData.pbxinfo.conferences[room]['dialStr'])
                {
                    u.new_action('delete', ASTGUI.contexts.CONFERENCES, 'exten', '', parent.sessionData.pbxinfo.conferences[room]['dialStr']);
                }


                u.callActions();
                u.clearActions('meetme.conf'); // flush u and point to a different file
                if (parent.sessionData.pbxinfo.conferences[room]['meetmeStr'])
                {
                    u.new_action('delete', 'rooms', 'conf', '', parent.sessionData.pbxinfo.conferences[room]['meetmeStr']);
                }
                u.callActions();
                delete parent.sessionData.pbxinfo.conferences[room];
                
                // database deltree AAUSER/{CONF_EXTEN} 
                ASTGUI.ASTDB.deletetree( { "dbname": 'AAUSER/' + room } );
                top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG816"), callback: function () { mWindow.location.reload(); } });
            }
        }
    });
};

var prepareAddItemForm = function ()
{
    isNewBridge = true;
    EDIT_BRIDGE = '';
    show_MeetMe_Form();
    _$("edit_public").checked = true;
    change_disable();
};

var prepareEditItemForm = function (room)
{
    isNewBridge = false;
    EDIT_BRIDGE = room;
    show_MeetMe_Form();
    change_disable();
};

var prepareAddUserItemForm = function (roomid)
{
    isNewBridge = true;
    EDIT_BRIDGE = '';
    $("#belong_room_id").val(roomid);
};

var prepareAddMcbItemForm = function (roomid)
{
    isNewBridge = true;
    EDIT_BRIDGE = '';
    $("#belong_room_id").val(roomid);
};

var show_MeetMe_Form = function ()
{
    $('#tr_moh_class').css('visibility', 'hidden');
    ASTGUI.updateFieldToValue(DOM_edit_moh_class, 'default');

    if (isNewBridge == true)
    {
        ASTGUI.resetTheseFields([DOM_edit_Ext, DOM_edit_PinCode, DOM_edit_AdminPinCode, DOM_edit_moh_firstcaller, DOM_edit_callerMenu, DOM_edit_announceCallers, 
                                DOM_edit_quietMode, DOM_edit_waitAdmin, DOM_edit_recording, /*DOM_edit_closeLastAdmin,*/ DOM_edit_userInvite, DOM_edit_skipAuth, DOM_edit_public]);
        var tmp_allextensions = parent.miscFunctions.getAllExtensions();
        DOM_edit_Ext.value = tmp_allextensions.firstAvailable(parent.sessionData.GUI_PREFERENCES.getProperty('mm_start'));
        DOM_edit_Ext.disabled = false;
        return;
    }

    var TMP_EDIT_BRIDGE = parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE];
    DOM_edit_Ext.value = EDIT_BRIDGE;
    DOM_edit_Ext.disabled = true;

    if (TMP_EDIT_BRIDGE!=undefined)
    {
        DOM_edit_PinCode.value = TMP_EDIT_BRIDGE['passwd'] || '';
        DOM_edit_AdminPinCode.value = TMP_EDIT_BRIDGE['adminPin'] || '';

        var stripOpt = TMP_EDIT_BRIDGE['stripOpt'] || '';
        // M(musicclass) should handle first for remove itself.
        if (TMP_EDIT_BRIDGE['mohClass'])
        {
            DOM_edit_moh_firstcaller.checked = true;
            $('#tr_moh_class').css('visibility', 'visible');
            ASTGUI.updateFieldToValue(DOM_edit_moh_class, TMP_EDIT_BRIDGE['mohClass']);
            if (DOM_edit_moh_class.selectedIndex == -1)      //former musicclass has been deleted 
            {
                ASTGUI.updateFieldToValue(DOM_edit_moh_class, 'default');
            }
        }
        else
        {
            DOM_edit_moh_firstcaller.checked = false;
        }
        /*Added by rzhang grandstream [Bug 14675] start */
        var pinless = ASTGUI.ASTDB.getKey({ dbname: 'AAUSER/' + EDIT_BRIDGE, key: 'pinless' });
        /*Added by rzhang grandstream [Bug 14675] end */
        var publicMode = 'NO';
        publicMode = ASTGUI.ASTDB.getKey({ dbname: 'AAUSER/' + EDIT_BRIDGE, key: 'PUBLIC' });

        DOM_edit_callerMenu.checked = (stripOpt.contains('s')) ? true : false;
        DOM_edit_announceCallers.checked = (stripOpt.contains('I')) ? true : false;
        DOM_edit_quietMode.checked = (stripOpt.contains('q')) ? true : false;
        DOM_edit_waitAdmin.checked = (stripOpt.contains('w')) ? true : false;
        //DOM_edit_closeLastAdmin.checked = (stripOpt.contains('x')) ? true : false;
        DOM_edit_recording.checked = (stripOpt.contains('r')) ? true : false;
        DOM_edit_userInvite.checked = (stripOpt.contains('X')) ? true : false;
        /*Added by rzhang grandstream [Bug 14675] start */
        DOM_edit_skipAuth.checked = ( pinless == 'NOPASSWD' ) ? true : false;
        /*Added by rzhang grandstream [Bug 14675] end */ 
        if ( DOM_edit_AdminPinCode.value != undefined && DOM_edit_AdminPinCode.value != '' )
        {
            /*created room*/
            DOM_edit_public.checked = ( publicMode == 'YES' ) ? true : false;
        }
        else
        {
            /*new room*/
            DOM_edit_public.checked = ( publicMode == 'NO' ) ? false : true;/*Default to enabled*/
        }
        
        if ( DOM_edit_public.checked == true )
        {
            DOM_edit_PinCode.value = '';
            //DOM_edit_AdminPinCode.value = '';

            $(DOM_edit_PinCode).attr("disabled", true);
            $(DOM_edit_AdminPinCode).attr("disabled", true);
        }

        /* Beacuse of history reason, we use 'JIT_BUF' to make local bridge to process conference jitter.
         * but we don't need it now.
         */
        var jb_enable = ASTGUI.ASTDB.getKey({ dbname: 'AAUSER/' + EDIT_BRIDGE, key: 'JIT_BUF' })
        if (jb_enable !== null)
        {
            ASTGUI.ASTDB.deleteKey({ dbname: 'AAUSER/' + EDIT_BRIDGE, key : 'JIT_BUF' });
        }
        
        $('#edit_quietMode').trigger('change');
        $('#edit_announceCallers').trigger('change');
        $('#edit_public').trigger('change');
        $('#edit_waitAdmin').trigger('change');
    }
};

var edit_meetMe_apply = function ()
{
    AskextensionRange(_$("edit_Ext").value, 'mm_start', 'mm_end');
};

var edit_meetMe_apply_work = function ()
{
    var new_MeetmeOptions = [];
    var new_stripOpt = [];
    var pinless = "NEEDAUTH";
    var publicMode = "NO";
    
    if (DOM_edit_callerMenu.checked) { new_MeetmeOptions.push('s'); }
    if (DOM_edit_announceCallers.checked) { new_MeetmeOptions.push('I'); }
    if (DOM_edit_quietMode.checked) { new_MeetmeOptions.push('q'); }
    if (DOM_edit_waitAdmin.checked) { new_MeetmeOptions.push('w'); }
    //if (DOM_edit_closeLastAdmin.checked) { new_MeetmeOptions.push('x'); }
    if (DOM_edit_recording.checked) { new_MeetmeOptions.push('r'); }
    if (DOM_edit_userInvite.checked) { new_MeetmeOptions.push('X'); }
    if (DOM_edit_skipAuth.checked) { pinless="NOPASSWD"; } 
    else { pinless="NEEDAUTH"; }
    if (DOM_edit_public.checked) { publicMode="YES"; } 
    else { publicMode="NO"; }

    new_stripOpt = new_MeetmeOptions.slice();
    if (DOM_edit_moh_firstcaller.checked)
    {
        new_stripOpt.push('M');
        new_MeetmeOptions.push('M(' + ASTGUI.getFieldValue(DOM_edit_moh_class) + ')');
    }

    var u = new listOfSynActions('extensions.conf');
    var w = new listOfSynActions('meetme.conf');

    var new_exten = DOM_edit_Ext.value;
    var new_MeetmeLanguageString = 'conf-' + new_exten + ",1,Macro(user-callerid,)";
    var new_MeetmeString = 'conf-' + new_exten + ",n,Macro(dial-conference,${EXTEN:5}," + new_MeetmeOptions.join('') + "," + DOM_edit_PinCode.value.trim() + "," + DOM_edit_AdminPinCode.value.trim() + ",confnocdr)";

    var new_meetmeStr = DOM_edit_Ext.value + ",,";
    var new_dialLanguageStr = new_exten + ",1,Macro(user-callerid,)";
    //exten => 6300,n,ExecIf($["${DB(AAUSER/${EXTEN}/JIT_BUF)}" = "ON"]?Dial(Local/conf-6300@ext-conference/nj):Macro(dial-conference,6300,,1234,6300,))
    var new_dialStr = '{0},n,ExecIf($["${DB(AAUSER/${EXTEN}/JIT_BUF)}" = "ON"]?Dial(Local/conf-{0}@{1}/nj):Macro(dial-conference,{0},{2},{3},{4},))'.format(new_exten,                            //{0}
                                  ASTGUI.contexts.CONFERENCES,          //{1}
                                  new_MeetmeOptions.join(''),           //{2}
                                  DOM_edit_PinCode.value.trim(),        //{3}
                                  DOM_edit_AdminPinCode.value.trim() ); //{4}

    if (isNewBridge == false)
    { // delete/update old bridge values

        if (parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['extenLangStr'])
        {
            u.new_action('delete', ASTGUI.contexts.CONFERENCES, 'exten', '', parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['extenLangStr']);
        }
        
        if (parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['extenStr'])
        {
            u.new_action('delete', ASTGUI.contexts.CONFERENCES, 'exten', '', parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['extenStr']);
        }
        
        if (parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['dialLangStr'])
        {
            u.new_action('delete', ASTGUI.contexts.CONFERENCES, 'exten', '', parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['dialLangStr']);
        }

        if (parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['dialStr'])
        {
            u.new_action('delete', ASTGUI.contexts.CONFERENCES, 'exten', '', parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['dialStr']);
        }


        u.callActions();
        u.clearActions();
        if (parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['meetmeStr'])
        {
            w.new_action('delete', 'rooms', 'conf', '', parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['meetmeStr']);
        }
        w.callActions();
        w.clearActions();

        // database deltree AAUSER/{CONF_EXTEN} 
        ASTGUI.ASTDB.deletetree( { "dbname": 'AAUSER/' +  DOM_edit_Ext.value } );


    } else
    {      //new conferece room
        EDIT_BRIDGE = ASTGUI.getFieldValue(DOM_edit_Ext);

        if (parent.miscFunctions.ifExtensionAlreadyExists(EDIT_BRIDGE))
        {
            ASTGUI.highlightField(DOM_edit_Ext, 'Extension already exists');
            top.dialog.clearDialog();
            return;
        }
        if (!ASTGUI.miscFunctions.isExtensionInRange(EDIT_BRIDGE, 'mm_start', 'mm_end'))
        {
            ASTGUI.highlightField(DOM_edit_Ext, 'Extension is not in preferred range');
            top.dialog.clearDialog();
            return;
        }
        parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE] = new ASTGUI.customObject;
    }

    parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['extenLangStr'] = new_MeetmeLanguageString;
    parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['extenStr'] = new_MeetmeString;
    parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['dialLangStr'] = new_dialLanguageStr;
    parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['dialStr'] = new_dialStr;
    parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['meetmeStr'] = new_meetmeStr;
    parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['stripOpt'] = new_stripOpt.join("");
    delete parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['mohClass'];
    delete parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['passwd'];
    if (DOM_edit_moh_firstcaller.checked)
    {
        parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['mohClass'] = ASTGUI.getFieldValue(DOM_edit_moh_class);
    }
    if (DOM_edit_PinCode.value.trim())
    {
        parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['passwd'] = DOM_edit_PinCode.value.trim();
    }
    if (DOM_edit_AdminPinCode.value)
    {
        parent.sessionData.pbxinfo.conferences[EDIT_BRIDGE]['adminPin'] = DOM_edit_AdminPinCode.value.trim();
    }

    u.new_action('append', ASTGUI.contexts.CONFERENCES, 'exten', new_dialLanguageStr);
    u.new_action('append', ASTGUI.contexts.CONFERENCES, 'exten', new_dialStr);
    u.new_action('append', ASTGUI.contexts.CONFERENCES, 'exten', new_MeetmeLanguageString);
    u.new_action('append', ASTGUI.contexts.CONFERENCES, 'exten', new_MeetmeString);
    w.new_action('append', 'rooms', 'conf', new_meetmeStr);

    u.callActions();
    w.callActions();
    
    /*Added by rzhang grandstream [Bug 14674] [Bug 14675] start */
    ASTGUI.ASTDB.updateKey({ dbname: 'AAUSER/' + DOM_edit_Ext.value, key: 'PERMISSION', keyvalue: 'internal-local-national-international' });
    ASTGUI.ASTDB.updateKey({ dbname: 'AAUSER/' + DOM_edit_Ext.value, key : 'pinless', keyvalue: pinless });
    ASTGUI.ASTDB.updateKey({ dbname: 'AAUSER/' + DOM_edit_Ext.value, key : 'PUBLIC', keyvalue: publicMode });
    
    ASTGUI.ASTDB.updateKey({ dbname: 'AAUSER/' + DOM_edit_Ext.value, key : 'RECORD', keyvalue: ASTGUI.getFieldValue(DOM_edit_recording)});

    top.dialog.dialogMessage({
        type: 'success',
        content: $P.lang("LANG815"),
        callback: function () { mWindow.location.reload(); }
    });

};

var create_add_user_work = function ()
{
    var adduserid = $('#create_user_id').val();
    var need_confirm = "0";
    if ( $('#need_confirm').attr("checked") == "checked" )
    {
        need_confirm = "1";
    }
    create_add_request($("#belong_room_id").val(), adduserid+"@"+need_confirm);
};

var create_add_mcb_work = function ()
{
    var mcbremoteid = $('#create_remote_room').val();
    var mcbremotepass = $('#create_remote_pass').val();
    create_add_mcb_request($("#belong_room_id").val(), mcbremoteid+"@"+mcbremotepass);
};

var create_add_request = function (roomid, userid)
{
    $.ajax({
        type:"GET",
        url:"../webcgi?action=inviteuser&target=" + roomid + "&conf-user=" + userid
    });
    top.dialog.dialogMessage({
        type: 'success',
        content: $P.lang("LANG922"),
        callback: function () { mWindow.location.reload(); }
    });
};

var create_add_mcb_request = function (roomid, remoteinfo)
{
    $.ajax({
        type:"GET",
        url:"../webcgi?action=inviteroom&target=" + roomid + "&conf-user=" + remoteinfo
    });
    top.dialog.dialogMessage({
        type: 'success',
        content: $P.lang("LANG922"),
        callback: function () { mWindow.location.reload(); }
    });
};

var change_ps = function(val)
{
    if (_$("edit_userInvite").checked && _$("edit_public").checked )
    {
        top.dialog.dialogConfirm({
            confirmStr: $P.lang("LANG2433").format($P.lang("LANG1027"), $P.lang("LANG2431")),
            buttons: {
                ok: function ()
                {
                    top.dialog.container.show();
                    top.dialog.shadeDiv.show();
                },
                cancel: function ()
                {
                    if (val == 'i')
                    {
                        _$("edit_userInvite").checked = false;
                    }
                    else
                    {
                        _$("edit_public").checked = false;
                    }

                    change_disable();
                    top.dialog.container.show();
                    top.dialog.shadeDiv.show();
                }
            }
        }); 
    }
    change_disable();
}

var change_disable = function()
{
    if ( _$("edit_public").checked )
    {
        $('#edit_PinCode').attr("disabled", true);
        $('#edit_AdminPinCode').attr("disabled", true);
        $('#edit_waitAdmin').attr("disabled", true);
    }
    else
    {
        $('#edit_PinCode').attr("disabled", false);
        $('#edit_AdminPinCode').attr("disabled", false);
        $('#edit_waitAdmin').attr("disabled", false);
    }
    top.Custom.init(document);
}
