/*
* Asterisk-GUI  - an Asterisk configuration interface
*
* menuprompts_record.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Remove digium related product definitions and procedures              
 * IVR Prompts:there is no "Apply Changes" when delete the prompts       
 * Support multiple languages                                            
 * 
 */
$P = top.$;
var mWindow = window;

var CURRENT_FILE; // whatever file that needs to be deleted/playback/record over
var ACTION; // 'PLAY'/'RECORD';
var FILENAME_LIST;

window.onload = function ()
{
    $P.lang(document, true);
    var mode = gup("mode");
    var item = decodeURIComponent(gup("item"));
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        initForm();
        if (mode == "addRecord")
        {
            displayAddRecord();
        }
        else if (mode == "editRecord")
        {
            displayEditRecord(item);
        }
        else if (mode == "playRecord")
        {
            displayPlayRecord(item);
        }
        init_validator();
        top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG674"));
        $("title").attr("locale", "LANG674");
        $('#recfilesTable').hide();
        updateTable();
    }
};

var check_filename = function (value, element)
{
    var filename = _$('newvmenu_name').value + _$('newvmenu_format').value;

    if (mWindow.FILENAME_LIST)
    {
        for (var i = 0; i < mWindow.FILENAME_LIST.length; i++)
        {
            if (filename === mWindow.FILENAME_LIST[i])
            {
                return false;
            }
        }
    }
    return true;
};

var init_validator = function () {
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "newvmenu_name": {
                required: true,
                customCallback1: [$P.lang("LANG2148"), function(){
                    return /^[^&#(\/;*?]*$/.test(_$("newvmenu_name").value);
                }],
                customCallback: [$P.lang("LANG2146"), check_filename]
            },
            "newvmenu_ext": {
                required: true
            },
            "newvmenu_format": {
                required: true
            }
        },
        submitHandler: function() {
            record_new_Verify();
        }
    });
};

var initForm = function ()
{
    var newvmenu_ext = _$('newvmenu_ext');
    var playVmenu_ext = _$('playVmenu_ext');

    var ul = parent.pbx.users.list(); ul = ul.sortNumbers();
    ul.each(function (user)
    { // list each user in table
        ASTGUI.selectbox.append(newvmenu_ext, user, user);
        ASTGUI.selectbox.append(playVmenu_ext, user, user);
    });

}


var updateTable = function ()
{
    var _rft = _$('recfilesTable');

    (function ()
    {
        if (_rft) {
            var newRow = _rft.insertRow(-1);
            newRow.className = "frow";
            ASTGUI.domActions.tr_addCell(newRow, { html: '', width: '50', align: 'center' });
            ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang("LANG135"), locale: "LANG135",width: '500' });
            ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang("LANG74"), locale: "LANG74"});
        }
    })();
    ASTGUI.listSystemFiles(top.sessionData.directories.menusRecord, function (recfiles)
    {
        for (var i = 0; i < recfiles.length; i++)
        {
            var filename = recfiles[i].stripTags();
            FILENAME_LIST = recfiles;
            if (_rft) {
                var newRow = _rft.insertRow(-1);
                newRow.className = ((_rft.rows.length) % 2 == 1) ? 'odd' : 'even';
                var tmp = "<button class='options record' onclick='showEditRecordForm(\"" + filename + "\")' localetitle=\"LANG784\" title='" + $P.lang("LANG784") + "'></button> &nbsp; " +
                          "<button class='options play' onclick='showPlayRecordForm(\"" + filename + "\")' localetitle=\"LANG777\" title=" + $P.lang("LANG777") + "></button> &nbsp; " + 
                          "<button class='options del' onclick='delete_file(\"" + filename + "\")' localetitle=\"LANG739\" title=" + $P.lang("LANG739") + "></button>";
                ASTGUI.domActions.tr_addCell(newRow, { html: _rft.rows.length - 1, align: 'center', width: '35' });
                ASTGUI.domActions.tr_addCell(newRow, { html: filename, width: '180' });
                ASTGUI.domActions.tr_addCell(newRow, { html: tmp, align: 'center' });
            }
        }
        if (_rft) {
            if (_rft.rows.length == 1)
            {
                ASTGUI.domActions.clear_table(_rft);
                var newRow = _rft.insertRow(-1);
                ASTGUI.domActions.tr_addCell(newRow, { locale: "LANG1610", html:  $P.lang("LANG1610")
                 , align: 'center'
                });
            }
        }
        $('#recfilesTable').show();
    });
}


var displayAddRecord = function ()
{
    $("#recordnew_content").show();
};

var displayPlayRecord = function (item)
{
    ACTION = 'PLAY';
    CURRENT_FILE = item;
    _$('playVmenu_ext_0').innerHTML = $P.lang("LANG1611");
    _$('play_record_button').innerHTML = $P.lang("LANG777");
    _$('playVmenu_name').innerHTML = CURRENT_FILE;
    $('#playFile_content').show();
};


var displayEditRecord = function (item)
{
    ACTION = 'RECORD';
    CURRENT_FILE = item;
    _$('playVmenu_ext_0').innerHTML = $P.lang("LANG1612");
    _$('play_record_button').innerHTML = $P.lang("LANG778");
    _$('playVmenu_name').innerHTML = CURRENT_FILE;
    $('#playFile_content').show();
};

var play_record_file = function ()
{
    var y = _$('playVmenu_ext').value;
    if (ACTION == 'PLAY')
    {
        if (CURRENT_FILE.endsWith('.gsm'))
        {
            CURRENT_FILE = CURRENT_FILE.withOut('.gsm');
        } 
        else if (CURRENT_FILE.endsWith('.alaw'))
        {
            CURRENT_FILE = CURRENT_FILE.withOut('.alaw');
        } 
        else if (CURRENT_FILE.endsWith('.g722'))
        {
            CURRENT_FILE = CURRENT_FILE.withOut('.g722');
        } 
        else if (CURRENT_FILE.endsWith('.g729'))
        {
            CURRENT_FILE = CURRENT_FILE.withOut('.g729');
        } 
        else if (CURRENT_FILE.endsWith('.ulaw'))
        {
            CURRENT_FILE = CURRENT_FILE.withOut('.ulaw');
        } 
        else if (CURRENT_FILE.endsWith('.wav'))
        {
            CURRENT_FILE = CURRENT_FILE.withOut('.wav');
        } 
        else if (CURRENT_FILE.endsWith('.mp3'))
        {
            CURRENT_FILE = CURRENT_FILE.withOut('.mp3');
        }
        makeRequest({
            action: 'originate',
            channel: 'Local/' + 'REC' + y + '@' + ASTGUI.contexts.guitools,
            context: ASTGUI.contexts.guitools,
            exten: 'play_file',
            priority: '1',
            Variable: 'var1=' + top.sessionData.directories.menusRecord + CURRENT_FILE,
            Async:true
        });
        top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG873") });
    }
    else if (ACTION == 'RECORD')
    {
        RecordFile(y);
    }
    CURRENT_FILE = '';
};

var delete_file = function (item)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(item),
        buttons: {
            ok: function ()
            {
                top.$.cookie('configFilesChanged', 'yes');
                top.$('#applyChanges_Button').css("visibility", "visible");
                top.$('#line_Button').css("visibility", "visible");
                top.$('#applyChanges_Button').effect("shake", {
                    direction: "up", distance: 2, times:10000 
                }, 400);
                top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG877")});
				var tmp_cmd = "action=fileOpDelete&type=ivr&data=" + encodeURI(item);
				var cb = function()
                {
                    top.dialog.clearDialog();
                    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG816"), callback: function () { mWindow.location.reload(); } });
                }
				$.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: cb});
            }
        }
    });
};

var record_new_Verify = function ()
{
    CURRENT_FILE = _$('newvmenu_name').value + _$('newvmenu_format').value;
    top.dialog.clearDialog();
    RecordFile(_$('newvmenu_ext').value);
};

var RecordFile = function (extension)
{ // uses/dials  extension to record into CURRENT_FILE
    makeRequest({
        action: 'originate',
        channel: 'Local/' + 'REC' + extension + '@' + ASTGUI.contexts.guitools,
        context: ASTGUI.contexts.guitools,
        exten: 'record_vmenu',
        priority: '1',
        Variable: 'var1=' + top.sessionData.directories.menusRecord + CURRENT_FILE,
        Async:true
    });
    top.dialog.dialogMessage({
        type: 'success',
        content: $P.lang("LANG873"),
        callback: function ()
        {
            top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG878")});
            setTimeout(
                function ()
                {
                    top.dialog.clearDialog();
                    top.dialog.dialogMessage({ type: 'info', content: $P.lang("LANG879") });
                }, 3000
	        );
        } 
    });
};

var ask_add_users = function()
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG880"),
        buttons: {
            ok: function ()
            {
                top.module.jumpMenu('users.html');
            },
            cancel: function ()
            {
                top.dialog.clearDialog();
            }
        }
    });
}

