/*
* Asterisk-GUI  - an Asterisk configuration interface
*
* Javascript functions for accessing manager over HTTP and Some UI components/functions used in AsteriskGUI.
*
* Copyright (C) 2006-2011, Digium, Inc.
*
* Mark Spencer <markster@digium.com>
* Pari Nannapaneni <pari@digium.com>
* Ryan Brindley <rbrindley@digium.com>
* Erin Spiceland <espiceland@digium.com>
*
* Thanks to Steven Levithan (http://stevenlevithan.com) for his
* implementation of trim()
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * modified the function of IVR       
 * 
 */
$P = top.$;
var mWindow = window;
var isNewVmenu, EDIT_VMENU, mode, item, ext;
var CURRENT_ACTIONS = [];
var LISTOFSOUNDFILES = [];
var LISTOFRECORDFILES = [];
var KEYPRESS_LIST = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', 't', 'i'];
function arrIndexOf(arr, val) {
    if ($.isArray(arr)) {
        var i = arr.length;
        while (i--)
        {
            if (arr[i] === val)
            {
                return i;
            }
        }
        return -1;
    }
    else{
        return arr.indexOf(val);
    }
}
window.onload = function ()
{
    $P.lang(document, true);
    mode = gup("mode");
    item = gup("item");
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        initForm();
        if (mode == "add")
        {
            prepareAddItemForm();
            $("#editForm").show();
        }
        else if (mode == "edit")
        {
            prepareEditItemForm(item);
            $("#editForm").show();
        }
        $("#ivr_name").focus();
        if (item != undefined)
        {
            ext = parent.sessionData.pbxinfo.voicemenus[item].getProperty('alias_exten').split(',')[0];
            if (ext.beginsWith('exten='))
            {
                ext = ext.lChop('exten=');
            }
        }
        top.Custom.init(document);
        init_validator();
        return false;
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG647"));
        $("title").attr("locale", "LANG647");
        if (!ASTGUI.miscFunctions.alertIfRangeisNotdefined('vme_start', 'vme_end', 'IVR'))
        {
            $('.top_buttons').hide();
            return;
        }
        updateVoiceMenus_Table();
        sortTable({
            sortTableToggle: false,
            "tableId": 'table_VoiceMenuslist',
            sortFuncs: {
                "sort_1": ASTGUI.extenStyleCmpFunc
            }

        }).sortTableInitial(1).bindTitles( 
            {idx: 0, isDis: true}, {idx: 1}
        );
    }

};
var init_validator = function () {
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules:{
            "ivr_name": {
                required: true,
                letterDigitUndHyphen: true,
                voicemenuExists: [mode, item]
            },
            "ivr_ext": {
                minlength: 2,
                digits: true,
                extensionRange: ["vme_start", "vme_end"],
                extensionExists: [ext]
            },
            "digit_timeout": {
                required: true,
                digits: true,
                range: [1,60]
            },
            "ivr_timeout": {
                required: true,
                digits: true,
                range: [1,180]//,
                //bigger: [$P.lang('LANG2540'), $P.lang('LANG2539'), $("input[name='digit_timeout']")]
            }
            /*,
            "ivr_trunkExt": {
                required: true,
                minlength: 2,
                digits: true
            } 
            */ 
        },
        submitHandler: function() {
            voiceMenu_saveChange();
        }
    });
}
var appendlist = function (id, keylist, valuelist, addnone)
{
    if (addnone)
    {
        ASTGUI.selectbox.append(id,  $P.lang("LANG133"), "");
    }
    else
    {
        if (!keylist.length)
        {
            ASTGUI.selectbox.append(id, $P.lang("LANG133"), "");
        }
    }

    for(var i = 0; i < keylist.length; i++)
    {
        var key = keylist[i];
        /*if (key.length > 25)
        {
            key = key.substring(0,20) + "...";
        }*/

        ASTGUI.selectbox.append(id, key, valuelist[i]);
    }
};

var appendsoundlist = function (id, keylist, addnone)
{
    if (addnone)
    {
        ASTGUI.selectbox.append(id, $P.lang("LANG133"), "");
    }
    else
    {
        if (!keylist.length)
        {
            ASTGUI.selectbox.append(id, $P.lang("LANG133"), "");
        }
    }

    for(var i = 0; i < keylist.length; i++)
    {
        var key = keylist[i];
        /*
        if (key.length > 25)
        {
            key = key.substring(0,20) + "...";
        }*/

        ASTGUI.selectbox.append(id, key, "playback(" + keylist[i] + ")");
    }
};

var initForm = function ()
{
    var EVENT_LIST = [$P.lang("LANG85"), $P.lang("LANG90"), $P.lang("LANG98"),  $P.lang("LANG89"), "IVR", $P.lang("LANG600"), $P.lang("LANG91"), $P.lang("LANG94"), $P.lang("LANG95"), $P.lang("LANG238"), $P.lang("LANG97"), $P.lang("LANG2353")];
    var VALUE_LIST = ["user", "voicemail", "conference", "vmgroup", "ivr", "rggroup", "queue", "pge", "fax", "prompt", "hangup", "disa"];
    var key;

    //ASTGUI.domActions.enableDisableByCheckBox('ivr_dialTrunk', ['ivr_trunkExt', 'ivr_disPin']);
    ASTGUI.domActions.enableDisableByCheckBox('ivr_dialTrunk', ['ivr_permission']);
    
    KEYPRESS_LIST.each(function (ef)
    {
        key = "keypress_" + ef;
        ASTGUI.selectbox.append(key, $P.lang("LANG1485"), "");
        appendlist(key, EVENT_LIST, VALUE_LIST, 0);
        $("#keypressevent_" + ef).hide();
    });

    VoiceMenus_miscFunctions.load_recordedSounds();
    init_sounds();
}

var init_sounds = function ()
{
    appendlist("prompt_welcome", LISTOFRECORDFILES, LISTOFRECORDFILES, 1);
    ASTGUI.selectbox.insert_before("prompt_welcome", "welcome", "welcome", 1);
    appendlist("prompt_timeout", LISTOFRECORDFILES, LISTOFRECORDFILES, 1);
    ASTGUI.selectbox.insert_before("prompt_timeout", "ivr-create-timeout", "ivr-create-timeout", 1);
    appendlist("prompt_invalid", LISTOFRECORDFILES, LISTOFRECORDFILES, 1);
    ASTGUI.selectbox.insert_before("prompt_invalid", "invalid", "invalid", 1);
}

var chop_Extension = function (a)
{
    if (!a) { return ''; }
    if (a.endsWith('.gsm')) { return a.rChop('.gsm'); }
    else if (a.endsWith('.wav')) { return a.rChop('.wav'); }
    else if (a.endsWith('.alaw')) { return a.rChop('.alaw'); }
    else if (a.endsWith('.g722')) { return a.rChop('.g722'); }
    else if (a.endsWith('.g729')) { return a.rChop('.g729'); }
    else if (a.endsWith('.ulaw')) { return a.rChop('.ulaw'); }
    else if (a.endsWith('.wav')) { return a.rChop('.wav'); }
    else if (a.endsWith('.mp3')) { return a.rChop('.mp3'); }
    return a;
}

var VoiceMenus_miscFunctions = {

    load_Sounds: function ()
    { // VoiceMenus_miscFunctions.load_Sounds();
        //top.dialog.dialogMessage({ type: 'loading', content: 'loading list of sounds ..' });
        var LANG_PATH = top.sessionData.directories.Sounds;

        var c = config2json({ filename: 'users.conf', usf: 1 });
        if (c.hasOwnProperty('general'))
        {
            if (c['general'].getProperty('language') != 'en')
            {
                var LANG_PATH = top.sessionData.directories.Sounds + c['general'].getProperty('language') + '/';
            }
        }

        ASTGUI.listSystemFiles(LANG_PATH, function (listOfFiles)
        {
            var tmp_obj = {};
            listOfFiles.each(function (CURRENT_FILE)
            {
                CURRENT_FILE = chop_Extension(CURRENT_FILE);
                if (!tmp_obj.hasOwnProperty(CURRENT_FILE))
                {
                    tmp_obj[CURRENT_FILE] = 1;
                    LISTOFSOUNDFILES.push(CURRENT_FILE);
                }
            });

            VoiceMenus_miscFunctions.load_recordedSounds();
        });
    },

    load_soundtodata: function()
    {
        ASTGUI.listSystemFiles(top.sessionData.directories.menusRecord, function (recfiles)
        {
            top.sessionData.pbxinfo.soundlist = recfiles;
        });
    },

    load_recordedSounds: function ()
    {
        var recfiles = top.sessionData.pbxinfo.soundlist;
        var tmp_obj = {};

        recfiles.each(function (CURRENT_FILE){
            CURRENT_FILE = chop_Extension(CURRENT_FILE);
            CURRENT_FILE = "record/" + CURRENT_FILE;
            if (!tmp_obj.hasOwnProperty(CURRENT_FILE))
            {
                tmp_obj[CURRENT_FILE] = 1;
                LISTOFRECORDFILES.push(CURRENT_FILE);
            }
        });
    },

    reset_VoicemenuFields: function ()
    { // VoiceMenus_miscFunctions.reset_VoicemenuFields();
        //ASTGUI.resetTheseFields(['ivr_name', 'ivr_ext', 'ivr_dialOther', 'ivr_dialTrunk', 'ivr_disPin', 'ivr_trunkExt', 'ivr_timeout']);
        ASTGUI.resetTheseFields(['ivr_name', 'ivr_ext', 'ivr_dialOther', 'ivr_dialTrunk', 'ivr_permission', 'ivr_timeout', 'ivr_language']);

        ASTGUI.selectbox.append('ivr_language', $P.lang("LANG257"), '');
    
        $.ajax({
            type: "GET",
            url: "./userdefined/language.json",
            dataType: "json",
            async: false,
            success: function (data)
            {		 
                var languageData = eval(data);
        
                for (var i = 0; i < languageData.length; i++)
                {
                    ASTGUI.selectbox.append('ivr_language', languageData[i].name, languageData[i].id );
                };
            }
        });
    
        _$('ivr_tloop').selectedIndex = 2;
        _$('ivr_iloop').selectedIndex = 2;
        _$('ivr_language').selectedIndex = 0;
        var key;

        KEYPRESS_LIST.each(function(el){
            key = "keypress_" + el;
            _$(key).selectedIndex = 0;
            key = "keypressevent_" + el;
            if (key != undefined)
            {
                _$(key).selectedIndex = 0;
            }
        });
    }
};


var delete_voiceMenu_confirm = function (item)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(parent.sessionData.pbxinfo.voicemenus[item].getProperty('comment')),
        buttons: {
            ok: function ()
            {
                EDIT_VMENU = item;

                if (!parent.pbx.voice_menus.remove(EDIT_VMENU))
                {
                    top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG820") });
                }
                else
                {
                    top.dialog.dialogMessage({
                        type: 'success',
                        content: $P.lang("LANG816"),
                        callback: function ()
                        {
                            mWindow.location.reload();
                        }
                    });
                }
            }
        }
    });
};

var voiceMenu_saveChanges = function ()
{
    AskextensionRange(_$("ivr_ext").value, "vme_start", "vme_end");
}
var voiceMenu_saveChange = function () {
    var levelVal = $("#ivr_permission").val();
    if ($("#ivr_dialTrunk").is(":checked") && levelVal != "internal") {
        top.dialog.dialogConfirm({
            type: "warning",
            confirmStr: $P.lang("LANG2537").format($P.lang("LANG1071"), $P.lang("LANG2372")),
            buttons: {
                ok:  function (){
                    doVoiceMenu_saveChange();
                },
                cancel:  function (){
                    top.dialog.container.show();
                    top.dialog.shadeDiv.show();
                }
            }
        }); 
    }
    else {
        doVoiceMenu_saveChange();
    }
}
var doVoiceMenu_saveChange = function () {
    var vm = {
        comment: ASTGUI.getFieldValue('ivr_name'),
        alias_exten: '',
        welcome_prompt: '',
        digit_timeout:'',
        timeout: '',
        timeout_prompt: '',
        invalid_prompt: '',
        tloop: '',
        iloop: '',
        includes: [],
        //trunk: '',
        //dispin:'',
        permission:'',
        language:'',
        keypress_events: {},
    };
    vm = ASTGUI.toCustomObject(vm);
    if (_$('ivr_dialOther').checked)
    {
        vm.includes.push(ASTGUI.contexts.ExtLocal);
        vm.includes.push(ASTGUI.contexts.CONFERENCES);
        vm.includes.push(ASTGUI.contexts.QUEUES);
        vm.includes.push(ASTGUI.contexts.ExtGroup);
        vm.includes.push(ASTGUI.contexts.VoiceMailGroups);
        vm.includes.push(ASTGUI.contexts.PageGroups);
        vm.includes.push(ASTGUI.contexts.ExtFax);
    }

    KEYPRESS_LIST.each(function (this_key)
    {
        var key = _$('keypress_' + this_key).value;
        if (key)
        {
            var value =  _$('keypressevent_' + this_key).value;
            var k = this_key;

            if (this_key == "11")
            {
                k = "*";
            }
            else if (this_key == "10")
            {
                k = "#"; 
            }

            if (key == "hangup")
            {
                vm.keypress_events[k] = "Hangup";
            }
            else if(value)
            {
                vm.keypress_events[k] =  value;
            }
        }
    });


    if (isNewVmenu == true)
    {
        var vmenu_name = parent.pbx.voice_menus.next();
    } else
    {
        var vmenu_name = EDIT_VMENU;
        parent.pbx.voice_menus.remove(vmenu_name);
    }

    var after = function ()
    {
        top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG815"), callback: function () { mWindow.location.reload(); } });
    };

    vm.alias_exten = ASTGUI.getFieldValue('ivr_ext');
    vm.welcome_prompt = ASTGUI.getFieldValue('prompt_welcome').replace(/\(/g, '\\(');
    vm.digit_timeout = ASTGUI.getFieldValue('digit_timeout');
    vm.timeout = ASTGUI.getFieldValue('ivr_timeout');
    vm.timeout_prompt = ASTGUI.getFieldValue('prompt_timeout').replace(/\(/g, '\\(');
    vm.invalid_prompt = ASTGUI.getFieldValue('prompt_invalid').replace(/\(/g, '\\(');
    vm.tloop = ASTGUI.getFieldValue('ivr_tloop');
    vm.iloop = ASTGUI.getFieldValue('ivr_iloop');
    vm.language = ASTGUI.getFieldValue('ivr_language');

    if ( _$('ivr_dialTrunk').checked)
    {
        vm.includes.push(ASTGUI.contexts.OutboundAllroutes);
        vm.permission = _$('ivr_permission').value;
    }
 
    parent.pbx.voice_menus.add(vmenu_name, vm, after);
}
var prepareAddItemForm = function ()
{
    isNewVmenu = true;
    EDIT_VMENU = '';
    VoiceMenus_miscFunctions.reset_VoicemenuFields();

    var tmp_allextensions = ASTGUI.cloneObject(parent.miscFunctions.getAllExtensions());

    _$('prompt_welcome').selectedIndex = 1;
    _$('prompt_invalid').selectedIndex = 1;
    _$('prompt_timeout').selectedIndex = 1;

    _$('ivr_ext').value = tmp_allextensions.firstAvailable(parent.sessionData.GUI_PREFERENCES.getProperty('vme_start'));
    _$('digit_timeout').value = 3;
    _$('ivr_timeout').value = 10;
    _$('ivr_permission').value = 'internal';
    _$('ivr_permission').disabled = true ;
};

var show_as = function(string)
{
    if (!string)
    {
        return "";
    }

    if (string.contains(ASTGUI.contexts.CallingPlan))
    {
        return "user";
    }
    else if (string.contains("simple_vm"))
    {
        return "voicemail";
    }
    else if (string.contains(ASTGUI.contexts.CONFERENCES))
    {
        return "conference";
    }
    else if (string.contains(ASTGUI.contexts.QUEUES))
    {
        return "queue";
    }
    else if (string.contains(ASTGUI.contexts.VoiceMenuPrefix))
    {
        return "ivr";
    }
    else if (string.contains(ASTGUI.contexts.ExtGroup))
    {
        return "rggroup";
    }
    else if (string.contains(ASTGUI.contexts.VoiceMailGroups))
    {
        return "vmgroup";
    }
    else if (string.contains(ASTGUI.contexts.PageGroups))
    {
        return "pge";
    }
    else if (string.contains(ASTGUI.contexts.ExtFax))
    {
        return "fax";
    }
    else if (string.contains("Hangup"))
    {
        return "hangup";
    }
    else if (string.contains("playback"))
    {
        return "prompt";
    }
    else if (string.contains("disa"))
    {
        return "disa";
    }
    else
    {
        return "";
    }
}

Array.prototype.in_array = function(e)  
{  
    if (e.beginsWith('playback('))
    { 
        e = e.lChop('playback(');
        e = e.rChop(')');
    }

    for(i=0;i<this.length;i++)  
    {  
        if(this[i] == e)  
            return true;  
    }  
    return false;  
}  

var prepareEditItemForm = function (this_menu)
{
    EDIT_VMENU = this_menu;
    isNewVmenu = false;

    VoiceMenus_miscFunctions.reset_VoicemenuFields();
    var THIS_VM = parent.sessionData.pbxinfo.voicemenus[EDIT_VMENU];
    ASTGUI.updateFieldToValue('ivr_name', THIS_VM.getProperty('comment'));
    VMEXT = ASTGUI.parseContextLine.getExten(THIS_VM.getProperty('alias_exten'));
    ASTGUI.updateFieldToValue('ivr_ext', VMEXT);
    
    var include_list = THIS_VM.getProperty('includes')
    if ( include_list && arrIndexOf(include_list, ASTGUI.contexts.OutboundAllroutes) != -1 )
    {
        _$('ivr_dialTrunk').checked = true;
        //ASTGUI.updateFieldToValue('ivr_trunkExt', THIS_VM.getProperty('trunk'));
        //_$('ivr_disPin').checked = THIS_VM.getProperty('dispin');
    }

    _$('ivr_permission').value = THIS_VM.getProperty('permission') ? THIS_VM.getProperty('permission') : "internal";
    
    //_$('ivr_trunkExt').disabled = !(_$('ivr_dialTrunk').checked);
    //_$('ivr_disPin').disabled = !(_$('ivr_dialTrunk').checked);
    _$('ivr_permission').disabled = !(_$('ivr_dialTrunk').checked);
    
    var value = THIS_VM.getProperty('welcome_prompt');
    value = value.replace(/\\\(/g, '\(');

    value = ((value == "welcome" || LISTOFRECORDFILES.in_array(value)) ? value : "");
    ASTGUI.updateFieldToValue('prompt_welcome', value);
    ASTGUI.updateFieldToValue('digit_timeout', THIS_VM.getProperty('digit_timeout'));
    ASTGUI.updateFieldToValue('ivr_timeout', THIS_VM.getProperty('timeout'));

    value = THIS_VM.getProperty('timeout_prompt');
    value = value.replace(/\\\(/g, '\(');
    value = (( value == "ivr-create-timeout" || LISTOFRECORDFILES.in_array(value)) ? value : "");
    ASTGUI.updateFieldToValue('prompt_timeout', value);

    value = THIS_VM.getProperty('invalid_prompt');
    value = value.replace(/\\\(/g, '\(');
    value = ((value == "invalid" || LISTOFRECORDFILES.in_array(value)) ? value : "");
    ASTGUI.updateFieldToValue('prompt_invalid', value);


    var num_list = ["1", "2", "3", "4", "5"];
    var loop = THIS_VM.getProperty('tloop');
    if (!num_list.contains(loop))
    {
        loop = "1";
    }

    ASTGUI.updateFieldToValue('ivr_tloop', loop);
    loop = THIS_VM.getProperty('iloop');
    if (!num_list.contains(loop))
    {
        loop = "1";
    }
    ASTGUI.updateFieldToValue('ivr_iloop', loop);

    _$('ivr_dialOther').checked = (THIS_VM.includes.length > 1) ? true : false;
    //CURRENT_ACTIONS = ASTGUI.cloneObject(THIS_VM.steps);

    var lan = THIS_VM.getProperty('language');
    var option = _$('ivr_language').options;
    var lan_exist = false;
    for(var i=0; i < option.length; i++)
    {
        if (option[i].value == lan)
        {
            lan_exist = true;
            break;
        }
    }
    
    if (lan)
    {
        if (lan_exist)
        {
            ASTGUI.updateFieldToValue('ivr_language', lan);
        }
        else
        {
            ASTGUI.updateFieldToValue('ivr_language', "en");
        }
    }
        
    (function ()
    {
        KEYPRESS_LIST.each(function (this_key)
        {
            var event_key = this_key;

            if (this_key == "10")
            {
                event_key = "#"
            }
            else if (this_key == "11")
            {
                event_key = "*"
            }
          
            var option_value = THIS_VM.keypress_events[event_key];
            if (option_value)
            {
                var value =show_as(option_value);
                var option_key = 'keypressevent_' + this_key;

                _$('keypress_' + this_key).value = value;

                if (value != "" && value != "hangup")
                {
                    var option_list;

                    if (value == "prompt")
                    {
                        option_list =  LISTOFRECORDFILES;
                        appendsoundlist(option_key, LISTOFRECORDFILES, 0);
                    }
                    else
                    {
                        option_list = [];
                        var list = parent.miscFunctions.getDestinations(value);
                        list.each(function(option){
                            option_list.push(option.optionValue);
                        });
                        ASTGUI.selectbox.populateArray(option_key, list);
                    }

                    option_value = ((option_list.in_array(option_value)) ? option_value : "");

                    if (option_value == "" && !option_list.in_array(""))
                    {
                        ASTGUI.selectbox.insert_before(option_key, $P.lang("LANG133"), '', 0);
                    }
                    $("#" + option_key).show();
                    _$(option_key).value =  option_value;
                }
                else
                {
                    $("#" + option_key).hide();
                }
            }
            else
            {
                _$('keypress_' + this_key).value = "";
                _$('keypressevent_' + this_key).value = "";
            }
        });
    })();

};

var edit_voiceMenu_KeyPressActions_form = function ()
{
    $('#div_vmenu_keyPressActions').show();
};


var updateVoiceMenus_Table = function ()
{
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function
    var lvms = parent.sessionData.pbxinfo.voicemenus.getOwnProperties();
    var TBL = _$('table_VoiceMenuslist');
    if (lvms.length == 0)
    {
        ASTGUI.domActions.clear_table(TBL);
        if (TBL) {
            var newRow = TBL.insertRow(-1);
            newRow.className = 'even';
            addCell(newRow, { html: $P.lang("LANG129").format($P.lang("LANG134")), locale: "LANG129 LANG134"});
        }
        return;
    } else
    {

        (function ()
        { // add first row
            if (TBL) {
                var newRow = TBL.insertRow(-1);
                newRow.className = "frow";
                //addCell(newRow, { html: '', width: '15px' });
                addCell(newRow, { html: "<span locale='LANG85'>"+$P.lang('LANG85')+"</span>"});
                addCell(newRow, { html: "<span locale='LANG135'>"+$P.lang('LANG135')+"</span>"});
                addCell(newRow, { html: "<span locale='LANG1445'>"+$P.lang('LANG1445')+"</span>"});
                addCell(newRow, { html: "<span locale='LANG1447'>"+$P.lang('LANG1447')+"</span>"});
                addCell(newRow, { html: "<span locale='LANG2540'>"+$P.lang('LANG2540')+"</span>"});
                addCell(newRow, { html: "<span locale='LANG74'>"+$P.lang('LANG74')+"</span>"});
            }
        })();

        lvms.each(function (vm_name)
        {
            var THIS_VM = parent.sessionData.pbxinfo.voicemenus[vm_name];
            var lbl = THIS_VM.comment || vm_name;
            var ae_line = THIS_VM.getProperty('alias_exten');
            var ae = (ae_line) ? ASTGUI.parseContextLine.getExten(ae_line) : '';
            if (TBL) {
                var newRow = TBL.insertRow(-1);
                newRow.className = ((TBL.rows.length) % 2 == 1) ? 'odd' : 'even';
                //addCell(newRow, { html: '', width: '15px' });
                addCell(newRow, { html: ae });
                addCell(newRow, { html: lbl });
                var str = (THIS_VM.includes.length > 1) ? "LANG136" : "LANG137";
                addCell(newRow, { html:  $P.lang(str),locale: str}); // dial other extensions
                str = (arrIndexOf(THIS_VM.includes, ASTGUI.contexts.OutboundAllroutes) != -1 )? "LANG136" : "LANG137";
                addCell(newRow, { html:  $P.lang(str), locale: str});
                addCell(newRow, { html: THIS_VM.timeout ? THIS_VM.timeout : '--'});
                var tmp = "<button class='options edit' onclick=\"showEditForm('" + vm_name + "')\" localetitle=\"LANG738\" title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                          "<button class='options del' onclick=\"delete_voiceMenu_confirm('" + vm_name + "')\" localetitle=\"LANG739\" title=" + $P.lang("LANG739") + "></button>";

                addCell(newRow, { html: tmp });
            }
        });
    }
};

var link_prompt= function()
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG843").format($P.lang("LANG238")),
        buttons: {
            ok: function ()
            {
                top.module.jumpMenu('menuprompts_record.html');
            },
            cancel: function ()
            {
                top.dialog.container.show();
                top.dialog.shadeDiv.show();
            }
        }
    });
};

var keypress_switch= function(id)
{
    var value = _$(id).value;
    var array = parent.miscFunctions.getDestinations(value);
    var eventid = "keypressevent_" + id.split('_')[1];
    var $eventid = $("#" + eventid);
    $eventid.empty(); 

    if (value == "hangup" || value == "")
    {
        $eventid.hide();
        $eventid.prev().hide();
    }
    else if (value== "prompt")
    {
        $eventid.show();
        $eventid.prev().show();
        appendsoundlist(eventid, LISTOFRECORDFILES, 0);
    }
    else
    {
        $eventid.show();
        $eventid.prev().show();
        if (array.length)
        {
            ASTGUI.selectbox.populateArray(eventid, array);
        }
        else
        {
            ASTGUI.selectbox.append(eventid, $P.lang("LANG133"), "");
        }
    }
    top.Custom.init(document, $eventid[0]);
};
