/*
* Asterisk-GUI	- an Asterisk configuration interface
*
* paging.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 *  have no limit member, when has many of members, The page display unsightly
 *  modify select,checkbox and radiobox elements style
 *
 */
$P = top.$;
var mWindow = top.window.frames["mainscreen"]; // this will be the main window

var isNewPgGrp;
var PgGrp_EDITING; // page group being edited
var listOfDialDevices = [];


window.onload = function ()
{
    $P.lang(document, true);
    var mode = gup("mode");
    var item = gup("item");
    loadDOMelements();
    if (mode)
    { 
        initForm();
        if (mode == "add")
        {
            prepareAddItemForm();
        }
        else if (mode == "edit")
        {
            prepareEditItemForm(item);
        }
        init_validator();
        top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG604"));
        $("title").attr("locale", "LANG604");
        update_PageGroupsTable();
        sortTable({
            "tableId": 'table_pageGroups_list',
            sortFuncs: {
                "sort_1": ASTGUI.extenStyleCmpFunc
            }
        }).sortTableInitial(1).bindTitles( 
            {idx: 0, isDis: true}, {idx: 1}
        );
    }
};


var init_validator = function () {
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "text_pgExten": {
                required: true,
                digits: true,
                extensionExists: [PgGrp_EDITING]
            },
            "select_pg_type": {
                required: true
            },
            "select_ringthesechannels": {
                selectItemMin: 1,
                //selectItemMax: max_member,
                selectMemberLength: 900 //consider AMI buffer
            }
        },
        submitHandler: function() {
            save_pageGroup_work();
        }
    });

};

var initForm = function ()
{
    try
    {
        (function ()
        {
            var t = parent.pbx.users.list();
            t.each(function (usr)
            {
                if (parent.sessionData.pbxinfo['users'][usr]['hassip'] && parent.sessionData.pbxinfo['users'][usr]['hassip'] == 'yes')
                {
                    listOfDialDevices.push('SIP/' + usr);
                }
                if (parent.sessionData.pbxinfo['users'][usr]['hasiax'] && parent.sessionData.pbxinfo['users'][usr]['hasiax'] == 'yes')
                {
                    listOfDialDevices.push('IAX2/' + usr);
                }
            });
            t = parent.sessionData.FXS_PORTS_DETECTED;
            t.each(function (fxs)
            {
                listOfDialDevices.push(parent.sessionData.DahdiDeviceString + '/' + fxs);
            });

            ASTGUI.events.add(DOM_button_add_toringlist, 'click', function ()
            {
            
                if (!DOM_select_fromlistofchannels.value)
                {
                    return;
                }

                $(':selected', DOM_select_fromlistofchannels).each( function (idx, item) {
                    var val = $(item).val();
                    var text = $(item).html() || val;
                    $(item).remove();
                    ASTGUI.selectbox.append(DOM_select_ringthesechannels, text, val);
                });

                $P("#select_ringthesechannels", document).valid();
            });
            ASTGUI.events.add(DOM_button_remove_fromringlist, 'click', function ()
            {

                if (!DOM_select_ringthesechannels.value)
                {
                    return;
                }

                $(':selected', DOM_select_ringthesechannels).each( function (idx, item) {
                    var val = $(item).val();
                    var text = $(item).html() || val;
                    $(item).remove();
                    ASTGUI.selectbox.append(DOM_select_fromlistofchannels, text, val);
                });

                $P("#select_ringthesechannels", document).valid();
            });
            ASTGUI.events.add(DOM_button_removeall_fromringlist, 'click', function ()
            {
                ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
                ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);
                listOfDialDevices.each(function (device)
                {
                    ASTGUI.selectbox.append(DOM_select_fromlistofchannels, GetDevice_UserName(device), device);
                });
            });

            ASTGUI.events.add('button_addall_toringlist', 'click', function ()
            {
                ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
                ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);
                listOfDialDevices.each(function (device)
                {
                    ASTGUI.selectbox.append(DOM_select_ringthesechannels, GetDevice_UserName(device), device);
                });
            });

        })();
    }
    catch (err)
    {

    }
}


var loadDOMelements = function ()
{
    DOM_pagegroup_editdiv = _$('pagegroup_editdiv');
    DOM_table_pageGroups_list = _$('table_pageGroups_list');

    DOM_text_pageGroup_Exten = _$('text_pgExten');
    DOM_select_pageGroup_Type = _$('select_pg_type');
    DOM_chk_pageGroup_beep = _$('chk_pgrp_beep');

    DOM_select_ringthesechannels = _$('select_ringthesechannels');
    DOM_select_fromlistofchannels = _$('select_fromlistofchannels');
    DOM_button_addall_toringlist = _$('button_addall_toringlist');
    DOM_button_add_toringlist = _$('button_add_toringlist');
    DOM_button_remove_fromringlist = _$('button_remove_fromringlist');
    DOM_button_removeall_fromringlist = _$('button_removeall_fromringlist');


};

var save_pageGroup_work = function ()
{

    var PAGE_EXTEN = ASTGUI.getFieldValue(DOM_text_pageGroup_Exten);

    var tmp_options = [];
    if (ASTGUI.getFieldValue(DOM_select_pageGroup_Type) == '2way') { tmp_options.push('d'); }
    if (!DOM_chk_pageGroup_beep.checked) { tmp_options.push('q'); }

    var TEMP_members = ASTGUI.selectbox.readAllValues(DOM_select_ringthesechannels);

    var tmp_new_line = PAGE_EXTEN + ',1,Macro(pagingintercom,' + TEMP_members.join('&') + ',' + tmp_options.join('') + ')';

    var tmp_after = function ()
    {

        var later = function ()
        {
            top.dialog.clearDialog();

            top.dialog.dialogMessage({
                type: 'success',
                content: $P.lang("LANG815"),
                callback: function ()
                {
                    mWindow.location.reload();
                }
            });
        };
        parent.pbx.paging.add(tmp_new_line, later);
    };

    if (!isNewPgGrp)
    { // if editing existing page group
        top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826')});
        parent.pbx.paging.remove(PgGrp_EDITING, tmp_after);
    } else
    {
        top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826')});
        tmp_after();
    }
};


var GetDevice_UserName = function (device)
{
    // device = SIP/6002 or DAHDI/1 or IAX2/6001
    var tmp_ext = device.afterChar('/');
    var tmp_techn = device.beforeChar('/');

    if (tmp_techn.toLowerCase() == 'dahdi')
    {
        /*
        for (var q in parent.sessionData.pbxinfo.users)
        {
            if (parent.sessionData.pbxinfo.users.hasOwnProperty(q))
            {
                if (parent.sessionData.pbxinfo.users[q].hasOwnProperty('dahdichan') && parent.sessionData.pbxinfo.users[q]['dahdichan'] == tmp_ext)
                {
                    // return q + '(AnalogPort ' + tmp_ext + ') ' + parent.sessionData.pbxinfo.users[q].getProperty('fullname');
                    var tmp_name = parent.sessionData.pbxinfo.users[q].getProperty('fullname');
                    return 'AnalogPort/' + tmp_ext + '(' + q + ')' + (tmp_name ? ' - ' + tmp_name : '');
                }
            }
        }
        */

        var idx = parent.sessionData.FXS_PORTS_DETECTED.indexOf(tmp_ext) + 1;
        return 'FXS ' + idx;
    }
    else
    {
        if (parent.sessionData.pbxinfo.users.hasOwnProperty(tmp_ext))
        {
            var tmp_name = parent.sessionData.pbxinfo.users[tmp_ext].getProperty('fullname');
            //return tmp_ext + '(' + tmp_techn + ')' + ' ' + tmp_name
            return tmp_techn + '/' + tmp_ext + (tmp_name ? ' \"' + tmp_name + '\"' : '');
        }
        return '';
    }
};


var resetFields = function ()
{
    if (isNewPgGrp)
    {
        // _$('pagegroup_editdiv_title').innerHTML = 'New Page/Intercom Group';

        ASTGUI.resetTheseFields([DOM_text_pageGroup_Exten, DOM_select_pageGroup_Type, DOM_chk_pageGroup_beep]);
        ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
        ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);
        listOfDialDevices.each(function (device)
        {
            ASTGUI.selectbox.append(DOM_select_fromlistofchannels, GetDevice_UserName(device), device);
        });

        var tmp_allextensions = ASTGUI.cloneObject(parent.miscFunctions.getAllExtensions());
        //ybiaozhou remove initial extension value
        //	DOM_text_pageGroup_Exten.value  = tmp_allextensions.firstAvailable( parent.sessionData.GUI_PREFERENCES.getProperty('rge_start') );
        return;
    }

    var c = parent.sessionData.pbxinfo['pagegroups'];
    // _$('pagegroup_editdiv_title').innerHTML = 'Edit Page/Intercom Group - ' + PgGrp_EDITING;

    var line = '';
    try
    {
        line = c[c.indexOfLike('exten=' + PgGrp_EDITING + ',1,Macro(pagingintercom')];
    } catch (err) { }

    if (!line) return;
    //var tmp_args = ASTGUI.parseContextLine.getArgs( line ); replace getArgs, for it will ignore empty arg
    var tmp_args = line.afterStr('Macro(') || '';
    tmp_args = tmp_args.split(',');
    tmp_args[2] = tmp_args[2] || '';  //ybiaozhou make sure tmp_args[2] - options  has value,  when exten like Marco(pagxxx, SIP/6000,);

    if (!tmp_args[1])
    {
        //        tmp_args[1] = '';
        var tmp_members = [];
    }
    else
    {
        var tmp_members = tmp_args[1].split('&');
    }


    ASTGUI.updateFieldToValue(DOM_text_pageGroup_Exten, PgGrp_EDITING);
    DOM_text_pageGroup_Exten.disabled = true;
    ASTGUI.updateFieldToValue(DOM_select_pageGroup_Type, tmp_args[2].contains('d') ? '2way' : '1way');
    DOM_chk_pageGroup_beep.checked = !tmp_args[2].contains('q');

    ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
    var mbrs = ASTGUI.cloneObject(tmp_members);

    if (tmp_members.length > 0)
    {
        mbrs.each(function (device)
        {
            ASTGUI.selectbox.append(DOM_select_ringthesechannels, GetDevice_UserName(device), device);
        });
    }


    ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);
    listOfDialDevices.each(function (device)
    {
        if (!mbrs.contains(device))
        {
            ASTGUI.selectbox.append(DOM_select_fromlistofchannels, GetDevice_UserName(device), device);
        }
    });
};


var delete_pageGroup_confirm = function (d)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(d),
        buttons: {
            ok: function ()
            {
                parent.pbx.paging.remove(d, function ()
                {
                    top.dialog.clearDialog();
                    top.dialog.dialogMessage({
                        type: 'success',
                        content: $P.lang("LANG816"),
                        callback: function ()
                        {
                            mWindow.location.reload();
                        }
                    });
                });
            }
        }
    });
};

var prepareAddItemForm = function ()
{
    PgGrp_EDITING = '';
    isNewPgGrp = true;
    resetFields();
    $("#editForm").show();
    $("#text_pgExten").focus();

};


var prepareEditItemForm = function (item)
{
    PgGrp_EDITING = item;
    isNewPgGrp = false;
    resetFields();

    $("#editForm").show();
};


var update_PageGroupsTable = function ()
{
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function
    (function ()
    { // add first row
        if (DOM_table_pageGroups_list) {
            var newRow = DOM_table_pageGroups_list.insertRow(-1);
            newRow.className = "frow";
            //addCell(newRow, { html: '', width: '10px' });
            addCell(newRow, { html: "<span locale='LANG85'>"+$P.lang('LANG85')+"</span>", width: "150px" });
            addCell(newRow, { html: "<span locale='LANG84'>"+$P.lang('LANG84')+"</span>", width: "200px" });
            addCell(newRow, { html: "<span locale='LANG128'>"+$P.lang('LANG128')+"</span>"});
            addCell(newRow, { html: "<span locale='LANG74'>"+$P.lang('LANG74')+"</span>", width: "80px" });
        }
    })();

    var c = parent.sessionData.pbxinfo['pagegroups'];
    c.each(function (line)
    {
        var this_exten = ASTGUI.parseContextLine.getExten(line);
        //	var tmp_args = ASTGUI.parseContextLine.getArgs( line ); replace getArgs, for it will ignore empty arg
        var tmp_args = line.afterStr('Macro(') || '';
        tmp_args = tmp_args.split(',');
        tmp_args[2] = tmp_args[2] || '';  //ybiaozhou make sure tmp_args[2] - options  has value,  when exten like Marco(pagxxx, SIP/6000,);

        if (!tmp_args[1])
        {
            tmp_args[1] = '';
            var tmp_members = [];
        }
        else
        {
            var tmp_members = tmp_args[1].split('&')
            var tmp_length = tmp_members.length;
            // DAHDI display conversion, e.g. DAHDI/17 should display as FXS 1
            for (var i = 0; i < tmp_length; i++)
            {
                var item = tmp_members[i];
                if (item.beginsWith('DAHDI/'))
                {
                    var port = item.split('/')[1] || '';
                    var idx = parent.sessionData.FXS_PORTS_DETECTED.indexOf(port) + 1;
                    tmp_members[i] = 'FXS ' + idx;
                }
            }
            tmp_members = tmp_members.join(', ');
        }
        if (DOM_table_pageGroups_list) {
            var newRow = DOM_table_pageGroups_list.insertRow(-1);
            var rn = DOM_table_pageGroups_list.rows.length;
            newRow.className = ((rn) % 2 == 1) ? 'odd' : 'even';

            //addCell(newRow, { html: '' });
            addCell(newRow, { html: this_exten });
            addCell(newRow, { html: (tmp_args[2].contains('d') ? $P.lang('LANG1162') : $P.lang('LANG1161') ), locale: (tmp_args[2].contains('d') ? 'LANG1162' : 'LANG1161') });
            addCell(newRow, { html: tmp_members });

            var tmp = "<button class='options edit' onclick=\"showEditForm('" + this_exten + "')\" localeTitle='LANG738' title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                      "<button class='options del' onclick=\"delete_pageGroup_confirm('" + this_exten + "')\" localeTitle='LANG739' title=" + $P.lang("LANG739") + "></button>";

            addCell(newRow, { html: tmp });
        }
    });
if (DOM_table_pageGroups_list) {
    if (DOM_table_pageGroups_list.rows.length == 1)
    {
        ASTGUI.domActions.clear_table(DOM_table_pageGroups_list);
        var newRow = DOM_table_pageGroups_list.insertRow(-1);
        newRow.className = 'even';
        addCell(newRow, { html: $P.lang('LANG129').format($P.lang('LANG604')), locale: 'LANG129 LANG604'});
        return;
    }    
}
};

