/*
* Asterisk-GUI	- an Asterisk configuration interface
*
* preferences.html functions
*
* Copyright (C) 2006-2011, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
* Erin Spiceland <espiceland@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 *  Added support of random password
 *  Modified the display of extension
 *  Modified the range
 *  There are some style inconsistencies and abbreviationissues
 *
 */
$P = top.$;
var OPEXTENSION = '';

var loadDOMelements = function ()
{
    DOM_obCid_input = _$('obCid_input');
    DOM_op_ext = _$('op_ext');
};

window.onload = function()
{
    $P.lang(document, true);
    top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG667"));
    $("title").attr("locale", "LANG667");
    var fd_text = 'Reset Configuration';
    var cusers = parent.pbx.users.list();
    var op_tmp;
    loadDOMelements();
    ASTGUI.selectbox.append(DOM_op_ext, $P.lang("LANG133"), '', 'LANG133');
    cusers.each(function (user)
    {
        op_tmp = 'Goto(' + ASTGUI.contexts.CallingPlan + ',' + user + ',1)';
        ASTGUI.selectbox.append(DOM_op_ext,  $P.lang("LANG85") +' ' + user, op_tmp, "LANG86" +" "+ "'"+user+"'");
    });
    (function ()
    {
        var c = context2json({ filename: 'extensions.conf', context: 'globals', usf: 1 });
        var ringtime = (c.hasOwnProperty('RINGTIME')) ? c['RINGTIME'] : '20';
        ASTGUI.updateFieldToValue('op_ringTimeOut_Voicemail', ringtime);
        try
        {
            ASTGUI.updateFieldToValue(DOM_obCid_input, c[ASTGUI.globals.obcidstr] || '');
            ASTGUI.updateFieldToValue('obCidName_input', c[ASTGUI.globals.obcidNamestr] || '');
        } catch (err) { }
        
        var f = context2json({ filename: 'features.conf', context: 'general', usf: 1 });
        _$("courtesytone").checked = (f.hasOwnProperty('recstarttone')) ? true : false;

    })();
    try
    {
        OPEXTENSION = (parent.sessionData.pbxinfo.localextensions && parent.sessionData.pbxinfo.localextensions.Operator) ? parent.sessionData.pbxinfo.localextensions.Operator : '';
        //Goto(from-internal,6000,1)
        ASTGUI.selectbox.selectOption(DOM_op_ext, OPEXTENSION);

    } catch (err) { }
    loadExtensions_ranges();
    init_validator();
    loadPickupGroup();
    top.Custom.init(document);
};

var loadPickupGroup = function ()
{
    var ct = ct = _$('pickup_div');
    var picklist = parent.sessionData.pickupgroup;
    for (var item in picklist)
    {
        if (!picklist.hasOwnProperty(item))
        {
            continue;
        }
        
        var h = document.createElement('div');
        h.style.position = "relative";

        var h_1 = document.createElement('div');
        var h_3 = document.createElement('div');
        //var h_2_orig = document.createElement('input');
        //var h_2 = h_2_orig.cloneNode(true);
        h.className = "field-cell";
        h_1.className = "field-label";

        h_1.setAttribute("glabel", "@LANG2512 '" + item+"'");
        h_3.className = "field-content";
        //h_2.id = item;
        h_3.innerHTML = "<input id="+item+" name=pickupgroup maxlength='32' type='text' value="+picklist[item]+">";
        h.appendChild(h_1);
        h.appendChild(h_3);
        //h_3.appendChild(h_2);
        
        ct.appendChild(h);
    }
    
    $P.lang(document, true);
        
    $P("[name=pickupgroup]", document).rules("add", {
        required: true,
        letterDigitUndHyphen: true
    });
}

$(function () {
    $("#disable_extension_ranges").bind("click", function (ev) {
        saveDisableExtensionRanges();
        ev.stopPropagation();
    });
});

var loadExtensions_ranges = function ()
{
    var c = context2json({ filename: ASTGUI.globals.configfile, context: 'general', usf: 1 });
    ASTGUI.updateFieldToValue('ue_start', c.getProperty('ue_start'));
    ASTGUI.updateFieldToValue('ue_end', c.getProperty('ue_end'));
    ASTGUI.updateFieldToValue('mm_start', c.getProperty('mm_start'));
    ASTGUI.updateFieldToValue('mm_end', c.getProperty('mm_end'));
    ASTGUI.updateFieldToValue('qe_start', c.getProperty('qe_start'));
    ASTGUI.updateFieldToValue('qe_end', c.getProperty('qe_end'));
    ASTGUI.updateFieldToValue('vme_start', c.getProperty('vme_start'));
    ASTGUI.updateFieldToValue('vme_end', c.getProperty('vme_end'));
    ASTGUI.updateFieldToValue('rge_start', c.getProperty('rge_start'));
    ASTGUI.updateFieldToValue('rge_end', c.getProperty('rge_end'));
    ASTGUI.updateFieldToValue('vmg_start', c.getProperty('vmg_start'));
    ASTGUI.updateFieldToValue('vmg_end', c.getProperty('vmg_end'));

    var disabled = c.getProperty('disable_extension_ranges');
    ASTGUI.updateFieldToValue('disable_extension_ranges', disabled == 'yes' ? 'on' : 'off');
    ASTGUI.updateFieldToValue('weak_password', top.sessionData.GUI_PREFERENCES.weak_password == 'no' ? 'no' : 'yes');
    ASTGUI.updateFieldToValue('rand_password', top.sessionData.GUI_PREFERENCES.rand_password == 'no' ? 'no' : 'yes');
    refreshExtensionRangesForm(disabled);
};



var verify_Ranges = function ()
{
    var arrF = ['ue_start', 'ue_end', 'mm_start', 'mm_end', 'qe_start', 'qe_end', 'vme_start', 'vme_end', 'rge_start', 'rge_end', 'vmg_start', 'vmg_end'];
    var arr = [];
    for (var i = 0; i < arrF.length; i++)
    {
        arr.push(_$(arrF[i]));
    }

    var obj = new Object();

    var obj = {
        "ue_start": 'users',
        "mm_start": 'conferences',
        "qe_start": 'queues',
        "vme_start": 'voicemenus',
        "rge_start": 'ringgroups',
        "vmg_start": 'vmgroups'
    };

    for (var i = 0; i < arrF.length; i++)
    {
        var tmp = arrF[i];
        if (!arrF[i].endsWith('_start')) { continue; }
        var start = ASTGUI.getFieldValue(arrF[i]), end = ASTGUI.getFieldValue(arrF[i + 1]);
        start = parseInt(start, 10);
        end = parseInt(end, 10);

        //check existed extensions out of range
        var conflicts = [];
        if (obj.hasOwnProperty(tmp))
        {
            var modules = new Object;
            modules[obj[tmp]] = true;
            //e.g. tmp: ue_start , obj[tmp]: 'user', moduleExtension : user's extensions
            var moduleExtension = top.miscFunctions.getAllExtensions({ "modules": modules});

            $.each(moduleExtension, function (idx, value) {
                var valueNum = parseInt(value, 10);
                if (!isNaN(valueNum) && !value.isValueInBetween(start, end))
                {
                    conflicts.push(value);
                }

            });
        }

        if (conflicts.length > 0)
        {
            exten_prompt = conflicts.slice(0,5).join(',');
            exten_prompt = conflicts.length > 5 ? '[' + exten_prompt + ', ...]' 
                                                : '[' + exten_prompt + ']';

            var msg = $P.lang("LANG1600") + exten_prompt + '<br/>'+ $P.lang("LANG1601") 
            + '<b>[' + ASTGUI.getFieldValue(arrF[i]) + ',' + ASTGUI.getFieldValue(arrF[i + 1]) + ']</b>.';

            top.dialog.dialogMessage({ type: 'error', content: msg});
            return false;
        }

    }

    return true;
};

var saveDisableExtensionRanges = function ()
{
    var disabled = ASTGUI.getFieldValue("disable_extension_ranges");
    ASTGUI.updateFieldToValue('disable_extension_ranges', 'no');
    if (disabled == 'yes')
    {
        top.dialog.dialogConfirm({
            confirmStr: $P.lang("LANG864"),
            buttons: {
                ok: function ()
                {
                    ASTGUI.updateFieldToValue('disable_extension_ranges', 'yes');
                    refreshExtensionRangesForm(disabled);
                }
            }
        });
    }
    else
    {
        refreshExtensionRangesForm(disabled);
    }
};

var refreshExtensionRangesForm = function (disabled)
{
    if (disabled == 'yes')
    {
        _$('ue_start').disabled = true;
        _$('ue_end').disabled = true;
        _$('mm_start').disabled = true;
        _$('mm_end').disabled = true;
        _$('vme_start').disabled = true;
        _$('vme_end').disabled = true;
        _$('rge_start').disabled = true;
        _$('rge_end').disabled = true;
        _$('qe_start').disabled = true;
        _$('qe_end').disabled = true;
        _$('vmg_start').disabled = true;
        _$('vmg_end').disabled = true;
        $('#reset_ranges_button').hide();
    } else
    {
        _$('ue_start').disabled = false;
        _$('ue_end').disabled = false;
        _$('mm_start').disabled = false;
        _$('mm_end').disabled = false;
        _$('vme_start').disabled = false;
        _$('vme_end').disabled = false;
        _$('rge_start').disabled = false;
        _$('rge_end').disabled = false;
        _$('qe_start').disabled = false;
        _$('qe_end').disabled = false;
        _$('vmg_start').disabled = false;
        _$('vmg_end').disabled = false;
        $('#reset_ranges_button').show();
    }
};

var save_changes = function ()
{
    var time = ASTGUI.getFieldValue("op_ringTimeOut_Voicemail");

    var disabled = ASTGUI.getFieldValue("disable_extension_ranges");
    if (disabled == 'no' && !verify_Ranges()) { return; }

    var processSaving = function ()
    {
        var x = new listOfSynActions(ASTGUI.globals.configfile);
        var weak_password = ASTGUI.getFieldValue("weak_password");
        x.new_action('update', 'general', 'disable_extension_ranges', disabled);
        x.new_action('update', 'general', 'weak_password', weak_password);
        x.new_action('update', 'general', 'rand_password', ASTGUI.getFieldValue("rand_password"));
        x.callActions();
        
        var f = new listOfSynActions("features.conf");

        if (_$("courtesytone").checked)
        {
            f.new_action('autoupdate', 'general', 'recstarttone', 'dialog-being-recorded');
            f.new_action('autoupdate', 'general', 'recstoptone', 'dialog-stop-recording');
        }
        else
        {
            f.new_action('delete', 'general', 'recstarttone', '');
            f.new_action('delete', 'general', 'recstoptone', '');
        }
        
        f.callActions();
        
        top.sessionData.GUI_PREFERENCES.disable_extension_ranges = disabled;
        top.sessionData.GUI_PREFERENCES.weak_password = ASTGUI.getFieldValue("weak_password");
        top.sessionData.GUI_PREFERENCES.rand_password = ASTGUI.getFieldValue("rand_password");
        parent.sessionData.pbxinfo.GLOBALS[ASTGUI.globals.obcidstr] = DOM_obCid_input.value;
        parent.sessionData.pbxinfo.GLOBALS['RINGTIME'] = time;

        var u = new listOfSynActions('extensions.conf');
        u.new_action('update', 'globals', ASTGUI.globals.obcidstr, ASTGUI.getFieldValue(DOM_obCid_input));
        u.new_action('update', 'globals', ASTGUI.globals.obcidNamestr, ASTGUI.getFieldValue('obCidName_input'));
        u.new_action('update', 'globals', 'RINGTIME', time);
        u.new_action('update', 'globals', 'RECORD_PROMPT', ASTGUI.getFieldValue("courtesytone"));

        if ( !u.FILE_CONTENT['default'][0] || u.FILE_CONTENT['default'][0].indexOf("Goto") != -1 )
        {
            u.new_action('append', 'default', 'exten', 'o,1,Set(__FROM_IVR=operator)');
        }
        
        if ( u.FILE_CONTENT['default'][1] && u.FILE_CONTENT['default'][1].indexOf(OPEXTENSION) != -1 )
        {
            u.new_action('delete', 'default', 'exten', '', 'o,n,' + OPEXTENSION);
        }

        if (ASTGUI.getFieldValue(DOM_op_ext) != '')
        {
            u.new_action('append', 'default', 'exten', 'o,n,' + ASTGUI.getFieldValue(DOM_op_ext));
        }
        var resp = u.callActions();

        u.clearActions(ASTGUI.globals.configfile);
        u.new_action('update', 'general', 'ue_start', ASTGUI.getFieldValue('ue_start'));
        u.new_action('update', 'general', 'ue_end', ASTGUI.getFieldValue('ue_end'));
        u.new_action('update', 'general', 'mm_start', ASTGUI.getFieldValue('mm_start'));
        u.new_action('update', 'general', 'mm_end', ASTGUI.getFieldValue('mm_end'));
        u.new_action('update', 'general', 'qe_start', ASTGUI.getFieldValue('qe_start'));
        u.new_action('update', 'general', 'qe_end', ASTGUI.getFieldValue('qe_end'));
        u.callActions();
        u.clearActions();
        u.new_action('update', 'general', 'vme_start', ASTGUI.getFieldValue('vme_start'));
        u.new_action('update', 'general', 'vme_end', ASTGUI.getFieldValue('vme_end'));
        u.new_action('update', 'general', 'rge_start', ASTGUI.getFieldValue('rge_start'));
        u.new_action('update', 'general', 'rge_end', ASTGUI.getFieldValue('rge_end'));
        u.new_action('update', 'general', 'vmg_start', ASTGUI.getFieldValue('vmg_start'));
        u.new_action('update', 'general', 'vmg_end', ASTGUI.getFieldValue('vmg_end'));
        u.new_action('update', 'pickupgroup', '1', ASTGUI.getFieldValue('1'));        
        u.new_action('update', 'pickupgroup', '2', ASTGUI.getFieldValue('2'));
        u.new_action('update', 'pickupgroup', '3', ASTGUI.getFieldValue('3'));
        u.new_action('update', 'pickupgroup', '4', ASTGUI.getFieldValue('4'));
        u.new_action('update', 'pickupgroup', '5', ASTGUI.getFieldValue('5'));
        u.new_action('update', 'pickupgroup', '6', ASTGUI.getFieldValue('6'));
        u.new_action('update', 'pickupgroup', '7', ASTGUI.getFieldValue('7'));
        u.new_action('update', 'pickupgroup', '8', ASTGUI.getFieldValue('8'));
        u.new_action('update', 'pickupgroup', '9', ASTGUI.getFieldValue('9'));
        u.new_action('update', 'pickupgroup', '10', ASTGUI.getFieldValue('10'));
        
        u.callActions();

        parent.sessionData.GUI_PREFERENCES.updateProperties({
            ue_start: ASTGUI.getFieldValue('ue_start'),
            ue_end: ASTGUI.getFieldValue('ue_end'),
            mm_start: ASTGUI.getFieldValue('mm_start'),
            mm_end: ASTGUI.getFieldValue('mm_end'),
            qe_start: ASTGUI.getFieldValue('qe_start'),
            qe_end: ASTGUI.getFieldValue('qe_end'),
            vme_start: ASTGUI.getFieldValue('vme_start'),
            vme_end: ASTGUI.getFieldValue('vme_end'),
            rge_start: ASTGUI.getFieldValue('rge_start'),
            rge_end: ASTGUI.getFieldValue('rge_end'),
            vmg_start: ASTGUI.getFieldValue('vmg_start'),
            vmg_end: ASTGUI.getFieldValue('vmg_end')
        });

        parent.sessionData.pickupgroup.updateProperties({
            1: ASTGUI.getFieldValue('1'),
            2: ASTGUI.getFieldValue('2'),
            3: ASTGUI.getFieldValue('3'),
            4: ASTGUI.getFieldValue('4'),
            5: ASTGUI.getFieldValue('5'),
            6: ASTGUI.getFieldValue('6'),
            7: ASTGUI.getFieldValue('7'),
            8: ASTGUI.getFieldValue('8'),
            9: ASTGUI.getFieldValue('9'),
            10: ASTGUI.getFieldValue('10')
        });
        
        parent.sessionData.pbxinfo.localextensions.Operator = DOM_op_ext.value;
        top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG844"), callback: function () { window.location.reload(); } });
    };

    /* we will need this function after add the followme */
    //if (time < parent.sessionData.pbxinfo.GLOBALS['RINGTIME'])
    //{
    //    top.dialog.dialogMessage({ type: 'warning', content: $P.lang("LANG865"), callback: processSaving });
    //}
    //else
    //{
        processSaving();
    //}
};


var reset_ranges_default = function ()
{
    ASTGUI.updateFieldToValue('ue_start', '5000');
    ASTGUI.updateFieldToValue('ue_end', '6299');
    ASTGUI.updateFieldToValue('mm_start', '6300');
    ASTGUI.updateFieldToValue('mm_end', '6399');
    ASTGUI.updateFieldToValue('vme_start', '7000');
    ASTGUI.updateFieldToValue('vme_end', '7100');
    ASTGUI.updateFieldToValue('rge_start', '6400');
    ASTGUI.updateFieldToValue('rge_end', '6499');
    ASTGUI.updateFieldToValue('qe_start', '6500');
    ASTGUI.updateFieldToValue('qe_end', '6599');
    ASTGUI.updateFieldToValue('vmg_start', '6600');
    ASTGUI.updateFieldToValue('vmg_end', '6699');
};


var isRangeConflict = function (value, element) {
    if ($('#disable_extension_ranges:checked').length)
    {
        return true;
    }

    var selfRange = $(element).parent().parent();
    var start = $('[id$="_start"]', selfRange).val();
    var end = $('[id$="_end"]', selfRange).val();
    start = Number(start);
    end = Number(end);

    var otherRange = $('.extRange').not(selfRange);

    var length = otherRange.length;
    for (var i = 0; i < length; i++)
    {
        var comp_start = $('[id$="_start"]', otherRange[i]).val();
        var comp_end = $('[id$="_end"]', otherRange[i]).val();
        comp_start = Number(comp_start);
        comp_end = Number(comp_end);
        if (start.isValueInBetween(comp_start, comp_end))
        {
            return false;
        }
        else if (end.isValueInBetween(comp_start, comp_end))
        {
            return false;
        }
    }
    return true;

};

var compareLength = function (value, element) {
    if ($('#disable_extension_ranges:checked').length)
    {
        return true;
    }

    var selfRange = $(element).parent().parent();
    var start = $('[id$="_start"]', selfRange).val();
    var end = $('[id$="_end"]', selfRange).val();
    if (start && end && start.length > end.length)
    {
        return false;
    }

    return true;
};

var init_validator = function () {
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "obCid_input": {
                digits: true,
                minlength: 2
            },
            "obCidName_input": {
                alphanumeric: true, 
                nowhitespace: true,
                minlength: 2
            },
            "op_ringTimeOut_Voicemail": {
                required: true,
                digits: true,
                minlength: 2,
                range: [1, 600]
            },
            "ue_start": {
                required: true,
                digits: true,
                minlength: 2,
                smaller: [$P.lang("LANG560"), $P.lang("LANG561"), $('#ue_end')],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict]
            },
            "ue_end": {
                required: true,
                digits: true,
                minlength: 2,
                bigger: [$P.lang("LANG561"), $P.lang("LANG560"), $('#ue_start')],
                customCallback: [$P.lang("LANG2145"), compareLength],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "mm_start": {
                required: true,
                digits: true,
                minlength: 2,
                smaller: [$P.lang("LANG560"), $P.lang("LANG561"), $('#mm_end')],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "mm_end": {
                required: true,
                digits: true,
                minlength: 2,
                bigger: [$P.lang("LANG561"), $P.lang("LANG560"), $('#mm_start')],
                customCallback: [$P.lang("LANG2145"), compareLength],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "vme_start": {
                required: true,
                digits: true,
                minlength: 2,
                smaller: [$P.lang("LANG560"), $P.lang("LANG561"), $('#vme_end')],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "vme_end": {
                required: true,
                digits: true,
                minlength: 2,
                bigger: [$P.lang("LANG561"), $P.lang("LANG560"), $('#vme_start')],
                customCallback: [$P.lang("LANG2145"), compareLength],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "rge_start": {
                required: true,
                digits: true,
                minlength: 2,
                smaller: [$P.lang("LANG560"), $P.lang("LANG561"), $('#rge_end')],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "rge_end": {
                required: true,
                digits: true,
                minlength: 2,
                bigger: [$P.lang("LANG561"), $P.lang("LANG560"), $('#rge_start')],
                customCallback: [$P.lang("LANG2145"), compareLength],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "qe_start": {
                required: true,
                digits: true,
                minlength: 2,
                smaller: [$P.lang("LANG560"), $P.lang("LANG561"), $('#qe_end')],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "qe_end": {
                required: true,
                digits: true,
                minlength: 2,
                bigger: [$P.lang("LANG561"), $P.lang("LANG560"), $('#qe_start')],
                customCallback: [$P.lang("LANG2145"), compareLength],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "vmg_start": {
                required: true,
                digits: true,
                minlength: 2,
                smaller: [$P.lang("LANG560"), $P.lang("LANG561"), $('#vmg_end')],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            },
            "vmg_end": {
                required: true,
                digits: true,
                minlength: 2,
                bigger: [$P.lang("LANG561"), $P.lang("LANG560"), $('#vmg_start')],
                customCallback: [$P.lang("LANG2145"), compareLength],
                customCallback2: [$P.lang("LANG2144"), isRangeConflict] 
            }
        },
        newValidator: true
    });
};


var save_entrance = function()
{
    if ($P("#form", document).valid())
    {
        save_changes();
    }
};




