/*
* Asterisk-GUI	- an Asterisk configuration interface
*
* queues.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 *  Added multiple select function
 *  Modified select max items tooltips
 *  Modified hideWithBg to hide
 *  
 */
$P = top.$;

var isNewQueue;
var EDIT_Queue;
var ag_chkbxClass = "agents_Chkboxes";
var QUEUES_CONF;
var ul_agents = {};
var agent_count = 0;


window.onload = function ()
{
    $P.lang(document, true);
    loadDOMElements();
    QUEUES_CONF = config2json({ filename: 'queues.conf', usf: 1 });
    var mode = gup("mode");
    var item = gup("item");
    mWindow = top.window.frames["mainscreen"]; // this will be the main window 
    if (mode)
    {
        initForm();
        if (mode == "add")
        {
            prepareAddItemForm(); 
        }
        else if (mode == "edit")
        {
            prepareEditItemForm(item);
        }
        //validator
        var max_member = (parent.sessionData.GUI_PREFERENCES['qe_maxmember'] && Number(parent.sessionData.GUI_PREFERENCES['qe_maxmember']));
		if ($("#form").tooltip) {
			$("#form").tooltip();
		}
        $P("#form", document).validate({
            rules: {
                "edit_Ext": {
                    required: true,
                    digits: true,
                    extensionRange: ['qe_start', 'qe_end'],
                    extensionExists: true
                },
                "edit_label": {
                    required: true,
                    letterDigitUndHyphen: true,
                    customCallback: [$P.lang("LANG2137"), QueueExistsbyThisName]
                },
                "edit_strategy": {
                    required: true
                },
                "edit_leavewhenempty": {
                    required: true
                },
                "edit_PIN": {
                    required: '#chk_PIN',
                    digits: true,
                    minlength: 4
                },
                "edit_timeout": {
                    required: true,
                    digits: true
                },
                "edit_wrapuptime": {
                    required: true,
                    digits: true
                },
                "edit_maxlen": {
                    required: true,
                    digits: true
                },
                "edit_waittime": {
                    required: '#chk_waittime',
                    digits: true
                },
                "select_ringthesechannels": {
                    selectItemMax: [max_member, $P.lang('LANG1191')]
                }
            },
            submitHandler: function() {
                $P('#edit_Ext', document).rules('remove')
                $P('#edit_label', document).rules('remove')
                edit_queue_apply_work();
            }
        });
        isEnableWeakPw();
        top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG607"));
        $("title").attr("locale", "LANG607");
        updateQueuesTable();
        sortTable({
            sortTableToggle: false,
            "tableId": 'table_queueslist',
            sortFuncs: {
                "sort_1": ASTGUI.extenStyleCmpFunc
            }

        }).sortTableInitial(1).bindTitles( 
            {idx: 0, isDis: true},
            {idx: 1}, 
            {idx: 2}
        );
        top.dialog.clearDialog();
    }
};

var isEnableWeakPw = function () {
    var GUI_PREFERENCES = top.sessionData.GUI_PREFERENCES;
    if (!GUI_PREFERENCES.getProperty('weak_password')) {
        GUI_PREFERENCES  = context2json({ filename: 'guipreferences.conf', context: 'general', usf: 1 });
    }
    if (GUI_PREFERENCES.weak_password == "yes") {
        var edit_Ext = $P.lang("LANG85");
        var DynamicLoginPw = $P.lang("LANG1169");
        $P("#edit_PIN", document).rules("add", {
          identical: [edit_Ext, DynamicLoginPw, $("#edit_Ext")],
          checkNumericPw: [ 
              showCheckPassword, {
                  type: "digital",
                  pwsId: "#edit_PIN"
              }
          ] 
        });
    }
}
var initForm = function ()
{
    (function ()
    {
        var mcls = config2json({ filename: 'musiconhold.conf', catlist: 'yes' });
        mcls.each(function (this_class)
        {
            if (this_class != "rbt" )
            {
                ASTGUI.selectbox.append('edit_musicclass', this_class, this_class);
            }
        });
        _$('edit_musicclass').selectedIndex = -1;
        /* Need to add list of voicemenus */

        ASTGUI.selectbox.append('edit_voicemenuclass', $P.lang('LANG133'), '');
        var vmcls = parent.sessionData.pbxinfo.voicemenus.getOwnProperties();
        vmcls.each(function (vmenu)
        {
            var vm_name = parent.sessionData.pbxinfo.voicemenus[vmenu].comment || vmenu;
            ASTGUI.selectbox.append('edit_voicemenuclass', $P.lang('LANG134') + vm_name, vmenu);
        });

        /* end */
    })();

    load_agents_checkboxes();

    $('#chk_waittime').click(function ()
    {
        if (this.checked)
        {
            $('#edit_waittime').show()
        }
        else
        {
            $('#edit_waittime').hide();
        }
    });
    $('#chk_PIN').click(function ()
    {
        if (this.checked)
        {
            $('#edit_PIN').show()
        }
        else
        {
            $('#edit_PIN').hide();
        }
    });

/*
    $('#edit_agents_chboxes').sortable();
    $('#edit_agents_chboxes').disableSelection();
*/


    ASTGUI.events.add(DOM_button_add_toringlist, 'click', function ()
    {
        if (!DOM_select_fromlistofchannels.value)
        {
            return;
        }

        $(':selected', DOM_select_fromlistofchannels).each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_select_ringthesechannels, text, val);
        });

        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add(DOM_button_remove_fromringlist, 'click', function ()
    {
        if (!DOM_select_ringthesechannels.value)
        {
            return;
        }

        $(':selected', DOM_select_ringthesechannels).each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            if (ul_agents[val])
            {
                ASTGUI.selectbox.append(DOM_select_fromlistofchannels, text, val);
            }
        });

        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add(DOM_button_removeall_fromringlist, 'click', function ()
    {
        ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
        ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);

        for ( var user in ul_agents)
        {
            if (!ul_agents.hasOwnProperty(user))
            {
                continue;
            }

            ASTGUI.selectbox.append(DOM_select_fromlistofchannels, ul_agents[user], user);

        }
        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add('button_addall_toringlist', 'click', function ()
    {
        // modified to do not override former order in selected box, just append it
        $(DOM_select_fromlistofchannels).children().each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_select_ringthesechannels, text, val);
        });

        $P("#select_ringthesechannels", document).valid();
    });

    //bind up/down/top/bottom button
    $('#button_up').click( function () {
        ASTGUI.domActions.selectScrollUp('select_ringthesechannels');
    });

    $('#button_down').click( function () {
        ASTGUI.domActions.selectScrollDown('select_ringthesechannels');
    });

    $('#button_top').click( function () {
        ASTGUI.domActions.selectScrollTop('select_ringthesechannels');
    });

    $('#button_bottom').click( function () {
        ASTGUI.domActions.selectScrollBottom('select_ringthesechannels');
    });

}


var loadDOMElements = function ()
{
    DOM_table_ql = _$('table_queueslist');
    //DOM_edit_agents_chboxes = _$('edit_agents_chboxes');
    DOM_edit_agents_chboxes = $('edit_agents_chboxes');

    DOM_edit_QueueDiv = _$('edit_QueueDiv');
    DOM_edit_QueueDiv_title = _$('edit_QueueDiv_title');
    DOM_Queue_Ext = _$('edit_Ext'); // text box
    DOM_edit_label = _$('edit_label'); // text box
    DOM_edit_strategy = _$('edit_strategy'); // select
    DOM_edit_musicclass = _$('edit_musicclass'); // select
    DOM_edit_timeout = _$('edit_timeout'); // text
    DOM_edit_wrapuptime = _$('edit_wrapuptime'); // text
    DOM_edit_maxlen = _$('edit_maxlen'); // text
    //DOM_edit_autofill = _$('edit_autofill'); // chkbox
    //DOM_edit_autopause = _$('edit_autopause'); // chkbox
    DOM_edit_joinempty = _$('edit_joinempty'); // chkbox
    DOM_edit_leavewhenempty = _$('edit_leavewhenempty'); // chkbox
    DOM_edit_reportholdtime = _$('edit_reportholdtime'); // chkbox
    DOM_edit_voicemenuclass = _$('edit_voicemenuclass'); // select
    DOM_edit_PIN = _$('edit_PIN'); // text 
    DOM_chk_PIN = _$('chk_PIN');
    DOM_edit_waittime = _$('edit_waittime'); // text 
    DOM_chk_waittime = _$('chk_waittime');

    DOM_select_ringthesechannels = _$('select_ringthesechannels');
    DOM_select_fromlistofchannels = _$('select_fromlistofchannels');
    DOM_button_add_toringlist = _$('button_add_toringlist');
    DOM_button_remove_fromringlist = _$('button_remove_fromringlist');
    DOM_button_removeall_fromringlist = _$('button_removeall_fromringlist');

};

var updateQueuesTable = function ()
{
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function
    if (DOM_table_ql) {
        var newRow = DOM_table_ql.insertRow(-1);
        newRow.className = "frow";
        //addCell(newRow, { html: '', width: '15px' });
        addCell(newRow, { html: "<span locale='LANG607'>"+$P.lang('LANG607')+"</span>"});
        addCell(newRow, { html: "<span locale='LANG135'>"+$P.lang('LANG135')+"</span>"});
        addCell(newRow, { html: "<span locale='LANG132'>"+$P.lang('LANG132')+"</span>"});
        addCell(newRow, { html: "<span locale='LANG74'>"+$P.lang('LANG74')+"</span>"});
    }

    var langMap = {
        "RINGALL"      :  'LANG1197',
        "LINEAR"       :  'LANG1198',
        "LEASTRECENT"  :  'LANG1199',
        "FEWESTCALLS"  :  'LANG1200',
        "RANDOM"       :  'LANG1201',
        "RRMEMORY"     :  'LANG1202'
    };


    var m = parent.sessionData.pbxinfo.queues.exten;
    for (l in m)
    {
        if (m.hasOwnProperty(l))
        {

            if (!QUEUES_CONF.hasOwnProperty(l))
            {
                delete_queue(l);
                ASTGUI.feedback({ msg: msg, showfor: 2, color: 'red', bgcolor: '#FFFFFF' });
                top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG838").format(l) });
                continue;
            }
            if (DOM_table_ql) {
                var newRow = DOM_table_ql.insertRow(-1);
                newRow.className = ((DOM_table_ql.rows.length) % 2 == 1) ? 'odd' : 'even';
                var tmp = "<button class='options edit' onclick=\"showEditForm('" + l + "')\" localeTitle='LANG738' title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                          "<button class='options del' onclick=\"delete_Queue_confirm('" + l + "')\" localeTitle='LANG739' title=" + $P.lang("LANG739") + "></button>";
                var strategyLang = langMap[QUEUES_CONF[l].getProperty('strategy').toUpperCase()];
                //addCell(newRow, { html: '' });
                addCell(newRow, { html: l });
                addCell(newRow, { html: QUEUES_CONF[l].getProperty('fullname') }); // Name
                addCell(newRow, { html: $P.lang(strategyLang) , locale: strategyLang}); // Strategy
                addCell(newRow, { html: tmp }); // Edit Options
            }
        }
    }
    if (DOM_table_ql) {
        if (DOM_table_ql.rows.length == 1)
        {
            ASTGUI.domActions.clear_table(DOM_table_ql);
            var newRow = DOM_table_ql.insertRow(-1);
            newRow.className = 'even';
            addCell(newRow, { html: $P.lang('LANG129').format($P.lang('LANG607')), locale: 'LANG129 LANG607'});
            return;
        }
    }
};


var delete_queue = function (q)
{
    var u = new listOfSynActions('extensions.conf');
    u.new_action('delcat', parent.sessionData.pbxinfo.queues.exten[q]['context'], '', '');
    u.new_action('delete', ASTGUI.contexts.QUEUES, 'include', '', parent.sessionData.pbxinfo.queues.exten[q]['context']);
    u.callActions();

    var w = new listOfSynActions('queues.conf');
    w.new_action('delcat', q, '', '');
    w.callActions();

    delete parent.sessionData.pbxinfo.queues.exten[q];
    //ASTGUI.ASTDB.deleteKey( { dbname: 'QUEUE/' + q, key : 'PIN' } );

    //delete astdb queue's setting
    ASTGUI.cliCommand('database deltree QUEUE/' + q);

    //try to delete persistent dynamic members.
    ASTGUI.ASTDB.deleteKey({ dbname: 'Queue/PersistentMembers', key: q });

    return true;
};

var delete_Queue_confirm = function (item)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(item),
        buttons:{
            ok: function ()
            {
                if (delete_queue(item))
                {
                    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG816"), callback: function () { mWindow.location.reload(); } });
                }
                else
                    top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG820") });
            }
        }
    });
};

var prepareAddItemForm = function ()
{
    isNewQueue = true;
    EDIT_Queue = '';
    show_Queue_Form();
    $("#editForm").show();
    $("#edit_Ext").focus();
};


var prepareEditItemForm = function (k)
{
    isNewQueue = false;
    EDIT_Queue = k;
    show_Queue_Form();
};



var show_Queue_Form = function ()
{
    //set agent users
    ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
    ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);

    var members = (EDIT_Queue && QUEUES_CONF[EDIT_Queue].getProperty('member')) ? QUEUES_CONF[EDIT_Queue]['member'].split(',') : [];
    members.each(function (user)
    {
        //TODO check deleted users? not necessary i think.
        ASTGUI.selectbox.append(DOM_select_ringthesechannels, ul_agents[user], user);
    });

    for ( var user in ul_agents)
    {
        if (!ul_agents.hasOwnProperty(user))
        {
            continue;
        }

        if (members.contains(user))
        {
            continue;
        }

        ASTGUI.selectbox.append(DOM_select_fromlistofchannels, ul_agents[user], user);

    }


    $('#chk_waittime').removeAttr('checked');
    $('#edit_waittime').hide();
    $('#chk_PIN').removeAttr('checked');
    $('#edit_PIN').hide();

    if (isNewQueue == true)
    {
        ASTGUI.resetTheseFields([DOM_Queue_Ext, DOM_edit_label, DOM_edit_strategy, DOM_edit_musicclass, DOM_edit_timeout, DOM_edit_wrapuptime, DOM_edit_maxlen, /* DOM_edit_autopause,*/DOM_edit_joinempty, DOM_edit_leavewhenempty, DOM_edit_reportholdtime, DOM_edit_waittime, DOM_chk_PIN, DOM_chk_waittime]); /* reset all fields */
        //ASTGUI.domActions.unCheckAll( ag_chkbxClass );

        DOM_Queue_Ext.disabled = false;
        var tmp_allextensions = ASTGUI.cloneObject(parent.miscFunctions.getAllExtensions());
        DOM_Queue_Ext.value = tmp_allextensions.firstAvailable(parent.sessionData.GUI_PREFERENCES.getProperty('qe_start'));
        //DOM_edit_QueueDiv_title.innerHTML = 'New Queue';
       
        ASTGUI.updateFieldToValue(DOM_edit_musicclass, 'default');
        ASTGUI.updateFieldToValue(DOM_edit_voicemenuclass, '');
        return;
    }

    DOM_Queue_Ext.value = EDIT_Queue; DOM_Queue_Ext.disabled = true;
    //DOM_edit_QueueDiv_title.innerHTML = 'Edit Queue ' + EDIT_Queue;
    // load values for all other fields

    ASTGUI.updateFieldToValue(DOM_edit_label, QUEUES_CONF[EDIT_Queue].getProperty('fullname'));
    ASTGUI.updateFieldToValue(DOM_edit_strategy, QUEUES_CONF[EDIT_Queue].getProperty('strategy'));
    ASTGUI.updateFieldToValue(DOM_edit_musicclass, QUEUES_CONF[EDIT_Queue].getProperty('musicclass'));
    if (DOM_edit_musicclass.selectedIndex == -1)      //former musicclass has been deleted 
    {
        ASTGUI.updateFieldToValue(DOM_edit_musicclass, 'default');
    }
    ASTGUI.updateFieldToValue(DOM_edit_timeout, QUEUES_CONF[EDIT_Queue].getProperty('timeout'));
    ASTGUI.updateFieldToValue(DOM_edit_wrapuptime, QUEUES_CONF[EDIT_Queue].getProperty('wrapuptime'));
    ASTGUI.updateFieldToValue(DOM_edit_maxlen, QUEUES_CONF[EDIT_Queue].getProperty('maxlen'));
    //ASTGUI.updateFieldToValue(DOM_edit_autofill, QUEUES_CONF[EDIT_Queue].getProperty('autofill') );
    //ASTGUI.updateFieldToValue(DOM_edit_autopause, QUEUES_CONF[EDIT_Queue].getProperty('autopause') );
    ASTGUI.updateFieldToValue(DOM_edit_joinempty, QUEUES_CONF[EDIT_Queue].getProperty('joinempty'));
    ASTGUI.updateFieldToValue(DOM_edit_leavewhenempty, QUEUES_CONF[EDIT_Queue].getProperty('leavewhenempty'));
    ASTGUI.updateFieldToValue(DOM_edit_reportholdtime, QUEUES_CONF[EDIT_Queue].getProperty('reportholdtime'));

    ASTGUI.updateFieldToValue(DOM_edit_voicemenuclass, QUEUES_CONF[EDIT_Queue].getProperty('context'));
    //ASTGUI.domActions.checkSelected( ag_chkbxClass, (QUEUES_CONF[EDIT_Queue].getProperty('member')) ? QUEUES_CONF[EDIT_Queue]['member'].split(',') :[] ) ;


    ASTGUI.updateFieldToValue(DOM_edit_PIN, ASTGUI.ASTDB.getKey({ dbname: 'QUEUE/' + EDIT_Queue, key: 'PIN' }));

    ASTGUI.updateFieldToValue(DOM_edit_waittime, ASTGUI.ASTDB.getKey({ dbname: 'QUEUE/' + EDIT_Queue, key: 'WAITTIME' }));

    if (DOM_edit_PIN.value)
    {
        DOM_chk_PIN.checked = true;
        $('#edit_PIN').show();
    }

    if (DOM_edit_waittime.value)
    {
        DOM_chk_waittime.checked = true;
        $('#edit_waittime').show();
    }
};


var edit_queue_apply = function ()
{
    AskextensionRange(_$("edit_Ext").value, "qe_start", "qe_end");
};

var edit_queue_apply_work = function ()
{
    var cat = ASTGUI.getFieldValue(DOM_Queue_Ext);
    var waittime = ASTGUI.getFieldValue(DOM_edit_waittime);
    var PIN = ASTGUI.getFieldValue(DOM_edit_PIN);
    var ags = ASTGUI.domActions.get_checked(ag_chkbxClass);

    if (isNewQueue == true)
    { // new QUEUE

        //add to extensions.conf for new queue
        if (!parent.pbx.queues.new_exten(cat))
        {
            top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG839"), callback: function() { mWindow.location.reload(); } });
            return;
        }

        parent.sessionData.pbxinfo.queues.exten[cat] = new ASTGUI.customObject;
        parent.sessionData.pbxinfo.queues.exten[cat]['context'] = 'queue-' + cat;

    }

    if (DOM_chk_PIN.checked)    //update key
    {
        ASTGUI.ASTDB.updateKey({ dbname: 'QUEUE/' + cat, key: 'PIN', keyvalue: PIN });
    }
    else
    {
        ASTGUI.ASTDB.deleteKey({ dbname: 'QUEUE/' + cat, key: 'PIN' });
    }

    if (DOM_chk_waittime.checked)    //update key
    {
        ASTGUI.ASTDB.updateKey({ dbname: 'QUEUE/' + cat, key: 'WAITTIME', keyvalue: waittime });
    }
    else
    {
        ASTGUI.ASTDB.deleteKey({ dbname: 'QUEUE/' + cat, key: 'WAITTIME' });
    }

    //edit queues.conf
    var x = new listOfActions('queues.conf');
    x.new_action('delcat', cat, '', '');
    x.new_action('newcat', cat, '', ''); // create new context
    x.new_action('append', cat, 'fullname', ASTGUI.getFieldValue(DOM_edit_label));
    x.new_action('append', cat, 'strategy', ASTGUI.getFieldValue(DOM_edit_strategy));
    x.new_action('append', cat, 'timeout', ASTGUI.getFieldValue(DOM_edit_timeout));
    x.new_action('append', cat, 'wrapuptime', ASTGUI.getFieldValue(DOM_edit_wrapuptime));
    //   x.new_action('append', cat, 'autofill', ASTGUI.getFieldValue(DOM_edit_autofill));
    //    x.new_action('append', cat, 'autopause', DOM_edit_autopause));
    x.new_action('append', cat, 'joinempty', ASTGUI.getFieldValue(DOM_edit_joinempty));
    x.new_action('append', cat, 'leavewhenempty', ASTGUI.getFieldValue(DOM_edit_leavewhenempty));
    x.new_action('append', cat, 'reportholdtime', ASTGUI.getFieldValue(DOM_edit_reportholdtime));
    x.new_action('append', cat, 'maxlen', ASTGUI.getFieldValue(DOM_edit_maxlen));
    x.new_action('append', cat, 'musicclass', ASTGUI.getFieldValue(DOM_edit_musicclass));
    x.new_action('append', cat, 'ringinuse', 'no'); //default set ringinguse=no
    x.new_action('append', cat, 'timeoutrestart', 'yes'); //always set timeoutrestart  = yes

    var s = ASTGUI.getFieldValue(DOM_edit_voicemenuclass);
    if (s != '') x.new_action('append', cat, 'context', s);
    else x.new_action('delete', cat, 'context');

    var add_members = ASTGUI.selectbox.readAllValues(DOM_select_ringthesechannels);

    var el = DOM_select_ringthesechannels;
    for (var i = 0; i < el.options.length; i++)
    {
        if (el.options[i].value.trim())
        {
            x.new_action('append', cat, 'member', el.options[i].value);
            //y.push(el.options[x].value);
        }
    }

    var after = function ()
    {
        top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG844"), callback: function() { mWindow.location.reload(); } });
    };

    x.callActions(after);
};


var QueueExistsbyThisName = function (thisName, element)
{
    if (!thisName)
    {
        return true;
    }

    var m = parent.sessionData.pbxinfo.queues;
    for (l in m)
    {
        if (m.hasOwnProperty(l) && QUEUES_CONF.hasOwnProperty(l))
        {
            if (isNewQueue == true && QUEUES_CONF[l].getProperty('fullname') == thisName)
            {
                return false;
            }
            if (isNewQueue == false && l != EDIT_Queue && QUEUES_CONF[l].getProperty('fullname') == thisName)
            {
                return false;
            }
        } 
    }
    return true;
};


var load_agents_checkboxes = function ()
{
    var ul = parent.pbx.users.list();
    var fxs_tmp = {};
    ul.each(function (user)
    {
        if (parent.sessionData.pbxinfo.users[user]['hassip'] && parent.sessionData.pbxinfo.users[user]['hassip'].isAstTrue())
        {
            agent_count++;
            var fullname = parent.sessionData.pbxinfo.users[user]['fullname'];
            ul_agents['SIP/' + user] = 'SIP/' + user + (fullname ? ' \"' + fullname + '\"' : '');
        }
        if (parent.sessionData.pbxinfo.users[user]['hasiax'] && parent.sessionData.pbxinfo.users[user]['hasiax'].isAstTrue())
        {
            agent_count++;
            var fullname = parent.sessionData.pbxinfo.users[user]['fullname'];
            ul_agents['IAX2/' + user] = 'IAX2/' + user + (fullname ? ' \"' + fullname + '\"': '');
        }

    });

    //add fxs port user at last
    parent.sessionData.FXS_PORTS_DETECTED.each(function (fxs)
    {
        var idx = parent.sessionData.FXS_PORTS_DETECTED.indexOf(fxs) + 1;
        ul_agents[parent.sessionData.DahdiDeviceString + '/' + fxs] = 'FXS ' + idx;
    });

};
