/*
* Asterisk-GUI	- an Asterisk configuration interface
*
* ringgroups.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 *  Delete the old ringgroup when update
 *  Modified the dialplan of ringgroup
 *
 */
$P = top.$;
var mWindow = top.window.frames["mainscreen"]; // this will be the main window 

var isNewRG;
var RG_EDITING; // ring group being edited
var listOfUserExtens = [];
function arrIndexOf(arr, val) {
    if ($.isArray(arr)) {
        var i = arr.length;
        while (i--)
        {
            if (arr[i] === val)
            {
                return i;
            }
        }
        return -1;
    }
    else{
        return arr.indexOf(val);
    }
}
function contains(arr, a) // firefox don't support contains function
{
    if (arrIndexOf(arr, a) > -1 )
        return true;
    else
        return false;
};
var ringGroupExistsbyThisName = function (thisName, element)
{
    if (!thisName)
    {
        return true;
    }
    var m = parent.sessionData.pbxinfo.ringgroups;
    for (l in m)
    {
        if (typeof m[l] == "object")
        {
            if (isNewRG == true && m[l]['NAME'] == thisName)
            {
                return false;
            }
            if (isNewRG == false && l != RG_EDITING)
            {
                var ml_name = m[l];
                if (ml_name['NAME'] == thisName) return false;
            }
        }
    }
    return true;
};
window.onload = function ()
{
    $P.lang(document, true);
    loadDOMelements();
    var mode = gup("mode");
    var item = gup("item");
    if (mode)
    {
        initForm();
        if (mode == "add")
        {
            prepareAddItemForm();
        }
        else if (mode == "edit")
        {
            prepareEditItemForm(item);
        }
        choose_voicemail();
        $("#text_rgname").focus();
        $('#select_strategy').trigger('change');
        var max_member = (parent.sessionData.GUI_PREFERENCES['rg_maxmember']
                        && Number(parent.sessionData.GUI_PREFERENCES['rg_maxmember']));
		if ($("#form").tooltip) {
			$("#form").tooltip();
		}
        $P("#form", document).validate({
            rules: {
                "text_rgname": {
                    required: true, 
                    letterDigitUndHyphen: true,
                    customCallback: [$P.lang("LANG2137"), ringGroupExistsbyThisName]
                },
                "text_rgExten": {
                    required: true,
                    digits: true,
                    extensionExists: [RG_EDITING] ,
                    extensionRange: ['rge_start', 'rge_end']
                },
                "select_ringthesechannels": {
                    selectItemMin: 1,
                    selectItemMax: [max_member, $P.lang('LANG128').toLowerCase()],
                    selectMemberLength: 200     //consider CLI buffer
                },
                "text_ringTime": {
                    required: true,
                    digits: true,
                    min: 0
                },
                "edit_vmsecret": {
                    required: "#edit_check_voicemail:checked",
                    digits: true,
                    minlength: 4
                },
                "edit_vm": {
                    email: true
                }
            },
            submitHandler: function() {
                $P('#text_rgExten', document).rules('remove');
                $P('#text_rgname', document).rules('remove');
                save_rg_work();
            }
        });
        isEnableWeakPw();
        top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG600"));
        $("title").attr("locale", "LANG600");
        // Logic for list
        if (!ASTGUI.miscFunctions.alertIfRangeisNotdefined('rge_start', 'rge_end', $P.lang('LANG600')))
        {
            $('.top_buttons').hide();
            return;
        }
        update_RingGroupsTable();
    }
};
var isEnableWeakPw = function () {
    var GUI_PREFERENCES = top.sessionData.GUI_PREFERENCES;
    if (!GUI_PREFERENCES.getProperty('weak_password')) {
        GUI_PREFERENCES  = context2json({ filename: 'guipreferences.conf', context: 'general', usf: 1 });
    }
    if (GUI_PREFERENCES.weak_password == "yes") {
        var new_crl_name = $P.lang("LANG85");
        var password = $P.lang("LANG127");
        $P("#edit_vmsecret", document).rules("add", {
          identical: [new_crl_name, password, $("#text_rgExten")],
          checkNumericPw: [ 
              showCheckPassword, {
                required: true,
                type: "digital",
                pwsId: "#edit_vmsecret"
              }
          ] 
        });
    }
}
var initForm = function ()
{
    listOfUserExtens = parent.pbx.users.list().sortExtension();
    ASTGUI.events.add(DOM_button_add_toringlist, 'click', function ()
    {
        if (!DOM_select_fromlistofchannels.value)
        {
            return;
        }

        $(':selected', DOM_select_fromlistofchannels).each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_select_ringthesechannels, text, val);
        });
        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add(DOM_button_remove_fromringlist, 'click', function ()
    {
        if (!DOM_select_ringthesechannels.value)
        {
            return;
        }
        $(':selected', DOM_select_ringthesechannels).each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_select_fromlistofchannels, text, val);
        });
        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add(DOM_button_removeall_fromringlist, 'click', function ()
    {
        ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
        ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);

        listOfUserExtens.each(function (user)
        {
            ASTGUI.selectbox.append(DOM_select_fromlistofchannels, getUserDisplay(user), user);
        });
        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add('button_addall_toringlist', 'click', function ()
    {
        // modified to do not override former order in selected box, just append it
        $(DOM_select_fromlistofchannels).children().each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_select_ringthesechannels, text, val);
        });

        $P("#select_ringthesechannels", document).valid();
    });

    //bind up/down/top/bottom button
    $('#button_up').click( function () {
        ASTGUI.domActions.selectScrollUp('select_ringthesechannels');
    });

    $('#button_down').click( function () {
        ASTGUI.domActions.selectScrollDown('select_ringthesechannels');
    });

    $('#button_top').click( function () {
        ASTGUI.domActions.selectScrollTop('select_ringthesechannels');
    });

    $('#button_bottom').click( function () {
        ASTGUI.domActions.selectScrollBottom('select_ringthesechannels');
    });

    ASTGUI.domActions.enableDisableByCheckBox('edit_check_voicemail', ['voicemail']);
    $('#select_strategy').change(function (){
        if ( this.value === 'ORDER')
        {
            $('#div_ringtime > > .label').html($P.lang("LANG1055") + ':');
        }
        else
        {
            $('#div_ringtime > > .label').html($P.lang("LANG1054") + ':');
        }
        if (window.frameElement)
            $(window.frameElement).css("height","0px");
        top.dialog.repositionDialog("none");
    });

    var list = parent.miscFunctions.getDestinations("voicemail");
    ASTGUI.selectbox.append("voicemail", $P.lang("LANG2223"), "");
    list.each(function(option)
    {
        ASTGUI.selectbox.append("voicemail", option.optionText, option.optionText.lChop($P.lang("LANG90") + ' -- ' ));
    });
}

var loadDOMelements = function ()
{
    DOM_table_rgoups_list = _$('table_rgoups_list');
    DOM_text_rgname = _$('text_rgname');
    DOM_select_strategy = _$('select_strategy');
    DOM_select_ringthesechannels = _$('select_ringthesechannels');
    DOM_select_fromlistofchannels = _$('select_fromlistofchannels');
    DOM_button_add_toringlist = _$('button_add_toringlist');
    DOM_button_remove_fromringlist = _$('button_remove_fromringlist');
    DOM_button_removeall_fromringlist = _$('button_removeall_fromringlist');
    DOM_text_rgExten = _$('text_rgExten');
    DOM_text_ringTime = _$('text_ringTime');
    DOM_rg_fb_select = _$('rg_fb_select');
    DOM_edit_ignoreRedir = _$('edit_ignoreRedir');
};

var save_rg = function()
{
    AskextensionRange(_$("text_rgExten").value, "rge_start", "rge_end");
}

var save_rg_work = function ()
{
    var RG_EXTEN = ASTGUI.getFieldValue(DOM_text_rgExten);

    var TEMP_members = ASTGUI.selectbox.readAllValues(DOM_select_ringthesechannels);

    if (!isNewRG)
    { // if editing existing ring group
        parent.pbx.ring_groups.remove(RG_EDITING);
    }

    var tmp_obj = {
        NAME: DOM_text_rgname.value,
        strategy: DOM_select_strategy.value,
        members: TEMP_members,
        extension: RG_EXTEN,
        ringtime: DOM_text_ringTime.value,
        context: 'group-' + RG_EXTEN,
        fallback: '',
        ignore: _$('edit_ignoreRedir').checked
    };

    tmp_obj = ASTGUI.toCustomObject(tmp_obj);
    tmp_obj.fallback = ASTGUI.getFieldValue(DOM_rg_fb_select);

    var reload = function ()
    {
        top.dialog.clearDialog();
        top.dialog.dialogMessage({
            type: 'success',
            content: $P.lang("LANG815"),
            callback: function ()
            {
                mWindow.location.reload();
            }
        });
    };

    var later = function ()
    {
        ASTGUI.ASTDB.updateKey({ dbname: 'GROUP/' + RG_EXTEN, key: 'HASVM', keyvalue: ASTGUI.getFieldValue('edit_check_voicemail')});
        var x = new listOfActions();
        x.filename('voicemail.conf');
        if (ASTGUI.getFieldValue('edit_check_voicemail') == 'yes')
        {
            ASTGUI.ASTDB.updateKey({ dbname: 'GROUP/' + RG_EXTEN, key: 'VMBOX', keyvalue: ASTGUI.getFieldValue('voicemail')});

            if (ASTGUI.getFieldValue('voicemail') == "")
            {
                var tmp = ASTGUI.getFieldValue('edit_vmsecret') + ',' + RG_EXTEN + ',' + ASTGUI.getFieldValue('edit_vm');
                x.new_action('autoupdate', 'default', RG_EXTEN, tmp);
            }
        }

        x.callActions(reload);
    };

    parent.pbx.ring_groups.add(RG_EDITING, tmp_obj, later);

};

var getUserDisplay = function (user)
{
    if (!parent.sessionData.pbxinfo.users[user])
    {
        return '';
    }
    var full_name = parent.sessionData.pbxinfo.users[user]['fullname'];
    if ((full_name && full_name.length <= 0) || !full_name)
    {
        return user;
    }
    else
    {
        return user + ' \"' + full_name + '\"';
    }
};


var resetFields = function ()
{

    ASTGUI.updateFieldToValue('edit_check_voicemail', 'no');
    ASTGUI.updateFieldToValue('edit_vmsecret', '');
    ASTGUI.updateFieldToValue('edit_vm', '');

    _$('voicemail').disabled = true;

    if (isNewRG)
    {
        DOM_text_rgname.value = '';
        DOM_edit_ignoreRedir.checked = true;
        //DOM_select_strategy.selectedIndex = -1 ;
        ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
        ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);
        listOfUserExtens.each(function (user)
        {
            ASTGUI.selectbox.append(DOM_select_fromlistofchannels, getUserDisplay(user), user);
        });

        DOM_text_rgExten.value = '';
        ASTGUI.selectbox.populateArray(DOM_rg_fb_select, top.miscFunctions.getAllDestinations());
        ASTGUI.resetTheseFields([DOM_rg_fb_select, DOM_select_strategy, DOM_text_ringTime]);
        _$("text_ringTime").value = "30";
        var tmp_allextensions = ASTGUI.cloneObject(top.miscFunctions.getAllExtensions());
        DOM_text_rgExten.value = tmp_allextensions.firstAvailable(top.sessionData.GUI_PREFERENCES['rge_start']);
        return;
    }

    var c = parent.sessionData.pbxinfo['ringgroups'][RG_EDITING];
    DOM_text_rgname.value = c.NAME;
    // TODO [AH] Dialog needs to have interface for other program to update dialog title
    //_$('rgedit_form_caption').innerHTML = 'Edit RingGroup - ' + c.NAME;
    ASTGUI.selectbox.selectOption(DOM_select_strategy, c['strategy']);
    ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
    var mbrs = ASTGUI.cloneObject(c['members']);
    mbrs.each(function (user)
    {
        if (parent.sessionData.pbxinfo.users[user])
        {
            ASTGUI.selectbox.append(DOM_select_ringthesechannels, getUserDisplay(user), user);
        }
    });
    //add selected attribute ybiaozhou
//    $('#select_ringthesechannels').children().prop('selected', 'selected');

    ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);
    listOfUserExtens.each(function (user)
    {
        if (contains(mbrs, user)) { return; }
        ASTGUI.selectbox.append(DOM_select_fromlistofchannels, getUserDisplay(user), user);
    });

    DOM_text_rgExten.value = (c['extension']) ? c['extension'] : '';
    DOM_text_rgExten.disabled = true;

    DOM_text_ringTime.value = (c['ringtime']) ? c['ringtime'] : '';
    DOM_edit_ignoreRedir.checked = (c['ignore'] == true);
    // select value from select box DOM_rg_fb_select
    var destinations = parent.miscFunctions.getAllDestinations();
    var destinations_WithOut_ThisRingGroup = [];

    destinations.each(function (this_object)
    {
        if (this_object.optionValue != 'Goto(' + RG_EDITING + '|s|1)' && this_object.optionValue != 'Goto(' + RG_EDITING + ',s,1)')
        {
            destinations_WithOut_ThisRingGroup.push(this_object);
        }
    });

    ASTGUI.selectbox.populateArray(DOM_rg_fb_select, destinations_WithOut_ThisRingGroup);
    ASTGUI.selectbox.selectDestinationOption(DOM_rg_fb_select, (c['fallback']) ? c['fallback'] : '');

    var a = ASTGUI.ASTDB.getKey({ dbname: 'GROUP/' + RG_EDITING, key: 'HASVM' });
    ASTGUI.updateFieldToValue('edit_check_voicemail', a);
    if(a == "yes")
    {
        _$('voicemail').disabled = false;
    }

    a = ASTGUI.ASTDB.getKey({ dbname: 'GROUP/' + RG_EDITING, key: 'VMBOX' });
    ASTGUI.updateFieldToValue('voicemail', a || "");

    var cxt = context2json({ filename: 'voicemail.conf', context: 'default', usf: 0 });

    if (!cxt) cxt = [];

    for (var t = 0; t < cxt.length; t++)
    {
        if (cxt[t].beginsWith(RG_EDITING + '='))
        {
            var tmp = cxt[t].afterChar('=').split(",");
            ASTGUI.updateFieldToValue('edit_vmsecret', tmp[0]);
            ASTGUI.updateFieldToValue('edit_vm', tmp[2]);
        }
    }

    //choose_voicemail();
};

var delete_rg_confirm = function (d)
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(d),
        buttons:{
            ok: function ()
            {
                parent.pbx.ring_groups.remove(d);

                var DELETE = new listOfActions('voicemail.conf');

                DELETE.new_action('delete', 'default', d, '');
                DELETE.callActions(function ()
                {
                    var reload = function()
                    {
                        top.dialog.dialogMessage({
                            type: 'success',
                            content: $P.lang("LANG816"),
                            callback: function ()
                            {
                                mWindow.location.reload();
                            }
                        });
                    }

                    top.dialog.dialogConfirm({
                        confirmStr: $P.lang("LANG822"),
                        buttons: {
                            ok: function ()
                            {
                                var tmp_cmd = "action=fileOpDelete&type=voicemails_dir&data=" + top.UrlEscape(d);
                                $.ajax({type: "GET", url: "/webcgi", data: tmp_cmd, success: function(){reload();}});
                            },
                            cancel: function()
                            {
                                reload();
                            }
                        }
                    });

                });
            }
        }
    });
};

var prepareAddItemForm = function ()
{
    RG_EDITING = '';
    isNewRG = true;
    resetFields();
};

var prepareEditItemForm = function (item)
{
    RG_EDITING = item;
    isNewRG = false;
    resetFields();
};

var update_RingGroupsTable = function ()
{
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function
    (function ()
    { // add first row
        if (DOM_table_rgoups_list) {
            var newRow = DOM_table_rgoups_list.insertRow(-1);
            newRow.className = "frow";
            //addCell(newRow, { html: '', width: '12px' });
            addCell(newRow, { html: "<span locale='LANG85'>"+$P.lang('LANG85')+"</span>"});
            addCell(newRow, { html: "<span locale='LANG1053'>"+$P.lang('LANG1053')+"</span>"});
            addCell(newRow, { html: "<span locale='LANG128'>"+$P.lang('LANG128')+"</span>"});
            addCell(newRow, { html: "<span locale='LANG74'>"+$P.lang('LANG74')+"</span>"});
        }
    })();

    var c = parent.sessionData.pbxinfo.ringgroups;
    for (var d in c)
    {
        if (c.hasOwnProperty(d))
        {
            if (DOM_table_rgoups_list) {
                var newRow = DOM_table_rgoups_list.insertRow(-1);
                var rn = DOM_table_rgoups_list.rows.length;
                newRow.className = ((rn) % 2 == 1) ? 'odd' : 'even';
                //addCell(newRow, { html: '' });
                addCell(newRow, { html: c[d]['extension'] });
                addCell(newRow, { html: c[d]['NAME'] });

                var TMP_MEMBERS = [];
                var CDM = ASTGUI.cloneObject(c[d]['members']);
                CDM.each(function (user)
                {
                    if (parent.sessionData.pbxinfo.users[user])
                    {
                        // TMP_MEMBERS.push( getUserDisplay(user)); display extension directly, same as voicemail group .
                        TMP_MEMBERS.push(user);
                    }
                });
                addCell(newRow, { html: TMP_MEMBERS.join(', ') });
                var tmp = "<button class='options edit' onclick=\"showEditForm('" + d + "')\" localeTitle='LANG738' title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                      "<button class='options del' onclick=\"delete_rg_confirm('" + d + "')\" localeTitle='LANG739' title=" + $P.lang("LANG739") + "></button>";
                addCell(newRow, { html: tmp });
            }
        } 
    }
    if (DOM_table_rgoups_list) {
        if (DOM_table_rgoups_list.rows.length == 1)
        {
            ASTGUI.domActions.clear_table(DOM_table_rgoups_list);
            var newRow = DOM_table_rgoups_list.insertRow(-1);
            newRow.className = 'even';
            //addCell(newRow, { html: $P.lang('LANG129').format($P.lang('LANG600')), locale: "LANG129 LANG600"}); 
            addCell(newRow, { html: $P.lang('LANG129').format($P.lang('LANG600')), locale: "LANG129 LANG600"}); 
            return;
        }
    }


    //sort Table by extension, and bind sort function
    sortTable({
        sortTableToggle: false,
        "tableId": 'table_rgoups_list',
        sortFuncs: {
            "sort_1": ASTGUI.extenStyleCmpFunc
        }

    }).sortTableInitial(1).bindTitles( 
        {idx: 0, isDis: true},
        {idx: 1}
    );

};

var choose_voicemail = function ()
{
    if (_$("edit_check_voicemail").checked == true && _$("voicemail").value == "" )
    {
        $("#vmoption").show();
    }
    else
    {
        $("#vmoption").hide();
    }

    if (window.frameElement)
        $(window.frameElement).css("height","0px");
    top.dialog.repositionDialog("none");
};


