/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Added for support Security functions
 * 
 */
$P = top.$;
var mWindow = window;
var isNew;
var FW_OLD_NAME = "";

window.onload = function()
{
    $P.lang(document, true);    
    //var url = top.$.url(window.location.href, true);
    var mode = gup("mode");
    var item = gup("item");
    load_DOMelements(mode ? true : false);
    security_init();
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        init_validator();
        if (mode == "add")
        {
            prepareAddItemForm();
        }
        else if (mode == "edit")
        {
            prepareEditItemForm(item);
        }
        //top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG38"));
        $("title").attr("locale", "LANG38");
        update_FirewallTable();
    }
    top.Custom.init(document);
};

var load_DOMelements = function (editor)
{
    /*  initialize interface  */
    interface_init();
    DOM_tbl_fw = _$('table_firewall_list');
    DOM_new_fw_name = _$('new_fw_name');
    DOM_new_fw_interface = _$('new_fw_interface');
    DOM_new_fw_action = _$('new_fw_action');
    DOM_new_fw_proto = _$('new_fw_proto');
    DOM_new_fw_type = _$('new_fw_type');
    DOM_new_fw_services = _$('new_fw_services');
    
    var type = $('#new_fw_type');
    type.change( function()
    {
        var el = _$('new_fw_type');
        if ( el.value == "in" )
        {
            $('#new_fw_interf').show();
        } 
        else if ( el.value == "out" )
        {
            $('#new_fw_interf').hide();
        }
        top.dialog.repositionDialog("none");
    } );

    var rule_service = $('#new_fw_services');
    rule_service.change( function()
    {
        var el = _$('new_fw_services');
        if ( el.value == "custom" ) {
            $('#new_fw_src').show();
            $('#new_fw_dst').show();
            $('#new_fw_proto_container').show();
        } else {
            $('#new_fw_src').hide();
            $('#new_fw_dst').hide();
            $('#new_fw_proto_container').hide();
        }
        if (window.frameElement)
        {
            $(window.frameElement).css("height","0px");
        }
        top.dialog.repositionDialog("none");
    } );
};

var init_validator = function()
{
    $P = top.$;
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "new_fw_name": {
                required: true,
                letterDigitUndHyphen: true,
                customCallback: ["The rule name already exists.", check_name_exist]
            },
            "new_fw_action": {
                required: true
            },
            "new_fw_proto": {
                required: true
            },
            "new_fw_type": {
                required: true
            },
            "new_fw_interface": {
                required: true
            },
            "new_fw_services": {
                required: true,
                customCallback: ["The settings will block anywhere with any port to access this pbx.", check_service]
            },
            "new_fw_src_ipaddr": {
                customCallback: ["Invalid format.", check_ipaddr]
            },
            "new_fw_src_port": {
                customCallback: ["Invalid format.", check_port]
            },
            "new_fw_dst_ipaddr": {
                customCallback: ["Invalid format.", check_ipaddr]
            },
            "new_fw_dst_port": {
                customCallback: ["Invalid format.", check_port]
            }
        },
        submitHandler: function(ev) {
            var target = this.submitButton;
            if ($(target).attr('id') !== 'save')
            {
                return;
            }
            new_fw_save_go(ev);
        }
    });
};

var update_FirewallTable = function()
{
    // simple firewall
    var fw_attr = parent.sessionData.pbxinfo['fw_attr'];
    if ( fw_attr.hasOwnProperty('ping_enable') && fw_attr['ping_enable'] != '' ) {
        var values = fw_attr['ping_enable'].split(',');
        for ( var i = 0; i < values.length; i++ )
        {
            var chkbox = $('#ping_container').find('input');
            for ( var j = 0; chkbox && j < chkbox.length; j++ )
            {
                if ( chkbox[j].value == values[i] )
                {
                    chkbox[j].checked = true;
                    break;
                }
            }
        }
    }
    if ( fw_attr.hasOwnProperty('ping_of_death') && fw_attr['ping_of_death'] != '' ) {
        var values = fw_attr['ping_of_death'].split(',');
        for ( var i = 0; i < values.length; i++ )
        {
            var chkbox = $('#deathping_container').find('input');
            for ( var j = 0; chkbox && j < chkbox.length; j++ )
            {
                if ( chkbox[j].value == values[i] )
                {
                    chkbox[j].checked = true;
                    break;
                }
            }
        }
    }
    if ( fw_attr.hasOwnProperty('syn_flood') && fw_attr['syn_flood'] != '' ) {
        var values = fw_attr['syn_flood'].split(',');
        for ( var i = 0; i < values.length; i++ )
        {
            var chkbox = $('#synflood_container').find('input');
            for ( var j = 0; chkbox && j < chkbox.length; j++ )
            {
                if ( chkbox[j].value == values[i] )
                {
                    chkbox[j].checked = true;
                    break;
                }
            }
        }
    }

    // update table 
    var addCell = ASTGUI.domActions.tr_addCell;

    (function ()
    { // add first row
        if ( DOM_tbl_fw ) {
            var newRow = DOM_tbl_fw.insertRow(-1);
            newRow.className = "frow";
            addCell(newRow, { html: $P.lang("LANG1957"), locale: "LANG1957", width: '100px' });
            addCell(newRow, { html: $P.lang("LANG1947"), locale: "LANG1947"});
            addCell(newRow, { html: $P.lang("LANG1948"), locale: "LANG1948" });
            addCell(newRow, { html: $P.lang("LANG1949"), locale: "LANG1949" });
            addCell(newRow, { html: $P.lang("LANG1950"), locale: "LANG1950" });
            addCell(newRow, { html: $P.lang("LANG1952"), locale: "LANG1952" });   //
            addCell(newRow, { html: $P.lang("LANG1953"), locale: "LANG1953" });
            addCell(newRow, { html: $P.lang("LANG1958"), locale: "LANG1958" });
        }
    })();

    var PreviousTRColor = 'odd'; // 'odd' : 'even' ;

    var firewall = parent.sessionData.pbxinfo['firewall'];
    var fw_rules = parent.sessionData.pbxinfo['fw_rules'];
    for ( var idx = 0; idx < fw_rules.length; idx++ )
    {
        var item = fw_rules[idx];
        if ( item.indexOf('rule=') > -1 )
            item = item.split('=')[1];
        else
            continue;
        if ( firewall.hasOwnProperty(item) )
        {
            var rule = firewall[item];
            var newRow = DOM_tbl_fw.insertRow(-1);
            newRow.className = PreviousTRColor;
            newRow.id = item;
            /*  append data  */

            var tmp_idx = idx+1;
            addCell(newRow, { html: ''+tmp_idx });
            for ( var i = 0; i < rule.length; i++ )
            {
                if ( rule[i][0] == 'flags' || rule[i][0] == 'interface' )
                    continue;
                addCell(newRow, { html: rule[i][1] });
            }

            /*  append button  */
            var btn = "<button type='button' class='options edit' onclick=\"showFirewallEditForm('" + item + "')\" title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                      "<button type='button' class='options del' onclick=\"delete_FW_confirm('" + item + "','')\" title=" + $P.lang("LANG739") + "></button>";
    /*                  "<button type='button' class='rule_clickTop options' title='top' onClick='rule_top(" + "\"" + item + "\")'></button>" +
                      "<button type='button' class='rule_clickUp options' title='up' onClick='rule_up(" + "\"" + item + "\")'></button>" +
                      "<button type='button' class='rule_clickDown options' title='down' onClick='rule_down(" + "\"" + item + "\")'></button>" +
                      "<button type='button' class='rule_clickBottom options' title='bottom' onClick='rule_bottom(" + "\"" + item + "\")'></button>";
    */
             addCell(newRow, { html: btn });
             $(newRow).attr('value', idx);
            
             if ( DOM_tbl_fw && DOM_tbl_fw.rows.length == 1 )
             {
                
                   ASTGUI.domActions.clear_table(DOM_tbl_fw);
                   var newRow = DOM_tbl_fw.insertRow(-1);
                   newRow.className = 'even';
                   addCell(newRow, { html: 'No Firewall Rules Defined !!' });
                   return;
             }

        }
    }

};

var prepareAddItemForm = function()
{
    isNew = true;
    FW_OLD_NAME = '';
    ASTGUI.resetTheseFields([/*DOM_tbl_fw, */DOM_new_fw_action, DOM_new_fw_name, DOM_new_fw_proto, DOM_new_fw_type, DOM_new_fw_interface, DOM_new_fw_services]);
    top.Custom.init(document);
    $("#editForm").show();
};

var prepareEditItemForm = function(item)
{
    isNew = false;
    ASTGUI.resetTheseFields([DOM_new_fw_action, DOM_new_fw_name, DOM_new_fw_proto, DOM_new_fw_type, DOM_new_fw_interface, DOM_new_fw_services]);

    var firewall = parent.sessionData.pbxinfo['firewall'];
    if ( firewall.hasOwnProperty(item) )
    {
        var tmp_flags = '';
        var rule = firewall[item];
        for ( var i = 0; i < rule.length; i++ )
        {
            switch ( rule[i][0] )
            {
                case 'rulename':
                   var tmpl = 'rule_';
                   var tmp_name = rule[i][1].substring(tmpl.length);
                   ASTGUI.updateFieldToValue(DOM_new_fw_name, tmp_name);   
                   FW_OLD_NAME = rule[i][1];
                   break;
                case 'action':
                   ASTGUI.updateFieldToValue(DOM_new_fw_action, rule[i][1]);   
                   break;
                case 'protocol':
                   ASTGUI.updateFieldToValue(DOM_new_fw_proto, rule[i][1]);   
                   break;
                case 'type':
                   ASTGUI.updateFieldToValue(DOM_new_fw_type, rule[i][1]);   
                   if ( rule[i][1] == "in" )
                   {
                       $('#new_fw_type').bind('change');
                       $('#new_fw_type').trigger('change');
                   }
                   break;
                case 'interface':
                   ASTGUI.updateFieldToValue(DOM_new_fw_interface, rule[i][1]);   
                   break;
                case 'source':
                   dealWith_address_port(rule[i][1], 'new_fw_src_ipaddr', 'new_fw_src_port');
                   break;
                case 'destination':
                   dealWith_address_port(rule[i][1], 'new_fw_dst_ipaddr', 'new_fw_dst_port');
                   break;
                case 'flags':
                   tmp_flags = rule[i][1];
                   break;
            }
        }

       var dport = _$('new_fw_dst_port').value;
       if ( tmp_flags == 'byService' )
       {
            var res = port2service(dport);
            ASTGUI.updateFieldToValue(DOM_new_fw_services, res);
       } else if ( tmp_flags == 'byAdvance' )
       {
            ASTGUI.updateFieldToValue(DOM_new_fw_services, 'custom');
            $('#new_fw_src').show();
            $('#new_fw_dst').show();
            $('#new_fw_proto_container').show();
       }
    }
    top.Custom.init(document);
    $("#editForm").show();
};

var service2port = function ( services )
{
    var table = {
        'ftp':'21',
        'ssh':'22',
        'telnet':'23',
        'tftp':'69',
        'http':'80',
        'ldap':'389'
    };
    if ( table.hasOwnProperty(services) )
        return table[services];
    return '';
}

var port2service = function ( port )
{
    var table = {
        '21':'ftp',
        '22':'ssh',
        '23':'telnet',
        '69':'tftp',
        '80':'http',
        '389':'ldap'
    };
    if ( table.hasOwnProperty(port) )
        return table[port];
    return '';
}

var port2proto = function ( port )
{
    var table = {
        '21': 'tcp',
        '22': 'tcp',
        '23': 'tcp',
        '69': 'udp',
        '80': 'tcp',
        '389': 'tcp'
    };
    
    return table[port];
}


var new_fw_save_go = function(ev)
{
    var tmp_fw_name = 'rule_' + DOM_new_fw_name.value.toLowerCase();
    
    var tmp_fw_action = DOM_new_fw_action.value;
    var tmp_fw_proto = DOM_new_fw_proto.value;
    var tmp_fw_type = DOM_new_fw_type.value;
    var tmp_fw_interface = DOM_new_fw_interface.value;

    var tmp_source = '';
    var tmp_destination = '';
    var tmp_fw_services = DOM_new_fw_services.value;
    var tmp_flags = '';

    if ( tmp_fw_services == "custom" )
    {
        var src_ipaddr = ASTGUI.getFieldValue('new_fw_src_ipaddr') || "Anywhere";
        var src_port = ASTGUI.getFieldValue('new_fw_src_port') || "Any";
        var dst_ipaddr = ASTGUI.getFieldValue('new_fw_dst_ipaddr') || "Anywhere";
        var dst_port = ASTGUI.getFieldValue('new_fw_dst_port') || "Any";

        tmp_source = "Address:" + src_ipaddr + ' Port:' + src_port;
        tmp_destination = "Address:" + dst_ipaddr + ' Port:' + dst_port;
        tmp_flags = 'byAdvance';
    } else
    {
        var tmp_dst = service2port(tmp_fw_services);
        tmp_fw_proto = port2proto(tmp_dst);
        tmp_source = 'Address:Anywhere Port:Any';
        tmp_destination = 'Address:Anywhere Port:' + tmp_dst;
        tmp_flags = 'byService';
    }

    var rule = [
        ['rulename', tmp_fw_name],
        ['action', tmp_fw_action],
        ['protocol', tmp_fw_proto],
        ['type', tmp_fw_type],
        ['source', tmp_source],
        ['destination', tmp_destination],
        ['interface', tmp_fw_interface],
        ['flags', tmp_flags]
    ];
    $P.cookie("disNoneApply", true);
    if ( isNew )
    {
        var actions = listOfSynActions('fw_rules.conf');
        actions.new_action('newcat', tmp_fw_name, '', '');
        for ( var i = 0; i < rule.length; i++ )
            actions.new_action('append', tmp_fw_name, rule[i][0], rule[i][1]);
        actions.new_action('append', 'firewall_rules_container', 'rule', tmp_fw_name);
        var resp = actions.callActions();
        if ( !resp.contains('Response: Success') )
        {
            top.log.error('pbx.call_plans.add: Error update extensions.conf.');
            return false;
        }
        parent.sessionData.pbxinfo['firewall'][tmp_fw_name] = rule;
        parent.sessionData.pbxinfo['fw_rules'].push('rule='+tmp_fw_name);
        top.dialog.dialogMessage({
            type: 'success',
            content: $P.lang("LANG815"),
            callback: function ()
            {
                mWindow.location.reload();
            }
        });
    } else if ( isNew == false )
    {
        var actions = listOfSynActions('fw_rules.conf');
        if ( tmp_fw_name != FW_OLD_NAME )
        {
            actions.new_action('delcat', FW_OLD_NAME, '', '');
            actions.new_action('newcat', tmp_fw_name, '', '');
        } else {
            actions.new_action('delcat', tmp_fw_name, '', '');
            actions.new_action('newcat', tmp_fw_name, '', '');
        }
        for ( var i = 0; i < rule.length; i++ )
            actions.new_action('append', tmp_fw_name, rule[i][0], rule[i][1]);
        if ( tmp_fw_name != FW_OLD_NAME )
        {
            actions.new_action('delete', 'firewall_rules_container', 'rule', FW_OLD_NAME);
            actions.new_action('append', 'firewall_rules_container', 'rule', tmp_fw_name);
        }
        var resp = actions.callActions();
        if ( !resp.contains('Response: Success') )
        {
            top.log.error('pbx.call_plans.add: Error update extensions.conf.');
            return false;
        }
        if ( tmp_fw_name != FW_OLD_NAME )
        {
            var firewall = parent.sessionData.pbxinfo['firewall'];
            delete firewall[FW_OLD_NAME];

            var idx = 0;
            var rules = parent.sessionData.pbxinfo['fw_rules'];
            var tmp_rule = 'rule='+ FW_OLD_NAME;
            for ( var i = 0; i < rules.length; i++ )
            {
                if ( rules[i] == tmp_rule )
                {
                    idx = i;
                    break;
                }
            }
            rules[idx] = 'rule='+tmp_fw_name;
        }
        parent.sessionData.pbxinfo['firewall'][tmp_fw_name] = rule;
        top.dialog.dialogMessage({
            type: 'success',
            content: $P.lang("LANG815"),
            callback: function ()
            {
                mWindow.location.reload();
            }
        });
    }
}

var delete_rule = function(item)
{
    var firewall = parent.sessionData.pbxinfo['firewall'];
    var rules = parent.sessionData.pbxinfo['fw_rules'];

    delete firewall[item];
    var rule = 'rule=' + item;
    for ( var i = 0; i < rules.length; i++ )
    {
        if ( rules[i] == rule )
        {
            rules.splice(i, 1);
            break;
        }
    }
}

var fw_remove = function(item)
{
    var actions = listOfSynActions('fw_rules.conf');
    actions.new_action('delcat', item, '', '');
    actions.new_action('delete', 'firewall_rules_container', 'rule', item);
    $P.cookie("disNoneApply", true);
    var resp = actions.callActions();
    if ( !resp.contains('Response: Success') )
    {
        top.log.error('pbx.call_plans.add: Error update extensions.conf.');
        return false;
    }
    return true;
}

var delete_FW_confirm = function(a, b)
{
    var tmp = '';
    tmp = '';
    tmp = '';
    tmp = '';
    tmp = '';
    tmp = '';
    tmp = '';
    tmp = '';

    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(a),
        buttons: {
            ok: function ()
            {
                delete_rule(a);
                var res = fw_remove(a);
                if ( res )
                    top.dialog.dialogMessage({
                        type: 'success',
                        content: $P.lang("LANG816"),
                        callback: function ()
                        {
                            mWindow.location.reload();
                        }
                    });
            }
        }
    });
}

var dealWith_address_port = function ( str, address, port )
{
    var tmpl = "Address:";
    if ( str.indexOf(tmpl) == 0 )
    {
        var tmp_str = str.substring(tmpl.length);
        var offset = tmp_str.indexOf(' ');
        var doc_address = document.getElementById(address);
        doc_address.value = tmp_str.substring(0, offset);
        var doc_port = document.getElementById(port);
        offset = tmp_str.lastIndexOf(':');
        doc_port.value = tmp_str.substring(offset+1);
    }
}


var applyChanges = function ()
{
    var fw_attr = parent.sessionData.pbxinfo['fw_attr'];

    var ping_container = $('#ping_container').find('input');
    var ping_enable_list = [];
    for ( var i = 0; ping_container && i < ping_container.length; i++ )
    {
        if ( ping_container[i].checked )
            ping_enable_list.push(ping_container[i].value);
    }
    var ping_enable = ping_enable_list.join(',');

    var synflood_container = $('#synflood_container').find('input');
    var synflood_list = [];
    for ( var i = 0; synflood_container && i < synflood_container.length; i++ )
    {
        if ( synflood_container[i].checked )
            synflood_list.push(synflood_container[i].value);
    }
    var syn_flood = synflood_list.join(',');

    var deathping_container = $('#deathping_container').find('input');
    var deathping_list = [];
    for ( var i = 0; deathping_container && i < deathping_container.length; i++ )
    {
        if ( deathping_container[i].checked )
            deathping_list.push(deathping_container[i].value);
    }
    var ping_of_death = deathping_list.join(',');

    var actions = listOfSynActions('fw_rules.conf');
    actions.new_action('delcat', 'general', '', '');
    actions.new_action('newcat', 'general', '', '');
    actions.new_action('append', 'general', 'ping_enable', ping_enable);
    actions.new_action('append', 'general', 'syn_flood', syn_flood);
    actions.new_action('append', 'general', 'ping_of_death', ping_of_death);
    $P.cookie("disNoneApply", true);
    var resp = actions.callActions();
    if ( !resp.contains('Response: Success') )
    {
        top.log.error('pbx.calling_rules.add: error adding ' + name + ' to extensions.conf');
        top.log.error(resp);
        return false;
    }

    /*  update the sessionData  */
    fw_attr['ping_enable'] = ping_enable;
    fw_attr['syn_flood'] = syn_flood;
    fw_attr['ping_of_death'] = ping_of_death;

    /*  call shell script  */
    //top.dialog.clearDialog();
    $.ajax({
        type: 'GET',
        url: '../webcgi?action=reloadSecurity&ipt_security=',
        success: function( response )
        {
            if (response.response.trim() == 'success') 
            {
                top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG945") });
            }
            else
            {
                top.dialog.dialogMessage({ type: 'error', content: response.body });
            }
        }
    });
}

var security_init = function()
{
    $('#button_expand').css('cursor', 'pointer');
    /*  request for webcgi  */
}

var interface_init = function()
{
    /*  */
    var value = "action=getparam&P231&P:wan_iface&P:lan_iface&P:lan1_iface&P:lan2_iface";
    $.ajax({
        type: "GET",
        url: "/webcgi",
        async: false,
        data: value,
        success: function( response )
        {
            var kv_pairs = response.split('\r\n');
            var mode = kv_pairs[0].split('=')[1];
            var interface_global = $('#firewall_interface');
            var interface_local = $('#new_fw_interface');

            var ping_container = $('#ping_container');
            var synflood_container = $('#synflood_container');
            var deathping_container = $('#deathping_container');

            if ( mode == 0 ) 
            {
                var wan = kv_pairs[1].split('=')[1];
                var lan = kv_pairs[2].split('=')[1];

                /*  ping syn_flood death_ping  */


                /*  modified by dcwei 2013 03 29  */
                append_chkbox( "ping", {"WAN": wan, "LAN": lan});
                append_chkbox( "synflood", {"WAN": wan, "LAN": lan});
                append_chkbox( "deathping", {"WAN": wan, "LAN": lan});

                interface_global.append($('<option>').html("WAN").val(wan));
                interface_global.append($('<option>').html("LAN").val(lan));
                interface_global.append($('<option>').attr('locale', 'LANG1959').html($P.lang("LANG1959")).val(lan+','+wan));
                interface_local.append($('<option>').html("WAN").val(wan));
                interface_local.append($('<option>').html("LAN").val(lan));
                interface_local.append($('<option>').attr('locale', 'LANG1959').html($P.lang("LANG1959")).val(lan+','+wan));
            } else if ( mode == 1 )
            {
                var lan = kv_pairs[1].split('=')[1];
                append_chkbox( "ping", {"LAN": lan});
                append_chkbox( "synflood", {"LAN": lan});
                append_chkbox( "deathping", {"LAN": lan});

                interface_global.append($('<option>').html("LAN").val(lan));
                interface_local.append($('<option>').html("LAN").val(lan));
            } else if ( mode == 2 )
            {
                var lan1 = kv_pairs[3].split('=')[1];
                var lan2 = kv_pairs[4].split('=')[1];
                append_chkbox( "ping", {"LAN1": lan1, "LAN2": lan2});
                append_chkbox( "synflood", {"LAN1": lan1, "LAN2": lan2});
                append_chkbox( "deathping", {"LAN1": lan1, "LAN2": lan2});

                interface_global.append($('<option>').html("LAN1").val(lan1));
                interface_global.append($('<option>').html("LAN2").val(lan2));
                interface_global.append($('<option>').attr('locale', 'LANG1959').html($P.lang("LANG1959")).val(lan1+','+lan2));
                interface_local.append($('<option>').html("LAN1").val(lan1));
                interface_local.append($('<option>').html("LAN2").val(lan2));
                interface_local.append($('<option>').attr('locale', 'LANG1959').html($P.lang("LANG1959")).val(lan1+','+lan2));
            }
        }
    });
}

var check_name_exist = function(value, element)
{
    var fw_rules = parent.sessionData.pbxinfo['fw_rules'];
    value = value.toLowerCase();
    for ( var i = 0; i < fw_rules.length; i++ )
    {
        var tmp_name = 'rule=rule_' + value;
        if ( tmp_name == 'rule='+FW_OLD_NAME )
            break;
        if ( fw_rules[i] == tmp_name )
        {
            return false;
        }
    }

    return true;
}

var check_service = function(value, element)
{
    if ( value == "custom" ) {
        var dst_ipaddr = _$('new_fw_dst_ipaddr').value || "Anywhere";
        var dst_port = _$('new_fw_dst_port').value || "Any";
        var src_ipaddr = _$('new_fw_src_ipaddr').value || "Anywhere";
        var src_port = _$('new_fw_src_port').value || "Any";

        if ( /^Anywhere$/.test(dst_ipaddr)
            && /^Anywhere$/.test(src_ipaddr)
            && /^Any$/.test(dst_port)
            && /^Any$/.test(src_port) )
            return false;

        return true;
    } else {
        return true;
    }
}

var check_ipaddr = function(value, element)
{
    if ( value == "" )
        return true;
    if ( !/^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$/.test(value) && !/^Anywhere$/.test(value) ) {
        return false;
    }

    return true;
}

var check_port = function(value, element)
{
    if ( /^Any$/.test(value) || value == "" )
        return true;
    if ( !/^[1-9][0-9]*$/.test(value) )
        return false;
    var cnt = parseInt(value);
    if ( cnt < 0 || cnt > 65535 )
        return false;

    return true;
}

var append_chkbox = function ( field, obj )
{
    var str = '#' + field + '_container';
    var container = $(str);
    $.each(
        obj, 
        function(key, value) {
            var div = $('<div>');
            div.addClass("divInline");
            var span = $('<span>');
            var chkbox = $('<input type="checkbox">');
            span.text(key);
            chkbox.val(value);
            div.append(span).append(chkbox);
            container.append(div);
        }
    );
}
