/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 *  Added for showing the CDR with a friendly diagram
 * 
 */
$P = top.$;
var records_too_big = false;
var backend = "CDR-CSV";
var records = [];
var uniqueIDs = [];
var condition = "";
var intDest = [];
var intExt_str = "('conf','',";
var total_count = "count(*)";
var recordsOutbound = [];
var recordsInbound = [];
var recordsInternal = [];
var recordsExternal = [];
var recordsAll = [];
var data = [];
var period_cond = [];
var calltypeContainer = $("#calltype_choices");
var periodContainer = $("#period_choices");
var serviceContainer = $("#service_choices");
var calltype_arr;
var calltype_key = "";
var period_key = "";//get which period option is selected;
var service_key = "";
var curr_date = new Date();
var curr_year = curr_date.getFullYear();
var curr_month = curr_date.getMonth() + 1;
var curr_month_str = (curr_month < 10 ) ? "0" + curr_month : curr_month;
var curr_day = curr_date.getDate();
var curr_day_str = (curr_day < 10) ? "0" + curr_day : curr_day;
var period_focused = "";
var search_count = 0;
var tick = 0;


function date_str(date)
{
    if(date < 10)
        return "0" + date;
    else
        return "" + date;
}


var year_options = function ()
{
    
    var options = "";
    var k;
    var num_year = 5;
    k = num_year - 1;
    while (k > -1)
    {
        if( k > 0 )
            options = options + '<option value="' + (curr_year - k) + '">' + (curr_year - k) + '</option>';
        else
            options = options + '<option value="' + (curr_year - k) + '" selected="selected">' + (curr_year - k) + '</option>';
        k--;
    }
    return options;
};

var period_keys = ["monthly", "weekly", "daily", "hourly", "acc_hourly"];
var service_keys = ["all", "sip", "pstn"];

var calltype = {
    "inbound": {
        id: "inbound",
        checked: false,
        locale: "LANG193",
        label: $P.lang("LANG193"),
        datasource: recordsInbound
        //callback: ib_dataBindCallback
    },
    "outbound": {
        id: "outbound",
        checked: false,
        locale: "LANG194",
        label: $P.lang("LANG194"),
        datasource: recordsOutbound
        //callback: ob_dataBindCallback
    },
    "internal": {
        id: "internal",
        checked: false,
        locale: "LANG195",
        label: $P.lang("LANG195"),
        datasource: recordsInternal
        //callback: it_dataBindCallback
    },
    "external": {
        id: "external",
        checked: false,
        locale: "LANG196",
        label: $P.lang("LANG196"),
        datasource: recordsExternal
        //callback: et_dataBindCallback        
    },
    "all_calltype": {
        id: "all_calltype",
        checked: true,
        locale: "LANG197",
        label: $P.lang("LANG197"),
        datasource: recordsAll
        //callback: all_dataBindCallback
    }
};


var period = {
    "monthly":
    {
        locale: "LANG198",
        label: $P.lang("LANG198"),
        type: "radio",
        checked: true,
        element: 
        [
            {
                row: 0,
                label: "",
                id: "monthly-date",
                name: "monthly-date",
                type: "select",
                onchange: getOnchangeFunc(),
                option: year_options()
            }
        ]
    },
    "weekly": 
    {
        locale: "LANG199",
        label: $P.lang("LANG199"),
        type: "radio",
        checked: false,
        element: 
        [
            {
                row: 0,
                label: "",
                id: "weekly-date",
                name: "weekly-date",
                type: "select",
                onchange: getOnchangeFunc(),
                option: year_options()
            }
        ]
    },
    "daily": {
        locale: "LANG200",
        label: $P.lang("LANG200"),
        type: "radio",
        checked: false,
        element: 
        [
            {
                row: 0,
                label: "",
                id: "daily-date",
                name: "daily-date",
                type: "text",
                defaults: curr_year + "-" + curr_month_str,
                onchange: getOnchangeFunc()
            }
        ]
    },
    "hourly": {
        locale: "LANG201",
        label: $P.lang("LANG201"),
        type: "radio",
        checked: false,
        element: 
        [
            {
                 row: 0,
                 label: "",
                 id: "hourly-date",
                 name: "hourly-date",
                 type: "text",
                 defaults: curr_year + "-" + curr_month_str + "-" + curr_day_str,
                 onchange: getOnchangeFunc()
             }
        ]
    },
    "acc_hourly": {
        locale: "LANG202",
        label: $P.lang("LANG202"),
        type: "radio",
        checked: false,
        element: 
        [
            {
                 row: 0,
                 label: "",
                 id: "acc_hourly-date-from",
                 name: "acc_hourly-date-from",
                 type: "text",
                 defaults: curr_year + "-" + curr_month_str,
                 onchange: getOnchangeFunc()
            },
               
            {
                 row: 1,
                 locale: "LANG171",
                 label: $P.lang("LANG171"),
                 id: "acc_hourly-date-to",
                 name: "acc_hourly-date-to",
                 type: "text",
                 defaults: curr_year + "-" + curr_month_str,
                 onchange: getOnchangeFunc()
                          
            }
        ]

    }
};

var service = {
    "all_service": {
        locale: "LANG104",
        label: $P.lang("LANG104"),
        type: "radio",
        checked: true
    },
    "sip": {
        locale: "LANG191",
        label: $P.lang("LANG191"),
        type: "radio",
        checked: false
    },
    "pstn": {
        locale: "LANG192",
        label: $P.lang("LANG192"),
        type: "radio",
        checked: false
    }
};

var showOutboundCalls = calltype["outbound"].checked;
var showInboundCalls = calltype["inbound"].checked; 
var showInternalCalls = calltype["internal"].checked;
var showExternalCalls = calltype["external"].checked;
var showAlltypeCalls = calltype["all_calltype"].checked;
var showSipCalls = service["sip"].checked;
var showPSTNCalls = service["pstn"].checked;
var showAllsvcCalls = service["all_service"].checked;

function and(cond)
{
    var and_cond = (cond.length > 0)? " AND " : "";
    return and_cond;
}

function where(cond)
{
    var where_cond = (cond.length > 0)? " WHERE " : "";
    return where_cond;
}

function or(cond)
{
    var or_cond = (cond.length > 0)? " OR " : "";
    return or_cond;
}

function getOnchangeFunc()
{
    return "setElementValue(this)";
}

function setElementValue(element)
{

    var id = $(element).attr("id");
    var ids = [];
    ids = id.split("-");
    //window.alert("setelementvalue id " + id);
    if (ids.length > 0)
    {
        var key = ids[0];
        var el_arr = period[key].element;
        for (var i = 0; i < el_arr.length; i++)
        {
            if (el_arr[i].id == id)
                el_arr[i].value = $(element).val();
        }
    }
}


function isset(obj)
{
    if (typeof obj != "object")
        return (typeof obj != "undefined");
    for (var i in obj)
        return true;
    return false;
}

function weekOfYear(year, month, day)
{
    //   year       ï¿œ?  
    //   month     ï¿œ?  
    //   day         ï¿œ?  
    //   æ¯åšä»åšæ¥åŒï¿œ?  
    var date1 = new Date(year, 0, 1);
    var date2 = new Date(year, month - 1, day, 1);
    var dayMS = 24 * 60 * 60 * 1000;
    var firstDay = (7 - date1.getDay()) * dayMS;
    var weekMS = 7 * dayMS;
    date1 = date1.getTime();
    date2 = date2.getTime();
    return Math.ceil((date2 - date1 - firstDay) / weekMS) + 1;
}

function processCDRResult(result, call_type)
{   
    var saved_data = [];
    var sub_datasets = [];
    var datasource = [];
    var color = "#000000";
    if(result.length > 0)
    {  
        var records = result.split("|");
        var total_num = records.length;
        for(var i = 0; i < total_num; i++ )
        {
            records[i] = records[i].split('$');
            saved_data[records[i][0]] = Math.floor(records[i][1]);
        } 
    }   
    
    if (period_key == "monthly") //monthly data
    {
        for (var i = 0; i < 12; i++)
        {
            var time = Date.UTC(period_cond[0], i);
            var time_str = period_cond[0] + "-" + date_str(i+1);
            if (saved_data[time_str] != undefined)
                datasource.push([time, saved_data[time_str]]);
            else
                datasource.push([time, 0]);
        }        
    }
    else if (period_key == "weekly")
    {
        var num_of_week = weekOfYear(period_cond[0], 12, 31);

        for (var i = 1; i < num_of_week; i++)
        {
            var time_str = period_cond[0] + "-" + date_str(i);

            if (saved_data[time_str] != undefined)
                datasource.push([i, saved_data[time_str]]);
            else
                datasource.push([i, 0]);
        }
        mode = null;
    }
    else if (period_key == "daily")   //daily data
    {
        var period_arr = period_cond[0].split("-");
        var year = parseInt(period_arr[0]);
        var month = parseInt(period_arr[1])-1;
        for (var i = 1; i <= 31; i++)
        {                    
            var time = Date.UTC(year, month, i);
            var time_str = period_cond[0] + "-" + date_str(i);
            if (saved_data[time_str] != undefined)
                datasource.push([time, saved_data[time_str]]);
            else
                datasource.push([time, 0]);
        }
    }
    else if (period_key == "hourly")  //hourly data
    {
        var period_arr = period_cond[0].split("-");
        var year = parseInt(period_arr[0]);
        var month = parseInt(period_arr[1])-1;
        var day = parseInt(period_arr[2]);
        for (var i = 0; i < 24; i++)
        {
            
            var time = Date.UTC(year, month, day, i);
            var time_str = period_cond[0] + "-" + date_str(i);
            if (saved_data[time_str] != undefined)
                datasource.push([time, saved_data[time_str]]);
            else
                datasource.push([time, 0]);
        }
    }
    else if (period_key == "acc_hourly")
    {
        for (var i = 0; i < 24; i++)
        {
            var key = Date.UTC(2012, 0, 1, i).toString();
            var time_str = date_str(i);
            if (saved_data[time_str] != undefined)
                datasource.push([key, saved_data[time_str]]);
            else
                datasource.push([key, 0]);
        }
    }
    
    if(call_type == "inbound")
    {
        color = "#EDC240";
    }
    else if(call_type == "outbound")
    {
        color = "#AFD8F8";
    }
    else if(call_type == "internal")
    {
        color = "#CB4B4B";
    }
    else if(call_type == "external")
    {
        color = "#4DA74D";
    }
    else if(call_type == "all_calltype")
    {
        color = "#9440ED";
    }
    
    if(call_type != "no_calltype") {
        var calltypeLocale = calltype[call_type].locale;
        sub_datasets[call_type] = { locale: calltypeLocale, label: $P.lang(calltypeLocale), data: datasource, color: color };
    }
    else
        sub_datasets[call_type] = { data: datasource};
    data.push(sub_datasets[call_type]);
    //console.log(data);
    /*
    $.plot($("#placeholder"), data, {
        yaxis: {
            min: 0,
            minTickSize: 1,              
            tickDecimals: 0   
        },
        xaxis: { 
            mode: mode
	    
	}
   });  
   */      
}

function onResponseError()
{
}

var plotAccordingToChoices = function()
{
    condition = "";
    search_count = 0;
    tick = 0;
    delete data;
    data = [];    
    period_cond = [];  //store year, month, day....if any;
    calltype_arr = calltypeContainer.find("input:checked");
    period_key = periodContainer.find("input:checked").attr("id"); //get which period option is selected;
    service_key = serviceContainer.find("input:checked").attr("id");
    var ob_cond = "(channel_ext in " + intExt_str + " AND dstchannel_ext not in " + intExt_str + ")";
    var ib_cond = "(channel_ext not in " + intExt_str + " AND dstchannel_ext in " + intExt_str + ")";
    var it_cond = "(channel_ext in " + intExt_str + " AND dstchannel_ext in " + intExt_str + ")";
    var et_cond = "(channel_ext not in " + intExt_str + " AND dstchannel_ext not in " + intExt_str + ")";
    var query_str = "";
    var groupby = "";
    var selectfield = "";
    var calltype_cond = ""; 
    var el_arr = period[period_key].element;
    var callbackFunc;
    
    var datasource = [];
    
    //top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG877") });
    $.each(el_arr, function (key, val)
    {  
        //get year/month/day.. value from the web element;
        period_cond.push($("#"+val.id).val());
    });
    
    if(service_key == "sip")
    {
        condition += " service = 's' ";
    }
    else if(service_key == "pstn")
    {
        condition += " service = 'p' ";
    }
    
    if(period_key == "monthly")
    {
        condition += and(condition) + " strftime('%Y',start) = '" + period_cond[0] + "' ";   
        groupby = " group by strftime('%m', start)"; 
        selectfield = " strftime('%Y-%m', start) ";            
    }
    else if(period_key == "weekly")
    {
        condition += and(condition) + " strftime('%Y',start) = '" + period_cond[0] + "' ";
        groupby = " group by strftime('%W', start)";
        selectfield = " strftime('%Y-%W', start) ";
    }
    else if(period_key == "daily")
    {
        condition += and(condition) + " strftime('%Y-%m', start) = '" + period_cond[0] + "' ";
        groupby = " group by strftime('%d', start)";
        selectfield = " strftime('%Y-%m-%d', start) ";
    }
    else if(period_key == "hourly")
    {
        condition += and(condition) + " strftime('%Y-%m-%d', start) = '" + period_cond[0] + "' ";
        groupby = " group by strftime('%H', start)";
        selectfield = " strftime('%Y-%m-%d-%H', start) ";
    }
    else if(period_key == "acc_hourly")
    {
        condition += and(condition) + " strftime('%Y-%m', start) >= '" + period_cond[0] + 
                     "' AND strftime('%Y-%m', start) <= '" + period_cond[1] + "' ";
        groupby = " group by strftime('%H', start)";
        selectfield = " strftime('%H', start) ";
    }
    
    condition = where(condition) + condition;
    
    calltype_arr.each(function()
    {
        search_count++;
    });
    
    
    if(calltype_arr.length > 0)
    {            
        calltype_arr.each(function()
        {
            
            var call_type = $(this).attr("id");
            if(call_type == "inbound")
                calltype_cond = and(condition) + ib_cond;
            else if(call_type == "outbound")
                calltype_cond = and(condition) + ob_cond;
            else if(call_type == "internal")
                calltype_cond = and(condition) + it_cond;
            else if(call_type == "external")
                calltype_cond = and(condition) + et_cond;
            else if(call_type == "all_calltype")
                calltype_cond = "";
                    
            query_str = "select " + selectfield + "," + total_count + " from cdr" + condition + calltype_cond + groupby;
            /*
	        $P.gs.actionRequest(document, "searchCDRDB", null, {
                    "request": ["cdr-query=" + query_str],
                    "auto": false,
                    "onInvalidResponse": onResponseError,
                    "onResponseProcessed": calltype[call_type].callback
            });  
            */
                        
            var request = $.ajax({
                url: "/webcgi",
                type: "GET",
                async: false,
                dataType: "json",
                data: "Action=searchCDRDB&cdr-query="+query_str,
                success: function (val)
                {                     
                    if ($P.gs.isObject(val) && $P.gs.isObject(val.body) && !$P.gs.isEmpty(val.body["cdr"]))
                    {        
                        var cdr_table = val.body["cdr"];
                        //console.log(cdr_table.result);
                        processCDRResult(cdr_table.result, call_type); 
                    
                    }
                    tick++;
                    /*
                    if(tick == search_count)
                    {
                         top.dialog.clearDialog(); 
                    }
                    */
                },
                error: function(msg)
                {
                }
            });
            
        }); 
    }
    else
    {
        processCDRResult("", "no_calltype"); 
    }
    
    var mode = (period_key == "weekly") ? null : 'time';
    $(document.body).show();
    $.plot($("#placeholder"), data, {
        yaxis: {
            min: 0,
            minTickSize: 1,              
            tickDecimals: 0   
        },
        xaxis: { 
            mode: mode,
            monthNames: ["LANG204", "LANG205", "LANG206", "LANG207", "LANG208", "LANG209", "LANG210", "LANG211", "LANG212", "LANG213", "LANG214", "LANG215"]  
	    
	}
   });
   //top.dialog.clearDialog();
             
}

if (!Array.indexOf)
{
    Array.prototype.indexOf = function (obj)
    {
        for (var i = 0; i < this.length; i++)
        {
            if (this[i] == obj)
            {
                return i;
            }
        }

        return -1;
    }
}

$P = top.$;

window.onload = function ()
{
    $P.lang(document, true);
    top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG593"));
    $("title").attr("locale", "LANG593");
    if (_$('engine') != null)
    {
        _$('engine').innerHTML = backend;
    }

    $.each(service, function (key, val)
    {
        var checked = '';
        if(val.checked == true)
            checked = 'checked="checked"';
        serviceContainer.append('<div style="line-height: 30px;"><input type="' + val.type + '" name="service" ' + checked + ' id="' + key + '">' +
                                        '<label for="id' + key + '" locale='+val.locale+'>'
                                        + val.label + '</label></div>');
    });

    serviceContainer.find("input").click(plotAccordingToChoices);


    var t = 0;
    $.each(calltype, function (key, val)
    {
        val.color = t;
        ++t;
    });

    // insert checkboxes 

    $.each(calltype, function (key, val)
    {
        var checked = '';
        if(val.checked == true)
            checked = 'checked="checked"';
        calltypeContainer.append('<div style="line-height: 30px;"><input type="checkbox" name="' + key +
                                        '" ' + checked + ' id="' + key + '">' +
                                        '<label for="id' + key + '" locale='+val.locale+'  class="label">'
                                        + val.label + '</label></div>');

    });

    calltypeContainer.find("input").click(plotAccordingToChoices);

    var newTable = $("<table/>");

    $.each(period, function (key, val)
    {
        //periodContainer.append('<tr>');
        var checked = '';
        if(val.checked == true)
            checked = 'checked="checked"';
        var newTr = $("<tr/>");
        newTr.css("line-height", "29px");
        newTr.append('<td style="text-align: left; position: relative;" valign="top">' +
                        '<input type="' + val.type + '" class="period" name="period" ' +  checked + ' id="' + key + '">' +
                        '<label for="id' + key + '" locale='+val.locale+'  class="label">' +
                            val.label + 
                        '</label>' +
                     '</td>');

        var el_arr = val.element;
        var td_str = '<td style="text-align: left; position: relative; top: 4px;" valign="top">';
        $.each(el_arr, function (index, el)
        {
            if(el.type == "text"){
                if (el.label) {
                    td_str = td_str + '<font locale='+el.locale+'>'+el.label+'</font>' + '<input type="text" id="' + el.id + '" name="' + el.name + '" class="special" value="' + el.defaults + '" oninput="plotAccordingToChoices()"/>';
                }
                else{
                    td_str = td_str + '<input type="text" id="' + el.id + '" name="' + el.name + '" class="special" value="' + el.defaults + '" oninput="plotAccordingToChoices()"/>';
                }              
            }
            else if(el.type == "select"){
                if (el.label) {
                    td_str = td_str + '<font locale='+el.locale+'>'+el.label+'</font>' + '<select id="' + el.id + '" name="' + el.name + '" onchange="plotAccordingToChoices()">' + el.option + '</select>';
                }
                else{
                    td_str = td_str + '</font>' + '<select id="' + el.id + '" name="' + el.name + '" onchange="plotAccordingToChoices()">' + el.option + '</select>';                    
                }
            }
            
        });
        td_str = td_str + "</td>";
        newTr.append(td_str);
        newTable.append(newTr);
    });
    $(".period").live("click", function () {
        plotAccordingToChoices();
    })
    periodContainer.append(newTable);
    var input_ids = ["daily-date", "acc_hourly-date-from", "acc_hourly-date-to"]; //these ones' datepicker needs to be customized. 
    $.each(input_ids, function(key, val) {
        $("#" + val).datepicker({
            dateFormat: 'yy-mm',
            changeMonth: true,
            changeYear: true,
            showButtonPanel: false,
           
            onClose: function() {
                var month = $("#ui-datepicker-div .ui-datepicker-month :selected").val();
                var year = $("#ui-datepicker-div .ui-datepicker-year :selected").val();
                $(this).val($.datepicker.formatDate('yy-mm', new Date(year, month, 1)));
                
            },
            
            beforeShow: function() {
               var selDate;
               if ((selDate = $(this).val()).length > 0)
               {
                   var month = parseInt(selDate.substring(5));
                   var year = parseInt(selDate.substring(0, 4));
                   $(this).datepicker('option', 'defaultDate', new Date(year, month-1, 1));
                   $(this).datepicker('setDate', new Date(year, month-1, 1));
               } 
            }   
        });

        $("#" + val ).focus(function () {
            if(period_focused == "" || period_focused == val)
            {
                var month = $("#ui-datepicker-div .ui-datepicker-month :selected").val();
                var year = $("#ui-datepicker-div .ui-datepicker-year :selected").val();
                $(this).val($.datepicker.formatDate('yy-mm', new Date(year, month, 1)));
            }
            var id_arr = val.split("-");
            if($("#"+id_arr[0]).attr("checked") == "checked") 
                plotAccordingToChoices();   
            period_focused = val;   
        });   
    });   
    
    $("#hourly-date").datepicker({
        dateFormat: 'yy-mm-dd',
        onSelect: function() {
            if($("#hourly").attr("checked") == "checked")
                plotAccordingToChoices();    
        }
    });
    $("#hourly-date").focus(function () {
        $(".ui-datepicker-calendar").show();
        period_focused = "hourly-date";
    });
    var jc = context2json({ filename: "cdr.conf", context: "general", usf: 1 });

    if (jc.hasOwnProperty('enable') && !jc['enable'].isAstTrue())
    {
        top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG990") });
    }

    var c = context2json({ filename: 'http.conf', context: 'general', usf: 1 });

    if (c.hasOwnProperty('enablestatic') && !c['enablestatic'].isAstTrue())
    {
        top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG991") });
    }
	var tmp_cmd = "action=systemcmdwithoutput&"+top.sessionData.directories.script_mastercsvexists;
	
	intDest = parent.pbx.users.list();
	var fxs_str = parent.sessionData.FXS_PORTS_DETECTED;
	if(fxs_str.length > 0)
	{
	    intDest = intDest.concat(fxs_str);
	}
	for(var i = 0; i < intDest.length; i++ )
	{
	    if(i < intDest.length - 1)
	        intExt_str += "'" + intDest[i] + "',";
	    else
	        intExt_str += "'" + intDest[i] + "'";
	}
	intExt_str += ")";
	if(intDest.length == 0)
	    intExt_str = "('conf','')";
    plotAccordingToChoices();
    top.Custom.init(document);
}


