/*
* Asterisk-GUI  - an Asterisk configuration interface
*
* trunks_analog.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 *  Removed call-id option from analog trunk web page.
 *  Marked the configchanged.conf when modify DAHDI and users
 *  Modified FXO volume option from a per-port option to a per-trunk one
 * 
 */
$P = top.$;
var mWindow = top.window.frames["mainscreen"]; // this will be the main window 
var zapchan_Before = '';
var FXOS = [];
var isNew, mode, EDIT_TRUNK, DAHDICHANNELSTRING, EDIT_TRUNKNAME;
var Electrical_Fields = ['busydetect', 'busycount', 'ringtimeout', 'polarityswitch', 'usecallerid', 'cidstart', 'pulsedial', 'cidsignalling', 'polarityonanswerdelay', 'countrytone', 'busyfrequencies', 'congestionfrequencies', 'busytone', 'congestiontone', 'busylevels', 'congestionlevels', 'busypattern', 'congestionpattern', 'congestiondetect', 'congestioncount', 'congestioncountview', 'enablecurrentdisconnectthreshold', 'rxgain', 'txgain', 'calleridscheme', 'faxdetect', 'echocancel' ];
var ch_chkbxClass = "FXO_ChkBoxes";
var type = "";
function arrIndexOf(arr, val) {
    if ($.isArray(arr)) {
        var i = arr.length;
        while (i--)
        {
            if (arr[i] === val)
            {
                return i;
            }
        }
        return -1;
    }
    else{
        return arr.indexOf(val);
    }
}
function contains(arr, a) // firefox don't support contains function
{
    if (arrIndexOf(arr, a) > -1 )
        return true;
    else
        return false;
};
window.onload = function ()
{
    $P.lang(document, true);
    DAHDICHANNELSTRING = top.sessionData.DahdiChannelString;
    FXOS = top.sessionData.FXO_PORTS_DETECTED;
    mode = gup("mode");
    var item = gup("item");
    loadDOMelements(mode ? true : false);
    if (mode)
    {
        populateGroupsSelect();
        $("#new_ATRNK_DIV").show();
        if (mode == "add")
        {
            new_analogTrunk_form();
        }
        else if (mode == "edit")
        {
            selectedTrunk_editOptions_form(item);
        }
        init_validator();
        top.Custom.init(document);
        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG639"));
        $("title").attr("locale", "LANG639");
        update_AnalogTrunksTable();
        sortTable({
            "tableId": 'table_ATRUNKS_list',
            sortFuncs: {
                "sort_0": ASTGUI.extenStyleCmpFunc
            }
        }).sortTableInitial(0).bindTitles( 
            {idx: 0, isDis: true}
        );
    }
};

var init_validator = function () {

	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "busypattern": {
                //pattern: '[1-9][0-9]+,[1-9][0-9]+',
				required: true,
                //I18N, the busypattern rules can be removed, since it has not been used.
                customCallback: ['Invalid Format ( format example: 500,500 )', function (value, element) {
                    if (!value)
                    {
                        return true;
                    }

                    if ( /^[0-9]+,[0-9]+$/.test(value) )
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }]

            },
            "congestionpattern": {
				required: true,
                //I18N, the congestionpattern rules can be removed, since it has not been used.
                customCallback: ["Invalid Format ( format example: X,X ), the range of 'X' is [100,2000]", function (value, element) {
                    if (!value)
                    {
                        return true;
                    }

                    if ( /^[0-9]+,[0-9]+$/.test(value) )
                    {
                        var a = Number(value.split(',')[0]);
                        var b = Number(value.split(',')[1]);

                        if (a >=100 && a <= 2000 && b >=100 && b <= 2000 )
                        {
                            return true;
                        }
                        else
                        {
                            return false;
                        }
                    }
                    else
                    {
                        return false;
                    }
                }]

            },
            "ringtimeout": {
                digits: true,
				range: [4000, 20000]
            },
            "edit_trunkName": {
                required: true,
                letterDigitUndHyphen: true,
                customCallback: [
                    $P.lang('LANG2135').format(1, $P.lang('LANG1329')),
                    function (value, element) {
                        if ( $('.' + ch_chkbxClass + ':checked').length )
                        {
                            if ( EDIT_TRUNKNAME != '' && value == EDIT_TRUNKNAME )
                                return true;
                            /*  checked if the name exists  */
                            var analog = parent.sessionData.pbxinfo.trunks['analog'];
                            for ( var el in analog )
                            {
                                if ( /^trunk_[1-9][0-9]*$/.test(el) && value == analog[el].trunkname )
                                {
                                    return false;
                                }
                            }
                            return true;
                        }
                        return false;
                    }
                ]

            },
            "busycount": {
                required: true,
                digits: true,
                range: [1, 2]
            },
            "congestioncountview": {
                required: true,
                digits: true,
                range: [1, 2]
            },
            "polarityonanswerdelay": {
                required: true,
                digits: true,
                range: [100, 2000]
            },
            "rxgain": {
                required: true,
                gain: [-13.5, 12]
            },
            "txgain": {
                required: true,
                gain: [-13.5, 12]
            },
            "busyfrequencies": {
                required: true,
                //I18N, the rule can be removed, since it has not been used.
                customCallback: ['Invalid format (format example: 450 or 480+620).', check_freq]
            },
            "congestionfrequencies": {
                required: true,
                //I18N, the rule can be removed, since it has not been used.
                customCallback: ['Invalid format (format example: 450 or 480+620).', check_freq]
            },
            "countrytone": {
                required: true
            },
            "busytone": {
                required: true,
                customCallback: [$P.lang('LANG2136'), check_tone]
            },
            "congestiontone": {
                required: true,
                customCallback: [$P.lang('LANG2136'), check_tone]
            },
            "currentdisconnectthreshold": {
                required: true,
                digits: true,
                range:[100, 2000]
            },
            "des_channels": {
                customCallback: [$P.lang('LANG2334'), function(){
                    return !(_$("src_channels").value == _$("des_channels").value);
                }]
            },
            "des_number": {
                required: true,
                digits: true,
                minlength: 2
            }
        },
        submitHandler: function() {
            if (type == "pstn")
            {
                pstnSave();
            }
            else
            {
                top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG2384') });
                setTimeout(function(){
                    new_ATRNK_save_go();
                }, 200);
            }
        }
    });

    $('.' + ch_chkbxClass).live("click", function () {
        $P('#edit_trunkName', document).valid();
        btn_disable();
    });

};

var check_freq = function (value, element) {
    if (!value)
    {
        return true;
    }

    if ( /^[0-9]+\+?[0-9]+$/.test(value) )
    {
        return true;
    }

    return false;
};

var check_tone = function (value, element) {
    if (!value)
    {
        return true;
    }
    var regex = /^f1=(\d+)?(@(-\d+))?(,f2=(\d+)(@(-\d+))?)?,c=((\d+)\/(\d+))(-(\d+)\/(\d+))?(-(\d+)\/(\d+))?;?$/;
    var match = value.match(regex);
    if (!match)
    {
        return false;
    }

    //match group fields example at pre_save_setting()
    //valid frequence range Tone frequency A in Hz; 0 <= Hz < 4000
    if (match[1] && match[1] > 4000) {
        return false;
    }
    if (match[5] && match[1] > 4000) {
        return false;
    }
    //valid levels range -300 < levelA < 0
    if (match[3] && (match[3] <= -300 || match[3] == 0)) {
        return false;
    }
    if (match[7] && (match[7] <= -300 || match[3] == 0)) {
        return false;
    }
    //valid pattern range  0 <= cadence <= 16383.
    var fields = [9, 10, 12, 13, 15, 16];
    for (var i = 0, len = fields.length; i < len; i++)
    { 
        var idx = fields[i];
        if (match[idx] && match[idx] > 16383)
        {
            return false;
        }
        
    }

    return true;
};

var getSelectedChannels = function ()
{
    return ASTGUI.domActions.get_checked(ch_chkbxClass);
};

var resetChannels = function ()
{
    ASTGUI.domActions.unCheckAll(ch_chkbxClass);
};

var checkChannels = function (channels)
{
    ASTGUI.domActions.checkSelected_analog_trunk(ch_chkbxClass, channels);
};

var populateGroupsSelect = function ()
{
    parent.pbx.trunks.listAllGroups().each(function (h)
    {
        var tr = parent.pbx.trunks.getTrunkNamesByGroup(h);
        var trstr = tr.join(", ");
        if (trstr.length > 30)
        {
            trstr = trstr.substr(30) + '...';
        }
        ASTGUI.selectbox.append(_$('edit_groups'), parent.pbx.trunks.getGroupDescription(h), h);
    });
};

var disable_usedChannels = function (trunk)
{ // trunk is (optional, used while editing a trunk, for new trunk leave blank)
    // FXOS -- list of all Analog channels
    var used = [];
    var c = parent.pbx.trunks.list({ analog: true });
    c.each(function (item)
    {
        if (trunk && trunk == item) return;
        used = used.concat(ASTGUI.miscFunctions.chanStringToArray(parent.sessionData.pbxinfo['trunks']['analog'][item][DAHDICHANNELSTRING]));
    });
    ASTGUI.domActions.unCheckAll(ch_chkbxClass);
    ASTGUI.domActions.disableSelected(ch_chkbxClass, used);
};

var countryToneToArraySort = function (obj) {
    var arr = [];
//    $(obj).each( function (key, value) {
//        arr.push(value);
//    });

    for (var item in obj)
    {
        if (obj.hasOwnProperty(item))
        {
            arr.push(obj[item]);
        }
    }

    arr.sort( function (left, right) {
        return left.desc.localeCompare(right.desc);
    });

    return arr;
};

var load_modal_form_pre = function () {
    var DOM_countryTone = _$('countrytone');
    var arr = countryToneToArraySort(top.sessionData.countryTone);

    var mapping_nation2lang = {
        "AD"     : 'LANG275',
        "AE"     : 'LANG276',
        "AF"     : 'LANG277',
        "AG"     : 'LANG278',
        "AI"     : 'LANG279',
        "AL"     : 'LANG280',
        "AM"     : 'LANG281',
        "AO"     : 'LANG282',
        "AQ"     : 'LANG283',
        "AR"     : 'LANG284',
        "AS"     : 'LANG285',
        "AT"     : 'LANG286',
        "AU"     : 'LANG287',
        "AW"     : 'LANG288',
        "AX"     : 'LANG289',
        "AZ"     : 'LANG290',
        "BA"     : 'LANG291',
        "BB"     : 'LANG292',
        "BD"     : 'LANG293',
        "BE"     : 'LANG294',
        "BF"     : 'LANG295',
        "BG"     : 'LANG296',
        "BG2"    : 'LANG297',
        "BG3"    : 'LANG298',
        "BH"     : 'LANG299',
        "BI"     : 'LANG300',
        "BJ"     : 'LANG301',
        "BL"     : 'LANG302',
        "BM"     : 'LANG303',
        "BN"     : 'LANG304',
        "BO"     : 'LANG305',
        "BQ"     : 'LANG306',
        "BR"     : 'LANG307',
        "BS"     : 'LANG308',
        "BT"     : 'LANG309',
        "BV"     : 'LANG310',
        "BW"     : 'LANG311',
        "BY"     : 'LANG312',
        "BZ"     : 'LANG313',
        "CA"     : 'LANG314',
        "CC"     : 'LANG315',
        "CD"     : 'LANG316',
        "CF"     : 'LANG317',
        "CG"     : 'LANG318',
        "CH"     : 'LANG319',
        "CI"     : 'LANG320',
        "CK"     : 'LANG321',
        "CL"     : 'LANG322',
        "CM"     : 'LANG323',
        "CN"     : 'LANG324',
        "CO"     : 'LANG325',
        "CR"     : 'LANG326',
        "CU"     : 'LANG327',
        "CV"     : 'LANG328',
        "CW"     : 'LANG329',
        "CX"     : 'LANG330',
        "CY"     : 'LANG331',
        "CZ"     : 'LANG332',
        "DE"     : 'LANG333',
        "DJ"     : 'LANG334',
        "DK"     : 'LANG335',
        "DK1"    : 'LANG336',
        "DM"     : 'LANG337',
        "DO"     : 'LANG338',
        "DO2"    : 'LANG339',
        "DZ"     : 'LANG340',
        "EC"     : 'LANG341',
        "EE"     : 'LANG342',
        "EG"     : 'LANG343',
        "EH"     : 'LANG344',
        "ER"     : 'LANG345',
        "ES"     : 'LANG346',
        "ET"     : 'LANG347',
        "FI"     : 'LANG348',
        "FJ"     : 'LANG349',
        "FK"     : 'LANG350',
        "FM"     : 'LANG351',
        "FO"     : 'LANG352',
        "FR"     : 'LANG353',
        "GA"     : 'LANG354',
        "GB"     : 'LANG355',
        "GD"     : 'LANG356',
        "GE"     : 'LANG357',
        "GF"     : 'LANG358',
        "GG"     : 'LANG359',
        "GH"     : 'LANG360',
        "GI"     : 'LANG361',
        "GL"     : 'LANG362',
        "GM"     : 'LANG363',
        "GN"     : 'LANG364',
        "GP"     : 'LANG365',
        "GQ"     : 'LANG366',
        "GR"     : 'LANG367',
        "GS"     : 'LANG368',
        "GT"     : 'LANG369',
        "GU"     : 'LANG370',
        "GW"     : 'LANG371',
        "GY"     : 'LANG372',
        "HK"     : 'LANG373',
        "HM"     : 'LANG374',
        "HN"     : 'LANG375',
        "HR"     : 'LANG376',
        "HT"     : 'LANG377',
        "HU"     : 'LANG378',
        "ID"     : 'LANG379',
        "IE"     : 'LANG380',
        "IL"     : 'LANG381',
        "IM"     : 'LANG382',
        "IN"     : 'LANG383',
        "IO"     : 'LANG384',
        "IQ"     : 'LANG385',
        "IR"     : 'LANG386',
        "IS"     : 'LANG387',
        "IT"     : 'LANG388',
        "JE"     : 'LANG389',
        "JM"     : 'LANG390',
        "JO"     : 'LANG391',
        "JP"     : 'LANG392',
        "KE"     : 'LANG393',
        "KG"     : 'LANG394',
        "KH"     : 'LANG395',
        "KI"     : 'LANG396',
        "KM"     : 'LANG397',
        "KN"     : 'LANG398',
        "KP"     : 'LANG399',
        "KR"     : 'LANG400',
        "KW"     : 'LANG401',
        "KY"     : 'LANG402',
        "KZ"     : 'LANG403',
        "LA"     : 'LANG404',
        "LB"     : 'LANG405',
        "LC"     : 'LANG406',
        "LI"     : 'LANG407',
        "LK"     : 'LANG408',
        "LR"     : 'LANG409',
        "LS"     : 'LANG410',
        "LS2"    : 'LANG411',
        "LT"     : 'LANG412',
        "LU"     : 'LANG413',
        "LV"     : 'LANG414',
        "LY"     : 'LANG415',
        "MA"     : 'LANG416',
        "MA2"    : 'LANG417',
        "MC"     : 'LANG418',
        "MD"     : 'LANG419',
        "ME"     : 'LANG420',
        "MF"     : 'LANG421',
        "MG"     : 'LANG422',
        "MH"     : 'LANG423',
        "MK"     : 'LANG424',
        "ML"     : 'LANG425',
        "MM"     : 'LANG426',
        "MN"     : 'LANG427',
        "MO"     : 'LANG428',
        "MP"     : 'LANG429',
        "MQ"     : 'LANG430',
        "MR"     : 'LANG431',
        "MS"     : 'LANG432',
        "MT"     : 'LANG433',
        "MU"     : 'LANG434',
        "MV"     : 'LANG435',
        "MW"     : 'LANG436',
        "MX"     : 'LANG437',
        "MY"     : 'LANG438',
        "MZ"     : 'LANG439',
        "NA"     : 'LANG440',
        "NC"     : 'LANG441',
        "NE"     : 'LANG442',
        "NF"     : 'LANG443',
        "NG"     : 'LANG444',
        "NI"     : 'LANG445',
        "NL"     : 'LANG446',
        "NO"     : 'LANG447',
        "NP"     : 'LANG448',
        "NR"     : 'LANG449',
        "NAURU"  : 'LANG450',
        "NU"     : 'LANG451',
        "NZ"     : 'LANG452',
        "OM"     : 'LANG453',
        "PA"     : 'LANG454',
        "PE"     : 'LANG455',
        "PF"     : 'LANG456',
        "PG"     : 'LANG457',
        "PH"     : 'LANG458',
        "PK"     : 'LANG459',
        "PL"     : 'LANG460',
        "PM"     : 'LANG461',
        "PN"     : 'LANG462',
        "PR"     : 'LANG463',
        "PS"     : 'LANG464',
        "PT"     : 'LANG465',
        "PT2"    : 'LANG466',
        "PW"     : 'LANG467',
        "PY"     : 'LANG468',
        "QA"     : 'LANG469',
        "QA1"    : 'LANG470',
        "QA2"    : 'LANG471',
        "QA3"    : 'LANG472',
        "RE"     : 'LANG473',
        "RO"     : 'LANG474',
        "RS"     : 'LANG475',
        "RU"     : 'LANG476',
        "RW"     : 'LANG477',
        "SA"     : 'LANG478',
        "SB"     : 'LANG479',
        "SC"     : 'LANG480',
        "SD"     : 'LANG481',
        "SE"     : 'LANG482',
        "SG"     : 'LANG483',
        "SH"     : 'LANG484',
        "SI"     : 'LANG485',
        "SJ"     : 'LANG486',
        "SK"     : 'LANG487',
        "SL"     : 'LANG488',
        "SM"     : 'LANG489',
        "SN"     : 'LANG490',
        "SO"     : 'LANG491',
        "SR"     : 'LANG492',
        "SS"     : 'LANG493',
        "ST"     : 'LANG494',
        "SV"     : 'LANG495',
        "SX"     : 'LANG496',
        "SY"     : 'LANG497',
        "SZ"     : 'LANG498',
        "TC"     : 'LANG499',
        "TD"     : 'LANG500',
        "TF"     : 'LANG501',
        "TG"     : 'LANG502',
        "TH"     : 'LANG503',
        "TJ"     : 'LANG504',
        "TK"     : 'LANG505',
        "TL"     : 'LANG506',
        "TM"     : 'LANG507',
        "TN"     : 'LANG508',
        "TO"     : 'LANG509',
        "TR"     : 'LANG510',
        "TT"     : 'LANG511',
        "TV"     : 'LANG512',
        "TW"     : 'LANG513',
        "TZ"     : 'LANG514',
        "TZ2"    : 'LANG515',
        "UA"     : 'LANG516',
        "UG"     : 'LANG517',
        "UG2"    : 'LANG518',
        "UK"     : 'LANG519',
        "UM"     : 'LANG520',
        "US"     : 'LANG521',
        "US_OLD" : 'LANG522',
        "US-OLD" : 'LANG523',
        "UY"     : 'LANG524',
        "UZ"     : 'LANG525',
        "VA"     : 'LANG526',
        "VC"     : 'LANG527',
        "VE"     : 'LANG528',
        "VG"     : 'LANG529',
        "VI"     : 'LANG530',
        "VN"     : 'LANG531',
        "VU"     : 'LANG532',
        "WF"     : 'LANG533',
        "WS"     : 'LANG534',
        "YE"     : 'LANG535',
        "YT"     : 'LANG536',
        "ZA"     : 'LANG537',
        "ZM"     : 'LANG538',
        "ZW"     : 'LANG539'
    };

    for (var i = 0, len = arr.length; i < len; i++)
    {
        var lang = mapping_nation2lang[arr[i].val.toUpperCase()];
        ASTGUI.selectbox.append(DOM_countryTone, $P.lang(lang), arr[i].val, lang);
    }
    ASTGUI.selectbox.append(DOM_countryTone, $P.lang('LANG231'), 'custom', 'LANG231');
    DOM_countryTone.selectedIndex = -1;

};

var load_modal_form_after = function ()
{
    var delayTime = 0;
    if ($('#polarityswitch').val() === 'yes')
    {
        $('#polarityfield').show();
    }
    else {
        $('#polarityfield').hide(); 
    }
    if ($('#countrytone').val() === 'custom')
    {
        $('.tone-setting').removeAttr('disabled');
    }
    else {
        $('.tone-setting').attr('disabled', true); 
    }
    $('#countrytone').bind('change', function (ev) {
        var $this = $(this);
        var val = $this.val();
        var freq_tmp;
        var busytone = '';
        var congestiontone = '';
        if (val === 'custom')
        {
            $('.tone-setting').removeAttr('disabled');
        }
        else
        {
            var countryTone = top.sessionData.countryTone[val];
            if (countryTone)
            {
                //f1=500[@-11][,f2=440[@-11]],c=500/500[-600/600[-700/700]]; 
                freq_tmp = countryTone.busyfreq.split('+');
                for ( var i = 0, len = freq_tmp.length; i < len; i++)
                {
                    busytone += 'f' + (i + 1) + '=' + freq_tmp[i] + '@-50' + ',';
                }
                busytone += 'c=' + countryTone.busypattern.replace(/,/g, '/');
                $('#busytone').val(busytone);

                freq_tmp = countryTone.congestionfreq.split('+');
                for ( var i = 0, len = freq_tmp.length; i < len; i++)
                {
                    congestiontone += 'f' + (i + 1) + '=' + freq_tmp[i] + '@-50' + ',';
                }
                congestiontone += 'c=' + countryTone.congestionpattern.replace(/,/g, '/');
                
                $('#congestiontone').val(congestiontone);

            }
            $('.tone-setting').attr('disabled', 'disabled').removeClass('ui-state-highlight');
        }
    });

    $('#enablecurrentdisconnectthreshold').change(function(){
        if($(this).val() == "no"){
            $("#currentdisconnectthreshold").hide();
            $("#currentdisconnectthreshold").attr('disabled', true);
        }
        else{
            $("#currentdisconnectthreshold").removeAttr('disabled');
            $("#currentdisconnectthreshold").show();
            var editTrunk = parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK];
            if (editTrunk && editTrunk['enablecurrentdisconnectthreshold'] === 'no')
            {
                //if enable = no, we actually set threshold = 3000.
                //reset to default value (200) to display.
                ASTGUI.resetTheseFields('currentdisconnectthreshold');
            }
        }
    });

    if($('#enablecurrentdisconnectthreshold').val() == 'no'){
        $('#enablecurrentdisconnectthreshold').change();
    }

    $('#calleridscheme').change( function (ev) {


/***********************************************************************
val | Caller ID Scheme                     | CID Start   |CID Signal
0   | Bellcore/Telcordia                   | Ring        |Bell
1   | ETSI-FSK during ringing              | Ring        |Bell
2   | ETSI-FSK prior to ringing with DTAS  | Ring        |Bell
3   | ETSI-FSK prior to ringing with LR    | Ring        |Bell
4   | ETSI-FSK prior to ringing with RP    | Ring        |Bell
5   | ETSI-DTMF during ringing             | Polarity    |DTMF
6   | ETSI-DTMF prior to ringing with DTAS | DTMF        |DTMF
7   | ETSI-DTMF prior to ringing with LR   | Polarity    |DTMF
8   | ETSI-DTMF prior to ringing with RP   | Polarity_in |DTMF
9   | SIN 227-BT                           | Polarity    |V23        
***********************************************************************/

        switch ($(this).val())
        {
        case '0':
            $('#cidstart').val('Ring');
            $('#cidsignalling').val('Bell');
            break;
        case '1':
            $('#cidstart').val('Ring');
            $('#cidsignalling').val('Bell');
            break;
        case '2':
            $('#cidstart').val('Ring');
            $('#cidsignalling').val('Bell');
            break;
        case '3':
            $('#cidstart').val('Ring');
            $('#cidsignalling').val('Bell');
            break;
        case '4':
            $('#cidstart').val('Ring');
            $('#cidsignalling').val('Bell');
            break;
        case '5':
            $('#cidstart').val('Polarity');
            $('#cidsignalling').val('DTMF');
            break;
        case '6':
            $('#cidstart').val('DTMF');
            $('#cidsignalling').val('DTMF');
            break;
        case '7':
            $('#cidstart').val('Polarity');
            $('#cidsignalling').val('DTMF');
            break;
        case '8':
            $('#cidstart').val('Polarity_in');
            $('#cidsignalling').val('DTMF');
            break;
        case '9':
            $('#cidstart').val('Polarity');
            $('#cidsignalling').val('V23');
            break;
        case '10':
            $('#cidstart').val('autodetect');
            $('#cidsignalling').val('Bell');
            break;
        default:
        }

    }); 

    $('#busydetect').change(function (ev) {
        $('#toneDiv').trigger('customchange');
        $('#busytone, #congestiontone').trigger("customDisable");
    });

    $('#congestiondetect').change(function (ev) {
        $('#toneDiv').trigger('customchange');
    });

    trunkAnalogBindToggle($('#busydetect'), $('#busycountfield'), delayTime);
    trunkAnalogBindToggle($('#congestiondetect'), $('#congestioncountfield'), delayTime);
    trunkAnalogBindToggle($('#polarityswitch'), $('#polarityfield'), delayTime);
    trunkAnalogBindToggle($('#usecallerid'), $('#calleridSchemeField'), delayTime);


    $('#busytone, #congestiontone').on('customDisable', function (ev) {
        var busydetect = $('#busydetect').val();
        var countrytone = $("#countrytone").val();
        if (busydetect === 'yes' && countrytone == "custom")
        {
            $('#busytone, #congestiontone').removeAttr("disabled", "disabled");
        }
        else
        {
            $('#busytone, #congestiontone').attr("disabled", "disabled");
        }
    });
    $('#toneDiv').on('customchange', function (ev) {
        var busydetect = $('#busydetect').val();
        var congestiondetect = $('#congestiondetect').val();

        if (busydetect === 'yes')
        {
            $('#busyToneField').show(delayTime);
        }
        else
        {
            $('#busyToneField').hide(delayTime);
        }

        if (congestiondetect === 'yes')
        {
            $('#congestionToneField').show(delayTime);
        }
        else
        {
            $('#congestionToneField').hide(delayTime);
        }

        if (busydetect !== 'yes' && congestiondetect !== 'yes')
        {
            $('#toneCountryField').hide(delayTime);
        }
        else
        {
            $('#toneCountryField').show(delayTime);
        }

        setTimeout(function () {
            if (window.frameElement)
            {
                $(window.frameElement).css("height", "0px");
                $(window.frameElement).css("width", "0px");
            }
            top.dialog.repositionDialog();
        }, delayTime + 10 );

    });

    //trigger bindings to show initial values
    setTimeout(function () {
        $('#countrytone').trigger('change');
        $('#calleridscheme').trigger('change');    
        $('#busydetect').trigger('change');
        $('#congestiondetect').trigger('change');
        $('#polarityswitch').trigger('change');
        $('#usecallerid').trigger('change');
    }, 20);
};


var new_analogTrunk_form = function ()
{
    isNew = true;
    EDIT_TRUNK = '';
    EDIT_TRUNKNAME = '';
    zapchan_Before = '';
    load_modal_form_pre();
    disable_usedChannels();
    //_$('new_ATRNK_addUpdateButton').innerHTML = $P.lang("LANG769");
    $('#HIDE_OnNEW_1').hide();
    $('#HIDE_OnNEW_2').hide();
    ASTGUI.updateFieldToValue('edit_trunkName', '');
    ASTGUI.resetTheseFields(Electrical_Fields);
    ASTGUI.resetTheseFields('currentdisconnectthreshold');
    DOM_edit_groups_select.selectedIndex = -1;
    load_modal_form_after();
};


var selectedTrunk_editOptions_form = function (w)
{
    EDIT_TRUNK = w;
    EDIT_TRUNKNAME = parent.sessionData.pbxinfo.trunks['analog'][w]['trunkname'];
    load_modal_form_pre();
    if (!FXOS.length)
    {
        top.dialog.dialogMessage({
            type: 'warning',
            content: $P.lang("LANG129").format($P.lang('LANG1724'))
        });
        return;
    }
    isNew = false;
    disable_usedChannels(EDIT_TRUNK);
    checkChannels(parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK][DAHDICHANNELSTRING]);
    zapchan_Before = parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK][DAHDICHANNELSTRING];
    var ct = ASTGUI.contexts.TrunkDIDPrefix + EDIT_TRUNK;
    var globals = globals = context2json({
        filename: 'extensions.conf',
        context: 'globals',
        usf: 1
    });
    var new_crl_pri_record_value = globals['OUT_AUTORECORD_' + EDIT_TRUNK.split("_")[1]];
    if (new_crl_pri_record_value) {
        ASTGUI.updateFieldToValue('auto_record', new_crl_pri_record_value, 'autoRecord');
    }
    ASTGUI.updateFieldToValue('edit_trunkName', getProperty(parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK], 'trunkname'));
    getPreviousVolumeForChannelsofThisTrunk(EDIT_TRUNK);
    Electrical_Fields.each(function (fld)
    {
        if (fld == 'polarityswitch')
            var fld_value = parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['answeronpolarityswitch'] || '';
        else{
            var fld_value = parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK][fld] || $('#'+fld).attr('dfalt');

            if(fld == 'enablecurrentdisconnectthreshold'){
                if( fld_value == "yes" ){
                    var value = parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['currentdisconnectthreshold'];
                    ASTGUI.updateFieldToValue(_$('currentdisconnectthreshold'), value);
                }
            }
        }

        ASTGUI.updateFieldToValue(_$(fld), fld_value);
    });


    DOM_edit_groups_select.selectedIndex = -1;
    var g = parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['group'].toString().split(',');
    g.each(function (group) { ASTGUI.updateFieldToValue(DOM_edit_groups_select, group); });

    ASTGUI.updateFieldToValue(_$('dummy_customCid'), '');
    ASTGUI.updateFieldToValue(_$('dummy_callerid'), 'asreceived');
    if (parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['callerid'] && parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['callerid'] != 'asreceived')
    {
        ASTGUI.updateFieldToValue(_$('dummy_customCid'), parent.sessionData.pbxinfo.trunks.analog[EDIT_TRUNK].callerid);
        ASTGUI.updateFieldToValue(_$('dummy_callerid'), 'custom');
    }

    if (parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['countryTone'])
    {
        $('#countrytone').val(parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['countryTone']);
    }

    $('#HIDE_OnNEW_1').hide();
    $('#HIDE_OnNEW_2').hide();
    load_modal_form_after();
    btn_disable();
};


var loadDOMelements = function (editor)
{
    DOM_table_ATRUNKS_list = _$('table_ATRUNKS_list');
    DOM_new_agtrnk_button = _$('new_agtrnk_button');
    DOM_new_ATRNK_DIV = _$('new_ATRNK_DIV');
    DOM_new_ATRNK_cls_container = _$('new_ATRNK_cls_container');
    DOM_trunk_OptionsList = _$('trunk_OptionsList');
    DOM_div_electrical = _$('div_electrical');
    DOM_div_audioLevels = _$('div_audioLevels');
    VOLUMES_TBL = _$('TABLE_PORTS_VOLUME');
    DOM_edit_groups_select = _$('edit_groups');

    if (editor)
    {
        FXOS.each(function (item)
        {
            var lbl = document.createElement('div');
            lbl.className = 'special';
            var label = document.createElement('label');
            var lbltext = document.createTextNode(item);
            var ncbx = document.createElement('input');
            ncbx.type = 'checkbox';
            ncbx.value = item;
            ncbx.className = ch_chkbxClass;
            label.appendChild(lbltext);
            lbl.appendChild(ncbx);
            lbl.appendChild(label);
            DOM_new_ATRNK_cls_container.appendChild(lbl);
        });
    }
};

//update hidden box of tone settings for auto update
// @param type:  ['busy', 'congestion']
var pre_save_tone = function (type) {
    var country = $('#countrytone').val();
    //var type = ['busy', 'congestion'];

    if (country === 'custom')
    {
        var regex = /^f1=(\d+)?(@(-\d+))?(,f2=(\d+)(@(-\d+))?)?,c=((\d+)\/(\d+))(-(\d+)\/(\d+))?(-(\d+)\/(\d+))?;?$/;
        type.each( function (type) {
            var match = $('#' + type + 'tone').val().match(regex);
            if (match)
            {
                //f1=350@-11,f2=440@-11,c=100/200-300/400-500/600;
                // Match Groups:
                //   |1 => 350 |2 => @-11 |3 => -11 |4 => ,f2=440@-11 |5 => 440 |6 => @-11 |7 => -11
                //   |8 => 100/200 |9 => 100 |10 => 200 |11 => -290/400 |12 => 300 |13 => 400
                //   |14 => -500/600 |15 => 500 |16 => 600

                var freq = match[1] + (match[5] ? ( '+' + match[5]) : '' );
                var levels = (match[3] ? match[3] : '-50') + ','
                             + (match[7] ? match[7] : '-50');
                var pattern = match[9] + '/' + match[10] 
                         + ( match[11] ? ( '-' + match[12] + '/' + match[13]) : '')
                         + ( match[14] ? ( '-' + match[15] + '/' + match[16] ) : '');
                $('#' + type + 'pattern').val(pattern);
                $('#' + type + 'frequencies').val(freq);
                $('#' + type + 'levels').val(levels);
            }
        });
    }
    else
    {
        var countryTone = top.sessionData.countryTone[country];
        type.each( function (type) {
            $('#' + type + 'pattern').val(countryTone[type + 'pattern']);
            $('#' + type + 'frequencies').val(countryTone[type + 'freq']);
            $('#' + type + 'levels').val('-50,-50');
        });
    }

};

var pre_save_setting = function () {
    var tone_type = [];
    var busydetect = $('#busydetect').val();
    var congestiondetect = $('#congestiondetect').val();

    if (busydetect === 'yes')
    {
        tone_type.push('busy');
    }
    else
    {
        //busydetect field itself is switch
        var $busycount = $('#busycount');
        $busycount.val( $busycount.attr('dfalt') || '0' );
        $('#busytone').val('');
    }

    if (congestiondetect === 'yes')
    {
        tone_type.push('congestion');
        $('#congestioncount').val( $('#congestioncountview').val() );
    }
    else
    {
        //congestioncount = 0 to switch off congestion detection
        $('#congestioncount').val('0'); 
        $('#congestiontone').val('');
    }

    if ($('#polarityswitch').val() !== 'yes')
    {
        var $delay = $('#polarityonanswerdelay');
        $delay.val( $delay.attr('dfalt') );
    }

    pre_save_tone(tone_type);
};

var new_ATRNK_save_go = function ()
{
	var pro_name = ASTGUI.getFieldValue('edit_trunkName');
	var len = pro_name.length;
    var scs = getSelectedChannels().join(',');
    var busycount = ASTGUI.getFieldValue('busycount');
    var polarityonanswerdelay = ASTGUI.getFieldValue('polarityonanswerdelay');

    pre_save_setting();

    // jcli modify
    // make fxs user/fxo trunk work right now, in fact, we only need unload and then load chan_dahdi.so here. 
    var c = listOfActions('configchanged.conf');
    c.new_action('update', 'configchanged', 'chandahdi', 'yes');
    c.callActions();
    // jcli end

    var groupstr = ASTGUI.getFieldValue(DOM_edit_groups_select) ? ASTGUI.getFieldValue(DOM_edit_groups_select).join(',') : 'New';
    groupstr = parent.pbx.trunks.formatGroupString(groupstr);
    var autoRecord = ASTGUI.getFieldValue('auto_record');
    if (ASTGUI.getFieldValue('edit_trunkName'))
    {
        var trunk_name = ASTGUI.getFieldValue('edit_trunkName');
    } else
    {
        var trunk_name = (getSelectedChannels().length) ? 'Ports ' + scs : 'Port ' + scs;
    }

    if (isNew)
    {
        var cbf = function ()
        {
            top.dialog.dialogMessage({
                type: 'success',
                content: $P.lang("LANG815"),
                callback: function ()
                {
                    mWindow.location.reload();
                }
            });
        };

        var tmp_object = { 'zapchan': scs, 'trunkname': trunk_name, 'group': groupstr, 'autoRecord': autoRecord };
        Electrical_Fields.each(function (fld)
        {
            if (fld == 'polarityswitch')
            {
                tmp_object['answeronpolarityswitch'] = ASTGUI.getFieldValue(_$(fld));
                tmp_object['hanguponpolarityswitch'] = ASTGUI.getFieldValue(_$(fld));
            }
            else{
                if(fld == 'enablecurrentdisconnectthreshold'){
                    if( ASTGUI.getFieldValue(_$(fld)) == "yes")
                        tmp_object['currentdisconnectthreshold'] = ASTGUI.getFieldValue(_$('currentdisconnectthreshold')); 
                    else
                        tmp_object['currentdisconnectthreshold'] = '3000'; 
                }

                tmp_object[fld] = ASTGUI.getFieldValue(_$(fld));
            }
        });

        if ($('#dummy_callerid').val() == 'asreceived')
        {
            tmp_object['callerid'] = 'asreceived';
        } else
        {
            tmp_object['callerid'] = $('#dummy_customCid').val();
        }
        //tmp_object['echocancel'] = "yes";
        parent.pbx.trunks.add1('analog', tmp_object, cbf);
        return;
    } 
    else
    {
        var new_groups = groupstr.split(",");

        var ded_group = parent.pbx.trunks.getDedicatedGroup(parent.pbx.trunks.getName(EDIT_TRUNK));
        if (!new_groups.contains(ded_group))
        {
            /* See if there is another dedicated group */
            var save_old_groups = parent.sessionData.pbxinfo.trunks['analog'][EDIT_TRUNK]['group'].toString().split(',');
            parent.sessionData.pbxinfo.trunks['analog'][EDIT_TRUNK]['group'] = new_groups.join(',');
            if (!parent.pbx.trunks.getDedicatedGroup(parent.pbx.trunks.getName(EDIT_TRUNK)))
            {
                top.log.debug("Can't remove a trunk from its dedicated group.");
                new_groups.push(ded_group);
                parent.sessionData.pbxinfo.trunks['analog'][EDIT_TRUNK]['group'] = new_groups.join(',');
            }
        }
    }

    /* If the user just put a trunk into another trunk's dedicated group, we will allow it
    but create a new dedicated group for the other trunk. */
    var ana_trunks = parent.pbx.trunks.list({ analog: true });

    /*
    ana_trunks.each(function (item)
    {
        var ded_group = parent.pbx.trunks.getDedicatedGroup(parent.pbx.trunks.getName(item));
        if (!ded_group || new_groups.contains(ded_group))
        {
            var new_ded_group = parent.pbx.trunks.makeDedicatedGroup();
            var g = parent.sessionData.pbxinfo.trunks['analog'][item]['group'].toString().split(',');
            g.push(new_ded_group);
            parent.sessionData.pbxinfo.trunks['analog'][item]['group'] = g.join(',');
            var x = new listOfSynActions('users.conf');
            x.new_action('update', item, 'group', parent.sessionData.pbxinfo.trunks['analog'][item]['group']);
            x.callActions();
        }
    });
    */
    var final_groups = [];
    for (var i = 0; i < new_groups.length; i++)
    {
        if (new_groups[i])
        {
            final_groups.push(new_groups[i]);
        }
    }
    delete new_groups;
    groupstr = final_groups.join(",");
    parent.sessionData.pbxinfo.trunks['analog'][EDIT_TRUNK]['group'] = groupstr;
    var chans = getSelectedChannels().join(',');
    var oldChans = parent.sessionData.pbxinfo.trunks.analog[EDIT_TRUNK]['dahdichan'];
    var oldChannels = ASTGUI.miscFunctions.chanStringToArray(oldChans);
    var channels = ASTGUI.miscFunctions.chanStringToArray(chans);
    if (channels.length < oldChannels.length) {
        oldChannels.each(function (ch) {
            if (contains(channels, ch)) {
                ASTGUI.ASTDB.updateKey({ dbname: 'TRUNK/DAHDI_' + ch, key: 'AUTORECORD', keyvalue: autoRecord == "ENABLE" ? "YES" : "NO" });
            }
            else {
                ASTGUI.ASTDB.deleteKey({ dbname: 'TRUNK/DAHDI_' + ch, key: 'AUTORECORD'});
            }
        })
    }
    else {
        channels.each(function (ch) {
            ASTGUI.ASTDB.updateKey({ dbname: 'TRUNK/DAHDI_' + ch, key: 'AUTORECORD', keyvalue: autoRecord == "ENABLE" ? "YES" : "NO" });
        })
    } 
    ASTGUI.updateaValue({ file: 'extensions.conf', context: 'globals', variable: 'OUT_AUTORECORD_' + EDIT_TRUNK.split("_")[1], value: autoRecord});
    var x = new listOfActions('extensions.conf');
    x.new_action('update', 'globals', EDIT_TRUNK, "DAHDI/g" + parent.pbx.trunks.getDedicatedGroup(parent.pbx.trunks.getName(EDIT_TRUNK)));
    x.callActions();

    // just update the selected channels
    /*
    (function ()
    {
        var x = new listOfSynActions('users.conf');
        x.new_action('update', EDIT_TRUNK, 'group', groupstr);
        x.new_action('delete', EDIT_TRUNK, 'gui_volume', '');
        x.new_action('delete', EDIT_TRUNK, 'gui_fxooffset', '');
        x.new_action('delete', EDIT_TRUNK, 'rxgain', '');
        x.callActions();
        x.clearActions();
        x.new_action('delete', EDIT_TRUNK, 'txgain', '');
        x.new_action('delete', EDIT_TRUNK, 'signalling', '');
        x.new_action('delete', EDIT_TRUNK, 'channel', '');
        x.callActions();
        x.clearActions();

        parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['group'] = groupstr;

        var zap_channels = ASTGUI.miscFunctions.chanStringToArray(scs);

        zap_channels.each(function (channel)
        {
            x.new_action('append', EDIT_TRUNK, 'channel', channel);
        });
        x.callActions();
    })();
    */
    save_audioLevels(save_electrical, trunk_name, scs);
};

var update_AnalogTrunksTable = function ()
{
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function

    if (!FXOS.length)
    {
        if (DOM_table_ATRUNKS_list) {
            var newRow = DOM_table_ATRUNKS_list.insertRow(-1);
            var ntd = document.createElement('TD');
            ntd.setAttribute('colspan', 4); ntd.colSpan = 4; // for IE
            ntd.align = 'center';
            ntd.innerHTML = $P.lang('LANG1394');
            newRow.appendChild(ntd);
            DOM_new_agtrnk_button.style.display = 'none';

            var c = parent.pbx.trunks.list({ analog: true });
            if (!c.length) { return; }
        }
    }

    (function ()
    { // add first row
        if (DOM_table_ATRUNKS_list) {
            var newRow = DOM_table_ATRUNKS_list.insertRow(-1);
            newRow.className = "frow";
            addCell(newRow, { html: "<span locale='LANG83'>"+$P.lang('LANG83')+"</span>"});
            addCell(newRow, { html: "<span locale='LANG232'>"+$P.lang('LANG232')+"</span>"});
            addCell(newRow, { html: "<span locale='LANG74'>"+$P.lang('LANG74')+"</span>"});
        }
    })();

    (function ()
    {
        var c = parent.pbx.trunks.list({ analog: true });
        c.each(function (item)
        {
            var tmp = "<button class='options edit' onclick=\"showEditForm('" + item + "')\" localeTitle='LANG738' title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                      "<button class='options del' onclick=\"delete_trunk_confirm('" + item + "')\" localeTitle='LANG739' title=" + $P.lang("LANG739") + "></button>";
            if (DOM_table_ATRUNKS_list) {
                var newRow = DOM_table_ATRUNKS_list.insertRow(-1);
                newRow.className = ((DOM_table_ATRUNKS_list.rows.length) % 2 == 1) ? 'odd' : 'even';
                addCell(newRow, { html: parent.sessionData.pbxinfo['trunks']['analog'][item]['trunkname'] });
                addCell(newRow, { html: parent.sessionData.pbxinfo['trunks']['analog'][item][DAHDICHANNELSTRING] });
                addCell(newRow, { html: tmp });
            }          
        });

        if (!c.length)
        {
            ASTGUI.domActions.clear_table(DOM_table_ATRUNKS_list);
            if (DOM_table_ATRUNKS_list) {
                var newRow = DOM_table_ATRUNKS_list.insertRow(-1);
                addCell(newRow, { html: $P.lang('LANG129').format($P.lang('LANG639')), locale: 'LANG129 LANG639' });                
            }

        }
    })();
};

var delete_trunk_confirm = function (a)
{
    EDIT_TRUNK = a;
    var chans = parent.sessionData.pbxinfo.trunks['analog'][EDIT_TRUNK]['dahdichan'];
    var channels = ASTGUI.miscFunctions.chanStringToArray(chans);
    var trunk_name = getProperty(parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK], 'trunkname') || EDIT_TRUNK;
    var tmp_rules = parent.sessionData.pbxinfo.callingRules.getOwnProperties();

    /*
    *	added by dcwei 2012-11-09
    *	Delete the relevant outgoing rules firstly when removed the trunk.
    */
    for (var i = 0; tmp_rules[i]; i++)
    {
        rule = parent.sessionData.pbxinfo.callingRules[tmp_rules[i]];
        if ( rule ) 
        {
            var item = '';
            for ( var j = 0; j < rule.length; j++ )
            {
                item = rule[j];
                if ( /Macro\(dialout-trunk,([1-9][0-9]*),/.test(item) )
                {
                    var idx = item.match(/Macro\(dialout-trunk,([1-9][0-9]*),/)[1];
                    trunk = "trunk_" + idx;
                    if (trunk == EDIT_TRUNK)
                    {
                        top.dialog.dialogMessage({ type: 'warning', content: $P.lang("LANG944").format(tmp_rules[i]) });
                        return;
                    }
                }
            }
        }
    }

    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG946").format(trunk_name),
        buttons: {
            ok: function ()
            {
                if (parent.pbx.trunks.remove1(EDIT_TRUNK))
                {
                    // jcli add to make fxs user/fxo trunk changes work right now
                    var u = new listOfSynActions('extensions.conf') ;
                    u.new_action('delete', 'globals', 'OUT_AUTORECORD_' + EDIT_TRUNK.split("_")[1], '', '');
                    u.callActions();
                    channels.each(function (ch)
                    {
                        ASTGUI.ASTDB.deleteKey({ dbname: 'TRUNK/DAHDI_' + ch, key: 'AUTORECORD'});
                    })
                    var c = listOfActions('configchanged.conf');
                    c.new_action('update', 'configchanged', 'chandahdi', 'yes');
                    c.callActions();
                    // jcli end

                    top.dialog.dialogMessage({
                        type: 'success',
                        content: $P.lang("LANG821").format(EDIT_TRUNK, $P.lang("LANG2471")),
                        callback: function ()
                        {
                            EDIT_TRUNK = '';
                            mWindow.location.reload();
                        }
                    });
                }

            }
        }
    });
};


var save_electrical = function ()
{
    var x = new listOfActions('users.conf');
    Electrical_Fields.each(function (fld)
    {
        if (fld == 'polarityswitch')
        {
            x.new_action('update', EDIT_TRUNK, 'answeronpolarityswitch', ASTGUI.getFieldValue(_$(fld)));
            x.new_action('update', EDIT_TRUNK, 'hanguponpolarityswitch', ASTGUI.getFieldValue(_$(fld)));
        }
        else{
            if(fld == 'enablecurrentdisconnectthreshold'){
                if( ASTGUI.getFieldValue(_$(fld)) == "yes"){
                    parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['currentdisconnectthreshold'] = ASTGUI.getFieldValue(_$('currentdisconnectthreshold'));
                    x.new_action('update', EDIT_TRUNK, 'currentdisconnectthreshold', ASTGUI.getFieldValue(_$('currentdisconnectthreshold')));
                }
                else{
                    x.new_action('update', EDIT_TRUNK, 'currentdisconnectthreshold', '3000');
                }
            }

            x.new_action('update', EDIT_TRUNK, fld, ASTGUI.getFieldValue(_$(fld)));
        }
    });

    if ($('#dummy_callerid').val() == 'asreceived')
    {
        parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['callerid'] = 'asreceived';
        x.new_action('update', EDIT_TRUNK, 'callerid', 'asreceived');
    } else
    {
        parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['callerid'] = $('#dummy_customCid').val();
        x.new_action('update', EDIT_TRUNK, 'callerid', $('#dummy_customCid').val());
    }

    var after = function ()
    {
        top.dialog.clearDialog();
        Electrical_Fields.each(function (fld)
        {
            if (fld == 'polarityswitch')
            {
                parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['answeronpolarityswitch'] = ASTGUI.getFieldValue(_$(fld));
                parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['hanguponpolarityswitch'] = ASTGUI.getFieldValue(_$(fld));
            }
            else
                parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK][fld] = ASTGUI.getFieldValue(_$(fld));
        });

        top.dialog.dialogMessage({
            type: 'success',
            content: $P.lang("LANG844"),
            callback: function ()
            {
                mWindow.location.reload();
            }
        });
    };

    top.dialog.dialogMessage({ type: 'loading', content: $P.lang('LANG826') });
    setTimeout(function () { x.callActions(after); }, 200);
};


var getPreviousVolumeForChannelsofThisTrunk = function (trunk)
{
    var c = context2json({ filename: 'users.conf', context: trunk, usf: 0 });
    var channels = ASTGUI.miscFunctions.chanStringToArray(parent.sessionData.pbxinfo.trunks.analog[trunk][DAHDICHANNELSTRING]);
    if (!channels.length) return;

    ASTGUI.domActions.clear_table(VOLUMES_TBL);
    var tmp_sel = document.createElement('select');
    ASTGUI.selectbox.append(tmp_sel, 'Low', '1');
    ASTGUI.selectbox.append(tmp_sel, 'Soft', '2');
    ASTGUI.selectbox.append(tmp_sel, 'Normal', '3');
    ASTGUI.selectbox.append(tmp_sel, 'Loud', '4');
    ASTGUI.selectbox.append(tmp_sel, 'Louder', '5');
    ASTGUI.selectbox.append(tmp_sel, 'Loudest', '6');

    //jcli modify
    var newRow = VOLUMES_TBL.insertRow(-1);
    ASTGUI.domActions.tr_addCell(newRow, { html: $P.lang('LANG1354') });
    var newcell = newRow.insertCell(newRow.cells.length);
    var sel = tmp_sel.cloneNode(true);
    sel.id = 'ports_guivolume';
    newcell.appendChild(sel);

    var t = c.indexOfLike('gui_volume='); // use indexOfLike other than indexOf.
    if (t == -1)
    {
        ASTGUI.selectbox.selectOption(sel, '2');
        return;
    }
    var s = c[t].afterChar('=');
    ASTGUI.selectbox.selectOption(sel, s);
    // jcli end
};


var save_audioLevels = function (CB_FN, trunk_name, dahdichannels)
{
    var get_guiVolValue = { '1': '-2', '2': '0', '3': '2', '4': '5', '5': '9', '6': '12' };

    var x = new listOfActions('users.conf');
    x.new_action('update', EDIT_TRUNK, 'trunkname', trunk_name.guiMetaData());
    x.new_action('update', EDIT_TRUNK, DAHDICHANNELSTRING, dahdichannels);
    x.new_action('update', EDIT_TRUNK, "channel", dahdichannels);
    parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK]['trunkname'] = trunk_name;
    parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK][DAHDICHANNELSTRING] = dahdichannels;
    x.new_action('delete', EDIT_TRUNK, 'gui_volume', '');
    x.new_action('delete', EDIT_TRUNK, 'gui_fxooffset', '');
    x.new_action('delete', EDIT_TRUNK, 'cidstart', '');
    x.new_action('delete', EDIT_TRUNK, 'cidsignalling', '');
    //x.new_action('delete', EDIT_TRUNK, 'rxgain', '');
    //x.new_action('delete', EDIT_TRUNK, 'txgain', '');
    x.new_action('delete', EDIT_TRUNK, 'signalling', '');

    var fx = 0;
    var gv = ASTGUI.getFieldValue(_$('ports_guivolume'));
    x.new_action('append', EDIT_TRUNK, 'gui_volume', gv.guiMetaData());
    x.new_action('append', EDIT_TRUNK, 'gui_fxooffset', fx.guiMetaData());
    x.new_action('append', EDIT_TRUNK, 'cidstart', $('#cidstart').val());           
    x.new_action('append', EDIT_TRUNK, 'cidsignalling', $('#cidsignalling').val());
    //x.new_action('append', EDIT_TRUNK, 'rxgain', fx + Number(get_guiVolValue[gv]));
    //x.new_action('append', EDIT_TRUNK, 'txgain', '0.0');

    var after = function ()
    {
        // top.dialog.clearDialog();
        // top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG844") });
        if (CB_FN) { CB_FN(); }
    };
    setTimeout(function () { x.callActions(after); }, 200);
};


var reset_calibration = function ()
{
    var zc = parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK][DAHDICHANNELSTRING].split(',');
    var x = new listOfActions('users.conf');
    x.new_action('delete', EDIT_TRUNK, 'gui_volume', '');
    x.new_action('delete', EDIT_TRUNK, 'gui_fxooffset', '');
    x.new_action('delete', EDIT_TRUNK, 'rxgain', '');
    x.new_action('delete', EDIT_TRUNK, 'txgain', '');
    x.new_action('delete', EDIT_TRUNK, 'channel', '');

    var after = function ()
    {
        var f = new listOfSynActions('fxotune.conf');
        zc.each(function (channel)
        {
            f.new_action('delcat', channel, '', '');
        });
        f.callActions();
        top.dialog.clearDialog();
        $("#div_calibrate").hide();
    };
    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG981") });
    x.callActions(after);
};


//do not add I18N for calibrate ports, since it has not been used.
var calibrate_ports = function ()
{
    var reqFailed = true;
    var checkOutPut = function ()
    {
        var op = ASTGUI.loadHTML('./fxotune_out').toLowerCase();
        var msg1 = 'starting fxo tuning';
        var msg2 = 'fxo tuning is complete';
        if (op.contains(msg1))
        {
            top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG982") });
            setTimeout(checkOutPut, 5000);
            return;
        }
        if (op.contains(msg2))
        {
            $("#div_calibrate").hide();

            // TODO updateUsers_fromFxotune();
            top.dialog.dialogMessage({ type: 'success', content: $P.lang("LANG947") });
            //ASTGUI.dialog.alertmsg('Finished Calibrating !! <BR> Click "Apply Changes" and restart your appliance ');
        }
    };
    var zc = parent.sessionData.pbxinfo['trunks']['analog'][EDIT_TRUNK][DAHDICHANNELSTRING];
    var k = zc.split(',').join(' ');
    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG983") });
    var st = setTimeout(
			function ()
			{
			    if (reqFailed)
			    {
			        top.dialog.clearDialog();
			    }
			}, 480000 // if there was no response in 8 minutes - hide the dialog so that the user can try again or navigate to a different page
		);

	/*	Coment by dcwei 2012-12-28 from AA50
    parent.ASTGUI.systemCmd('/bin/fxotune_from_gui ' + k, function ()
    {
        clearTimeout(st);
        reqFailed = false;
        setTimeout(checkOutPut, 4000);
    });
	*/
};

function pstnDetection()
{
    if ($("#detect").attr("disabled") == "disabled")
    {
        return;
    }  
    $("#new_ATRNK_DIV").hide();
    $("#pstn_div").show();

    if (window.frameElement)
    {
        $(window.frameElement).css("height", "0px");
        $(window.frameElement).css("width", "0px");
    }
    top.dialog.repositionDialog();
    
    _$("src_number").value = "6000";
    _$("des_number").value = "";    
    ASTGUI.selectbox.clear('src_channels');
    ASTGUI.selectbox.clear('des_channels');
    selectChannel = getSelectedChannels();
        
    selectChannel.each(function (item)
    {
        ASTGUI.selectbox.append('src_channels', item, item);
    });
    
    FXOS.each(function (item)
    {
        ASTGUI.selectbox.append('des_channels', item, item);
    });
    top.Custom.init(document, $("#pstn_div")[0]);
    
    type = "pstn";
};

var pstnCancel = function ()
{
    $("#pstn_div").hide();
    $("#new_ATRNK_DIV").show();
    type = "edit";
    
    if (window.frameElement)
    {
        $(window.frameElement).css("height", "0px");
        $(window.frameElement).css("width", "0px");
    }
    top.dialog.currentDialogType = "iframe";
    top.dialog.repositionDialog();
};

var pstnAfter = function()
{
    top.dialog.container.show();
    top.dialog.shadeDiv.show();
    pstnCancel();
};

var pstnStop = function()
{
    $.ajax({ type: "post", url: "/webcgi", data: "action=stopPSTNDetecting&pstn-cpt=", success: pstnAfter});
};

var mappingErrCode = {
    0 : ""        ,        // "no error return"
    1 : "LANG2415",        // "invaid extension number"
    2 : "",                // "invaid fxo channel num"
    3 : "LANG2416",        // "fxo absent/busy"
    4 : "LANG2417",        // "fxo disconnect"
    5 : "",                // "monitor task init failed"
    6 : "LANG2418",        // "fxo configure failed"
    7 : "LANG2419",        // "fxo offhook failed"
    8 : "LANG2420",        // "fxo dial failed"
    9 : "LANG2421",        // "no dial tone detected"
    10: "LANG2422",        // "pickup the call failed"
    11: "LANG2423",        // "user pickup the call timeout"
    12: "",                // "no ringback tone detected"
    13: "LANG2424",        // "fxo dtmf send failed"
    14: "LANG2425",        // "fxo no ringing, call setup failed"
    15: "LANG2426",        // "hang up the call failed"
    16: "LANG2427",        // "user hangup the call timeout"
    17: "",                // "no busy tone detected"
    18: "",                // "thread creat failed"
    19: ""                 // "end of error string"
}
var pstnSave = function()
{
    var pstnOver = function ()
    {               
        var i = 120;
        var status_type = 1;
        var getResponce = function(flag)
        {       
            setTimeout(function(){
                  
                $.ajax({ type: "post", url: "/webcgi", data: "action=getPSTNResult&pstn-cpt=", success: function(res){
                    if (res && res.body && res.body["pstn-cpt"] )
                    {
                        var str = res.body["pstn-cpt"];
                        var start = str.split("\n")[0].split("=")[6];
                        var status = 0;
                        var err = 0;
                        if ( str.split("\n")[1] && str.split("\n")[1].split("!") )
                        {
                            status = str.split("\n")[1].split("!")[0].split("=")[1];
                            err = str.split("\n")[1].split("!")[1].split("=")[1];
                            status = Number(status);
                            err = Number(err);
                        }
                        //var start = "done";
                        
                        if (start == "done")
                        {   
                            flag = 0;
                            if ( !err && str.indexOf("toneType=BUSY TONE!") != -1)
                            {
                                var elapsed = "";
                                var polarity = "";
                                var busy_tone = "";
                                    
                                str.split("\n").each(function(val){

                                    if (val && val.split("!"))
                                    {
                                        if ( val.indexOf("elapsedtime") != -1 && val.split("=")[2])
                                        {
                                            elapsed = Number(val.split("=")[2].rChop("ms"));
                                            elapsed = Math.round(elapsed * 0.085) * 10;
                                        }
                                        else if ( val.indexOf("No Current Disconnect Detecte") != -1 )
                                        {
                                            elapsed = "no";
                                        }
                                        else if ( val.indexOf("Polarity") != -1 )
                                        {
                                            if ( val.indexOf("No Polarity Reversal Detected") != -1 )
                                            {
                                                polarity = "no";
                                            }
                                            else
                                            {
                                                polarity = "yes";
                                            }
                                        }
                                        else if ( val.indexOf("toneType=BUSY TONE!") != -1 && val.split("!")[1] )
                                        {
                                             busy_tone = val.split("!")[1];
                                        }
                                    }
                                });
                                
                                var buf = "";
                                if ( elapsed )
                                {
                                    buf += $P.lang("LANG1694") + ": " + ( elapsed == "no"? $P.lang("LANG137") : elapsed ) + "<br/>";
                                }
                                
                                if ( polarity )
                                {
                                    buf += $P.lang("LANG1340") + ": " + 
                                        (polarity == "yes" ? $P.lang("LANG136") : $P.lang("LANG137") ) + "<br/>";
                                }
                                                                
                                if ( busy_tone )
                                {
                                    buf += $P.lang("LANG1325") + ": " + busy_tone + "<br/>";
                                }
                                
                                top.dialog.dialogConfirm({
                                    confirmStr: $P.lang("LANG2338").format(buf),
                                    buttons: {
                                        ok: function ()
                                        {
                                            if ( elapsed )
                                            {
                                                if ( elapsed != "no" )
                                                {
                                                    _$("enablecurrentdisconnectthreshold").value = "yes";
                                                    _$("currentdisconnectthreshold").value = elapsed;
                                                    $("#currentdisconnectthreshold").removeAttr('disabled');
                                                    $("#currentdisconnectthreshold").show();
                                                }
                                                else
                                                {
                                                    _$("enablecurrentdisconnectthreshold").value = "no";
                                                    $("#currentdisconnectthreshold").attr('disabled', true);
                                                    $("#currentdisconnectthreshold").hide();
                                                }
                                            }
                                            
                                            if ( polarity )
                                            {
                                                _$("polarityswitch").value = polarity;
                                            }
                                            
                                            if ( busy_tone )
                                            {
                                                _$("countrytone").value = "custom";
                                                _$("busytone").value = busy_tone;
                                                _$("busydetect").value = "yes";
                                                $('.tone-setting').removeAttr('disabled');
                                            }
                                            top.Custom.init(document);
                                            pstnAfter();
                                        },
                                        cancel: function ()
                                        {
                                            pstnAfter();
                                        }
                                    }
                                });
                            }
                            else
                            {
                                top.dialog.clearDialog(); 
                                top.dialog.dialogMessage({ 
                                    type: 'error', 
                                    content: $P.lang("LANG2339") + (mappingErrCode[err] ? $P.lang(mappingErrCode[err]) : ""), 
                                    callback: pstnAfter});
                            }
                        }
                        else if (  _$("detect_model").value == "1" )
                        {
                            if ( status == 4 && status_type  == 1)
                            {
                                flag = 0;
                                top.dialog.dialogConfirm({
                                    confirmStr: $P.lang("LANG2412"),
                                    buttons: {
                                        ok: function ()
                                        {
                                            $.ajax({ type: "post", url: "/webcgi", data: "action=startPSTNDetecting&pstn-pickup=1", success: function (){
                                                top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG2337")});
                                                status_type = 2;
                                                getResponce(1);
                                            }
                                            });
                                        },
                                        cancel: function ()
                                        {
                                            pstnStop();
                                        }
                                    }
                                });
                            }
                            else if ( status == 5 && status_type == 2 )
                            {
                                flag = 0;
                                top.dialog.dialogConfirm({
                                    confirmStr: $P.lang("LANG2413"),
                                    buttons: {
                                        ok: function ()
                                        {
                                            $.ajax({ type: "post", url: "/webcgi", data: "action=startPSTNDetecting&pstn-hangup=1", success: function (){
                                                top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG2337")});
                                                status_type = 3;
                                                getResponce(1);
                                            }
                                            });
                                        },
                                        cancel: function ()
                                        {
                                            pstnStop();
                                        }
                                    }
                                });
                            }
                            
                        }
                    }
                    
                    i--;
                
                    if (flag && i > 0 )
                    {
                        getResponce(1);
                    }
                    else if ( i <= 0 )
                    {
                        /* time out */
                        top.dialog.clearDialog(); 
                        top.dialog.dialogMessage({ type: 'warning', content: $P.lang("LANG2339"), callback: pstnAfter});
                    }
                    
                } });

                            
            }, 1000);
        };
        
        getResponce(1);
    }
     
    if ( _$("detect_model").value == "1" )
    {
        var des_channel = "-1";
    }
    else
    {
        var des_channel = _$("des_channels").value;
    }
        
    var buf = "action=startPSTNDetecting&pstn-cpt=(" + _$("src_channels").value + "," + _$("src_number").value
        + ")(" + des_channel + "," + _$("des_number").value + ")";
    $.ajax({ type: "post", url: "/webcgi", data: buf, success: pstnOver });
    //pstnCancel();
    //SHOW_FLAG = true;
    top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG2337")});
                
    /*     
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG2336"),
        buttons: {
            ok: function ()
            {
                var buf = "action=startPSTNDetecting&pstn-cpt=(" + _$("src_channels").value + "," + _$("src_number").value
                    + ")(" + _$("des_channels").value + "," + _$("des_number").value + ")";
                $.ajax({ type: "post", url: "/webcgi", data: buf, success: pstnOver });
                //pstnCancel();
                //SHOW_FLAG = true;
                top.dialog.dialogMessage({ type: 'loading', content: $P.lang("LANG2337")});
            },
            cancel: function ()
            {
                pstnAfter();
            }
        }
    });
    */
};

function btn_disable()
{   
    selectChannel = getSelectedChannels();
    if (selectChannel && selectChannel.length > 0 )
    {
        $("#detect").removeClass("disabled").removeAttr("disabled");
    }
    else
    {
        $("#detect").addClass("disabled").attr("disabled", "disabled");
    }
};

var trunkAnalogBindToggleBuilder = function (bindTarget, actionTarget, delay) {
    return (function (ev) {
        if (this.value === 'yes')
        {
            $(actionTarget).show(delay);
        }
        else
        {
            $(actionTarget).hide(delay);
        }
        if (window.frameElement)
        {
            $(window.frameElement).css("height", "0px");
            $(window.frameElement).css("width", "0px");
        }
        top.dialog.repositionDialog();
    });

}

function change_detect_model()
{
    if ( _$("detect_model").value == "1")
    {
        $("#dev_deschannel").hide();
    }
    else
    {
        $("#dev_deschannel").show();
    }

    if (window.frameElement)
    {
        $(window.frameElement).css("height", "0px");
        $(window.frameElement).css("width", "0px");
    }
    top.dialog.currentDialogType = "iframe";
    top.dialog.repositionDialog();
}

var trunkAnalogBindToggle = function (bindTarget, actionTarget, delay) {
    $(bindTarget).change( trunkAnalogBindToggleBuilder(bindTarget, actionTarget, delay) );
};
