/*
 * Asterisk-GUI  - an Asterisk configuration interface
 *
 * trunks_voip.html functions
 *
 * Copyright (C) 2006-2008, Digium, Inc.
 *
 * Pari Nannapaneni <pari@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc.
 * Please see http://www.asterisk.org for more information about the Asterisk
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Support multiple languages
 * Modified the register trunk callerid and removed the mwi code
 *
 */
$P = top.$;
var mWindow = top.window.frames["mainscreen"]; // this will be the main window 

var isNewTrunk;
var EDIT_TRUNK;
var EDIT_DID = '';
var EDIT_DID_rules = [];
var NEED_EDIT_SIP_CONF = 0; // added by dcwei 2012-10-15
var OLD_SIP_REGISTER_INFO = ''; // added by dcwei 2012-10-16
var OLD_SIP_REGISTER_INFO_OTHER = '';
var OLD_IAX_REGISTER_INFO = ''; // added by dcwei 2012-10-16
var OLD_TRUNK_NAME = ''; // added by dcwei 2012-10-18

function arrIndexOf(arr, val) {
    if ($.isArray(arr)) {
        var i = arr.length;
        while (i--) {
            if (arr[i] === val) {
                return i;
            }
        }
        return -1;
    } else {
        return arr.indexOf(val);
    }
}

var ldap_append_outrt_option = function(idx) {
    /*  list the relevant outbound-route rules  */
    var outrt_rules = parent.sessionData.pbxinfo.callingRules;
    var outrt_rule_names = outrt_rules.getOwnProperties();
    var primary_pattern = RegExp('Macro\\\(dialout-trunk,');
    var trunk_pattern = RegExp('Macro\\\(dialout-trunk,' + idx);
    var outrt_list = [];
    for (var i = 0; i < outrt_rule_names.length; i++) {
        var index = outrt_rule_names[i];
        var outrt_item = outrt_rules[index];
        for (var j = 0; j < outrt_item.length; j++) {
            if (primary_pattern.test(outrt_item[j]) && trunk_pattern.test(outrt_item[j])) {
                outrt_list.push(index);
                break;
            } else if (primary_pattern.test(outrt_item[j]) && !trunk_pattern.test(outrt_item[j])) {
                break;
            }
        }
    }

    var default_outrt = $('#edit_VOIPTrunk_ldap_default_outrt');
    default_outrt.append($('<option>').html($P.lang('LANG133')).val('none'));
    for (var i = 0; i < outrt_list.length; i++) {
        var rule = outrt_list[i];
        var rule_name = rule.match(/outrt_(.+)/)[1];
        default_outrt.append($('<option>').html(rule_name).val(rule));
    }
    default_outrt.append($('<option>').html($P.lang('LANG2526')).val('custom'));

    // defualt value is whitespace
    _$('edit_VOIPTrunk_ldap_default_outrt').value = "";
}
// for peer trunk ldap configuration
var ldap_sync_init = function() {
    var enable = $('#edit_VOIPTrunk_ldap_sync_enable');
    if (enable.length > 0) {
        var passwd = $('#TR_ldap_sync_passwd');
        var port = $('#TR_ldap_sync_port');
        var outrt = $('#TR_ldap_default_outrt');
        var prefix = $('#TR_ldap_default_outrt_prefix');
        var date = $('#TR_ldap_last_sync_date');
        var manual = $('#ldap_manual_sync');
        enable.change(function() {
            if (_$('edit_VOIPTrunk_ldap_sync_enable').checked) {
                passwd.show();
                port.show();
                outrt.show();
                prefix.show();
                date.show();
            } else {
                passwd.hide();
                port.hide();
                outrt.hide();
                prefix.hide();
                date.hide();
            }
        });
    }

    /*
    var outrt_prefix = _$('edit_VOIPTrunk_ldap_default_outrt_prefix');
    outrt_prefix.onchange = function()
    {
        var tag = document.getElementById("edit_VOIPTrunk_ldap_default_outrt");
        tag.selectedIndex = -1;
        tag.previousSibling.innerHTML = "";
    };
    */

}

var initForm = function() {
    var type = $('#edit_VOIPTrunk_Type');
    type.change(function() {
        if ($(this).val() === 'sip2') {
            $('#outboundproxy_field').show();
            $('#authid_field').show();
        } else {
            $('#outboundproxy_field').hide();
            $('#authid_field').hide();
        }
        if ($(this).val() === 'sip1' || $(this).val() === 'sip2') {
            $("#auto_record_field").show();
        } else {
            $("#auto_record_field").hide();
        }
        if ($(this).val() === 'sip1' || $(this).val() === 'iax1') {
            $('#TR_username').hide();
            $('#TR_password').hide();
        } else {
            $('#TR_username').show();
            $('#TR_password').show();
        }
        if (window.frameElement)
            $(window.frameElement).css("height", "0px");
        top.dialog.repositionDialog("none");
    });

}

var initOutboundProxy = function() {
    var proxy = $('#trunk_is_outboundproxy');
    proxy.change(function() {
        if (_$('trunk_is_outboundproxy').checked) {
            $('#outboundproxy_field').show();
        } else {
            $('#outboundproxy_field').hide();
        }
    });
}
var isSelfIp = function(ele) {
    var selfIp = window.location.hostname;
    var inputIp = $(ele).val();
    if (inputIp.contains(selfIp)) {
        return false;
    } else {
        return true;
    }
}

window.onload = function() {
    $P.lang(document, true);
    loadDOMelements();
    var mode = gup("mode");
    var item = gup("item");
    initOutboundProxy();
    if (mode) {
        if (mode == "add") {
            new_trunk_form();
        } else if (mode == "edit") {
            edit_trunk_form(item);
        }
        (function() {
            var tmp = [];
            var destination = function() {
                this.optionText = '';
                this.optionValue = '';
            };

            var y = top.pbx.users.list();
            y.each(function(user) {
                var f = new destination;
                f.optionText = f.optionValue = user;
                tmp.push(f);
            });
        })();
        init_validator();
        isEnableWeakPw();
        var add_validatorToType = function() {
            var edit_VOIPTrunk_Type = $("#edit_VOIPTrunk_Type").val();
            if (edit_VOIPTrunk_Type == "sip2" || edit_VOIPTrunk_Type == "iax2") {
                $P("#edit_VOIPTrunk_Username", document).rules("add", {
                    required: true,
                    authid: true
                });
            } else {
                $P("#edit_VOIPTrunk_Username", document).rules("remove", "required alphanumeric");
                $P("#form", document).validate().element($("#edit_VOIPTrunk_Username"));
            }
        }
        add_validatorToType();
        $("#edit_VOIPTrunk_Type").bind("change", function(ev) {
            add_validatorToType();
            ev.stopPropagation();
        });
        var add_validatorToCallerid = function() {
            if ($P("#form", document).validate().element($("#edit_VOIPTrunk_Callername")) && $("#edit_VOIPTrunk_Callername").val()) {
                $P("#edit_VOIPTrunk_Callerid", document).rules("add", {
                    required: true,
                    minlength: 2,
                    alphanumeric: true
                });
                $P("#form", document).validate().element($("#edit_VOIPTrunk_Callerid"));
            } else {
                $P("#edit_VOIPTrunk_Callerid", document).rules("remove", "required minlength alphanumeric");
                $P("#form", document).validate().element($("#edit_VOIPTrunk_Callerid"));
            }
        }
        add_validatorToCallerid();
        $("#edit_VOIPTrunk_Callername").bind("change", function(ev) {
            add_validatorToCallerid();
            ev.stopPropagation();
        });
        top.Custom.init(document);
        return false; // bypass reload dialog
    } else {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG641"));
        $("title").attr("locale", "LANG641");
        load_VOIPTrunksTable();
        sortTable({
            "tableId": 'table_VOIPTrunks_list',
            sortFuncs: {
                "sort_0": ASTGUI.extenStyleCmpFunc
            }
        }).sortTableInitial(0).bindTitles({
            idx: 0,
            isDis: true
        }, {
            idx: 1
        }, {
            idx: 2
        });
    }
};
var isEnableWeakPw = function() {
    var GUI_PREFERENCES = top.sessionData.GUI_PREFERENCES;
    if (!GUI_PREFERENCES.getProperty('weak_password')) {
        GUI_PREFERENCES = context2json({
            filename: 'guipreferences.conf',
            context: 'general',
            usf: 1
        });
    }
    if (GUI_PREFERENCES.weak_password == "yes") {
        var obj = {
            pwsId: "#edit_VOIPTrunk_ldap_sync_passwd"
        }
        $P("#edit_VOIPTrunk_ldap_sync_passwd", document).rules("add", {
            minlength: 4,
            checkAlphanumericPw: [showCheckPassword, obj]
        });
    }
}
var check_video_codec = function() {
    //var list = ["ulaw", "alaw", "gsm", "g726", "ilbc", "g722", "adpcm", "lpc10", "g729", "g723"];
    var list = ["ulaw", "alaw", "gsm", "g726", "g726aal2", "ilbc", "g722", "adpcm", "g729", "g723"];
    var res = false;
    var valuelist = ASTGUI.selectbox.readAllValues('select_allow_codecs');

    valuelist.each(function(value) {
        if (res == true) {
            return res;
        }
        list.each(function(codec) {
            if (value == codec)
                res = true;
            return res;
        });
    });

    return res;
}

var init_validator = function() {
    //init validator
    $P = top.$;
    if ($("#form").tooltip) {
        $("#form").tooltip();
    }
    $P("#form", document).validate({
        rules: {
            "edit_VOIPTrunk_Type": {
                required: true
            },
            "edit_VOIPTrunk_Providername": {
                required: true,
                letterDigitUndHyphen: true,
                customCallback: [$P.lang('LANG2137'), checkNameExists]
            },
            "edit_VOIPTrunk_Hostname": {
                required: true,
                host: [$P.lang('LANG1373').toLowerCase()],
                isExist: [$P.lang("LANG2542").format($P.lang("LANG1373")), isSelfIp]
            },
            "edit_VOIPTrunk_Callername": {
                minlength: 2,
                //required: true,
                alphanumeric: true //Unknown validation
            },
            "trunk_fromuser": {
                //alphanumeric: true     //Unknown validation
                authid: true
            },
            "select_allow_codecs": {
                selectItemMin: 1,
                customCallback: [$P.lang('LANG2138'), check_video_codec]
            },
            "qualify_timeout": {
                required: true,
                digits: true,
                min: 1000
            },
            "qualify_freq": {
                required: true,
                digits: true,
                min: 60
            },
            "trunk_fromdomain": {
                host: [$P.lang('LANG1369')]
            },
            "trunk_outboundproxy": {
                host: [$P.lang('LANG1378')],
                customCallback: ["outbound proxy is required.", check_outboundproxy],
                isExist: [$P.lang("LANG2542").format($P.lang("LANG1378")), isSelfIp]
            },
            "trunk_authid": {
                authid: true
            },
            "edit_VOIPTrunk_ldap_sync_passwd": {
                required: true,
                alphanumeric: true
            },
            "edit_VOIPTrunk_ldap_default_outrt_prefix": {
                //digits: true,
                alphanumeric: true,
                customCallback: ['prefix is required.', check_ldap_prefix]
            },
            "edit_VOIPTrunk_ldap_sync_port": {
                required: true,
                digits: true,
                range: [1, 65534]
            }
        },
        submitHandler: function() {
            edit_VOIPTrunk_save_go();
        }
    });

};

var loadDOMelements = function() {
    DOM_table_VOIPTrunks_list = _$('table_VOIPTrunks_list');
    DOM_edit_VOIPTrunk_DIV = _$('editForm');
    //DOM_edit_VOIPTrunk_DIV_Title = _$('edit_VOIPTrunk_DIV_Title');
    DOM_edit_VOIPTrunk_Type = _$('edit_VOIPTrunk_Type');
    /*  modified by dcwei 2012-09-12    used default namespace  */
    //  DOM_edit_VOIPTrunk_Context_Basis = _$('edit_VOIPTrunk_Context_Basis');
    DOM_edit_VOIPTrunk_Hostname = _$('edit_VOIPTrunk_Hostname');
    DOM_edit_VOIPTrunk_Callerid = _$('edit_VOIPTrunk_Callerid');
    DOM_edit_VOIPTrunk_Callername = _$('edit_VOIPTrunk_Callername');
    DOM_edit_VOIPTrunk_Username = _$('edit_VOIPTrunk_Username');
    DOM_edit_VOIPTrunk_Password = _$('edit_VOIPTrunk_Password');
    DOM_edit_VOIPTrunk_transport = _$('edit_VOIPTrunk_transport');
    DOM_edit_VOIPTrunk_srtp = _$('edit_VOIPTrunk_srtp');
    DOM_trunk_outboundproxy = _$('trunk_outboundproxy');

    DOM_allow = _$('select_allow_codecs');
    DOM_available = _$('select_available_codecs');

};

var check_ldap_sync_progress = function(ldap_ou, count) {
    if (count <= 0) {
        top.dialog.dialogMessage({
            type: 'warning',
            content: errArray[1]
        });
        return;
    }
    count--;
    $.ajax({
        type: "GET",
        url: "/webcgi",
        async: false,
        data: "action=getparam&Pldap_sync_progress_" + ldap_ou,
        success: function(result) {
            var verdata = decodeURIComponent(result).split("\r\n");
            var data = verdata[0].split("=");

            /*The progress*/
            var index = parseInt(data[1]);
            if (isNaN(index)) {
                setTimeout(function() {
                    check_ldap_sync_progress(ldap_ou, count)
                }, 1000);
                return;
            }

            if (index >= 0) {
                if (index == 7) {
                    top.dialog.clearDialog();
                    /*successful done*/
                    top.dialog.dialogMessage({
                        type: 'success',
                        content: msgArray[index]
                    });
                } else {
                    count = 90;
                    top.dialog.dialogMessage({
                        type: 'loading',
                        content: msgArray[index]
                    });
                    setTimeout(function() {
                        check_ldap_sync_progress(ldap_ou, count)
                    }, 1000);
                }
            } else {
                top.dialog.clearDialog();
                var number = Math.abs(index);
                top.dialog.dialogMessage({
                    type: 'warning',
                    content: errArray[number]
                });
            }

        }

    });
}

var msgArray = new Array($P.lang("LANG2656"), $P.lang("LANG2657"), $P.lang("LANG2658"), $P.lang("LANG2659"),
    $P.lang("LANG2660"), $P.lang("LANG2661"), $P.lang("LANG2662"), $P.lang("LANG2663"));
var errArray = new Array("", $P.lang("LANG2664"), $P.lang("LANG2665"), $P.lang("LANG2666"));

var manual_ldap_sync_progress = function(EDIT_TRUNK) {
    var tinfo = parent.sessionData.pbxinfo.trunks[parent.pbx.trunks.getType(EDIT_TRUNK)][EDIT_TRUNK];
    var ldap_ou = tinfo['trunkname'];

    /*Get the LDAP SYNC date and progress*/
    var count = 120;
    check_ldap_sync_progress(ldap_ou, count);

    if (count <= 0) {
        top.dialog.dialogMessage({
            type: 'warning',
            content: errArray[1]
        });
    }

};

var manual_ldap_sync = function(EDIT_TRUNK) {
    var ttype = parent.pbx.trunks.getType(EDIT_TRUNK);
    var tinfo = parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK];
    if (!EDIT_TRUNK) {
        top.dialog.clearDialog();
        top.dialog.dialogMessage({
            type: 'warning',
            content: "ERROR ITEM!!!!"
        });
        return;
    }

    var a = (tinfo["ldap_sync_enable"] == "yes") ? 1 : 0;
    if (a != 1) {
        // top.dialog.clearDialog();
        // top.dialog.dialogMessage({ type: 'warning', content: $P.lang("LANG2674") });
        return;
    }


    //if ( $P("#form", document).valid() ) {
    top.dialog.dialogMessage({
        type: 'loading',
        content: $P.lang("LANG2655")
    });

    /*Start to sync*/
    $.ajax({
        type: "Post",
        url: "/webcgi?",
        data: "action=startLDAPSync&ldap-sync=" + EDIT_TRUNK,
        dataType: "json",
        error: function() {
            top.dialog.clearDialog();
            top.dialog.dialogMessage({
                type: 'error',
                content: $P.lang("LANG909")
            });
        },
        success: function(data) {
            var data = eval(data);

            if (data.response == "success") {
                manual_ldap_sync_progress(EDIT_TRUNK);
            } else {
                top.dialog.dialogMessage({
                    type: 'error',
                    content: data.body
                });
            }
        }
    });
    //}
};

var new_trunk_form = function() {
    $('.editTrunk_Field').hide();
    $('#TR_username').hide();
    $('#TR_password').hide();
    $('#TR_transport').hide();
    $('#TR_srtp').hide();
    $('#ldap_manual_sync').hide();

    initForm();
    isNewTrunk = true;
    EDIT_TRUNK = '';
    show_Edit_Trunk();
};

var edit_trunk_form = function(ET) {
    $('.editTrunk_Field').show();
    //$('.editTrunk_Field_ermwi').show();
    EDIT_TRUNK = ET;
    isNewTrunk = false;
    /* trunk ldap */
    ldap_sync_init();
    var idx = ET.match(/trunk_([0-9]+)/)[1];
    ldap_append_outrt_option(idx);
    show_Edit_Trunk(ET);
};



var show_Edit_Trunk = function(trunkIndex) {
    //bind codecs select widget
    bind_codecs_action();

    $('#trunk_qualify').change(function() {
        if ($(this).attr('checked')) {
            $('#div_qualify').show();
            ASTGUI.updateFieldToValue('qualify_timeout', qualify);
            ASTGUI.updateFieldToValue('qualify_freq', getProperty(tinfo, 'qualifyfreq'));
        } else {
            $('#div_qualify').hide();
        }
        top.dialog.repositionDialog("none");
    });


    $('#outboundproxy_field').hide();
    $('#authid_field').hide();
    $('#is_outboundproxy_field').hide();
    //$('#TR_override_cid').hide();//Modified by rzhang for adding DOD feature for SIP register trunk.
    /*  hide the trunk ldap detail */
    $('#TR_ldap_sync_enable').hide();
    $('#TR_ldap_sync_passwd').hide();
    $('#TR_ldap_sync_port').hide();
    $('#TR_ldap_default_outrt').hide();
    $('#TR_ldap_default_outrt_prefix').hide();
    $('#TR_ldap_last_sync_date').hide();


    if (isNewTrunk == true) {
        ASTGUI.resetTheseFields([DOM_edit_VOIPTrunk_Type, DOM_edit_VOIPTrunk_Hostname, DOM_edit_VOIPTrunk_transport, DOM_edit_VOIPTrunk_srtp, DOM_edit_VOIPTrunk_Callerid, DOM_edit_VOIPTrunk_Callername, DOM_edit_VOIPTrunk_Username, DOM_edit_VOIPTrunk_Password /*, 'trunk_obcid'*/ , 'edit_VOIPTrunk_Providername', 'trunk_fromdomain', 'trunk_fromuser', /*'trunk_authuser', */ 'trunk_insecure', 'trunk_qualify', 'trunk_faxdetect', 'edit_VOIPTrunk_authid', 'edit_VOIPTrunk_ldap_sync_enable', 'edit_VOIPTrunk_ldap_sync_passwd', 'edit_VOIPTrunk_ldap_sync_port', 'edit_VOIPTrunk_ldap_default_outrt', 'edit_VOIPTrunk_ldap_default_outrt_prefix']);
        _$('trunk_is_outboundproxy').checked = false;
        _$('edit_override_cid').checked = false;
        DOM_edit_VOIPTrunk_Type.disabled = false;
        DOM_edit_VOIPTrunk_Password.value = '';
        NEED_EDIT_SIP_CONF = 0;
        OLD_SIP_REGISTER_INFO = '';
        OLD_IAX_REGISTER_INFO = '';
        OLD_TRUNK_NAME = '';
        $('#more_details').show()
        $('#TR_trunktype').show();
        $('#TR_callerid').hide();
        $('#TR_callername').hide();

        reset_form_codecs('default', top.sessionData.listOfCodecs);
    } else {
        /** disable the provider name **/
        _$('edit_VOIPTrunk_Providername').disabled = true;
        $('#more_details').hide()
        $('#TR_trunktype').hide();
        var ttype = parent.pbx.trunks.getType(EDIT_TRUNK);
        var tinfo = parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK];
        if (ttype != "analog")
            $('#TR_override_cid').show();

        var flags = parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['trunkflags'];

        if (ttype == 'iax') {
            $('#TR_transport').hide();
            $('#TR_srtp').hide();
        } else {
            $('#TR_transport').show();
            $('#TR_srtp').show();
        }

        /*  ldap only for peer trunk */
        if (flags == 'sip1') {
            $('#TR_ldap_sync_enable').show();
            var ldap_sync_enable = tinfo['ldap_sync_enable'];
            if (ldap_sync_enable == 'yes') {
                _$('edit_VOIPTrunk_ldap_sync_enable').checked = true;

                var ldap_ou = tinfo['trunkname'];

                /*Get the LDAP SYNC date and progress*/
                $.ajax({
                    type: "GET",
                    url: "/webcgi",
                    data: "action=getparam&Pldap_sync_date_" + ldap_ou + "&Pldap_sync_progress_" + ldap_ou,
                    success: function(result) {

                        var verdata = decodeURIComponent(result).split("\r\n");
                        var i;
                        for (i = 0; i < verdata.length; ++i) {
                            var data = verdata[i].split("=");
                            if (data != undefined) {
                                if (i == 0) {
                                    /*last successful date*/
                                    _$('label_ldap_last_sync_date').innerHTML = data[1] ? data[1] : $P.lang("LANG2403");
                                } else if (i == 1) {
                                    /*The progress*/
                                    ;
                                }
                            }
                        }
                    }
                });
            } else
                _$('edit_VOIPTrunk_ldap_sync_enable').checked = false;

            // add onchange for ldap outbound route
            var tag_outrt = _$('edit_VOIPTrunk_ldap_default_outrt');
            tag_outrt.onchange = function() {
                var outrt_rules = parent.sessionData.pbxinfo.callingRules;
                var idx = EDIT_TRUNK.match(/trunk_([0-9]+)/)[1];
                var reg_pat = RegExp('Macro\\\(dialout-trunk,' + idx + ',\\\${EXTEN:([0-9]+)},\\\)');
                var res = [];
                var ldap_default_outrt = _$('edit_VOIPTrunk_ldap_default_outrt').value;
                if (outrt_rules[ldap_default_outrt]) {
                    if ((res = outrt_rules[ldap_default_outrt].toString().match(reg_pat))) {
                        var pattern = outrt_rules[ldap_default_outrt][0].match(/exten=_(.+?),/)[1];
                        var prefix = generate_prefix_with_pattern_and_strip(pattern, res[1]);
                        /*
                    x.new_action('update', EDIT_TRUNK, 'ldap_default_outrt_prefix', prefix);
                    */
                        _$('edit_VOIPTrunk_ldap_default_outrt_prefix').value = prefix;
                        _$('edit_VOIPTrunk_ldap_default_outrt_prefix').disabled = true;
                    }
                } else {
                    if (ldap_default_outrt == 'custom') {
                        _$('edit_VOIPTrunk_ldap_default_outrt_prefix').value = "";
                        _$('edit_VOIPTrunk_ldap_default_outrt_prefix').disabled = false;
                    } else {
                        _$('edit_VOIPTrunk_ldap_default_outrt_prefix').value = "";
                        _$('edit_VOIPTrunk_ldap_default_outrt_prefix').disabled = true;
                    }
                }
            }

            ASTGUI.updateFieldToValue('edit_VOIPTrunk_ldap_sync_passwd', tinfo['ldap_sync_passwd']);
            ASTGUI.updateFieldToValue('edit_VOIPTrunk_ldap_sync_port', tinfo['ldap_sync_port']);
            if (tinfo['ldap_default_outrt'] == '' || tinfo['ldap_default_outrt'] == 'none') {
                ASTGUI.updateFieldToValue('edit_VOIPTrunk_ldap_default_outrt', 'none');
                _$('edit_VOIPTrunk_ldap_default_outrt_prefix').disabled = true;
                ASTGUI.updateFieldToValue('edit_VOIPTrunk_ldap_default_outrt_prefix', tinfo['ldap_default_outrt_prefix']);
            } else if (tinfo['ldap_default_outrt'] == 'custom') {
                ASTGUI.updateFieldToValue('edit_VOIPTrunk_ldap_default_outrt', 'custom');
                _$('edit_VOIPTrunk_ldap_default_outrt_prefix').disabled = false;
                ASTGUI.updateFieldToValue('edit_VOIPTrunk_ldap_default_outrt_prefix', tinfo['ldap_default_outrt_prefix']);
            } else {
                ASTGUI.updateFieldToValue('edit_VOIPTrunk_ldap_default_outrt', tinfo['ldap_default_outrt']);
                $('#edit_VOIPTrunk_ldap_default_outrt').trigger('change');
            }
            $('#edit_VOIPTrunk_ldap_sync_enable').trigger('change');
        }

        if (flags == 'iax1' || flags == 'sip1') {
            $('#TR_username').hide();
            $('#TR_password').hide();
            $('#TR_fromdomain').hide();
            $('#TR_fromuser').hide();
            $('#TR_insecure').hide();
        }

        if (flags == 'iax2') {
            $('#TR_fromdomain').hide();
            $('#TR_fromuser').hide();
        }

        if (flags == 'sip2') {
            $('#ldap_manual_sync').hide();
            _$('trunk_is_outboundproxy').checked = false;
            $('#is_outboundproxy_field').show();
            if (tinfo.hasOwnProperty('outboundproxy') && tinfo['outboundproxy'] != "") {
                ASTGUI.updateFieldToValue(DOM_trunk_outboundproxy, tinfo['outboundproxy']);
                _$('trunk_is_outboundproxy').checked = true;
                $('#outboundproxy_field').show();
            }

            $('#authid_field').show();
            /*
        if ( tinfo.hasOwnProperty('register') && (reg = tinfo['register']) != "" ) {
            tmp = reg.split("@")[0];
            if ( tmp )
            {
                authid = tmp.split(":")[2];
            }
            if ( authid )
            {
                ASTGUI.updateFieldToValue('edit_VOIPTrunk_authid', authid);
            }
        }*/
            if (tinfo.hasOwnProperty('authid') && tinfo['authid'] != "") {
                ASTGUI.updateFieldToValue('edit_VOIPTrunk_authid', tinfo['authid']);
            }

        }

        DOM_edit_VOIPTrunk_Type.disabled = true;
        if (trunkIndex && ttype == "sip") {
            $("#auto_record_field, #didmode_field").show();
            var globals = globals = context2json({
                filename: 'extensions.conf',
                context: 'globals',
                usf: 1
            });
            var new_crl_pri_record_value = globals['OUT_AUTORECORD_' + trunkIndex.split("_")[1]];
            if (new_crl_pri_record_value) {
                ASTGUI.updateFieldToValue(_$('editTrunk_auto_record'), new_crl_pri_record_value);
            }
        }
        if (flags != undefined) {
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_Type, flags);
        };
        if (getProperty(tinfo, 'port'))
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_Hostname, getProperty(tinfo, 'host') + ":" + getProperty(tinfo, 'port'));
        else
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_Hostname, getProperty(tinfo, 'host'));
        if (flags == 'sip1' || flags == 'sip2') {
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_transport, getProperty(tinfo, 'transport'));
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_srtp, getProperty(tinfo, 'encryption'));
        }

        /*  config override cide  */
        var idx = EDIT_TRUNK.split('_')[1];
        var outkeepcid = parent.sessionData.pbxinfo['GLOBALS']['OUTKEEPCID_' + idx];
        if (outkeepcid && outkeepcid == 'on') {
            _$('edit_override_cid').checked = true;
        } else {
            _$('edit_override_cid').checked = false;
        }
        if (flags == 'sip2') {
            /* Modified by rzhang for adding DOD feature for SIP register trunk.
            _$('edit_override_cid').checked = true;
            _$('edit_override_cid').disabled = true;
            */
            _$('edit_VOIPTrunk_Callerid').disabled = true;
            _$('edit_VOIPTrunk_Callername').disabled = true;
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_Callerid, getProperty(tinfo, 'username'));
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_Callername, getProperty(tinfo, 'username'));

            // hid the callerid number and name by dcwei for bug 13144
            //$('#TR_override_cid').hide(); //Modified by rzhang for adding dod feature for SIP register trunk, this option is removed on bug 15011.
            $('#TR_callerid').hide();
            $('#TR_callername').hide();
        } else {
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_Callerid, getProperty(tinfo, 'callerid'));
            ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_Callername, getProperty(tinfo, 'callername'));
        }
        // ended by dcwei

        ASTGUI.updateFieldToValue(DOM_edit_VOIPTrunk_Username, getProperty(tinfo, 'username'));

        if (getProperty(tinfo, 'username') === '') {
            NEED_EDIT_SIP_CONF = 0;
        } else {
            NEED_EDIT_SIP_CONF = 1;
        }
        OLD_SIP_REGISTER_INFO = '';
        OLD_IAX_REGISTER_INFO = '';
        if (NEED_EDIT_SIP_CONF) {
            var tmp_transport = getProperty(tinfo, 'transport');
            var tmp_username = getProperty(tinfo, 'username');
            var tmp_fromusr = getProperty(tinfo, 'fromuser');
            var tmp_passwd = getProperty(tinfo, 'secret');
            var tmp_hostname = getProperty(tinfo, 'host');
            var tmp_fromdomain = getProperty(tinfo, 'fromdomain');
            var tmp_authuser = '';
            var tmp_authid = getProperty(tinfo, 'authid');
            var tmp_proxy = getProperty(tinfo, 'outboundproxy');
            var peername = EDIT_TRUNK;
            var new_sip_register = peername + '?';
            var tmp_port = getProperty(tinfo, 'port');
            var tmp_hostname_other = "";
            var new_sip_register_other = "";
            if (tmp_port && tmp_port != "") {
                if (tmp_proxy != "") {
                    tmp_hostname_other = tmp_hostname + ':' + tmp_port;
                    tmp_hostname = tmp_hostname + '#' + tmp_port;
                } else
                    tmp_hostname = tmp_hostname + ':' + tmp_port;
            }
            if (tmp_transport && /^tls/.test(tmp_transport)) {
                new_sip_register += 'tls://';
            } else if (tmp_transport && /^tcp/.test(tmp_transport)) {
                new_sip_register += 'tcp://';
            }

            if (tmp_fromdomain != undefined && tmp_fromdomain != '') {
                if (/:/.test(tmp_fromdomain))
                    tmp_fromdomain = tmp_fromdomain.replace(':', '#');
                new_sip_register = new_sip_register + tmp_username + '@' + tmp_fromdomain;
            } else {
                new_sip_register = new_sip_register + tmp_username;
            }
            if (tmp_username) {
                tmp_username = tmp_username.replace(/\[/g, '\\\[');
                tmp_username = tmp_username.replace(/\]/g, '\\\]');
                tmp_username = tmp_username.replace(/\!/g, '\\\!');
                tmp_username = tmp_username.replace(/\*/g, '\\\*');
                tmp_username = tmp_username.replace(/\./g, '\\\.');
                tmp_username = tmp_username.replace(/\+/g, '\\\+');
                tmp_username = tmp_username.replace(/\(/g, '\\\(');
                tmp_username = tmp_username.replace(/\)/g, '\\\)');
                tmp_username = tmp_username.replace(/\</g, '\\\<');
                tmp_username = tmp_username.replace(/\>/g, '\\\>');
            }
            var pat = RegExp('\\\?' + tmp_username + '$');
            if (tmp_proxy && tmp_proxy != '' && pat.test(new_sip_register)) {
                var tmp_host = '';
                if (tmp_hostname_other) {
                    tmp_hostname_other = (tmp_fromdomain != undefined && tmp_fromdomain != '') ? tmp_fromdomain : tmp_hostname_other;
                    new_sip_register_other = new_sip_register + '@' + tmp_hostname_other
                }
                tmp_host = (tmp_fromdomain != undefined && tmp_fromdomain != '') ? tmp_fromdomain : tmp_hostname;
                new_sip_register = new_sip_register + '@' + tmp_host;
            }

            if (tmp_passwd && tmp_passwd != '') {
                if (tmp_hostname_other) {
                    new_sip_register_other = new_sip_register_other + ':' + tmp_passwd;
                }
                new_sip_register = new_sip_register + ':' + tmp_passwd;
            }

            if (tmp_authuser && tmp_authuser != '') {
                if (tmp_hostname_other) {
                    new_sip_register_other = new_sip_register_other + ':' + tmp_authuser;
                }
                new_sip_register = new_sip_register + ':' + tmp_authuser;
            }

            if (flags == 'sip2' && tmp_authid && tmp_authid != '') {
                if (tmp_hostname_other) {
                    new_sip_register_other = new_sip_register_other + ':' + tmp_authid;
                }
                new_sip_register = new_sip_register + ':' + tmp_authid;
            }

            if (tmp_proxy && tmp_proxy != '') {
                if (tmp_hostname_other) {
                    new_sip_register_other = new_sip_register_other + '@' + tmp_proxy;
                }
                new_sip_register = new_sip_register + '@' + tmp_proxy;
            } else {
                if (tmp_hostname_other) {
                    new_sip_register_other = new_sip_register_other + '@' + tmp_hostname_other;
                }
                new_sip_register = new_sip_register + '@' + tmp_hostname;
            }

            if (flags == 'sip2' && tmp_fromusr && tmp_fromusr != '') {
                if (tmp_hostname_other) {
                    new_sip_register_other = new_sip_register_other + '/' + tmp_fromusr;
                }
                new_sip_register = new_sip_register + '/' + tmp_fromusr;
            } else if (flags == 'sip2') {
                if (tmp_username) {
                    tmp_username = tmp_username.replace(/\\\[/g, '[');
                    tmp_username = tmp_username.replace(/\\\]/g, ']');
                    tmp_username = tmp_username.replace(/\\\!/g, '!');
                    tmp_username = tmp_username.replace(/\\\*/g, '*');
                    tmp_username = tmp_username.replace(/\\\./g, '.');
                    tmp_username = tmp_username.replace(/\\\+/g, '+');
                    tmp_username = tmp_username.replace(/\\\(/g, '(');
                    tmp_username = tmp_username.replace(/\\\)/g, ')');
                    tmp_username = tmp_username.replace(/\\\</g, '<');
                    tmp_username = tmp_username.replace(/\\\>/g, '>');
                }
                if (tmp_hostname_other) {
                    new_sip_register_other = new_sip_register_other + '/' + tmp_username;
                }
                new_sip_register = new_sip_register + '/' + tmp_username;
            }

            if (flags == 'sip2') {
                if (tmp_hostname_other) {
                    OLD_SIP_REGISTER_INFO_OTHER = new_sip_register_other;
                }
                OLD_SIP_REGISTER_INFO = new_sip_register;
            } else if (flags == 'iax2')
                OLD_IAX_REGISTER_INFO = new_sip_register;
        }

        ASTGUI.updateFieldToValue('edit_VOIPTrunk_Password', getProperty(tinfo, 'secret'));
        DOM_edit_VOIPTrunk_Password.value = tinfo.secret;
        ASTGUI.updateFieldToValue('edit_VOIPTrunk_Providername', getProperty(tinfo, 'trunkname'));
        OLD_TRUNK_NAME = getProperty(tinfo, 'trunkname');
        ASTGUI.updateFieldToValue('trunk_fromdomain', getProperty(tinfo, 'fromdomain'));
        ASTGUI.updateFieldToValue('trunk_fromuser', getProperty(tinfo, 'fromuser'));
        ASTGUI.updateFieldToValue('trunk_insecure', getProperty(tinfo, 'insecure'));
        var didmodeVal = getProperty(tinfo, 'didmode')
        if (didmodeVal) {
            ASTGUI.updateFieldToValue('editTrunk_didmode', didmodeVal);
        } else {
            ASTGUI.updateFieldToValue('editTrunk_didmode', 'request-line');
        }
        var qualify = getProperty(tinfo, 'qualify');
        if (qualify && qualify !== 'no') {
            $('#trunk_qualify').attr('checked', 'checked');
        } else {
            $('#trunk_qualify').removeAttr('checked');
        }

        $('#trunk_qualify').trigger('change');


        ASTGUI.updateFieldToValue('trunk_faxdetect', getProperty(tinfo, 'faxdetect'));

        var allow_codecs = getProperty(tinfo, 'allow') ? getProperty(tinfo, 'allow').split(',') : [];
        reset_form_codecs(allow_codecs, top.sessionData.listOfCodecs);

        /* Don't allow editing field if the asterisk [context] is based on it. */
        DOM_edit_VOIPTrunk_Username.disabled = false;
        _$('edit_VOIPTrunk_Providername').disabled = false;
        if (EDIT_TRUNK == getProperty(tinfo, 'username')) DOM_edit_VOIPTrunk_Username.disabled = true;
    }
};
var checkNameExists = function(value, element) {
    var field = 'trunkname';
    if (OLD_TRUNK_NAME === value) {
        return true;
    }

    type = DOM_edit_VOIPTrunk_Type.value;
    if (!type) {
        //type check required will take it
        return true;
    }

    type = type.toLowerCase();
    if (type === 'sip1' || type === 'sip2') {
        type = 'sip';
    }

    if (type === 'iax1' || type === 'iax2') {
        type = 'iax';
    }

    var tmp_data = parent.sessionData.pbxinfo.trunks[type];
    if (!tmp_data) {
        return true;
    }
    var ttype = tmp_data.getOwnProperties();
    var i;
    for (i = 0; ttype[i] != undefined; i++) {
        if (tmp_data[ttype[i]][field] == value) {
            return false;
        }
    }

    return true;
}

var edit_VOIPTrunk_save_go = function() {

    var topreload = false;
    var tmp_username = ASTGUI.getFieldValue('edit_VOIPTrunk_Username');
    var tmp_trunkname = ASTGUI.getFieldValue('edit_VOIPTrunk_Providername');
    var len = tmp_trunkname.length;
    var ttype = DOM_edit_VOIPTrunk_Type.value.toLowerCase();
    var tmp_flags = ttype;
    var autoRecord = _$('editTrunk_auto_record').value;
    /*  config override cid  */
    var outkeepcid = 'off';
    if (_$('edit_override_cid').checked) {
        outkeepcid = 'on';
    }
    if (isNewTrunk == true) {

        var ttv = DOM_edit_VOIPTrunk_Type.value;
        if (ttype == 'sip1') {
            var trp = {
                host: DOM_edit_VOIPTrunk_Hostname.value,
                callerid: DOM_edit_VOIPTrunk_Callerid.value,
                callername: DOM_edit_VOIPTrunk_Callername.value,
                username: DOM_edit_VOIPTrunk_Username.value,
                secret: DOM_edit_VOIPTrunk_Password.value,
                trunkname: ASTGUI.getFieldValue('edit_VOIPTrunk_Providername'),
                autoRecord: ASTGUI.getFieldValue('editTrunk_auto_record'),
                didmode: ASTGUI.getFieldValue('editTrunk_didmode'),
                outkeepcid: outkeepcid,
                trunkflags: "sip1"
            };
        } else if (ttype == 'sip2') {
            var trp = {
                host: DOM_edit_VOIPTrunk_Hostname.value,
                callerid: DOM_edit_VOIPTrunk_Callerid.value,
                callername: DOM_edit_VOIPTrunk_Callername.value,
                username: DOM_edit_VOIPTrunk_Username.value,
                secret: DOM_edit_VOIPTrunk_Password.value,
                trunkname: ASTGUI.getFieldValue('edit_VOIPTrunk_Providername'),
                outboundproxy: DOM_trunk_outboundproxy.value,
                authid: ASTGUI.getFieldValue('edit_VOIPTrunk_authid'),
                autoRecord: ASTGUI.getFieldValue('editTrunk_auto_record'),
                didmode: ASTGUI.getFieldValue('editTrunk_didmode'),
                outkeepcid: outkeepcid,
                trunkflags: "sip2"
            };
        } else if (ttype == 'iax1') {
            var trp = {
                host: DOM_edit_VOIPTrunk_Hostname.value,
                callerid: DOM_edit_VOIPTrunk_Callerid.value,
                callername: DOM_edit_VOIPTrunk_Callername.value,
                username: DOM_edit_VOIPTrunk_Username.value,
                secret: DOM_edit_VOIPTrunk_Password.value,
                trunkname: ASTGUI.getFieldValue('edit_VOIPTrunk_Providername'),
                outkeepcid: outkeepcid,
                trunkflags: "iax1"
            };
        } else {
            var trp = {
                host: DOM_edit_VOIPTrunk_Hostname.value,
                callerid: DOM_edit_VOIPTrunk_Callerid.value,
                callername: DOM_edit_VOIPTrunk_Callername.value,
                username: DOM_edit_VOIPTrunk_Username.value,
                secret: DOM_edit_VOIPTrunk_Password.value,
                trunkname: ASTGUI.getFieldValue('edit_VOIPTrunk_Providername'),
                outkeepcid: outkeepcid,
                trunkflags: "iax2"
            };
        }
        var cbf = function() {
            top.dialog.dialogMessage({
                type: 'success',
                content: $P.lang("LANG943"),
                callback: function() {
                    mWindow.location.reload();
                }
            });
        };

        if (ttv == 'sip1' || ttv == 'sip2') {
            var retval = parent.pbx.trunks.add1('sip', trp, cbf, 'GUIAssigned');
        } else if (ttv == 'iax1' || ttv == 'iax2') {
            var retval = parent.pbx.trunks.add1('iax', trp, cbf, 'GUIAssigned');
        }
    } else if (isNewTrunk == false) {
        // Edit Existing Trunk
        var codecs = ASTGUI.selectbox.readAllValues('select_allow_codecs').join(',') || '';
        top.dialog.dialogMessage({
            type: 'loading',
            content: $P.lang("LANG826")
        });
        var ttype = parent.pbx.trunks.getType(EDIT_TRUNK);
        var x = new listOfActions('users.conf');
        var z = new listOfSynActions('sip.conf');
        var old_trunkUsername = parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['username'];

        if (_$('trunk_is_outboundproxy').checked == true) {
            if (!ASTGUI.validateFields(['trunk_outboundproxy'])) return;
            x.new_action('update', EDIT_TRUNK, 'outboundproxy', $('#trunk_outboundproxy').val());
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['outboundproxy'] = $('#trunk_outboundproxy').val();
        } else {
            x.new_action('update', EDIT_TRUNK, 'outboundproxy', '');
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['outboundproxy'] = '';
        }

        var hostname = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Hostname);
        var pos = arrIndexOf(hostname, ':');
        var parse_hostname = hostname;
        var parse_port = "";
        if (pos > -1) {
            parse_hostname = hostname.substring(0, pos);
            parse_port = hostname.substring(pos + 1);
            x.new_action('update', EDIT_TRUNK, 'host', parse_hostname);
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['host'] = parse_hostname;
            if (hostname.length > pos) {
                x.new_action('update', EDIT_TRUNK, 'port', parse_port);
                parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['port'] = parse_port;
            } else {
                if (getProperty(parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK], 'port')) {
                    delete parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['port'];
                    x.new_action('delete', EDIT_TRUNK, 'port', '');
                }
            }
        } else {
            x.new_action('update', EDIT_TRUNK, 'host', hostname);
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['host'] = hostname;
            if (getProperty(parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK], 'port')) {
                delete parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['port'];
                x.new_action('delete', EDIT_TRUNK, 'port', '');
            }
        }

        if (tmp_flags == "sip1") {
            var ldap_sync_enable = _$('edit_VOIPTrunk_ldap_sync_enable').checked ? 'yes' : 'no';
            var ldap_sync_passwd = "";
            var ldap_sync_port = "";
            var ldap_default_outrt = "";
            var ldap_default_outrt_prefix = "";
            if (ldap_sync_enable == "yes") {
                ldap_sync_passwd = $('#edit_VOIPTrunk_ldap_sync_passwd').val() || "";
                ldap_sync_port = $('#edit_VOIPTrunk_ldap_sync_port').val() || "";
                ldap_default_outrt = $('#edit_VOIPTrunk_ldap_default_outrt').val();
                ldap_default_outrt_prefix = $('#edit_VOIPTrunk_ldap_default_outrt_prefix').val();
            }

            x.new_action('update', EDIT_TRUNK, 'ldap_sync_enable', ldap_sync_enable);
            x.new_action('update', EDIT_TRUNK, 'ldap_sync_passwd', ldap_sync_passwd);
            x.new_action('update', EDIT_TRUNK, 'ldap_sync_port', ldap_sync_port);
            x.new_action('update', EDIT_TRUNK, 'ldap_default_outrt', ldap_default_outrt);
            x.new_action('update', EDIT_TRUNK, 'ldap_default_outrt_prefix', ldap_default_outrt_prefix);
            /*  update the global array sessionData  */
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['ldap_sync_enable'] = ldap_sync_enable;
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['ldap_sync_passwd'] = ldap_sync_passwd;
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['ldap_sync_port'] = ldap_sync_port;
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['ldap_default_outrt'] = ldap_default_outrt;
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['ldap_default_outrt_prefix'] = ldap_default_outrt_prefix;

            /*  find the strip number
            var outrt_rules = parent.sessionData.pbxinfo.callingRules;
            var idx = EDIT_TRUNK.match(/trunk_([0-9]+)/)[1];
            var reg_pat = RegExp('Macro\\\(dialout-trunk,'+idx+',\\\${EXTEN:([0-9]+)},\\\)');
            var res = [];
            if ( outrt_rules[ldap_default_outrt] )
                if ( (res = outrt_rules[ldap_default_outrt].toString().match(reg_pat)) ) {
                    var pattern = outrt_rules[ldap_default_outrt][0].match(/exten=_(.+?),/)[1];
                    var prefix = generate_prefix_with_pattern_and_strip(pattern, res[1]);
                    x.new_action('update', EDIT_TRUNK, 'ldap_default_outrt_prefix', prefix);
                }
            */
        }

        var tmp_callerid_name;
        tmp_callerid_name = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Callername) + '<' + ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Callerid) + '>';
        //x.new_action('update', EDIT_TRUNK, 'callerid', ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Callerid));
        //x.new_action('update', EDIT_TRUNK, 'callername', ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Callername));

        if (ttype == 'sip') {
            var index = EDIT_TRUNK.split("_")[1];
            ASTGUI.ASTDB.updateKey({
                dbname: 'TRUNK/SIP_' + index,
                key: 'AUTORECORD',
                keyvalue: autoRecord == "ENABLE" ? "YES" : "NO"
            });
            ASTGUI.updateaValue({
                file: 'extensions.conf',
                context: 'globals',
                variable: 'OUT_AUTORECORD_' + index,
                value: autoRecord
            });
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['transport'] = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_transport);
            x.new_action('update', EDIT_TRUNK, 'transport', ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_transport));
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['encryption'] = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_srtp);
            x.new_action('update', EDIT_TRUNK, 'encryption', ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_srtp));
        }
        parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['callerid'] = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Callerid);
        parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['callername'] = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Callername);

        var v = new listOfActions('extensions.conf');
        if (DOM_edit_VOIPTrunk_Callerid.value) {
            /// for BUG 14789
            if (tmp_flags == 'sip2') {
                var name = DOM_edit_VOIPTrunk_Username.value;
                v.new_action('update', 'globals', ASTGUI.globals.obcidTrunkPrefix + (EDIT_TRUNK.split('_'))[1], name + '<' + name + '>');
            } else {
                v.new_action('update', 'globals', ASTGUI.globals.obcidTrunkPrefix + (EDIT_TRUNK.split('_'))[1], tmp_callerid_name);
            }
        } else {
            v.new_action('delete', 'globals', ASTGUI.globals.obcidTrunkPrefix + (EDIT_TRUNK.split('_'))[1], '', '');
        }

        var tmp_idx = EDIT_TRUNK.split('_')[1];
        v.new_action('update', 'globals', 'OUTKEEPCID_' + tmp_idx, outkeepcid);
        parent.sessionData.pbxinfo['GLOBALS']['OUTKEEPCID_' + tmp_idx] = outkeepcid;

        (function() {
            var tmp_fromdomain = ASTGUI.getFieldValue('trunk_fromdomain');
            var tmp_fromuser = ASTGUI.getFieldValue('trunk_fromuser');
            if (tmp_fromdomain) {
                if (!ASTGUI.validateFields(['trunk_fromdomain'])) return;
                x.new_action('update', EDIT_TRUNK, 'fromdomain', tmp_fromdomain);
                parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['fromdomain'] = tmp_fromdomain;
            } else {
                if (parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK].hasOwnProperty('fromdomain')) {
                    x.new_action('delete', EDIT_TRUNK, 'fromdomain', parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['fromdomain']);
                    delete parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['fromdomain'];
                }
            }

            if (tmp_fromuser) {
                if (!ASTGUI.validateFields(['trunk_fromuser'])) return;
                x.new_action('update', EDIT_TRUNK, 'fromuser', tmp_fromuser);
                parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['fromuser'] = tmp_fromuser;
            } else {
                if (parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK].hasOwnProperty('fromuser')) {
                    x.new_action('delete', EDIT_TRUNK, 'fromuser', parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['fromuser']);
                    delete parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['fromuser'];
                }
            }
            var tmp_insecure = ASTGUI.getFieldValue('trunk_insecure');
            x.new_action('update', EDIT_TRUNK, 'insecure', tmp_insecure);
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['insecure'] = tmp_insecure;
            var tmp_didmode = ASTGUI.getFieldValue('editTrunk_didmode');
            x.new_action('update', EDIT_TRUNK, 'didmode', tmp_didmode);
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['didmode'] = tmp_didmode;
            if (_$('trunk_qualify').checked) {
                var timeout = $('#qualify_timeout').val();
                var freq = $('#qualify_freq').val();

                x.new_action('update', EDIT_TRUNK, 'qualify', timeout);
                x.new_action('update', EDIT_TRUNK, 'qualifyfreq', freq);
                parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['qualify'] = timeout;
                parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['qualifyfreq'] = freq;
            } else {
                var timeout = parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK].qualify;
                var freq = parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK].qualifyfreq;
                if (timeout) {
                    x.new_action('delete', EDIT_TRUNK, 'qualify', timeout.toString());
                    delete parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK].qualify;
                }
                if (freq) {
                    x.new_action('delete', EDIT_TRUNK, 'qualifyfreq', freq.toString());
                    delete parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK].qualifyfreq;
                }

            }
            if (_$('trunk_faxdetect').checked) {
                x.new_action('update', EDIT_TRUNK, 'faxdetect', 'yes');
                parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['faxdetect'] = 'yes';
            } else {
                if (parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK].hasOwnProperty('faxdetect')) {
                    x.new_action('delete', EDIT_TRUNK, 'faxdetect', 'yes');
                    delete parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['faxdetect'];
                }
            }
        })();
        x.new_action('update', EDIT_TRUNK, 'secret', ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Password));
        parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['secret'] = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Password);
        x.new_action('update', EDIT_TRUNK, 'username', ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Username));
        parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['username'] = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Username);
        x.new_action('update', EDIT_TRUNK, 'trunkname', ASTGUI.getFieldValue('edit_VOIPTrunk_Providername').guiMetaData());
        parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['trunkname'] = ASTGUI.getFieldValue('edit_VOIPTrunk_Providername');
        x.new_action('delete', EDIT_TRUNK, 'disallow', '');
        x.new_action('delete', EDIT_TRUNK, 'allow', '');
        x.new_action('append', EDIT_TRUNK, 'disallow', 'all');
        parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['disallow'] = 'all';
        x.new_action('append', EDIT_TRUNK, 'allow', codecs);
        parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['allow'] = codecs;

        if (tmp_flags == "sip2") {
            x.new_action('update', EDIT_TRUNK, 'authid', ASTGUI.getFieldValue("edit_VOIPTrunk_authid"));
            parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['authid'] = ASTGUI.getFieldValue('edit_VOIPTrunk_authid');
        }
        // remote MWI - only for SIP trunk
        if (tmp_flags == 'sip2' || tmp_flags == 'iax2') {
            var tmp_username = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Username);
            if (tmp_username != '') {
                NEED_EDIT_SIP_CONF = 1;
            }

            var tmp_transport = ASTGUI.getFieldValue('edit_VOIPTrunk_transport');
            var tmp_fromusr = ASTGUI.getFieldValue('trunk_fromuser');
            var tmp_passwd = ASTGUI.getFieldValue(DOM_edit_VOIPTrunk_Password);
            var tmp_hostname = parse_hostname;
            var tmp_port = parse_port;
            var tmp_fromdomain = ASTGUI.getFieldValue('trunk_fromdomain');
            var tmp_authuser = '';
            var tmp_proxy = '';
            var tmp_authid = _$("edit_VOIPTrunk_authid").value;
            if (_$('trunk_is_outboundproxy').checked)
                tmp_proxy = $('#trunk_outboundproxy').val();
            var peername = EDIT_TRUNK;
            var new_sip_register = peername + '?';
            var new_sip_register_other = "";
            var iax_conf = new listOfSynActions('iax.conf');

            if (NEED_EDIT_SIP_CONF && tmp_username != '') {
                if (tmp_port && tmp_port != "") {
                    if (tmp_proxy != "")
                        tmp_hostname = tmp_hostname + '#' + tmp_port;
                    else
                        tmp_hostname = tmp_hostname + ':' + tmp_port;
                }
                /*  update information into sip.conf  */
                if (tmp_flags == 'sip2' && OLD_SIP_REGISTER_INFO && OLD_SIP_REGISTER_INFO != '') {
                    if (OLD_SIP_REGISTER_INFO_OTHER) {
                        z.new_action('delete', 'general', 'register', OLD_SIP_REGISTER_INFO_OTHER, '');
                        z.callActions();
                        z.clearActions();
                    }
                    z.new_action('delete', 'general', 'register', OLD_SIP_REGISTER_INFO, '');
                    z.callActions();
                    z.clearActions();
                }

                if (tmp_flags == 'iax2' && OLD_IAX_REGISTER_INFO && OLD_IAX_REGISTER_INFO != '') {
                    iax_conf.new_action('delete', 'general', 'register', OLD_IAX_REGISTER_INFO, '');
                }

                if (tmp_transport && /^tls/.test(tmp_transport)) {
                    new_sip_register += 'tls://';
                } else if (tmp_transport && /^tcp/.test(tmp_transport)) {
                    new_sip_register += 'tcp://';
                }

                if (tmp_fromdomain != undefined && tmp_fromdomain != '') {
                    if (/:/.test(tmp_fromdomain))
                        tmp_fromdomain = tmp_fromdomain.replace(':', '#');
                    new_sip_register = new_sip_register + tmp_username + '@' + tmp_fromdomain;
                } else {
                    new_sip_register = new_sip_register + tmp_username;
                }
                if (tmp_username) {
                    tmp_username = tmp_username.replace(/\[/g, '\\\[');
                    tmp_username = tmp_username.replace(/\]/g, '\\\]');
                    tmp_username = tmp_username.replace(/\!/g, '\\\!');
                    tmp_username = tmp_username.replace(/\*/g, '\\\*');
                    tmp_username = tmp_username.replace(/\./g, '\\\.');
                    tmp_username = tmp_username.replace(/\+/g, '\\\+');
                    tmp_username = tmp_username.replace(/\(/g, '\\\(');
                    tmp_username = tmp_username.replace(/\)/g, '\\\)');
                    tmp_username = tmp_username.replace(/\</g, '\\\<');
                    tmp_username = tmp_username.replace(/\>/g, '\\\>');
                }
                var pat = RegExp('\\\?' + tmp_username + '$');
                if (tmp_proxy && tmp_proxy != '' && pat.test(new_sip_register)) {
                    var tmp_host = '';
                    tmp_host = (tmp_fromdomain != undefined && tmp_fromdomain != '') ? tmp_fromdomain : tmp_hostname;
                    new_sip_register = new_sip_register + '@' + tmp_host;
                }

                if (tmp_passwd != '') {
                    new_sip_register = new_sip_register + ':' + tmp_passwd;
                }

                if (tmp_authuser != '') {
                    new_sip_register = new_sip_register + ':' + tmp_authuser;
                }

                if (tmp_flags == 'sip2' && tmp_authid != '') {
                    new_sip_register = new_sip_register + ':' + tmp_authid;
                }

                if (tmp_proxy != '') {
                    new_sip_register = new_sip_register + '@' + tmp_proxy;
                } else {
                    new_sip_register = new_sip_register + '@' + tmp_hostname;
                }

                if (tmp_flags == 'sip2' && tmp_fromusr && tmp_fromusr != '') {
                    new_sip_register = new_sip_register + '/' + tmp_fromusr;
                } else if (tmp_flags == 'sip2') {
                    if (tmp_username) {
                        tmp_username = tmp_username.replace(/\\\[/g, '[');
                        tmp_username = tmp_username.replace(/\\\]/g, ']');
                        tmp_username = tmp_username.replace(/\\\!/g, '!');
                        tmp_username = tmp_username.replace(/\\\*/g, '*');
                        tmp_username = tmp_username.replace(/\\\./g, '.');
                        tmp_username = tmp_username.replace(/\\\+/g, '+');
                        tmp_username = tmp_username.replace(/\\\(/g, '(');
                        tmp_username = tmp_username.replace(/\\\)/g, ')');
                        tmp_username = tmp_username.replace(/\\\</g, '<');
                        tmp_username = tmp_username.replace(/\\\>/g, '>');
                    }
                    new_sip_register = new_sip_register + '/' + tmp_username;
                }
                if (tmp_flags == 'sip2') {
                    var sipConf = config2json({
                        filename: 'sip.conf',
                        usf: 0
                    });
                    var sipGeneral = sipConf["general"];
                    for (var i = 0; i < sipGeneral.length; i++) {
                        if (sipGeneral[i].contains("register=" + EDIT_TRUNK)) {
                            z.new_action('delete', 'general', 'register', sipGeneral[i].split("register=")[1], '');
                            z.callActions();
                            z.clearActions();
                        }
                    };
                    z.new_action('append', 'general', 'register', new_sip_register);
                } else if (tmp_flags == 'iax2') {
                    var iaxConf = config2json({
                        filename: 'iax.conf',
                        usf: 0
                    });
                    var iaxGeneral = iaxConf["general"];
                    for (var i = 0; i < iaxGeneral.length; i++) {
                        if (iaxGeneral[i].contains("register=" + EDIT_TRUNK)) {
                            iax_conf.new_action('delete', 'general', 'register', iaxGeneral[i].split("register=")[1], '');
                            iax_conf.callActions();
                            iax_conf.clearActions();
                        }
                    };
                    iax_conf.new_action('append', 'general', 'register', new_sip_register);
                }

            } else if (NEED_EDIT_SIP_CONF) {
                /*  delete the information in the sip.conf  */
                if (tmp_flags == 'sip2') {
                    z.new_action('delete', 'general', 'register', OLD_SIP_REGISTER_INFO_OTHER, '');
                    z.callActions();
                    z.clearActions();
                    z.new_action('delete', 'general', 'register', OLD_SIP_REGISTER_INFO, '');
                } else if (tmp_flags == 'iax2')
                    iax_conf.new_action('delete', 'general', 'register', OLD_IAX_REGISTER_INFO, '');
            }

            if (tmp_flags == 'sip2')
                z.callActions();
            else(tmp_flags == 'iax2')
            iax_conf.callActions();
        }
        // End of remote MWI

        var after = function() {
            v.callActions(function() {
                top.dialog.clearDialog();
                top.dialog.dialogMessage({
                    type: 'success',
                    content: $P.lang("LANG844"),
                    callback: function() {
                        if (topreload) {
                            top.window.location.reload();
                        } else {
                            mWindow.location.reload();
                        }
                    }
                });


            });
        };

        x.callActions(after);

    }
};



var delete_trunk_confirm = function(ET) {
    EDIT_TRUNK = ET;
    if (!EDIT_TRUNK) {
        return;
    }
    var ttype = parent.pbx.trunks.getType(EDIT_TRUNK);
    var tmp_name = parent.sessionData.pbxinfo.trunks[ttype][EDIT_TRUNK]['trunkname'] + ' -- ' + EDIT_TRUNK;
    /*
     *  added by dcwei 2012-11-09
     *  Delete the relevant outgoing rules firstly when removed the trunk.
     */
    var tmp_rules = parent.sessionData.pbxinfo.callingRules.getOwnProperties();
    var rule = '';
    for (var i = 0; tmp_rules[i]; i++) {
        rule = parent.sessionData.pbxinfo.callingRules[tmp_rules[i]];
        if (rule) {
            var item = '';
            for (var j = 0; j < rule.length; j++) {
                item = rule[j];
                if (/Macro\(dialout-trunk,([1-9][0-9]*),/.test(item)) {
                    var idx = item.match(/Macro\(dialout-trunk,([1-9][0-9]*),/)[1];
                    trunk = "trunk_" + idx;
                    if (trunk == EDIT_TRUNK) {
                        top.dialog.dialogMessage({
                            type: 'warning',
                            content: $P.lang("LANG944").format(tmp_rules[i])
                        });
                        return;
                    }
                }
            }
        }
    }

    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG818").format(tmp_name),
        buttons: {
            ok: function() {
                var ttype = parent.pbx.trunks.getType(EDIT_TRUNK);
                if (ttype == "sip") {
                    var index = EDIT_TRUNK.split("_")[1];
                    ASTGUI.ASTDB.deleteKey({
                        dbname: 'TRUNK/SIP_' + index,
                        key: 'AUTORECORD'
                    });
                    var u = new listOfSynActions('extensions.conf');
                    u.new_action('delete', 'globals', 'OUT_AUTORECORD_' + index, '', '');
                    u.callActions();
                }
                parent.pbx.trunks.remove1(EDIT_TRUNK);
                ASTGUI.cliCommand('database deltree ' + EDIT_TRUNK.toUpperCase());
                top.dialog.dialogMessage({
                    type: 'success',
                    content: $P.lang("LANG821").format(tmp_name, $P.lang("LANG2471")),
                    callback: function() {
                        mWindow.location.reload();
                    }
                });
            }
        }
    });
};


var load_VOIPTrunksTable = function() {
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function
    (function() { // add first row
        if (DOM_table_VOIPTrunks_list) {
            var newRow = DOM_table_VOIPTrunks_list.insertRow(-1);
            newRow.className = "frow";
            addCell(newRow, {
                html: "<span locale='LANG1382'>" + $P.lang('LANG1382') + "</span>"
            });
            addCell(newRow, {
                html: "<span locale='LANG84'>" + $P.lang('LANG84') + "</span>"
            }); // sip/iax
            addCell(newRow, {
                html: "<span locale='LANG1395'>" + $P.lang('LANG1395') + "</span>"
            }); // Hostname
            addCell(newRow, {
                html: "<span locale='LANG72'>" + $P.lang('LANG72') + "</span>"
            }); // Hostname
            addCell(newRow, {
                html: "<span locale='LANG74'>" + $P.lang('LANG74') + "</span>"
            });
        }
    })();
    (function() {
        var c = [];
        var d = parent.pbx.trunks.list({
            sip: true,
            iax: true
        });
        d.each(function(item) {
            var ttype = parent.pbx.trunks.getType(item);
            var tinfos = parent.sessionData.pbxinfo.trunks[ttype][item];
            var flags = tinfos['trunkflags'];
            if (DOM_table_VOIPTrunks_list) {
                var newRow = DOM_table_VOIPTrunks_list.insertRow(-1);
                newRow.className = ((DOM_table_VOIPTrunks_list.rows.length) % 2 == 1) ? 'odd' : 'even';

                addCell(newRow, {
                    html: tinfos['trunkname']
                });
                if (flags == 'sip1') {
                    addCell(newRow, {
                        html: $P.lang('LANG233'),
                        locale: 'LANG233'
                    });
                } else if (flags == 'sip2') {
                    addCell(newRow, {
                        html: $P.lang('LANG234'),
                        locale: 'LANG234'
                    });
                } else if (flags == 'iax1') {
                    addCell(newRow, {
                        html: $P.lang('LANG235'),
                        locale: 'LANG235'
                    });
                } else {
                    addCell(newRow, {
                        html: $P.lang('LANG236'),
                        locale: 'LANG236'
                    });
                }
                if (getProperty(tinfos, 'port')) {
                    addCell(newRow, {
                        html: tinfos['host'] + ":" + tinfos['port']
                    });
                } else {
                    addCell(newRow, {
                        html: tinfos['host']
                    });
                }
                addCell(newRow, {
                    html: tinfos['username'] || ''
                });

                var tmp_editString = "<button class='options edit' onclick=\"showEditForm('" + item + "')\" localeTitle='LANG738' title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                    "<button class='options dod' onclick=\"edit_dod('" + item + "')\" localeTitle='LANG2677' title=" + $P.lang("LANG2677") + "></button>&nbsp;" +
                    "<button class='options sync' onclick=\"manual_ldap_sync('" + item + "')\" localeTitle='LANG2654' title='" + $P.lang("LANG2654") + "'></button>&nbsp;" +
                    "<button class='options del' onclick=\"delete_trunk_confirm('" + item + "')\"localeTitle='LANG739' title=" + $P.lang("LANG739") + "></button>";
                addCell(newRow, {
                    html: tmp_editString
                });
                if (tinfos['ldap_sync_enable'] == "no" || !tinfos['ldap_sync_enable']) {
                    $(newRow).children().last().children().eq(2).addClass("disabled");
                }
            }
        });
    })();
    if (DOM_table_VOIPTrunks_list) {
        if (DOM_table_VOIPTrunks_list.rows.length == 1) {
            ASTGUI.domActions.clear_table(DOM_table_VOIPTrunks_list);
            var newRow = DOM_table_VOIPTrunks_list.insertRow(-1);
            addCell(newRow, {
                html: $P.lang('LANG129').format($P.lang('LANG641')),
                locale: 'LANG129 LANG641'
            });
        }
    }
};

var bind_codecs_action = function() {
    ASTGUI.events.add('button_add_toringlist', 'click', function() {
        if (!DOM_available.value) {
            return;
        }
        $(':selected', DOM_available).each(function(idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_allow, text, val);
        });
        $P("#select_allow_codecs", document).valid();
    });

    ASTGUI.events.add('button_remove_fromringlist', 'click', function() {
        if (!DOM_allow.value) {
            return;
        }

        $(':selected', DOM_allow).each(function(idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_available, text, val);
        });
        $P("#select_allow_codecs", document).valid();
    });

    ASTGUI.events.add('button_removeall_fromringlist', 'click', function() {
        ASTGUI.selectbox.clear(DOM_allow);
        ASTGUI.selectbox.clear(DOM_available);
        var listOfCodecs = top.sessionData.listOfCodecs;

        for (var item in listOfCodecs) {
            if (!listOfCodecs.hasOwnProperty(item)) {
                continue;
            }
            ASTGUI.selectbox.append(DOM_available, listOfCodecs[item], item);
        }
        $P("#select_allow_codecs", document).valid();
    });

    ASTGUI.events.add('button_addall_toringlist', 'click', function() {
        // modified to do not override former order in selected box, just append it
        $(DOM_available).children().each(function(idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_allow, text, val);
        });
        $P("#select_allow_codecs", document).valid();
    });

    //bind up/down/top/bottom button
    $('#button_up').click(function() {
        ASTGUI.domActions.selectScrollUp('select_allow_codecs');
    });

    $('#button_down').click(function() {
        ASTGUI.domActions.selectScrollDown('select_allow_codecs');
    });

    $('#button_top').click(function() {
        ASTGUI.domActions.selectScrollTop('select_allow_codecs');
    });

    $('#button_bottom').click(function() {
        ASTGUI.domActions.selectScrollBottom('select_allow_codecs');
    });

};

var reset_form_codecs = function(allow_codecs, listOfCodecs) {

    if (typeof allow_codecs == 'string' && allow_codecs === 'default') {
        allow_codecs = ['ulaw', 'alaw', 'gsm', 'g726', 'g729'];
    }


    allow_codecs.each(function(item) {
        if (listOfCodecs.hasOwnProperty(item)) {
            ASTGUI.selectbox.append('select_allow_codecs', listOfCodecs[item], item);
        }
    });

    for (var item in listOfCodecs) {
        if (!listOfCodecs.hasOwnProperty(item)) {
            continue;
        }

        if (allow_codecs.contains(item)) {
            continue;
        } else {
            ASTGUI.selectbox.append('select_available_codecs', listOfCodecs[item], item);
        }
    }

};

var check_outboundproxy = function(value, element) {
    if (isNewTrunk)
        return true;

    var outboundproxy_chkbox = _$('trunk_is_outboundproxy');
    if (outboundproxy_chkbox && outboundproxy_chkbox.checked && value == "")
        return false;

    return true;
}

var check_ldap_prefix = function(value, element) {
    var default_ob = _$('edit_VOIPTrunk_ldap_default_outrt').value;
    if (default_ob && default_ob == 'custom' && value == "")
        return false;

    return true;
}

var edit_dod = function(EDIT_TRUNK) {
    top.sessionData.userPage = 1;
    top.dialog.dialogInnerhtml({
        dialogTitle: $P.lang('LANG2675'),
        frameSrc: "trunk_dod_modal.html?trunk=" + EDIT_TRUNK
    });
}