/*
* Asterisk-GUI  - an Asterisk configuration interface
*
* vmgroups.html functions
*
* Copyright (C) 2006-2008, Digium, Inc.
*
* Pari Nannapaneni <pari@digium.com>
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/
/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 *  change amount limit statement display style
 *  add amount limit to voicemail group umber and member
 *  Modify the javascript error of the ie10
 *
 */
$P = top.$;
var mWindow = window;

var isNEW_VMG;
var EDIT_VMG;
var vmg_chkbxClass = "VMG_Chkboxes";
var listOfUserExtens = [];
function arrIndexOf(arr, val) {
    if ($.isArray(arr)) {
        var i = arr.length;
        while (i--)
        {
            if (arr[i] === val)
            {
                return i;
            }
        }
        return -1;
    }
    else{
        return arr.indexOf(val);
    }
}
function contains(arr, a) // firefox don't support contains function
{
    if (arrIndexOf(arr, a) > -1 )
        return true;
    else
        return false;
};
window.onload = function ()
{
    $P.lang(document, true);
    loadDOMelements();
    var mode = gup("mode");
    var item = gup("item");
    if (mode)
    {
        mWindow = top.window.frames["mainscreen"]; // this will be the main window 
        initForm();
        if (mode == "add")
        {
            vmgroups_miscFunctions.show_NewVMG_form();
            $("#editForm").show();
        }
        else if (mode == "edit")
        {
            vmgroups_miscFunctions.edit_VMG_form(item);
            $("#editForm").show();
        }
        $("#edit_vmgroup_exten").focus();
        init_validator();

        return false; // bypass reload dialog
    }
    else
    {
        top.document.title = $P.lang("LANG584").format(top.model_info.model_name, $P.lang("LANG660"));
        $("title").attr("locale", "LANG660");
        updateTable();
        sortTable({
            sortTableToggle: false,
            "tableId": 'table_VMGroups_list',
            sortFuncs: {
                "sort_0": ASTGUI.extenStyleCmpFunc
            }

        }).sortTableInitial(0).bindTitles( 
            {idx: 0, isDis: true}, {idx: 1}
        );
    }
};


var loadDOMelements = function ()
{
    DOM_select_ringthesechannels = _$('select_ringthesechannels');
    DOM_select_fromlistofchannels = _$('select_fromlistofchannels');
    DOM_button_add_toringlist = _$('button_add_toringlist');
    DOM_button_remove_fromringlist = _$('button_remove_fromringlist');
    DOM_button_removeall_fromringlist = _$('button_removeall_fromringlist');
};

var link= function()
{
    top.dialog.dialogConfirm({
        confirmStr: $P.lang("LANG843").format($P.lang("LANG87")),
        buttons: {
            ok: function ()
            {
                top.module.jumpMenu('users.html');
            },
            cancel: function ()
            {
                top.dialog.container.show();
                top.dialog.shadeDiv.show();
            }
        }
    });
};

var initForm = function ()
{
    // load list of all voicemail boxes available on the system as checkboxes in 'edit_includeCheckboxes_div'
    var g = parent.pbx.users.list();
    g.each(function (this_user)
    {
        var hasvoicemail = parent.sessionData.pbxinfo['users'][this_user]['hasvoicemail']
        if (hasvoicemail && hasvoicemail.isAstTrue())
        {
            listOfUserExtens.push(this_user);
        }
    });
    listOfUserExtens.sortExtension();

    if (!listOfUserExtens.length)
    {
        var tmp_span = document.createElement('div');
        tmp_span.innerHTML = $P.lang("LANG1570") + '<br/> <a onclick=link(); class="link" >' 
            + $P.lang("LANG258") + '</a>' + $P.lang("LANG1571");
        /*ASTGUI.events.add(tmp_span, 'click', function ()
        {
            top.dialog.clearDialog();
            parent.miscFunctions.click_panel('users.html');
        });*/

        $('#edit_includeCheckboxes_div').css('border', '1px solid #CDCDCD');
        $('#edit_includeCheckboxes_div').css('text-align', 'center');
        _$('edit_includeCheckboxes_div').appendChild(tmp_span);
    }

    ASTGUI.events.add(DOM_button_add_toringlist, 'click', function ()
    {
        if (!DOM_select_fromlistofchannels.value)
        {
            return;
        }

        $(':selected', DOM_select_fromlistofchannels).each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            ASTGUI.selectbox.append(DOM_select_ringthesechannels, text, val);
        });
        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add(DOM_button_remove_fromringlist, 'click', function ()
    {
        if (!DOM_select_ringthesechannels.value)
        {
            return;
        }

        $(':selected', DOM_select_ringthesechannels).each( function (idx, item) {
            var val = $(item).val();
            var text = $(item).html() || val;
            $(item).remove();
            if (listOfUserExtens.contains(val))
            {
                ASTGUI.selectbox.append(DOM_select_fromlistofchannels, text, val);
            }
        });

        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add(DOM_button_removeall_fromringlist, 'click', function ()
    {
        ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
        ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);

        listOfUserExtens.each(function (user)
        {
            ASTGUI.selectbox.append(DOM_select_fromlistofchannels, getUserDisplay(user), user);
        });
        $P("#select_ringthesechannels", document).valid();
    });

    ASTGUI.events.add('button_addall_toringlist', 'click', function ()
    {
        ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
        ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);

        listOfUserExtens.each(function (user)
        {
            ASTGUI.selectbox.append(DOM_select_ringthesechannels, getUserDisplay(user), user);
        });
        $P("#select_ringthesechannels", document).valid();
    });


}

var init_validator = function () {
    //init validator
    var max_member = (parent.sessionData.GUI_PREFERENCES['vmg_maxmember']
                    && Number(parent.sessionData.GUI_PREFERENCES['vmg_maxmember']));
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules: {
            "edit_vmgroup_exten": {
                required: true,
                digits: true,
                minlength: 2,
                extensionExists: [EDIT_VMG],
                extensionRange: ['vmg_start', 'vmg_end']
            },
            "edit_vmgroup_label": {
                required: true,
                letterDigitUndHyphen: true,
                customCallback: [$P.lang("LANG2143"), vmgroups_miscFunctions.anotherVMgroupsExistbyThisLabel]
            },
            "select_ringthesechannels": {
                selectItemMin: 1,
                selectItemMax: [max_member, 'mailboxes']
            }

        },
        submitHandler: function() {
            $P('#edit_vmgroup_exten', document).rules('remove');
            $P('#edit_vmgroup_label', document).rules('remove');
            vmgroups_miscFunctions.save_VMGroup();
        }
    });
};


var save_entrance = function()
{
    AskextensionRange(_$("edit_vmgroup_exten").value, 'vmg_start', 'vmg_end');
}

var updateTable = function ()
{
    var addCell = ASTGUI.domActions.tr_addCell;

    // title row
    var t = _$('table_VMGroups_list');
    if (t) {
        var newRow = t.insertRow(-1);
        newRow.className = "frow";
        addCell(newRow, { html: "<span locale='LANG1569'>"+$P.lang('LANG1569')+"</span>"});
        addCell(newRow, { html: "<span locale='LANG135'>"+$P.lang('LANG135')+"</span>"});
        addCell(newRow, { html: "<span locale='LANG1572'>"+$P.lang('LANG1572')+"</span>"});
        addCell(newRow, { html: "<span locale='LANG74'>"+$P.lang('LANG74')+"</span>"});
    }
    // process body
    var vmgroups = parent.sessionData.pbxinfo.vmgroups.getOwnProperties();
    var tbl = _$('table_VMGroups_list');
    vmgroups.each(function (this_vmg_exten)
    {
        var tmp = "<button class='options edit' onclick=\"showEditForm('" + this_vmg_exten + "')\" localetitle=\"LANG738\" title=" + $P.lang("LANG738") + "></button>&nbsp;" +
                  "<button class='options del' onclick=\"vmgroups_miscFunctions.delete_VMG('" + this_vmg_exten + "')\" localetitle=\"LANG739\" title=" + $P.lang("LANG739") + "></button>";
        if (tbl) {
            var newRow = tbl.insertRow(-1);
            newRow.className = ((tbl.rows.length) % 2 == 1) ? 'odd' : 'even';
            addCell(newRow, { html: this_vmg_exten });
            addCell(newRow, { html: parent.sessionData.pbxinfo.vmgroups[this_vmg_exten].label });
            addCell(newRow, { html: parent.sessionData.pbxinfo.vmgroups[this_vmg_exten].mailboxes.join(', ') });
            addCell(newRow, { html: tmp });
        }
    });

    if (tbl) {
        if (tbl.rows.length == 1)
        {
            ASTGUI.domActions.clear_table(tbl);
            var newRow = tbl.insertRow(-1);
            newRow.className = 'even';
            addCell(newRow, { html: $P.lang("LANG129").format($P.lang("LANG89")),locale: "LANG129 LANG89"});
            return;
        }
    }
}


var getUserDisplay = function (user)
{
    if (!parent.sessionData.pbxinfo.users[user])
    {
        return '';
    }
    var full_name = parent.sessionData.pbxinfo.users[user]['fullname'];
    if (full_name.length <= 0)
    {
        return user;
    }
    else
    {
        return user + ' \"' + full_name + '\"';
    }

};


var vmgroups_miscFunctions = {

    reset_allFields: function ()
    { // vmgroups_miscFunctions.reset_allFields();
        if (isNEW_VMG)
        {
            var tmp_allextensions = ASTGUI.cloneObject(parent.miscFunctions.getAllExtensions());
            var NEW_EXT = tmp_allextensions.firstAvailable(parent.sessionData.GUI_PREFERENCES['vmg_start']);
            _$('edit_vmgroup_exten').value = NEW_EXT;
            _$('edit_vmgroup_label').value = '';
            _$('edit_vmgroup_exten').disabled = false;
            //ASTGUI.domActions.unCheckAll(vmg_chkbxClass);
            //_$('Edit_dialog_title').innerHTML = ' New Voice Mail Group ';

            ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
            ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);
            listOfUserExtens.each(function (user)
            {
                ASTGUI.selectbox.append(DOM_select_fromlistofchannels, getUserDisplay(user), user);
            });
            return;
        }

        var tmp_this_VMG = ASTGUI.cloneObject(parent.sessionData.pbxinfo.vmgroups[EDIT_VMG]);
        _$('edit_vmgroup_exten').value = EDIT_VMG;
        _$('edit_vmgroup_label').value = tmp_this_VMG.label;
        _$('edit_vmgroup_exten').disabled = true;
        //ASTGUI.domActions.unCheckAll(vmg_chkbxClass);
        //ASTGUI.domActions.checkSelected(vmg_chkbxClass, tmp_this_VMG.mailboxes);

        ASTGUI.selectbox.clear(DOM_select_ringthesechannels);
        var mbrs = tmp_this_VMG.mailboxes;
        for (var i = 0; i < mbrs.length; i++) {
            var user = mbrs[i];
            if (parent.sessionData.pbxinfo.users[user])
            {
                ASTGUI.selectbox.append(DOM_select_ringthesechannels, getUserDisplay(user), user);
            }
        };
        ASTGUI.selectbox.clear(DOM_select_fromlistofchannels);
        listOfUserExtens.each(function (user)
        {
            if (contains(mbrs, user)) { return; }
            ASTGUI.selectbox.append(DOM_select_fromlistofchannels, getUserDisplay(user), user);
        });




        //_$('Edit_dialog_title').innerHTML = ' Edit Voice Mail Group - ' + EDIT_VMG;
    },

    show_NewVMG_form: function ()
    { // vmgroups_miscFunctions.show_NewVMG_form()
        if (parent.sessionData.GUI_PREFERENCES['vmg_maxnumber']
            && parent.sessionData.pbxinfo.vmgroups.getOwnProperties().length >= Number(parent.sessionData.GUI_PREFERENCES['vmg_maxnumber']))
        {
            top.dialog.dialogMessage({
                type: 'warning',
                content: $P.lang("LANG863").format(parent.sessionData.GUI_PREFERENCES['vmg_maxnumber'])
            });
            //var msg = "The max number of voicemail groups is " + parent.sessionData.GUI_PREFERENCES['vmg_maxnumber'];
            //ASTGUI.feedback({ msg: msg, showfor: 3 });
            return;
        }
        isNEW_VMG = true;
        EDIT_VMG = '';
        vmgroups_miscFunctions.reset_allFields();
    },

    edit_VMG_form: function (vmgexten)
    { //vmgroups_miscFunctions.edit_VMG_form(exten)
        isNEW_VMG = false;
        EDIT_VMG = vmgexten;
        vmgroups_miscFunctions.reset_allFields();
    },

    anotherVMgroupsExistbyThisLabel: function (value, element)
    {		//	VoiceMenus_miscFunctions.anotherMenuExistbyThisName(a)
        var m = parent.sessionData.pbxinfo.vmgroups;
        for (var l in m)
        {
            var vmgroupsObj = parent.sessionData.pbxinfo.vmgroups[l];
            if (m.hasOwnProperty(l) && vmgroupsObj)
            {
                if (isNEW_VMG == true && vmgroupsObj['label'] == value)
                {
                    return false;
                }
                if (isNEW_VMG == false && l != EDIT_VMG && vmgroupsObj['label'] == value)
                {
                    return false;
                }
            }
        }
        return true;
    },

    save_VMGroup: function ()
    { // vmgroups_miscFunctions.save_VMGroup(exten)

        if (isNEW_VMG)
        {
            var NU_EXT = ASTGUI.getFieldValue('edit_vmgroup_exten');
        }

        var tmp_mailboxes = ASTGUI.selectbox.readAllValues(DOM_select_ringthesechannels);

        var vm_exten = ASTGUI.getFieldValue('edit_vmgroup_exten');
        var a = new ASTGUI.customObject;
        a.label = ASTGUI.getFieldValue('edit_vmgroup_label');
        //	a.mailboxes = ASTGUI.domActions.get_checked( vmg_chkbxClass );
        a.mailboxes = tmp_mailboxes;

        if (isNEW_VMG == false)
        { // delete the previous vmg
            parent.pbx.vm_groups.remove(EDIT_VMG);
        }

        parent.pbx.vm_groups.add(vm_exten, a);

        top.dialog.dialogMessage({
            type: 'success',
            content: $P.lang("LANG815"),
            callback: function ()
            {
                mWindow.location.reload();
            }
        });
    },

    delete_VMG: function (vmgexten)
    { // vmgroups_miscFunctions.delete_VMG(exten)

        top.dialog.dialogConfirm({
            confirmStr: $P.lang("LANG818").format(vmgexten),
            buttons: {
                ok: function ()
                {
                    parent.pbx.vm_groups.remove(vmgexten);
                    setTimeout(function ()
                    {
                        top.dialog.dialogMessage({
                            type: 'success',
                            content: $P.lang("LANG816"),
                            callback: function ()
                            {
                                mWindow.location.reload();
                            }
                        });
                    }, 500);
                }
            }
        });
    }
};
