/*
* Asterisk-GUI  - an Asterisk configuration interface
*
* Javascript functions for accessing manager over HTTP and Some UI components/functions used in AsteriskGUI.
*
* Copyright (C) 2006-2011, Digium, Inc.
*
* Mark Spencer <markster@digium.com>
* Pari Nannapaneni <pari@digium.com>
* Ryan Brindley <rbrindley@digium.com>
* Erin Spiceland <espiceland@digium.com>
*
* Thanks to Steven Levithan (http://stevenlevithan.com) for his
* implementation of trim()
*
* See http://www.asterisk.org for more information about
* the Asterisk project. Please do not directly contact
* any of the maintainers of this project for assistance;
* the project provides a web site, mailing lists and IRC
* channels for your use.
*
* This program is free software, distributed under the terms of
* the GNU General Public License Version 2. See the LICENSE file
* at the top of the source tree.
*
*/

/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Added support for local voiceprompts package uploading
 * 
 */
$P = top.$;


var upload_Filename = ""; // will be updated by upload_form.html
var starteduploading = 0;
var upload_Path; // path for 'uploads' as defined in http.conf - this variable will be automatically updated from http.conf

onUploadForm_load = function ()
{
    if (!top.sessionData.httpConf.postmappings_defined || !top.sessionData.httpConf.uploadPaths['voicepromptspackage'])
    {
        $('#uploadForm_container').hide();
        top.dialog.dialogMessage({ type: 'loading', content: 'Creating configuration needed for uploading voice prompts package ...' });
        var tmp_a = top.sessionData.directories.menusRecord.rChop('/');
        var u = new listOfSynActions('http.conf');
        if (!top.sessionData.httpConf.postmappings_defined)
        {
            u.new_action('newcat', 'post_mappings', '', '');
        }
        u.new_action('append', 'post_mappings', 'voicepromptspackage', tmp_a);
        u.callActions();
        top.sessionData.httpConf.postmappings_defined = true;
        top.sessionData.httpConf.uploadPaths['voicepromptspackage'] = tmp_a;

        ASTGUI.cliCommand('reload');
        setTimeout(
			function ()
			{
			    top.dialog.clearDialog();
			    window.location.reload();
			}
		, 1000); // reload page
        return;
    }

    var upload_action_path = (top.sessionData.httpConf.prefix) ? '/' + top.sessionData.httpConf.prefix + '/voicepromptspackage' : '/voicepromptspackage';
    _$('uploadiframe').contentWindow.document.getElementById('form22').action = upload_action_path;
};


onUploadForm_beforeUploading = function ()
{
    var tmp_fname = upload_Filename.toLowerCase();
    if (tmp_fname.endsWith('.tar.bz2') || tmp_fname.endsWith('.tar.gz') || tmp_fname.endsWith('.tar.Z') || tmp_fname.endsWith('.tgz')
	    || tmp_fname.endsWith('.tar') || tmp_fname.endsWith('.bz2') || tmp_fname.endsWith('.zip') || tmp_fname.endsWith('.gz'))
    {
        starteduploading = 1;
        //top.dialog.dialogMessage({ type: 'loading', content: 'File Upload in progress, please wait ..' });
        top.dialog.dialogMessage({ type: 'upload', callback: onProgress});
        return true;
    } else
    {
        starteduploading = 0;
        top.dialog.dialogMessage({ type: 'warning', content: $P.lang("LANG908") });
        return false;
    }
};

onUploadForm_unload = function ()
{
    //if (!starteduploading) { return; }

    // $('#uploadForm_container').hide();

    // top.dialog.dialogMessage({
    //     type: 'success',
    //     content: $P.lang("LANG906"),
    //     callback: function ()
    //     {
    //         top.dialog.clearDialog();
    //         window.location.href = 'language.html';
    //     } 
    // });
};
var progressTotal;
onProgress = function(){
    $.ajax({
        type: "GET",
        async: true,
        url: "/webcgi?action=getparam&:up_total",
        error: function ()
        {
            top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG909") });
        },
        success: function (data)
        {
            if (data) {
                var totalPos = data.indexOf("=") + 1;
                var total = data.substring(totalPos);
                if (total.length != 2) {
                    progressTotal = parseInt(total);
                    getReceive();
                   
                }
                else{
                    top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG909") });
                    return;    
                }
            }
            else{
                top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG909") });
                return;    
            }
        }
    });
}
function getReceive(){
    var timer = setTimeout(getReceive, 1000);
    $.ajax({
        type: "GET",
        async: true,
        url: "/webcgi?action=getparam&:up_receive",
        error: function ()
        {
            top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG909") });
        },
        success: function (data)
        {
            if (data) {
                var receivePos = data.indexOf("=") + 1;
                var receive = data.substring(receivePos);
                if (receive.length != 2) {
                    var progressReceive = parseInt(receive);
                    var progress = progressReceive / progressTotal;
                    if(progress == 1){
                        progress = 100;
                    }
                    else{
                        progress = progress + "";
                        var len = progress.indexOf(".") + 1;
                        progress = progress.substring(len);
                        progress = progress.substring(0, 2);
                        if (progress.substring(0, 1) == "0") {
                            progress = progress.substring(1);
                        }
                        progress = parseInt(progress);
                    }
                    $P("#spaceused1").progressBar(progress);
                    if(progress == 100){
                        clearTimeout(timer);
                        setTimeout(function() {
                            top.dialog.clearDialog();
                            top.dialog.dialogMessage({
                                type: 'success',
                                content: $P.lang("LANG906"),
                                callback: function ()
                                {
                                    top.dialog.clearDialog();
                                    window.location.href = 'language.html';
                                } 
                            });
                            $P("#spaceused1").progressBar(1);
                            $P("#spaceused1").hide();
                            return;
                        }, 1000)
                    }
                }
                else{
                    clearTimeout(timer);
                    top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG909") });
                    return;    
                }
            }
            else{
                clearTimeout(timer);
                top.dialog.dialogMessage({ type: 'error', content: $P.lang("LANG909") });
                return;    
            }
        }
    });
}
function localajaxinit()
{
    $P.lang(document, true);
    ASTGUI.events.add(_$('BACKBUTTON'), 'click', function () { window.location.href = 'language.html'; });
    //var url = top.$.url(window.location.href, true);
    var mode = gup("mode");
    if (mode === "upload")
    {
        return false; // bypass reload dialog
    }

};

