/*
 * Description: UCM6100 WebGUI
 * Copyright (C) 2013 Grandstream Networks, Inc.
 *
 * This project extends Asterisk GUI project and contains Grandstream Network's
 * implementation which uses Asterisk GUI.
 *
 * Asterisk and Asterisk GUI are registered trademarks of Digium, Inc. 
 * Please see http://www.asterisk.org for more information about the Asterisk 
 * project.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 * Modifications:
 * Added support for Zeroconfig function
 * 
 */
$P = top.$;

var modulelist = new Array();
var mWindow = top.window.frames["mainscreen"];
window.onload = function ()
{
    $P.lang(document, true);
    $.ajax({ type: "GET", url: "/database.zccgi", data: "DoCgi=ApLoad", async: false, success: load_ap_setting, error: loadDataError });
	if ($("#form").tooltip) {
		$("#form").tooltip();
	}
    $P("#form", document).validate({
        rules:{
            "start_account": {
                required: true,
                digits: true,
                minlength: 2,
                extensionRange: ['ue_start', 'ue_end']
            },
            "start_ext": {
                required: true,
                digits: true,
                minlength: 2,
                extensionRange: ['ue_start', 'ue_end']
            },
            "end_ext": {
                required: true,
                digits: true,
                minlength: 2,
                extensionRange: ['ue_start', 'ue_end'],
                strbigger: [$P.lang("LANG561"),$P.lang("LANG560"),$("#start_ext")]
            },
            "pick_period": {
                required: true,
                number: true,
                min: 0.1,
                max: 72                
            },
            "sip_secret": {
                required: true,
                keyboradNoSpacesemicolon: true,
                minlength: 2  
            }
        },
        submitHandler: function() {
            save_date();
        }
    });
    isEnableWeakPw();
    top.Custom.init(document);
    return false;
}
var isEnableWeakPw = function () {
    var GUI_PREFERENCES = top.sessionData.GUI_PREFERENCES;
    if (!GUI_PREFERENCES.getProperty('weak_password')) {
        GUI_PREFERENCES  = context2json({ filename: 'guipreferences.conf', context: 'general', usf: 1 });
    }
    if (GUI_PREFERENCES.weak_password == "yes") {
        var obj = {
            pwsId: "#sip_secret"
        }    
        $P("#sip_secret", document).rules("add", { 
            required: true,
            checkAlphanumericPw: [ showCheckPassword, obj] 
        });

    }
}
var update = function ()
{
    AskextensionRange(_$("start_account").value, "ue_start", "ue_end");
}

var save_date = function(){
    var autoAssign = _$('auto_assign');
    if (autoAssign.checked && !autoAssign.disabled)
    {
        top.dialog.dialogConfirm({
            confirmStr: $P.lang("LANG849"),
            buttons: {
                ok:  function (){
                    updates();
                    },
                cancel: function (){
                    top.dialog.container.show();
                    top.dialog.shadeDiv.show();
                    }
            }
        });
    }
    else
    {
        updates();
    }
}

var save_enable = function(id)
{
    if (_$(id).disabled == false)
    {
        var value;
        if (_$(id).type == "checkbox")
        {
            value = _$(id).checked ? 1: 0;
        }
        else
        {
            value = encodeURIComponent(_$(id).value);
        }
        return "&" + id + "=" + value;
    }
    else
        return "";
}

var updates = function () {

    var buf = "DoCgi=ApUpdate";
    var a = (_$('zeroconfig').checked == true) ? 1 : 0;
    buf += "&zeroconfig=" + a;
    buf = buf + save_enable("auto_assign");
    buf = buf + save_enable("start_account");
    buf = buf + save_enable("enable_pick");
    buf = buf + save_enable("start_ext");
    buf = buf + save_enable("end_ext");
    buf = buf + save_enable("random_secret");
    buf = buf + save_enable("sip_secret");    
    buf = buf + save_enable("pick_period");

    /*
    var module;
    var module_buf = "";

    for (i = 2; i < modulelist.length - 1; i++)
    {
        module = modulelist[i].split('&');
        a = (_$(module[0]).checked == true) ? 1 : 0;
        module_buf += "&" + module[0] + "=" + a;
    }

    buf += module_buf;
    */
    
    $.ajax({
        type: "get",
        url: "/database.zccgi",
        data: buf,
        error: function (jqXHR, textStatus, errorThrown) { top.dialog.dialogMessage({ type: 'error', content: errorThrown }); },
        success: function(){
            top.dialog.dialogMessage({ type: 'success', content:$P.lang("LANG844")});
        }
    });
    
}

function loadDataError()
{
    load_ap_setting("");
}

function load_module_table()
{
    var TBL = _$('table_modulelist');
    var addCell = ASTGUI.domActions.tr_addCell; // temporarily store the function

    if (modulelist.length < 4)
    {
        ASTGUI.domActions.clear_table(TBL);
        var newRow = TBL.insertRow(-1);
        newRow.className = 'even';
        addCell(newRow, { html: $P.lang("LANG1314") });
        return;
    }
    (function ()
    { // add first row
        var CA = document.createElement('input'); CA.type = 'checkbox'; CA.id = 'CHECKALL';
        ASTGUI.events.add(CA, 'click', function ()
        {
            if (_$('CHECKALL').checked)
            {
                ASTGUI.domActions.CheckAll('selected_module');
            } else
            {
                ASTGUI.domActions.unCheckAll('selected_module');
            }
        });

        var newRow = TBL.insertRow(-1);
        newRow.className = "frow";

        var newcell = newRow.insertCell(newRow.cells.length);
        newcell.width = '15px';
        newcell.appendChild(CA);

        addCell(newRow, { html: $P.lang("LANG1315") });
        addCell(newRow, { html: $P.lang("LANG1298") });
    })();

    for (i = 2; i < modulelist.length - 1; i++)
    {
        module_list(modulelist[i]);
    }

    function module_list(buf)
    { // list each user in table

        var module = buf.split("&");

        if (!module[0])
        {
            return false;
        }

        var newRow = TBL.insertRow(-1); //newRow.className = "frow";
        newRow.className = ((TBL.rows.length) % 2 == 1) ? 'odd' : 'even';

        addCell(newRow, { html: "<input type=checkbox  class='selected_module' id='" + module[0] + "'>" });
        addCell(newRow, { html: module[0] });
        addCell(newRow, { html: module[1] ? module[1] : '--' });
        _$(module[0]).checked = (module[2] == 1) ? true : false;
    }
    disablecheck();
}

function load_ap_setting(tmp)
{
    if (tmp && tmp.response == "0" && tmp.body)
    {
        var res = tmp.body;
        _$('zeroconfig').checked = (res.zeroconfig == "1") ? true : false;
        _$('auto_assign').checked = (res.auto_assign == "1") ? true : false;
        var enable_pick = _$('enable_pick').checked = (res.enable_pick == "1") ? true : false;              

        if (res.start_ext)
        {
            _$('start_ext').value = res.start_ext;
        }

        if (res.end_ext)
        {
            _$('end_ext').value = res.end_ext;
        }
        
        if (res.start_account)
        {
            _$('start_account').value = res.start_account;
        }
        
        if (res.pick_period)
        {
            _$('pick_period').value = res.pick_period;
        }
        else
        {
            _$('pick_period').value = "";
        }
        
        if (res.pick_until_time)
        {
             _$('warning_time').innerHTML = $P.lang("LANG2671") + res.pick_until_time;
             $('#warning_div').show();
        }

        _$('random_secret').checked = (res.random_secret == "1") ? true : false;

        if (res.sip_secret )
        {
            _$('sip_secret').value = decodeURIComponent(res.sip_secret);
        }
        //modulelist = tmp.split("\r\n");
    }
    //else
        //modulelist = [];
    //load_module_table();
    disablecheck();
}

function disablecheck()
{
    var module;
    var checked = !_$("zeroconfig").checked;

    /*
    for (i = 2; i < modulelist.length - 1; i++)
    {
        module = modulelist[i].split("&");
        _$(module[0]).disabled = checked;
    }
    
    _$('CHECKALL').disabled = checked;
    */
    
    enableCheckbox('random_secret', checked);
    enableCheckbox('auto_assign', checked);
    enableCheckbox('enable_pick', checked);
    enableCheckbox('start_ext', checked);
    enableCheckbox('end_ext', checked);    
    
    if (_$('auto_assign').disabled == false && _$('auto_assign').checked == true)
    {
        enableCheckbox('start_account', false);
    }
    else
    {
        enableCheckbox('start_account', true);
    }
        
    if (_$('auto_assign').disabled == false && _$('auto_assign').checked == true || _$('enable_pick').disabled == false && _$('enable_pick').checked == true)
    {
        enableCheckbox('random_secret', false);
    }
    else
    {
        enableCheckbox('random_secret', true);
    }

    if (_$('enable_pick').disabled == false && _$('enable_pick').checked == true)
    {
        enableCheckbox('start_ext', false);
        enableCheckbox('end_ext', false);         
        enableCheckbox('pick_period', false);       
    }
    else
    {
        enableCheckbox('start_ext', true);
        enableCheckbox('end_ext', true);        
        enableCheckbox('pick_period', true);  
    }    
    
    if (_$('random_secret').disabled == false && _$('random_secret').checked == false)
    {
        enableCheckbox('sip_secret', false);
    }
    else
    {
        enableCheckbox('sip_secret', true);
    }
}
function enableCheckbox(el, isChecked) {
    _$(el).disabled = isChecked;
    var el = _$(el);
    var span = el.previousSibling;
    if(span) {
        if (span.tagName != "DIV") {
            return false;
        }
        if (!isChecked) {
            if (el.tagName == "INPUT") {
                if (el.checked) {
                    span.style.backgroundPosition = "0 -50px";
                }
                else {
                    span.style.backgroundPosition = "0 0";
                }
                span.onmouseover = top.Custom.pushed;
                span.onmouseout = top.Custom.unpushed;
                span.onmouseup = top.Custom.check;
            }
            $(span).removeClass("disabled");
        }
        else {
            if (el.tagName == "INPUT") {
                if (el.checked) {
                    span.style.backgroundPosition = "0 -100px";
                }
                else {
                    span.style.backgroundPosition = "0 -125px";
                }
                span.onmouseover = null;
                span.onmouseout = null;
                span.onmouseup = null;
            }
            else {
                span.style.backgroundPosition = "0 -44px";
                $(el).unbind();
            }
            $(span).addClass("disabled");
        }
    }
}